// $ANTLR 3.4 com/senseidb/bql/parsers/BQL.g 2013-03-24 21:11:43

package com.senseidb.bql.parsers;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;
import org.json.JSONObject;
import org.json.JSONArray;
import org.json.JSONException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import com.senseidb.util.JSONUtil.FastJSONArray;
import com.senseidb.util.JSONUtil.FastJSONObject;
import com.senseidb.util.Pair;
import com.senseidb.search.req.BQLParserUtils;


import org.antlr.runtime.*;
import java.util.Stack;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

import org.antlr.runtime.tree.*;


/*******************************************************************************************

BNF Grammar for BQL
===================

<statement> ::= ( <select_stmt> | <describe_stmt> ) [';']

<select_stmt> ::= SELECT <select_list> [<from_clause>] [<where_clause>] [<given_clause>]
                  [<additional_clauses>]

<describe_stmt> ::= ( DESC | DESCRIBE ) [<index_name>]

<select_list> ::= '*' | (<column_name>|<aggregation_function>)( ',' <column_name>|<aggregation_function> )*

<column_name_list> ::= <column_name> ( ',' <column_name> )*

<aggregation_function> ::= <function_name> '(' <column_name> ')'

<function_name> ::= <column_name>

<from_clause> ::= FROM <index_name>

<index_name> ::= <identifier> | <quoted_string>

<where_clause> ::= WHERE <search_expr>

<search_expr> ::= <term_expr> ( OR <term_expr> )*

<term_expr> ::= <facet_expr> ( AND <facet_expr> )*

<facet_expr> ::= <predicate> 
               | '(' <search_expr> ')'

<predicates> ::= <predicate> ( AND <predicate> )*

<predicate> ::= <in_predicate>
              | <contains_all_predicate>
              | <equal_predicate>
              | <not_equal_predicate>
              | <query_predicate>
              | <between_predicate>
              | <range_predicate>
              | <time_predicate>
              | <match_predicate>
              | <like_predicate>
              | <null_predicate>

<in_predicate> ::= <column_name> [NOT] IN <value_list> [<except_clause>] [<predicate_props>]

<contains_all_predicate> ::= <column_name> CONTAINS ALL <value_list> [<except_clause>]
                             [<predicate_props>]

<equal_predicate> ::= <column_name> '=' <value> [<predicate_props>]

<not_equal_predicate> ::= <column_name> '<>' <value> [<predicate_props>]

<query_predicate> ::= QUERY IS <quoted_string>

<between_predicate> ::= <column_name> [NOT] BETWEEN <value> AND <value>

<range_predicate> ::= <column_name> <range_op> <numeric>

<time_predicate> ::= <column_name> IN LAST <time_span>
                   | <column_name> ( SINCE | AFTER | BEFORE ) <time_expr>

<match_predicate> ::= [NOT] MATCH '(' <column_name_list> ')' AGAINST '(' <quoted_string> ')'

<like_predicate> ::= <column_name> [NOT] LIKE <quoted_string>

<null_predicate> ::= <column_name> IS [NOT] NULL

<value_list> ::= <non_variable_value_list> | <variable>

<non_variable_value_list> ::= '(' <value> ( ',' <value> )* ')'

<python_style_list> ::= '[' <python_style_value>? ( ',' <python_style_value> )* ']'

<python_style_dict> ::= '{''}' 
                       | '{' <key_value_pair> ( ',' <key_value_pair> )* '}'

<python_style_value> ::= <value>
                       | <python_style_list>
                       | <python_style_dict>

<value> ::= <numeric>
          | <quoted_string>
          | TRUE
          | FALSE
          | <variable>

<range_op> ::= '<' | '<=' | '>=' | '>'

<except_clause> ::= EXCEPT <value_list>

<predicate_props> ::= WITH <prop_list>

<prop_list> ::= '(' <key_value_pair> ( ',' <key_value_pair> )* ')'

<key_value_pair> ::= <quoted_string> ':' 
                     ( <value> | <python_style_list> | <python_style_dict> )

<given_clause> ::= GIVEN FACET PARAM <facet_param_list>

<facet_param_list> ::= <facet_param> ( ',' <facet_param> )*

<facet_param> ::= '(' <facet_name> <facet_param_name> <facet_param_type> <facet_param_value> ')'

<facet_param_name> ::= <quoted_string>

<facet_param_type> ::= BOOLEAN | INT | LONG | STRING | BYTEARRAY | DOUBLE

<facet_param_value> ::= <quoted_string>

<additional_clauses> ::= ( <additional_clause> )+

<additional_clause> ::= <order_by_clause>
                      | <limit_clause>
                      | <group_by_clause>
                      | <distinct_clause>
                      | <execute_clause>
                      | <browse_by_clause>
                      | <fetching_stored_clause>
                      | <route_by_clause>
                      | <relevance_model_clause>

<order_by_clause> ::= ORDER BY <sort_specs>

<sort_specs> ::= <sort_spec> ( ',' <sort_spec> )*

<sort_spec> ::= <column_name> [<ordering_spec>]

<ordering_spec> ::= ASC | DESC

<group_by_clause> ::= GROUP BY <group_spec>

<distinct_clause> ::= DISTINCT <distinct_spec>

<execute_clause> ::= EXECUTE '(' function_name ((',' python_style_dict) | (',' key_value_pair)*) ')'

<group_spec> ::= <comma_column_name_list> [TOP <max_per_group>]

<distinct_spec> ::= <or_column_name_list>

<or_column_name_list> ::= <column_name> ( OR <column_name> )*

<comma_column_name_list> ::= <column_name> ( (OR | ',') <column_name> )*

<limit_clause> ::= LIMIT [<offset> ','] <count>

<offset> ::= ( <digit> )+

<count> ::= ( <digit> )+

<browse_by_clause> ::= BROWSE BY <facet_specs>

<facet_specs> ::= <facet_spec> ( ',' <facet_spec> )*

<facet_spec> ::= <facet_name> [<facet_expression>]

<facet_expression> ::= '(' <expand_flag> <count> <count> <facet_ordering> ')'

<expand_flag> ::= TRUE | FALSE

<facet_ordering> ::= HITS | VALUE

<fetching_stored_clause> ::= FETCHING STORED [<fetching_flag>]

<fetching_flag> ::= TRUE | FALSE

<route_by_clause> ::= ROUTE BY <quoted_string>

<relevance_model_clause> ::= USING RELEVANCE MODEL <identifier> <prop_list>
                             [<relevance_model>]

<relevance_model> ::= DEFINED AS <formal_parameters> BEGIN <model_block> END

<formal_parameters> ::= '(' <formal_parameter_decls> ')'

<formal_parameter_decls> ::= <formal_parameter_decl> ( ',' <formal_parameter_decl> )*

<formal_parameter_decl> ::= <variable_modifiers> <type> <variable_declarator_id>

<variable_modifiers> ::= ( <variable_modifier> )*

<variable_modifier> ::= 'final'

<type> ::= <class_or_interface_type> ('[' ']')*
         | <primitive_type> ('[' ']')*
         | <boxed_type> ('[' ']')*
         | <limited_type> ('[' ']')*

<class_or_interface_type> ::= <fast_util_data_type>

<fast_util_data_type> ::= 'IntOpenHashSet'
                        | 'FloatOpenHashSet'
                        | 'DoubleOpenHashSet'
                        | 'LongOpenHashSet'
                        | 'ObjectOpenHashSet'
                        | 'Int2IntOpenHashMap'
                        | 'Int2FloatOpenHashMap'
                        | 'Int2DoubleOpenHashMap'
                        | 'Int2LongOpenHashMap'
                        | 'Int2ObjectOpenHashMap'
                        | 'Object2IntOpenHashMap'
                        | 'Object2FloatOpenHashMap'
                        | 'Object2DoubleOpenHashMap'
                        | 'Object2LongOpenHashMap'
                        | 'Object2ObjectOpenHashMap'

<primitive_type> ::= 'boolean' | 'char' | 'byte' | 'short' 
                   | 'int' | 'long' | 'float' | 'double'

<boxed_type> ::= 'Boolean' | 'Character' | 'Byte' | 'Short' 
               | 'Integer' | 'Long' | 'Float' | 'Double'

<limited_type> ::= 'String' | 'System' | 'Math'

<model_block> ::= ( <block_statement> )+

<block_statement> ::= <local_variable_declaration_stmt>
                    | <java_statement>

<local_variable_declaration_stmt> ::= <local_variable_declaration> ';'

<local_variable_declaration> ::= <variable_modifiers> <type> <variable_declarators>

<java_statement> ::= <block>
                   | 'if' <par_expression> <java_statement> [ <else_statement> ]
                   | 'for' '(' <for_control> ')' <java_statement>
                   | 'while' <par_expression> <java_statement>
                   | 'do' <java_statement> 'while> <par_expression> ';'
                   | 'switch' <par_expression> '{' <switch_block_statement_groups> '}'
                   | 'return' <expression> ';'
                   | 'break' [<identifier>] ';'
                   | 'continue' [<identifier>] ';'
                   | ';'
                   | <statement_expression> ';'

<block> ::= '{' ( <block_statement> )* '}'

<else_statement> ::= 'else' <java_statement>

<switch_block_statement_groups> ::= ( <switch_block_statement_group> )*

<switch_block_statement_group> ::= ( <switch_label> )+ ( <block_statement> )*

<switch_label> ::= 'case' <constant_expression> ':'
                 | 'case' <enum_constant_name> ':'
                 | 'default' ':'

<for_control> ::= <enhanced_for_control>
                | [<for_init>] ';' [<expression>] ';' [<for_update>]

<for_init> ::= <local_variable_declaration>
             | <expression_list>

<enhanced_for_control> ::= <variable_modifiers> <type> <identifier> ':' <expression>

<for_update> ::= <expression_list>

<par_expression> ::= '(' <expression> ')'

<expression_list> ::= <expression> ( ',' <expression> )*

<statement_expression> ::= <expression>

<constant_expression> ::= <expression>

<enum_constant_name> ::= <identifier>

<variable_declarators> ::= <variable_declarator> ( ',' <variable_declarator> )*

<variable_declarator> ::= <variable_declarator_id> '=' <variable_initializer>

<variable_declarator_id> ::= <identifier> ('[' ']')*

<variable_initializer> ::= <array_initializer>
                         | <expression>

<array_initializer> ::= '{' [ <variable_initializer> ( ',' <variable_initializer> )* [','] ] '}'

<expression> ::= <conditional_expression> [ <assignment_operator> <expression> ]

<assignment_operator> ::= '=' | '+=' | '-=' | '*=' | '/=' | '&=' | '|=' | '^=' |
                        | '%=' | '<<=' | '>>>=' | '>>='

<conditional_expression> ::= <conditional_or_expression> [ '?' <expression> ':' <expression> ]

<conditional_or_expression> ::= <conditional_and_expression> ( '||' <conditional_and_expression> )*

<conditional_and_expression> ::= <inclusive_or_expression> ('&&' <inclusive_or_expression> )*

<inclusive_or_expression> ::= <exclusive_or_expression> ('|' <exclusive_or_expression> )*

<exclusive_or_expression> ::= <and_expression> ('^' <and_expression> )*

<and_expression> ::= <equality_expression> ( '&' <equality_expression> )*

<equality_expression> ::= <instanceof_expression> ( ('==' | '!=') <instanceof_expression> )*

<instanceof_expression> ::= <relational_expression> [ 'instanceof' <type> ]

<relational_expression> ::= <shift_expression> ( <relational_op> <shift_expression> )*

<shift_expression> ::= <additive_expression> ( <shift_op> <additive_expression> )*

<relational_op> ::= '<=' | '>=' | '<' | '>'

<shift_op> ::= '<<' | '>>>' | '>>'

<additive_expression> ::= <multiplicative_expression> ( ('+' | '-') <multiplicative_expression> )*

<multiplicative_expression> ::= <unary_expression> ( ( '*' | '/' | '%' ) <unary_expression> )*

<unary_expression> ::= '+' <unary_expression>
                     | '-' <unary_expression>
                     | '++' <unary_expression>
                     | '--' <unary_expression>
                     | <unary_expression_not_plus_minus>

<unary_expression_not_plus_minus> ::= '~' <unary_expression>
                                    | '!' <unary_expression>
                                    | <cast_expression>
                                    | <primary> <selector>* [ ('++'|'--') ]

<cast_expression> ::= '(' <primitive_type> ')' <unary_expression>
                    | '(' (<type> | <expression>) ')' <unary_expression_not_plus_minus>

<primary> ::= <par_expression>
            | <literal>
            | java_method identifier_suffix
            | <java_ident> ('.' <java_method>)* [<identifier_suffix>]

<java_ident> ::= <boxed_type>
               | <limited_type>
               | <identifier>

<java_method> ::= <identifier>

<identifier_suffix> ::= ('[' ']')+ '.' 'class'
                      | <arguments>
                      | '.' 'class'
                      | '.' 'this'
                      | '.' 'super' <arguments>

<literal> ::= <integer>
            | <real>
            | <floating_point_literal>
            | <character_literal>
            | <quoted_string>
            | <boolean_literal>
            | 'null'

<boolean_literal> ::= 'true' | 'false'

<selector> ::= '.' <identifier> <arguments>
             | '.' 'this'
             | '[' <expression> ']'

<arguments> ::= '(' [<expression_list>] ')'

<quoted_string> ::= '"' ( <char> )* '"'
                  | "'" ( <char> )* "'"

<identifier> ::= <identifier_start> ( <identifier_part> )*

<identifier_start> ::= <alpha> | '-' | '_'

<identifier_part> ::= <identifier_start> | <digit>

<variable> ::= '$' ( <alpha> | <digit> | '_' )+

<column_name> ::= <identifier> | <quoted_string>

<facet_name> ::= <identifier>

<alpha> ::= <alpha_lower_case> | <alpha_upper_case>

<alpha_upper_case> ::= A | B | C | D | E | F | G | H | I | J | K | L | M | N | O
                     | P | Q | R | S | T | U | V | W | X | Y | Z

<alpha_lower_case> ::= a | b | c | d | e | f | g | h | i | j | k | l | m | n | o
                     | p | q | r | s | t | u | v | w | x | y | z

<digit> ::= 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9

<numeric> ::= <time_expr> 
            | <integer>
            | <real>

<integer> ::= ( <digit> )+

<real> ::= ( <digit> )+ '.' ( <digit> )+

<time_expr> ::= <time_span> AGO
              | <date_time_string>
              | NOW

<time_span> ::= [<time_week_part>] [<time_day_part>] [<time_hour_part>]
                [<time_minute_part>] [<time_second_part>] [<time_millisecond_part>]

<time_week_part> ::= <integer> ( 'week' | 'weeks' )

<time_day_part>  ::= <integer> ( 'day'  | 'days' )

<time_hour_part> ::= <integer> ( 'hour' | 'hours' )

<time_minute_part> ::= <integer> ( 'minute' | 'minutes' | 'min' | 'mins')

<time_second_part> ::= <integer> ( 'second' | 'seconds' | 'sec' | 'secs')

<time_millisecond_part> ::= <integer> ( 'millisecond' | 'milliseconds' | 'msec' | 'msecs')

<date_time_string> ::= <date> [<time>]

<date> ::= <digit><digit><digit><digit> ('-' | '/' | '.') <digit><digit>
           ('-' | '/' | '.') <digit><digit>

<time> ::= DIGIT DIGIT ':' DIGIT DIGIT ':' DIGIT DIGIT

*******************************************************************************************/
@SuppressWarnings({"all", "warnings", "unchecked"})
public class BQLParser extends Parser {
    public static final String[] tokenNames = new String[] {
        "<invalid>", "<EOR>", "<DOWN>", "<UP>", "AFTER", "AGAINST", "AGO", "ALL", "ALPHA", "AND", "AND_PRED", "AS", "ASC", "BEFORE", "BEGIN", "BETWEEN", "BOOLEAN", "BROWSE", "BY", "BYTE", "BYTEARRAY", "CHARACTER_LITERAL", "COLON", "COLUMN_LIST", "COMMA", "COMMENT", "CONTAINS", "DATE", "DAYS", "DEFINED", "DESC", "DESCRIBE", "DIGIT", "DISTINCT", "DOUBLE", "ELSE", "EMPTY", "END", "EQUAL", "EQUAL_PRED", "ESCAPE_SEQUENCE", "EXCEPT", "EXECUTE", "EXPONENT", "FACET", "FALSE", "FAST_UTIL_DATA_TYPE", "FETCHING", "FLOATING_POINT_LITERAL", "FLOAT_TYPE_SUFFIX", "FROM", "GIVEN", "GROUP", "GT", "GTE", "HEX_DIGIT", "HEX_LITERAL", "HITS", "HOURS", "IDENT", "IN", "INT", "INTEGER", "INTEGER_TYPE_SUFFIX", "IS", "LAST", "LIKE", "LIMIT", "LINE_COMMENT", "LONG", "LPAR", "LT", "LTE", "MATCH", "MILLISECONDS", "MINS", "MINUTES", "MODEL", "MSECS", "NOT", "NOT_EQUAL", "NOW", "NULL", "OCTAL_ESCAPE", "OCTAL_LITERAL", "OR", "ORDER", "OR_PRED", "PARAM", "QUERY", "RANGE_PRED", "REAL", "RELEVANCE", "ROUTE", "RPAR", "SECONDS", "SECS", "SELECT", "SEMI", "SINCE", "STORED", "STRING", "STRING_LITERAL", "TIME", "TOP", "TRUE", "UNICODE_ESCAPE", "USING", "VALUE", "VARIABLE", "WEEKS", "WHERE", "WITH", "WS", "'!'", "'!='", "'%'", "'%='", "'&&'", "'&'", "'&='", "'*'", "'*='", "'+'", "'++'", "'+='", "'-'", "'--'", "'-='", "'.'", "'/'", "'/='", "'=='", "'?'", "'Character'", "'Float'", "'Integer'", "'Math'", "'Short'", "'String'", "'System'", "'['", "']'", "'^'", "'^='", "'break'", "'case'", "'char'", "'class'", "'continue'", "'default'", "'do'", "'extends'", "'final'", "'float'", "'for'", "'if'", "'instanceof'", "'min'", "'return'", "'short'", "'super'", "'switch'", "'this'", "'while'", "'{'", "'|'", "'|='", "'||'", "'}'", "'~'"
    };

    public static final int EOF=-1;
    public static final int T__114=114;
    public static final int T__115=115;
    public static final int T__116=116;
    public static final int T__117=117;
    public static final int T__118=118;
    public static final int T__119=119;
    public static final int T__120=120;
    public static final int T__121=121;
    public static final int T__122=122;
    public static final int T__123=123;
    public static final int T__124=124;
    public static final int T__125=125;
    public static final int T__126=126;
    public static final int T__127=127;
    public static final int T__128=128;
    public static final int T__129=129;
    public static final int T__130=130;
    public static final int T__131=131;
    public static final int T__132=132;
    public static final int T__133=133;
    public static final int T__134=134;
    public static final int T__135=135;
    public static final int T__136=136;
    public static final int T__137=137;
    public static final int T__138=138;
    public static final int T__139=139;
    public static final int T__140=140;
    public static final int T__141=141;
    public static final int T__142=142;
    public static final int T__143=143;
    public static final int T__144=144;
    public static final int T__145=145;
    public static final int T__146=146;
    public static final int T__147=147;
    public static final int T__148=148;
    public static final int T__149=149;
    public static final int T__150=150;
    public static final int T__151=151;
    public static final int T__152=152;
    public static final int T__153=153;
    public static final int T__154=154;
    public static final int T__155=155;
    public static final int T__156=156;
    public static final int T__157=157;
    public static final int T__158=158;
    public static final int T__159=159;
    public static final int T__160=160;
    public static final int T__161=161;
    public static final int T__162=162;
    public static final int T__163=163;
    public static final int T__164=164;
    public static final int T__165=165;
    public static final int T__166=166;
    public static final int T__167=167;
    public static final int T__168=168;
    public static final int T__169=169;
    public static final int T__170=170;
    public static final int AFTER=4;
    public static final int AGAINST=5;
    public static final int AGO=6;
    public static final int ALL=7;
    public static final int ALPHA=8;
    public static final int AND=9;
    public static final int AND_PRED=10;
    public static final int AS=11;
    public static final int ASC=12;
    public static final int BEFORE=13;
    public static final int BEGIN=14;
    public static final int BETWEEN=15;
    public static final int BOOLEAN=16;
    public static final int BROWSE=17;
    public static final int BY=18;
    public static final int BYTE=19;
    public static final int BYTEARRAY=20;
    public static final int CHARACTER_LITERAL=21;
    public static final int COLON=22;
    public static final int COLUMN_LIST=23;
    public static final int COMMA=24;
    public static final int COMMENT=25;
    public static final int CONTAINS=26;
    public static final int DATE=27;
    public static final int DAYS=28;
    public static final int DEFINED=29;
    public static final int DESC=30;
    public static final int DESCRIBE=31;
    public static final int DIGIT=32;
    public static final int DISTINCT=33;
    public static final int DOUBLE=34;
    public static final int ELSE=35;
    public static final int EMPTY=36;
    public static final int END=37;
    public static final int EQUAL=38;
    public static final int EQUAL_PRED=39;
    public static final int ESCAPE_SEQUENCE=40;
    public static final int EXCEPT=41;
    public static final int EXECUTE=42;
    public static final int EXPONENT=43;
    public static final int FACET=44;
    public static final int FALSE=45;
    public static final int FAST_UTIL_DATA_TYPE=46;
    public static final int FETCHING=47;
    public static final int FLOATING_POINT_LITERAL=48;
    public static final int FLOAT_TYPE_SUFFIX=49;
    public static final int FROM=50;
    public static final int GIVEN=51;
    public static final int GROUP=52;
    public static final int GT=53;
    public static final int GTE=54;
    public static final int HEX_DIGIT=55;
    public static final int HEX_LITERAL=56;
    public static final int HITS=57;
    public static final int HOURS=58;
    public static final int IDENT=59;
    public static final int IN=60;
    public static final int INT=61;
    public static final int INTEGER=62;
    public static final int INTEGER_TYPE_SUFFIX=63;
    public static final int IS=64;
    public static final int LAST=65;
    public static final int LIKE=66;
    public static final int LIMIT=67;
    public static final int LINE_COMMENT=68;
    public static final int LONG=69;
    public static final int LPAR=70;
    public static final int LT=71;
    public static final int LTE=72;
    public static final int MATCH=73;
    public static final int MILLISECONDS=74;
    public static final int MINS=75;
    public static final int MINUTES=76;
    public static final int MODEL=77;
    public static final int MSECS=78;
    public static final int NOT=79;
    public static final int NOT_EQUAL=80;
    public static final int NOW=81;
    public static final int NULL=82;
    public static final int OCTAL_ESCAPE=83;
    public static final int OCTAL_LITERAL=84;
    public static final int OR=85;
    public static final int ORDER=86;
    public static final int OR_PRED=87;
    public static final int PARAM=88;
    public static final int QUERY=89;
    public static final int RANGE_PRED=90;
    public static final int REAL=91;
    public static final int RELEVANCE=92;
    public static final int ROUTE=93;
    public static final int RPAR=94;
    public static final int SECONDS=95;
    public static final int SECS=96;
    public static final int SELECT=97;
    public static final int SEMI=98;
    public static final int SINCE=99;
    public static final int STORED=100;
    public static final int STRING=101;
    public static final int STRING_LITERAL=102;
    public static final int TIME=103;
    public static final int TOP=104;
    public static final int TRUE=105;
    public static final int UNICODE_ESCAPE=106;
    public static final int USING=107;
    public static final int VALUE=108;
    public static final int VARIABLE=109;
    public static final int WEEKS=110;
    public static final int WHERE=111;
    public static final int WITH=112;
    public static final int WS=113;

    // delegates
    public Parser[] getDelegates() {
        return new Parser[] {};
    }

    // delegators


    public BQLParser(TokenStream input) {
        this(input, new RecognizerSharedState());
    }
    public BQLParser(TokenStream input, RecognizerSharedState state) {
        super(input, state);
        this.state.ruleMemo = new HashMap[378+1];
         

    }

protected TreeAdaptor adaptor = new CommonTreeAdaptor();

public void setTreeAdaptor(TreeAdaptor adaptor) {
    this.adaptor = adaptor;
}
public TreeAdaptor getTreeAdaptor() {
    return adaptor;
}
    public String[] getTokenNames() { return BQLParser.tokenNames; }
    public String getGrammarFileName() { return "com/senseidb/bql/parsers/BQL.g"; }



        private static enum KeyType {
          STRING_LITERAL,
          IDENT,
          STRING_LITERAL_AND_IDENT
        }

        private static final int DEFAULT_REQUEST_OFFSET = 0;
        private static final int DEFAULT_REQUEST_COUNT = 10;
        private static final int DEFAULT_REQUEST_MAX_PER_GROUP = 10;
        private static final int DEFAULT_FACET_MINHIT = 1;
        private static final int DEFAULT_FACET_MAXHIT = 10;
        private static final Map<String, String> _fastutilTypeMap;
        private static final Map<String, String> _internalVarMap;
        private static final Map<String, String> _internalStaticVarMap;    
        private static final Set<String> _supportedClasses;
        private static Map<String, Set<String>> _compatibleFacetTypes;

        private Map<String, String[]> _facetInfoMap;
        private long _now;
        private HashSet<String> _variables;
        private SimpleDateFormat[] _format1 = new SimpleDateFormat[2];
        private SimpleDateFormat[] _format2 = new SimpleDateFormat[2];

        private LinkedList<Map<String, String>> _symbolTable;
        private Map<String, String> _currentScope;
        private Set<String> _usedFacets; // Facets used by relevance model
        private Set<String> _usedInternalVars; // Internal variables used by relevance model

        static {
            _fastutilTypeMap = new HashMap<String, String>();
            _fastutilTypeMap.put("IntOpenHashSet", "set_int");
            _fastutilTypeMap.put("FloatOpenHashSet", "set_float");
            _fastutilTypeMap.put("DoubleOpenHashSet", "set_double");
            _fastutilTypeMap.put("LongOpenHashSet", "set_long");
            _fastutilTypeMap.put("ObjectOpenHashSet", "set_string");

            _fastutilTypeMap.put("Int2IntOpenHashMap", "map_int_int");
            _fastutilTypeMap.put("Int2FloatOpenHashMap", "map_int_float");
            _fastutilTypeMap.put("Int2DoubleOpenHashMap", "map_int_double");
            _fastutilTypeMap.put("Int2LongOpenHashMap", "map_int_long");
            _fastutilTypeMap.put("Int2ObjectOpenHashMap", "map_int_string");

            _fastutilTypeMap.put("Object2IntOpenHashMap", "map_string_int");
            _fastutilTypeMap.put("Object2FloatOpenHashMap", "map_string_float");
            _fastutilTypeMap.put("Object2DoubleOpenHashMap", "map_string_double");
            _fastutilTypeMap.put("Object2LongOpenHashMap", "map_string_long");
            _fastutilTypeMap.put("Object2ObjectOpenHashMap", "map_string_string");

            _internalVarMap = new HashMap<String, String>();
            _internalVarMap.put("_NOW", "long");
            _internalVarMap.put("_INNER_SCORE", "float");
            _internalVarMap.put("_RANDOM", "java.util.Random");
            
            _internalStaticVarMap = new HashMap<String, String>();
            _internalStaticVarMap.put("_RANDOM", "java.util.Random");

            _supportedClasses = new HashSet<String>();
            _supportedClasses.add("Boolean");
            _supportedClasses.add("Byte");
            _supportedClasses.add("Character");
            _supportedClasses.add("Double");
            _supportedClasses.add("Integer");
            _supportedClasses.add("Long");
            _supportedClasses.add("Short");

            _supportedClasses.add("Math");
            _supportedClasses.add("String");
            _supportedClasses.add("System");
            
            _compatibleFacetTypes = new HashMap<String, Set<String>>();
            _compatibleFacetTypes.put("range", new HashSet<String>(Arrays.asList(new String[]
                                                                   {
                                                                       "simple",
                                                                       "multi"
                                                                   })));
        }

        public BQLParser(TokenStream input, Map<String, String[]> facetInfoMap)
        {
            this(input);
            _facetInfoMap = facetInfoMap;
            _facetInfoMap.put("_uid", new String[]{"simple", "long"});
        }

        // The following two overridden methods are used to force ANTLR to
        // stop parsing upon the very first error.

        // @Override
        // protected void mismatch(IntStream input, int ttype, BitSet follow)
        //     throws RecognitionException
        // {
        //     throw new MismatchedTokenException(ttype, input);
        // }

        @Override
        protected Object recoverFromMismatchedToken(IntStream input, int ttype, BitSet follow)
            throws RecognitionException
        {
            throw new MismatchedTokenException(ttype, input);
        }

        @Override
        public Object recoverFromMismatchedSet(IntStream input,
                                               RecognitionException e,
                                               BitSet follow)
            throws RecognitionException
        {
            throw e;
        }

        @Override
        public String getErrorMessage(RecognitionException err, String[] tokenNames) 
        {
            List stack = getRuleInvocationStack(err, this.getClass().getName());
            String msg = null; 
            if (err instanceof NoViableAltException) {
                NoViableAltException nvae = (NoViableAltException) err;
                // msg = "No viable alt; token=" + err.token.getText() +
                //     " (decision=" + nvae.decisionNumber +
                //     " state "+nvae.stateNumber+")" +
                //     " decision=<<" + nvae.grammarDecisionDescription + ">>";
                msg = "[line:" + err.line + ", col:" + err.charPositionInLine + "] " +
                    "No viable alternative (token=" + err.token.getText() + ")" + " (stack=" + stack + ")";
            }
            else if (err instanceof MismatchedTokenException) {
                MismatchedTokenException mte = (MismatchedTokenException) err;
                String tokenName = (mte.expecting == Token.EOF) ? "EOF" : tokenNames[mte.expecting];
                msg = "[line:" + mte.line + ", col:" + mte.charPositionInLine + "] " +
                    "Expecting " + tokenName +
                    " (token=" + err.token.getText() + ")";
            }
            else if (err instanceof FailedPredicateException) {
                FailedPredicateException fpe = (FailedPredicateException) err;
                msg = "[line:" + fpe.line + ", col:" + fpe.charPositionInLine + "] " +
                    fpe.predicateText +
                    " (token=" + fpe.token.getText() + ")";
            }
            else if (err instanceof MismatchedSetException) {
                MismatchedSetException mse = (MismatchedSetException) err;
                msg = "[line:" + mse.line + ", col:" + mse.charPositionInLine + "] " +
                    "Mismatched input (token=" + mse.token.getText() + ")";
            }
            else {
                msg = super.getErrorMessage(err, tokenNames); 
            }
            return msg;
        } 

        @Override
        public String getTokenErrorDisplay(Token t)
        {
            return t.toString();
        }

        private String predType(JSONObject pred)
        {
            return (String) pred.keys().next();
        }

        private String predField(JSONObject pred) throws JSONException
        {
            String type = (String) pred.keys().next();
            JSONObject fieldSpec = pred.getJSONObject(type);
            return (String) fieldSpec.keys().next();
        }

        private boolean verifyFacetType(final String field, final String expectedType)
        {
            String[] facetInfo = _facetInfoMap.get(field);
            if (facetInfo != null) {
                Set<String> compatibleTypes = _compatibleFacetTypes.get(expectedType);
                return (expectedType.equals(facetInfo[0]) ||
                        "custom".equals(facetInfo[0]) ||
                        (compatibleTypes != null && compatibleTypes.contains(facetInfo[0])));
            }
            else {
                return true;
            }
        }

        private boolean verifyValueType(Object value, final String columnType)
        {
            if (value instanceof String &&
                !((String) value).isEmpty() &&
                ((String) value).matches("\\$[^$].*"))
            {
                // This "value" is a variable, return true always
                return true;
            }

            if (columnType.equals("long") || columnType.equals("aint") || columnType.equals("int") || columnType.equals("short")) {
                return !(value instanceof Float || value instanceof String || value instanceof Boolean);
            }
            else if (columnType.equals("float")  || columnType.equals("int") || columnType.equals("double")) {
                return !(value instanceof String || value instanceof Boolean);
            }
            else if (columnType.equals("string") || columnType.equals("char")) {
                return (value instanceof String);
            }
            else if (columnType.equals("boolean")) {
                return (value instanceof Boolean);
            }
            else if (columnType.isEmpty()) {
                // For a custom facet, the data type is unknown (empty
                // string).  We accept all value types here.
                return true;
            }
            else {
                return false;
            }
        }

        private boolean verifyFieldDataType(final String field, Object value)
            throws FailedPredicateException
        {
            String[] facetInfo = _facetInfoMap.get(field);

            if (value instanceof String &&
                !((String) value).isEmpty() &&
                ((String) value).matches("\\$[^$].*"))
            {
                // This "value" is a variable, return true always
                return true;
            }
            else if (value instanceof JSONArray)
            {
                try {
                    if (facetInfo != null) {
                        String columnType = facetInfo[1];
                        for (int i = 0; i < ((JSONArray) value).length(); ++i) {
                            if (!verifyValueType(((JSONArray) value).get(i), columnType)) {
                                return false;
                            }
                        }
                    }
                    return true;
                }
                catch (JSONException err) {
                    throw new FailedPredicateException(input, "verifyFieldDataType", "JSONException: " + err.getMessage());
                }
            }
            else
            {
                if (facetInfo != null) {
                    return verifyValueType(value, facetInfo[1]);
                }
                else {
                    // Field is not a facet
                    return true;
                }
            }
        }

        private boolean verifyFieldDataType(final String field, Object[] values)
        {
            String[] facetInfo = _facetInfoMap.get(field);
            if (facetInfo != null) {
                String columnType = facetInfo[1];
                for (Object value: values) {
                    if (!verifyValueType(value, columnType)) {
                        return false;
                    }
                }
            }
            return true;
        }

        private void extractSelectionInfo(JSONObject where,
                                          JSONArray selections,
                                          JSONObject filter,
                                          JSONObject query) throws JSONException {

            JSONObject queryPred = where.optJSONObject("query");
            JSONArray andPreds = null;

            if (queryPred != null) {
                query.put("query", queryPred);
            }
            else if ((andPreds = where.optJSONArray("and")) != null) {
                JSONArray filter_list = new FastJSONArray();
                for (int i = 0; i < andPreds.length(); ++i) {
                    JSONObject pred = andPreds.getJSONObject(i);
                    queryPred = pred.optJSONObject("query");
                    if (queryPred != null) {
                        if (!query.has("query")) {
                            query.put("query", queryPred);
                        }
                        else {
                            filter_list.put(pred);
                        }
                    }
                    else if (pred.has("and") || pred.has("or") || pred.has("isNull")) {
                        filter_list.put(pred);
                    }
                    else {
                        String[] facetInfo = _facetInfoMap.get(predField(pred));
                        if (facetInfo != null) {
                            if ("range".equals(predType(pred)) && !"range".equals(facetInfo[0])) {
                                filter_list.put(pred);
                            }
                            else {
                                selections.put(pred);
                            }
                        }
                        else {
                            filter_list.put(pred);
                        }
                    }
                }
                if (filter_list.length() > 1) {
                    filter.put("filter", new FastJSONObject().put("and", filter_list));
                }
                else if (filter_list.length() == 1) {
                    filter.put("filter", filter_list.get(0));
                }
            }
            else if (where.has("or") || where.has("isNull")) {
                filter.put("filter", where);
            }
            else {
                String[] facetInfo = _facetInfoMap.get(predField(where));
                if (facetInfo != null) {
                    if ("range".equals(predType(where)) && !"range".equals(facetInfo[0])) {
                        filter.put("filter", where);
                    }
                    else {
                        selections.put(where);
                    }
                }
                else {
                    filter.put("filter", where);
                }
            }
        }

        private int compareValues(Object v1, Object v2)
        {
            if (v1 instanceof String) {
                return ((String) v1).compareTo((String) v2);
            }
            else if (v1 instanceof Integer) {
                return ((Integer) v1).compareTo((Integer) v2);
            }
            else if (v1 instanceof Long) {
                return ((Long) v1).compareTo((Long) v2);
            }
            else if (v1 instanceof Float) {
                return ((Float) v1).compareTo((Float) v2);
            }
            return 0;
        }

        private Object[] getMax(Object value1, boolean include1, Object value2, boolean include2)
        {
            Object value;
            Boolean include;
            if (value1 == null) {
                value = value2;
                include = include2;
            }
            else if (value2 == null) {
                value = value1;
                include = include1;
            }
            else {
                int comp = compareValues(value1, value2);
                if (comp > 0) {
                    value = value1;
                    include = include1;
                }
                else if (comp == 0) {
                    value = value1;
                    include = (include1 && include2);
                }
                else {
                    value = value2;
                    include = include2;
                }
            }
            return new Object[]{value, include};
        }

        private Object[] getMin(Object value1, boolean include1, Object value2, boolean include2)
        {
            Object value;
            Boolean include;
            if (value1 == null) {
                value = value2;
                include = include2;
            }
            else if (value2 == null) {
                value = value1;
                include = include1;
            }
            else {
                int comp = compareValues(value1, value2);
                if (comp > 0) {
                    value = value2;
                    include = include2;
                }
                else if (comp == 0) {
                    value = value1;
                    include = (include1 && include2);
                }
                else {
                    value = value1;
                    include = include1;
                }
            }
            return new Object[]{value, include};
        }

        private void accumulateRangePred(TokenStream input, JSONObject fieldMap, JSONObject pred) 
            throws JSONException, RecognitionException
        {
            String field = predField(pred);
            if (!fieldMap.has(field)) {
                fieldMap.put(field, pred);
                return;
            }
            JSONObject oldRange = (JSONObject) fieldMap.get(field);
            JSONObject oldSpec = (JSONObject) ((JSONObject) oldRange.get("range")).get(field);
            Object oldFrom = oldSpec.opt("from");
            Object oldTo = oldSpec.opt("to");
            Boolean oldIncludeLower = oldSpec.optBoolean("include_lower", false);
            Boolean oldIncludeUpper = oldSpec.optBoolean("include_upper", false);

            JSONObject curSpec = (JSONObject) ((JSONObject) pred.get("range")).get(field);
            Object curFrom = curSpec.opt("from");
            Object curTo = curSpec.opt("to");
            Boolean curIncludeLower = curSpec.optBoolean("include_lower", false);
            Boolean curIncludeUpper = curSpec.optBoolean("include_upper", false);

            Object[] result = getMax(oldFrom, oldIncludeLower, curFrom, curIncludeLower);
            Object newFrom = result[0];
            Boolean newIncludeLower = (Boolean) result[1];
            result = getMin(oldTo, oldIncludeUpper, curTo, curIncludeUpper);
            Object newTo = result[0];
            Boolean newIncludeUpper = (Boolean) result[1];

            if (newFrom != null && newTo != null && !newFrom.toString().startsWith("$")&& !newTo.toString().startsWith("$")) {
                if (compareValues(newFrom, newTo) > 0 ||
                    (compareValues(newFrom, newTo) == 0) && (!newIncludeLower || !newIncludeUpper)) {
                    // This error is in general detected late, so the token
                    // can be a little off, but hopefully the col index info
                    // is good enough.
                    throw new FailedPredicateException(input, "", "Inconsistent ranges detected for column: " + field);
                }
            }
            
            JSONObject newSpec = new FastJSONObject();
            if (newFrom != null) {
                newSpec.put("from", newFrom);
                newSpec.put("include_lower", newIncludeLower);
            }
            if (newTo != null) {
                newSpec.put("to", newTo);
                newSpec.put("include_upper", newIncludeUpper);
            }

            fieldMap.put(field, new FastJSONObject().put("range",
                                                         new FastJSONObject().put(field, newSpec)));
        }
        
        private void processRelevanceModelParam(TokenStream input,
                                                JSONObject json,
                                                Set<String> params,
                                                String typeName,
                                                final String varName)
            throws JSONException, RecognitionException
        {
            if (_facetInfoMap.containsKey(varName)) {
                throw new FailedPredicateException(input, "", "Facet name \"" + varName + "\" cannot be used as a relevance model parameter.");
            }

            if (_internalVarMap.containsKey(varName)) {
                throw new FailedPredicateException(input, "", "Internal variable \"" + varName + "\" cannot be used as a relevance model parameter.");
            }

            if (params.contains(varName)) {
                throw new FailedPredicateException(input, "", "Parameter name \"" + varName + "\" has already been used.");
            }

            if ("String".equals(typeName)) {
                typeName = "string";
            }

            JSONArray funcParams = json.optJSONArray("function_params");
            if (funcParams == null) {
                funcParams = new FastJSONArray();
                json.put("function_params", funcParams);
            }

            funcParams.put(varName);
            params.add(varName);

            JSONObject variables = json.optJSONObject("variables");
            if (variables == null) {
                variables = new FastJSONObject();
                json.put("variables", variables);
            }

            JSONArray varsWithSameType = variables.optJSONArray(typeName);
            if (varsWithSameType == null) {
                varsWithSameType = new FastJSONArray();
                variables.put(typeName, varsWithSameType);
            }
            varsWithSameType.put(varName);
        }

        // Check whether a variable is defined.
        private boolean verifyVariable(final String variable) {
            Iterator<Map<String, String>> itr = _symbolTable.descendingIterator();
            while (itr.hasNext()) {
                Map<String, String> scope = itr.next();
                if (scope.containsKey(variable)) {
                    return true;
                }
            }
            return false;
        }


    public static class statement_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "statement"
    // com/senseidb/bql/parsers/BQL.g:1214:1: statement returns [Object json] : ( select_stmt | describe_stmt ) ( SEMI )? EOF ;
    public final BQLParser.statement_return statement() throws RecognitionException {
        BQLParser.statement_return retval = new BQLParser.statement_return();
        retval.start = input.LT(1);

        int statement_StartIndex = input.index();

        Object root_0 = null;

        Token SEMI3=null;
        Token EOF4=null;
        BQLParser.select_stmt_return select_stmt1 =null;

        BQLParser.describe_stmt_return describe_stmt2 =null;


        Object SEMI3_tree=null;
        Object EOF4_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 1) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1215:5: ( ( select_stmt | describe_stmt ) ( SEMI )? EOF )
            // com/senseidb/bql/parsers/BQL.g:1215:9: ( select_stmt | describe_stmt ) ( SEMI )? EOF
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:1215:9: ( select_stmt | describe_stmt )
            int alt1=2;
            switch ( input.LA(1) ) {
            case SELECT:
                {
                alt1=1;
                }
                break;
            case DESCRIBE:
                {
                alt1=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 1, 0, input);

                throw nvae;

            }

            switch (alt1) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1215:13: select_stmt
                    {
                    pushFollow(FOLLOW_select_stmt_in_statement3650);
                    select_stmt1=select_stmt();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, select_stmt1.getTree());

                    if ( state.backtracking==0 ) { retval.json = (select_stmt1!=null?select_stmt1.json:null); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1216:13: describe_stmt
                    {
                    pushFollow(FOLLOW_describe_stmt_in_statement3666);
                    describe_stmt2=describe_stmt();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, describe_stmt2.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1217:13: ( SEMI )?
            int alt2=2;
            switch ( input.LA(1) ) {
                case SEMI:
                    {
                    alt2=1;
                    }
                    break;
            }

            switch (alt2) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1217:13: SEMI
                    {
                    SEMI3=(Token)match(input,SEMI,FOLLOW_SEMI_in_statement3680); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI3_tree = 
                    (Object)adaptor.create(SEMI3)
                    ;
                    adaptor.addChild(root_0, SEMI3_tree);
                    }

                    }
                    break;

            }


            EOF4=(Token)match(input,EOF,FOLLOW_EOF_in_statement3683); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            EOF4_tree = 
            (Object)adaptor.create(EOF4)
            ;
            adaptor.addChild(root_0, EOF4_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 1, statement_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "statement"


    public static class select_stmt_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "select_stmt"
    // com/senseidb/bql/parsers/BQL.g:1220:1: select_stmt returns [Object json] : SELECT ( '*' |cols= selection_list ) ( FROM ( IDENT | STRING_LITERAL ) )? (w= where )? (given= given_clause )? (order_by= order_by_clause |limit= limit_clause |group_by= group_by_clause |distinct= distinct_clause |executeMapReduce= execute_clause |browse_by= browse_by_clause |fetch_stored= fetching_stored_clause |route_param= route_by_clause |rel_model= relevance_model_clause )* ;
    public final BQLParser.select_stmt_return select_stmt() throws RecognitionException {
        BQLParser.select_stmt_return retval = new BQLParser.select_stmt_return();
        retval.start = input.LT(1);

        int select_stmt_StartIndex = input.index();

        Object root_0 = null;

        Token SELECT5=null;
        Token char_literal6=null;
        Token FROM7=null;
        Token set8=null;
        BQLParser.selection_list_return cols =null;

        BQLParser.where_return w =null;

        BQLParser.given_clause_return given =null;

        BQLParser.order_by_clause_return order_by =null;

        BQLParser.limit_clause_return limit =null;

        BQLParser.group_by_clause_return group_by =null;

        BQLParser.distinct_clause_return distinct =null;

        BQLParser.execute_clause_return executeMapReduce =null;

        BQLParser.browse_by_clause_return browse_by =null;

        BQLParser.fetching_stored_clause_return fetch_stored =null;

        BQLParser.route_by_clause_return route_param =null;

        BQLParser.relevance_model_clause_return rel_model =null;


        Object SELECT5_tree=null;
        Object char_literal6_tree=null;
        Object FROM7_tree=null;
        Object set8_tree=null;


            _now = System.currentTimeMillis();
            _variables = new HashSet<String>();
            boolean seenOrderBy = false;
            boolean seenLimit = false;
            boolean seenGroupBy = false;
             boolean seenExecuteMapReduce = false;
            boolean seenDistinct = false;
            boolean seenBrowseBy = false;
            boolean seenFetchStored = false;
            boolean seenRouteBy = false;
            boolean seenRelevanceModel = false;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 2) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1234:5: ( SELECT ( '*' |cols= selection_list ) ( FROM ( IDENT | STRING_LITERAL ) )? (w= where )? (given= given_clause )? (order_by= order_by_clause |limit= limit_clause |group_by= group_by_clause |distinct= distinct_clause |executeMapReduce= execute_clause |browse_by= browse_by_clause |fetch_stored= fetching_stored_clause |route_param= route_by_clause |rel_model= relevance_model_clause )* )
            // com/senseidb/bql/parsers/BQL.g:1234:9: SELECT ( '*' |cols= selection_list ) ( FROM ( IDENT | STRING_LITERAL ) )? (w= where )? (given= given_clause )? (order_by= order_by_clause |limit= limit_clause |group_by= group_by_clause |distinct= distinct_clause |executeMapReduce= execute_clause |browse_by= browse_by_clause |fetch_stored= fetching_stored_clause |route_param= route_by_clause |rel_model= relevance_model_clause )*
            {
            root_0 = (Object)adaptor.nil();


            SELECT5=(Token)match(input,SELECT,FOLLOW_SELECT_in_select_stmt3711); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            SELECT5_tree = 
            (Object)adaptor.create(SELECT5)
            ;
            adaptor.addChild(root_0, SELECT5_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:1234:16: ( '*' |cols= selection_list )
            int alt3=2;
            switch ( input.LA(1) ) {
            case 121:
                {
                alt3=1;
                }
                break;
            case IDENT:
            case STRING_LITERAL:
            case 158:
                {
                alt3=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 3, 0, input);

                throw nvae;

            }

            switch (alt3) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1234:17: '*'
                    {
                    char_literal6=(Token)match(input,121,FOLLOW_121_in_select_stmt3714); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal6_tree = 
                    (Object)adaptor.create(char_literal6)
                    ;
                    adaptor.addChild(root_0, char_literal6_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1234:23: cols= selection_list
                    {
                    pushFollow(FOLLOW_selection_list_in_select_stmt3720);
                    cols=selection_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, cols.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1235:9: ( FROM ( IDENT | STRING_LITERAL ) )?
            int alt4=2;
            switch ( input.LA(1) ) {
                case FROM:
                    {
                    alt4=1;
                    }
                    break;
            }

            switch (alt4) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1235:10: FROM ( IDENT | STRING_LITERAL )
                    {
                    FROM7=(Token)match(input,FROM,FOLLOW_FROM_in_select_stmt3732); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    FROM7_tree = 
                    (Object)adaptor.create(FROM7)
                    ;
                    adaptor.addChild(root_0, FROM7_tree);
                    }

                    set8=(Token)input.LT(1);

                    if ( input.LA(1)==IDENT||input.LA(1)==STRING_LITERAL ) {
                        input.consume();
                        if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                        (Object)adaptor.create(set8)
                        );
                        state.errorRecovery=false;
                        state.failed=false;
                    }
                    else {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        MismatchedSetException mse = new MismatchedSetException(null,input);
                        throw mse;
                    }


                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1236:10: (w= where )?
            int alt5=2;
            switch ( input.LA(1) ) {
                case WHERE:
                    {
                    alt5=1;
                    }
                    break;
            }

            switch (alt5) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1236:10: w= where
                    {
                    pushFollow(FOLLOW_where_in_select_stmt3754);
                    w=where();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, w.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1237:14: (given= given_clause )?
            int alt6=2;
            switch ( input.LA(1) ) {
                case GIVEN:
                    {
                    alt6=1;
                    }
                    break;
            }

            switch (alt6) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1237:14: given= given_clause
                    {
                    pushFollow(FOLLOW_given_clause_in_select_stmt3767);
                    given=given_clause();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, given.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1238:9: (order_by= order_by_clause |limit= limit_clause |group_by= group_by_clause |distinct= distinct_clause |executeMapReduce= execute_clause |browse_by= browse_by_clause |fetch_stored= fetching_stored_clause |route_param= route_by_clause |rel_model= relevance_model_clause )*
            loop7:
            do {
                int alt7=10;
                switch ( input.LA(1) ) {
                case ORDER:
                    {
                    alt7=1;
                    }
                    break;
                case LIMIT:
                    {
                    alt7=2;
                    }
                    break;
                case GROUP:
                    {
                    alt7=3;
                    }
                    break;
                case DISTINCT:
                    {
                    alt7=4;
                    }
                    break;
                case EXECUTE:
                    {
                    alt7=5;
                    }
                    break;
                case BROWSE:
                    {
                    alt7=6;
                    }
                    break;
                case FETCHING:
                    {
                    alt7=7;
                    }
                    break;
                case ROUTE:
                    {
                    alt7=8;
                    }
                    break;
                case USING:
                    {
                    alt7=9;
                    }
                    break;

                }

                switch (alt7) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1238:13: order_by= order_by_clause
            	    {
            	    pushFollow(FOLLOW_order_by_clause_in_select_stmt3786);
            	    order_by=order_by_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, order_by.getTree());

            	    if ( state.backtracking==0 ) { 
            	                    if (seenOrderBy) {
            	                        throw new FailedPredicateException(input, "select_stmt", "ORDER BY clause can only appear once.");
            	                    }
            	                    else {
            	                        seenOrderBy = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 2 :
            	    // com/senseidb/bql/parsers/BQL.g:1247:13: limit= limit_clause
            	    {
            	    pushFollow(FOLLOW_limit_clause_in_select_stmt3819);
            	    limit=limit_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, limit.getTree());

            	    if ( state.backtracking==0 ) { 
            	                    if (seenLimit) {
            	                        throw new FailedPredicateException(input, "select_stmt", "LIMIT clause can only appear once.");
            	                    }
            	                    else {
            	                        seenLimit = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 3 :
            	    // com/senseidb/bql/parsers/BQL.g:1256:13: group_by= group_by_clause
            	    {
            	    pushFollow(FOLLOW_group_by_clause_in_select_stmt3851);
            	    group_by=group_by_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, group_by.getTree());

            	    if ( state.backtracking==0 ) { 
            	                    if (seenGroupBy) {
            	                        throw new FailedPredicateException(input, "select_stmt", "GROUP BY clause can only appear once.");
            	                    }
            	                    else {
            	                        seenGroupBy = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 4 :
            	    // com/senseidb/bql/parsers/BQL.g:1265:13: distinct= distinct_clause
            	    {
            	    pushFollow(FOLLOW_distinct_clause_in_select_stmt3883);
            	    distinct=distinct_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, distinct.getTree());

            	    if ( state.backtracking==0 ) { 
            	                    if (seenDistinct) {
            	                        throw new FailedPredicateException(input, "select_stmt", "DISTINCT clause can only appear once.");
            	                    }
            	                    else {
            	                        seenDistinct = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 5 :
            	    // com/senseidb/bql/parsers/BQL.g:1274:13: executeMapReduce= execute_clause
            	    {
            	    pushFollow(FOLLOW_execute_clause_in_select_stmt3915);
            	    executeMapReduce=execute_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, executeMapReduce.getTree());

            	    if ( state.backtracking==0 ) { 
            	                    if (seenExecuteMapReduce) {
            	                        throw new FailedPredicateException(input, "select_stmt", "EXECUTE clause can only appear once.");
            	                    }
            	                    else {
            	                        seenExecuteMapReduce = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 6 :
            	    // com/senseidb/bql/parsers/BQL.g:1283:13: browse_by= browse_by_clause
            	    {
            	    pushFollow(FOLLOW_browse_by_clause_in_select_stmt3947);
            	    browse_by=browse_by_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, browse_by.getTree());

            	    if ( state.backtracking==0 ) { 
            	                    if (seenBrowseBy) {
            	                        throw new FailedPredicateException(input, "select_stmt", "BROWSE BY clause can only appear once.");
            	                    }
            	                    else {
            	                        seenBrowseBy = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 7 :
            	    // com/senseidb/bql/parsers/BQL.g:1292:13: fetch_stored= fetching_stored_clause
            	    {
            	    pushFollow(FOLLOW_fetching_stored_clause_in_select_stmt3979);
            	    fetch_stored=fetching_stored_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, fetch_stored.getTree());

            	    if ( state.backtracking==0 ) { 
            	                    if (seenFetchStored) {
            	                        throw new FailedPredicateException(input, "select_stmt", "FETCHING STORED clause can only appear once.");
            	                    }
            	                    else {
            	                        seenFetchStored = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 8 :
            	    // com/senseidb/bql/parsers/BQL.g:1301:13: route_param= route_by_clause
            	    {
            	    pushFollow(FOLLOW_route_by_clause_in_select_stmt4011);
            	    route_param=route_by_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, route_param.getTree());

            	    if ( state.backtracking==0 ) {
            	                    if (seenRouteBy) {
            	                        throw new FailedPredicateException(input, "select_stmt", "ROUTE BY clause can only appear once.");
            	                    }
            	                    else {
            	                        seenRouteBy = true;
            	                    }
            	                }

            	    }
            	    break;
            	case 9 :
            	    // com/senseidb/bql/parsers/BQL.g:1310:13: rel_model= relevance_model_clause
            	    {
            	    pushFollow(FOLLOW_relevance_model_clause_in_select_stmt4043);
            	    rel_model=relevance_model_clause();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, rel_model.getTree());

            	    if ( state.backtracking==0 ) {
            	                    if (seenRelevanceModel) {
            	                        throw new FailedPredicateException(input, "select_stmt", "USING RELEVANCE MODEL clause can only appear once.");
            	                    }
            	                    else {
            	                        seenRelevanceModel = true;
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop7;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        JSONObject jsonObj = new FastJSONObject();
                        JSONArray selections = new FastJSONArray();
                        JSONObject filter = new FastJSONObject();
                        JSONObject query = new FastJSONObject();

                        try {
                            JSONObject metaData = new FastJSONObject();
                            if (cols == null) {
                                metaData.put("select_list", new FastJSONArray().put("*"));
                            }
                            else {
                               metaData.put("select_list", (cols!=null?cols.json:null));
                               if ((cols!=null?cols.fetchStored:false)) {
                                   jsonObj.put("fetchStored", true);
                               }
                            }

                            if (_variables.size() > 0)
                            {
                                metaData.put("variables", new FastJSONArray(_variables));
                            }

                            jsonObj.put("meta", metaData);

                            if (order_by != null) {
                                if ((order_by!=null?order_by.isRelevance:false)) {
                                    jsonObj.put("sort", "relevance");
                                }
                                else {
                                    jsonObj.put("sort", (order_by!=null?order_by.json:null));
                                }
                            }
                            if (limit != null) {
                                jsonObj.put("from", (limit!=null?limit.offset:0));
                                jsonObj.put("size", (limit!=null?limit.count:0));
                            }
                            if (group_by != null) {
                                jsonObj.put("groupBy", (group_by!=null?group_by.json:null));
                                
                            }  
                            List<Pair<String, String>> aggregateFunctions = null;
                             if (cols != null) {
                                aggregateFunctions = (cols!=null?cols.aggregationFunctions:null);
                             }
                            
                            
                            if (distinct != null) {
                                jsonObj.put("distinct", (distinct!=null?distinct.json:null));
                            }
                            if (browse_by != null) {
                                jsonObj.put("facets", (browse_by!=null?browse_by.json:null));
                            }
                            if (executeMapReduce != null) {
                               if (group_by != null) {
                                  BQLParserUtils.decorateWithMapReduce(jsonObj, (cols!=null?cols.aggregationFunctions:null), (group_by!=null?group_by.json:null), (executeMapReduce!=null?executeMapReduce.functionName:null), (executeMapReduce!=null?executeMapReduce.properties:null));
                               } else {
                                  BQLParserUtils.decorateWithMapReduce(jsonObj, (cols!=null?cols.aggregationFunctions:null), null, (executeMapReduce!=null?executeMapReduce.functionName:null), (executeMapReduce!=null?executeMapReduce.properties:null));
                               }
                            } else {
                               if (group_by != null) {
                                  BQLParserUtils.decorateWithMapReduce(jsonObj, (cols!=null?cols.aggregationFunctions:null), (group_by!=null?group_by.json:null), null, null);
                               } else {
                                  BQLParserUtils.decorateWithMapReduce(jsonObj, (cols!=null?cols.aggregationFunctions:null), null, null, null);
                               }
                            }
                            if (fetch_stored != null) {
                                if (!(fetch_stored!=null?fetch_stored.val:false) && (cols != null && (cols!=null?cols.fetchStored:false))) {
                                    throw new FailedPredicateException(input, 
                                                                       "select_stmt",
                                                                       "FETCHING STORED cannot be false when _srcdata is selected.");
                                }
                                else if ((fetch_stored!=null?fetch_stored.val:false)) {
                                    // Default is false
                                    jsonObj.put("fetchStored", (fetch_stored!=null?fetch_stored.val:false));
                                }
                            }
                            if (route_param != null) {
                                jsonObj.put("routeParam", (route_param!=null?route_param.val:null));
                            }

                            if (w != null) {
                                extractSelectionInfo((JSONObject) (w!=null?w.json:null), selections, filter, query);
                                JSONObject queryPred = query.optJSONObject("query");
                                if (queryPred != null) {
                                    jsonObj.put("query", queryPred);
                                }
                                if (selections.length() > 0) {
                                    jsonObj.put("selections", selections);
                                }
                                JSONObject f = filter.optJSONObject("filter");
                                if (f != null) {
                                    jsonObj.put("filter", f);
                                }
                            }

                            if (given != null) {
                                jsonObj.put("facetInit", (given!=null?given.json:null));
                            }

                            if (rel_model != null) {
                                JSONObject queryPred = jsonObj.optJSONObject("query");
                                if (queryPred == null) {
                                    queryPred = new FastJSONObject().put("query_string",
                                                                         new FastJSONObject().put("query", "")
                                                                                             .put("relevance", (rel_model!=null?rel_model.json:null)));
                                    jsonObj.put("query", queryPred);
                                }
                            }
                            
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "select_stmt", "JSONException: " + err.getMessage());
                        }

                        retval.json = jsonObj;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 2, select_stmt_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "select_stmt"


    public static class describe_stmt_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "describe_stmt"
    // com/senseidb/bql/parsers/BQL.g:1441:1: describe_stmt : DESCRIBE ( IDENT | STRING_LITERAL ) ;
    public final BQLParser.describe_stmt_return describe_stmt() throws RecognitionException {
        BQLParser.describe_stmt_return retval = new BQLParser.describe_stmt_return();
        retval.start = input.LT(1);

        int describe_stmt_StartIndex = input.index();

        Object root_0 = null;

        Token DESCRIBE9=null;
        Token set10=null;

        Object DESCRIBE9_tree=null;
        Object set10_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 3) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1442:5: ( DESCRIBE ( IDENT | STRING_LITERAL ) )
            // com/senseidb/bql/parsers/BQL.g:1442:9: DESCRIBE ( IDENT | STRING_LITERAL )
            {
            root_0 = (Object)adaptor.nil();


            DESCRIBE9=(Token)match(input,DESCRIBE,FOLLOW_DESCRIBE_in_describe_stmt4107); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            DESCRIBE9_tree = 
            (Object)adaptor.create(DESCRIBE9)
            ;
            adaptor.addChild(root_0, DESCRIBE9_tree);
            }

            set10=(Token)input.LT(1);

            if ( input.LA(1)==IDENT||input.LA(1)==STRING_LITERAL ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                (Object)adaptor.create(set10)
                );
                state.errorRecovery=false;
                state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 3, describe_stmt_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "describe_stmt"


    public static class selection_list_return extends ParserRuleReturnScope {
        public boolean fetchStored;
        public JSONArray json;
        public List<Pair<String, String>> aggregationFunctions;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "selection_list"
    // com/senseidb/bql/parsers/BQL.g:1445:1: selection_list returns [boolean fetchStored, JSONArray json, List<Pair<String, String>> aggregationFunctions] : (col= column_name |agrFunction= aggregation_function ) ( COMMA (col= column_name |agrFunction= aggregation_function ) )* ;
    public final BQLParser.selection_list_return selection_list() throws RecognitionException {
        BQLParser.selection_list_return retval = new BQLParser.selection_list_return();
        retval.start = input.LT(1);

        int selection_list_StartIndex = input.index();

        Object root_0 = null;

        Token COMMA11=null;
        BQLParser.column_name_return col =null;

        BQLParser.aggregation_function_return agrFunction =null;


        Object COMMA11_tree=null;


            retval.fetchStored = false;
            retval.json = new FastJSONArray();
            retval.aggregationFunctions = new ArrayList<Pair<String, String>>();
            
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 4) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1451:5: ( (col= column_name |agrFunction= aggregation_function ) ( COMMA (col= column_name |agrFunction= aggregation_function ) )* )
            // com/senseidb/bql/parsers/BQL.g:1451:10: (col= column_name |agrFunction= aggregation_function ) ( COMMA (col= column_name |agrFunction= aggregation_function ) )*
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:1451:10: (col= column_name |agrFunction= aggregation_function )
            int alt8=2;
            alt8 = dfa8.predict(input);
            switch (alt8) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1451:11: col= column_name
                    {
                    pushFollow(FOLLOW_column_name_in_selection_list4147);
                    col=column_name();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, col.getTree());

                    if ( state.backtracking==0 ) {
                                String colName = (col!=null?col.text:null);
                                if (colName != null) {
                                    retval.json.put((col!=null?col.text:null)); 
                                    if ("_srcdata".equals(colName) || colName.startsWith("_srcdata.")) {
                                        retval.fetchStored = true;
                                    }
                                }
                            }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1460:13: agrFunction= aggregation_function
                    {
                    pushFollow(FOLLOW_aggregation_function_in_selection_list4163);
                    agrFunction=aggregation_function();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, agrFunction.getTree());

                    if ( state.backtracking==0 ) {
                                   retval.aggregationFunctions.add(new Pair((agrFunction!=null?agrFunction.function:null), (agrFunction!=null?agrFunction.column:null)));
                                }

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1464:9: ( COMMA (col= column_name |agrFunction= aggregation_function ) )*
            loop10:
            do {
                int alt10=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt10=1;
                    }
                    break;

                }

                switch (alt10) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1464:10: COMMA (col= column_name |agrFunction= aggregation_function )
            	    {
            	    COMMA11=(Token)match(input,COMMA,FOLLOW_COMMA_in_selection_list4190); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA11_tree = 
            	    (Object)adaptor.create(COMMA11)
            	    ;
            	    adaptor.addChild(root_0, COMMA11_tree);
            	    }

            	    // com/senseidb/bql/parsers/BQL.g:1464:18: (col= column_name |agrFunction= aggregation_function )
            	    int alt9=2;
            	    alt9 = dfa9.predict(input);
            	    switch (alt9) {
            	        case 1 :
            	            // com/senseidb/bql/parsers/BQL.g:1464:19: col= column_name
            	            {
            	            pushFollow(FOLLOW_column_name_in_selection_list4197);
            	            col=column_name();

            	            state._fsp--;
            	            if (state.failed) return retval;
            	            if ( state.backtracking==0 ) adaptor.addChild(root_0, col.getTree());

            	            if ( state.backtracking==0 ) {
            	                        String colName = (col!=null?col.text:null);
            	                        if (colName != null) {
            	                            retval.json.put((col!=null?col.text:null)); 
            	                            if ("_srcdata".equals(colName) || colName.startsWith("_srcdata.")) {
            	                                retval.fetchStored = true;
            	                            }
            	                        }
            	                    }

            	            }
            	            break;
            	        case 2 :
            	            // com/senseidb/bql/parsers/BQL.g:1473:13: agrFunction= aggregation_function
            	            {
            	            pushFollow(FOLLOW_aggregation_function_in_selection_list4213);
            	            agrFunction=aggregation_function();

            	            state._fsp--;
            	            if (state.failed) return retval;
            	            if ( state.backtracking==0 ) adaptor.addChild(root_0, agrFunction.getTree());

            	            if ( state.backtracking==0 ) {
            	                            retval.aggregationFunctions.add(new Pair((agrFunction!=null?agrFunction.function:null), (agrFunction!=null?agrFunction.column:null)));
            	                        }

            	            }
            	            break;

            	    }


            	    }
            	    break;

            	default :
            	    break loop10;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 4, selection_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "selection_list"


    public static class aggregation_function_return extends ParserRuleReturnScope {
        public String function;
        public String column;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "aggregation_function"
    // com/senseidb/bql/parsers/BQL.g:1478:1: aggregation_function returns [String function, String column] : (id= function_name LPAR (columnVar= column_name | '*' ) RPAR ) ;
    public final BQLParser.aggregation_function_return aggregation_function() throws RecognitionException {
        BQLParser.aggregation_function_return retval = new BQLParser.aggregation_function_return();
        retval.start = input.LT(1);

        int aggregation_function_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR12=null;
        Token char_literal13=null;
        Token RPAR14=null;
        BQLParser.function_name_return id =null;

        BQLParser.column_name_return columnVar =null;


        Object LPAR12_tree=null;
        Object char_literal13_tree=null;
        Object RPAR14_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 5) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1479:2: ( (id= function_name LPAR (columnVar= column_name | '*' ) RPAR ) )
            // com/senseidb/bql/parsers/BQL.g:1479:6: (id= function_name LPAR (columnVar= column_name | '*' ) RPAR )
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:1479:6: (id= function_name LPAR (columnVar= column_name | '*' ) RPAR )
            // com/senseidb/bql/parsers/BQL.g:1479:7: id= function_name LPAR (columnVar= column_name | '*' ) RPAR
            {
            pushFollow(FOLLOW_function_name_in_aggregation_function4249);
            id=function_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, id.getTree());

            LPAR12=(Token)match(input,LPAR,FOLLOW_LPAR_in_aggregation_function4251); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR12_tree = 
            (Object)adaptor.create(LPAR12)
            ;
            adaptor.addChild(root_0, LPAR12_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:1479:29: (columnVar= column_name | '*' )
            int alt11=2;
            switch ( input.LA(1) ) {
            case IDENT:
            case STRING_LITERAL:
                {
                alt11=1;
                }
                break;
            case 121:
                {
                alt11=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 11, 0, input);

                throw nvae;

            }

            switch (alt11) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1479:30: columnVar= column_name
                    {
                    pushFollow(FOLLOW_column_name_in_aggregation_function4256);
                    columnVar=column_name();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, columnVar.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1479:54: '*'
                    {
                    char_literal13=(Token)match(input,121,FOLLOW_121_in_aggregation_function4260); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal13_tree = 
                    (Object)adaptor.create(char_literal13)
                    ;
                    adaptor.addChild(root_0, char_literal13_tree);
                    }

                    }
                    break;

            }


            RPAR14=(Token)match(input,RPAR,FOLLOW_RPAR_in_aggregation_function4263); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR14_tree = 
            (Object)adaptor.create(RPAR14)
            ;
            adaptor.addChild(root_0, RPAR14_tree);
            }

            }


            if ( state.backtracking==0 ) {
                retval.function = (id!=null?id.text:null);    
                if (columnVar != null) {
                    retval.column = (columnVar!=null?columnVar.text:null);
                } else {
                    retval.column = ""; 
                }    
             }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 5, aggregation_function_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "aggregation_function"


    public static class column_name_return extends ParserRuleReturnScope {
        public String text;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "column_name"
    // com/senseidb/bql/parsers/BQL.g:1488:1: column_name returns [String text] : (id= IDENT |str= STRING_LITERAL ) ( '.' (id2= IDENT |str2= STRING_LITERAL ) )* ;
    public final BQLParser.column_name_return column_name() throws RecognitionException {
        BQLParser.column_name_return retval = new BQLParser.column_name_return();
        retval.start = input.LT(1);

        int column_name_StartIndex = input.index();

        Object root_0 = null;

        Token id=null;
        Token str=null;
        Token id2=null;
        Token str2=null;
        Token char_literal15=null;

        Object id_tree=null;
        Object str_tree=null;
        Object id2_tree=null;
        Object str2_tree=null;
        Object char_literal15_tree=null;


            StringBuilder builder = new StringBuilder();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 6) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1492:5: ( (id= IDENT |str= STRING_LITERAL ) ( '.' (id2= IDENT |str2= STRING_LITERAL ) )* )
            // com/senseidb/bql/parsers/BQL.g:1492:9: (id= IDENT |str= STRING_LITERAL ) ( '.' (id2= IDENT |str2= STRING_LITERAL ) )*
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:1492:9: (id= IDENT |str= STRING_LITERAL )
            int alt12=2;
            switch ( input.LA(1) ) {
            case IDENT:
                {
                alt12=1;
                }
                break;
            case STRING_LITERAL:
                {
                alt12=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 12, 0, input);

                throw nvae;

            }

            switch (alt12) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1492:10: id= IDENT
                    {
                    id=(Token)match(input,IDENT,FOLLOW_IDENT_in_column_name4292); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    id_tree = 
                    (Object)adaptor.create(id)
                    ;
                    adaptor.addChild(root_0, id_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1492:21: str= STRING_LITERAL
                    {
                    str=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_column_name4298); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    str_tree = 
                    (Object)adaptor.create(str)
                    ;
                    adaptor.addChild(root_0, str_tree);
                    }

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        if (id != null) {
                            builder.append((id!=null?id.getText():null));
                        }
                        else {
                            String orig = (str!=null?str.getText():null);
                            builder.append(orig.substring(1, orig.length() - 1));
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:1502:9: ( '.' (id2= IDENT |str2= STRING_LITERAL ) )*
            loop14:
            do {
                int alt14=2;
                switch ( input.LA(1) ) {
                case 129:
                    {
                    alt14=1;
                    }
                    break;

                }

                switch (alt14) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1502:10: '.' (id2= IDENT |str2= STRING_LITERAL )
            	    {
            	    char_literal15=(Token)match(input,129,FOLLOW_129_in_column_name4320); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal15_tree = 
            	    (Object)adaptor.create(char_literal15)
            	    ;
            	    adaptor.addChild(root_0, char_literal15_tree);
            	    }

            	    // com/senseidb/bql/parsers/BQL.g:1502:14: (id2= IDENT |str2= STRING_LITERAL )
            	    int alt13=2;
            	    switch ( input.LA(1) ) {
            	    case IDENT:
            	        {
            	        alt13=1;
            	        }
            	        break;
            	    case STRING_LITERAL:
            	        {
            	        alt13=2;
            	        }
            	        break;
            	    default:
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        NoViableAltException nvae =
            	            new NoViableAltException("", 13, 0, input);

            	        throw nvae;

            	    }

            	    switch (alt13) {
            	        case 1 :
            	            // com/senseidb/bql/parsers/BQL.g:1502:15: id2= IDENT
            	            {
            	            id2=(Token)match(input,IDENT,FOLLOW_IDENT_in_column_name4325); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            id2_tree = 
            	            (Object)adaptor.create(id2)
            	            ;
            	            adaptor.addChild(root_0, id2_tree);
            	            }

            	            }
            	            break;
            	        case 2 :
            	            // com/senseidb/bql/parsers/BQL.g:1502:27: str2= STRING_LITERAL
            	            {
            	            str2=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_column_name4331); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            str2_tree = 
            	            (Object)adaptor.create(str2)
            	            ;
            	            adaptor.addChild(root_0, str2_tree);
            	            }

            	            }
            	            break;

            	    }


            	    if ( state.backtracking==0 ) {
            	                builder.append(".");
            	                if (id2 != null) {
            	                    builder.append((id2!=null?id2.getText():null));
            	                }
            	                else {
            	                    String orig = (str2!=null?str2.getText():null);
            	                    builder.append(orig.substring(1, orig.length() - 1));
            	                }
            	            }

            	    }
            	    break;

            	default :
            	    break loop14;
                }
            } while (true);


            if ( state.backtracking==0 ) { retval.text = builder.toString(); }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 6, column_name_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "column_name"


    public static class function_name_return extends ParserRuleReturnScope {
        public String text;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "function_name"
    // com/senseidb/bql/parsers/BQL.g:1516:1: function_name returns [String text] : (min= 'min' |colName= column_name ) ;
    public final BQLParser.function_name_return function_name() throws RecognitionException {
        BQLParser.function_name_return retval = new BQLParser.function_name_return();
        retval.start = input.LT(1);

        int function_name_StartIndex = input.index();

        Object root_0 = null;

        Token min=null;
        BQLParser.column_name_return colName =null;


        Object min_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 7) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1518:5: ( (min= 'min' |colName= column_name ) )
            // com/senseidb/bql/parsers/BQL.g:1518:9: (min= 'min' |colName= column_name )
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:1518:9: (min= 'min' |colName= column_name )
            int alt15=2;
            switch ( input.LA(1) ) {
            case 158:
                {
                alt15=1;
                }
                break;
            case IDENT:
            case STRING_LITERAL:
                {
                alt15=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 15, 0, input);

                throw nvae;

            }

            switch (alt15) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1518:10: min= 'min'
                    {
                    min=(Token)match(input,158,FOLLOW_158_in_function_name4391); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    min_tree = 
                    (Object)adaptor.create(min)
                    ;
                    adaptor.addChild(root_0, min_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1518:21: colName= column_name
                    {
                    pushFollow(FOLLOW_column_name_in_function_name4395);
                    colName=column_name();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, colName.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                     if (min != null) {
                       retval.text = "min";
                     } else {
                        retval.text = (colName!=null?colName.text:null); 
                     }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 7, function_name_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "function_name"


    public static class where_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "where"
    // com/senseidb/bql/parsers/BQL.g:1527:1: where returns [Object json] : WHERE ^ search_expr ;
    public final BQLParser.where_return where() throws RecognitionException {
        BQLParser.where_return retval = new BQLParser.where_return();
        retval.start = input.LT(1);

        int where_StartIndex = input.index();

        Object root_0 = null;

        Token WHERE16=null;
        BQLParser.search_expr_return search_expr17 =null;


        Object WHERE16_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 8) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1528:5: ( WHERE ^ search_expr )
            // com/senseidb/bql/parsers/BQL.g:1528:9: WHERE ^ search_expr
            {
            root_0 = (Object)adaptor.nil();


            WHERE16=(Token)match(input,WHERE,FOLLOW_WHERE_in_where4428); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            WHERE16_tree = 
            (Object)adaptor.create(WHERE16)
            ;
            root_0 = (Object)adaptor.becomeRoot(WHERE16_tree, root_0);
            }

            pushFollow(FOLLOW_search_expr_in_where4431);
            search_expr17=search_expr();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, search_expr17.getTree());

            if ( state.backtracking==0 ) {
                        retval.json = (search_expr17!=null?search_expr17.json:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 8, where_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "where"


    public static class order_by_clause_return extends ParserRuleReturnScope {
        public boolean isRelevance;
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "order_by_clause"
    // com/senseidb/bql/parsers/BQL.g:1534:1: order_by_clause returns [boolean isRelevance, Object json] : ORDER BY ( RELEVANCE | sort_specs ) ;
    public final BQLParser.order_by_clause_return order_by_clause() throws RecognitionException {
        BQLParser.order_by_clause_return retval = new BQLParser.order_by_clause_return();
        retval.start = input.LT(1);

        int order_by_clause_StartIndex = input.index();

        Object root_0 = null;

        Token ORDER18=null;
        Token BY19=null;
        Token RELEVANCE20=null;
        BQLParser.sort_specs_return sort_specs21 =null;


        Object ORDER18_tree=null;
        Object BY19_tree=null;
        Object RELEVANCE20_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 9) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1535:5: ( ORDER BY ( RELEVANCE | sort_specs ) )
            // com/senseidb/bql/parsers/BQL.g:1535:9: ORDER BY ( RELEVANCE | sort_specs )
            {
            root_0 = (Object)adaptor.nil();


            ORDER18=(Token)match(input,ORDER,FOLLOW_ORDER_in_order_by_clause4464); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            ORDER18_tree = 
            (Object)adaptor.create(ORDER18)
            ;
            adaptor.addChild(root_0, ORDER18_tree);
            }

            BY19=(Token)match(input,BY,FOLLOW_BY_in_order_by_clause4466); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            BY19_tree = 
            (Object)adaptor.create(BY19)
            ;
            adaptor.addChild(root_0, BY19_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:1535:18: ( RELEVANCE | sort_specs )
            int alt16=2;
            switch ( input.LA(1) ) {
            case RELEVANCE:
                {
                alt16=1;
                }
                break;
            case IDENT:
            case STRING_LITERAL:
                {
                alt16=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 16, 0, input);

                throw nvae;

            }

            switch (alt16) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1535:19: RELEVANCE
                    {
                    RELEVANCE20=(Token)match(input,RELEVANCE,FOLLOW_RELEVANCE_in_order_by_clause4469); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    RELEVANCE20_tree = 
                    (Object)adaptor.create(RELEVANCE20)
                    ;
                    adaptor.addChild(root_0, RELEVANCE20_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1535:31: sort_specs
                    {
                    pushFollow(FOLLOW_sort_specs_in_order_by_clause4473);
                    sort_specs21=sort_specs();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, sort_specs21.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        if (RELEVANCE20 != null) {
                            retval.isRelevance = true;
                        }
                        else {
                            retval.isRelevance = false;
                            retval.json = (sort_specs21!=null?sort_specs21.json:null);
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 9, order_by_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "order_by_clause"


    public static class sort_specs_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "sort_specs"
    // com/senseidb/bql/parsers/BQL.g:1547:1: sort_specs returns [Object json] : sort= sort_spec ( COMMA sort= sort_spec )* ;
    public final BQLParser.sort_specs_return sort_specs() throws RecognitionException {
        BQLParser.sort_specs_return retval = new BQLParser.sort_specs_return();
        retval.start = input.LT(1);

        int sort_specs_StartIndex = input.index();

        Object root_0 = null;

        Token COMMA22=null;
        BQLParser.sort_spec_return sort =null;


        Object COMMA22_tree=null;


            JSONArray sortArray = new FastJSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 10) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1551:5: (sort= sort_spec ( COMMA sort= sort_spec )* )
            // com/senseidb/bql/parsers/BQL.g:1551:9: sort= sort_spec ( COMMA sort= sort_spec )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_sort_spec_in_sort_specs4514);
            sort=sort_spec();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, sort.getTree());

            if ( state.backtracking==0 ) {
                        sortArray.put((sort!=null?sort.json:null));
                    }

            // com/senseidb/bql/parsers/BQL.g:1555:9: ( COMMA sort= sort_spec )*
            loop17:
            do {
                int alt17=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt17=1;
                    }
                    break;

                }

                switch (alt17) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1555:10: COMMA sort= sort_spec
            	    {
            	    COMMA22=(Token)match(input,COMMA,FOLLOW_COMMA_in_sort_specs4535); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA22_tree = 
            	    (Object)adaptor.create(COMMA22)
            	    ;
            	    adaptor.addChild(root_0, COMMA22_tree);
            	    }

            	    pushFollow(FOLLOW_sort_spec_in_sort_specs4539);
            	    sort=sort_spec();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, sort.getTree());

            	    if ( state.backtracking==0 ) {
            	                    sortArray.put((sort!=null?sort.json:null));
            	                }

            	    }
            	    break;

            	default :
            	    break loop17;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        retval.json = sortArray;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 10, sort_specs_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "sort_specs"


    public static class sort_spec_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "sort_spec"
    // com/senseidb/bql/parsers/BQL.g:1565:1: sort_spec returns [JSONObject json] : column_name (ordering= ASC |ordering= DESC )? ;
    public final BQLParser.sort_spec_return sort_spec() throws RecognitionException {
        BQLParser.sort_spec_return retval = new BQLParser.sort_spec_return();
        retval.start = input.LT(1);

        int sort_spec_StartIndex = input.index();

        Object root_0 = null;

        Token ordering=null;
        BQLParser.column_name_return column_name23 =null;


        Object ordering_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 11) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1566:5: ( column_name (ordering= ASC |ordering= DESC )? )
            // com/senseidb/bql/parsers/BQL.g:1566:9: column_name (ordering= ASC |ordering= DESC )?
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_column_name_in_sort_spec4600);
            column_name23=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name23.getTree());

            // com/senseidb/bql/parsers/BQL.g:1566:21: (ordering= ASC |ordering= DESC )?
            int alt18=3;
            switch ( input.LA(1) ) {
                case ASC:
                    {
                    alt18=1;
                    }
                    break;
                case DESC:
                    {
                    alt18=2;
                    }
                    break;
            }

            switch (alt18) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1566:22: ordering= ASC
                    {
                    ordering=(Token)match(input,ASC,FOLLOW_ASC_in_sort_spec4605); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    ordering_tree = 
                    (Object)adaptor.create(ordering)
                    ;
                    adaptor.addChild(root_0, ordering_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1566:37: ordering= DESC
                    {
                    ordering=(Token)match(input,DESC,FOLLOW_DESC_in_sort_spec4611); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    ordering_tree = 
                    (Object)adaptor.create(ordering)
                    ;
                    adaptor.addChild(root_0, ordering_tree);
                    }

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        retval.json = new FastJSONObject();
                        try {
                            if (ordering == null) {
                                retval.json.put((column_name23!=null?column_name23.text:null), "asc");
                            }
                            else {
                                retval.json.put((column_name23!=null?column_name23.text:null), (ordering!=null?ordering.getText():null).toLowerCase());
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "sort_spec", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 11, sort_spec_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "sort_spec"


    public static class limit_clause_return extends ParserRuleReturnScope {
        public int offset;
        public int count;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "limit_clause"
    // com/senseidb/bql/parsers/BQL.g:1583:1: limit_clause returns [int offset, int count] : LIMIT (n1= INTEGER COMMA )? n2= INTEGER ;
    public final BQLParser.limit_clause_return limit_clause() throws RecognitionException {
        BQLParser.limit_clause_return retval = new BQLParser.limit_clause_return();
        retval.start = input.LT(1);

        int limit_clause_StartIndex = input.index();

        Object root_0 = null;

        Token n1=null;
        Token n2=null;
        Token LIMIT24=null;
        Token COMMA25=null;

        Object n1_tree=null;
        Object n2_tree=null;
        Object LIMIT24_tree=null;
        Object COMMA25_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 12) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1584:5: ( LIMIT (n1= INTEGER COMMA )? n2= INTEGER )
            // com/senseidb/bql/parsers/BQL.g:1584:9: LIMIT (n1= INTEGER COMMA )? n2= INTEGER
            {
            root_0 = (Object)adaptor.nil();


            LIMIT24=(Token)match(input,LIMIT,FOLLOW_LIMIT_in_limit_clause4646); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LIMIT24_tree = 
            (Object)adaptor.create(LIMIT24)
            ;
            adaptor.addChild(root_0, LIMIT24_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:1584:15: (n1= INTEGER COMMA )?
            int alt19=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    switch ( input.LA(2) ) {
                        case COMMA:
                            {
                            alt19=1;
                            }
                            break;
                    }

                    }
                    break;
            }

            switch (alt19) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1584:16: n1= INTEGER COMMA
                    {
                    n1=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_limit_clause4651); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    n1_tree = 
                    (Object)adaptor.create(n1)
                    ;
                    adaptor.addChild(root_0, n1_tree);
                    }

                    COMMA25=(Token)match(input,COMMA,FOLLOW_COMMA_in_limit_clause4653); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    COMMA25_tree = 
                    (Object)adaptor.create(COMMA25)
                    ;
                    adaptor.addChild(root_0, COMMA25_tree);
                    }

                    }
                    break;

            }


            n2=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_limit_clause4659); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            n2_tree = 
            (Object)adaptor.create(n2)
            ;
            adaptor.addChild(root_0, n2_tree);
            }

            if ( state.backtracking==0 ) {
                        if (n1 != null) {
                            retval.offset = Integer.parseInt((n1!=null?n1.getText():null));
                        }
                        else {
                            retval.offset = DEFAULT_REQUEST_OFFSET;
                        }
                        retval.count = Integer.parseInt((n2!=null?n2.getText():null));
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 12, limit_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "limit_clause"


    public static class comma_column_name_list_return extends ParserRuleReturnScope {
        public JSONArray json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "comma_column_name_list"
    // com/senseidb/bql/parsers/BQL.g:1595:1: comma_column_name_list returns [JSONArray json] : col= column_name ( ( OR | COMMA ) col= column_name )* ;
    public final BQLParser.comma_column_name_list_return comma_column_name_list() throws RecognitionException {
        BQLParser.comma_column_name_list_return retval = new BQLParser.comma_column_name_list_return();
        retval.start = input.LT(1);

        int comma_column_name_list_StartIndex = input.index();

        Object root_0 = null;

        Token set26=null;
        BQLParser.column_name_return col =null;


        Object set26_tree=null;


            retval.json = new FastJSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 13) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1599:5: (col= column_name ( ( OR | COMMA ) col= column_name )* )
            // com/senseidb/bql/parsers/BQL.g:1599:9: col= column_name ( ( OR | COMMA ) col= column_name )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_column_name_in_comma_column_name_list4698);
            col=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, col.getTree());

            if ( state.backtracking==0 ) {
                        String colName = (col!=null?col.text:null);
                        if (colName != null) {
                            retval.json.put((col!=null?col.text:null)); 
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:1606:9: ( ( OR | COMMA ) col= column_name )*
            loop20:
            do {
                int alt20=2;
                switch ( input.LA(1) ) {
                case COMMA:
                case OR:
                    {
                    alt20=1;
                    }
                    break;

                }

                switch (alt20) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1606:10: ( OR | COMMA ) col= column_name
            	    {
            	    set26=(Token)input.LT(1);

            	    if ( input.LA(1)==COMMA||input.LA(1)==OR ) {
            	        input.consume();
            	        if ( state.backtracking==0 ) adaptor.addChild(root_0, 
            	        (Object)adaptor.create(set26)
            	        );
            	        state.errorRecovery=false;
            	        state.failed=false;
            	    }
            	    else {
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        MismatchedSetException mse = new MismatchedSetException(null,input);
            	        throw mse;
            	    }


            	    pushFollow(FOLLOW_column_name_in_comma_column_name_list4729);
            	    col=column_name();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, col.getTree());

            	    if ( state.backtracking==0 ) {
            	                    String colName = (col!=null?col.text:null);
            	                    if (colName != null) {
            	                        retval.json.put((col!=null?col.text:null)); 
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop20;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 13, comma_column_name_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "comma_column_name_list"


    public static class or_column_name_list_return extends ParserRuleReturnScope {
        public JSONArray json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "or_column_name_list"
    // com/senseidb/bql/parsers/BQL.g:1615:1: or_column_name_list returns [JSONArray json] : col= column_name ( OR col= column_name )* ;
    public final BQLParser.or_column_name_list_return or_column_name_list() throws RecognitionException {
        BQLParser.or_column_name_list_return retval = new BQLParser.or_column_name_list_return();
        retval.start = input.LT(1);

        int or_column_name_list_StartIndex = input.index();

        Object root_0 = null;

        Token OR27=null;
        BQLParser.column_name_return col =null;


        Object OR27_tree=null;


            retval.json = new FastJSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 14) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1619:5: (col= column_name ( OR col= column_name )* )
            // com/senseidb/bql/parsers/BQL.g:1619:9: col= column_name ( OR col= column_name )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_column_name_in_or_column_name_list4783);
            col=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, col.getTree());

            if ( state.backtracking==0 ) {
                        String colName = (col!=null?col.text:null);
                        if (colName != null) {
                            retval.json.put((col!=null?col.text:null)); 
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:1626:9: ( OR col= column_name )*
            loop21:
            do {
                int alt21=2;
                switch ( input.LA(1) ) {
                case OR:
                    {
                    alt21=1;
                    }
                    break;

                }

                switch (alt21) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1626:10: OR col= column_name
            	    {
            	    OR27=(Token)match(input,OR,FOLLOW_OR_in_or_column_name_list4804); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    OR27_tree = 
            	    (Object)adaptor.create(OR27)
            	    ;
            	    adaptor.addChild(root_0, OR27_tree);
            	    }

            	    pushFollow(FOLLOW_column_name_in_or_column_name_list4808);
            	    col=column_name();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, col.getTree());

            	    if ( state.backtracking==0 ) {
            	                    String colName = (col!=null?col.text:null);
            	                    if (colName != null) {
            	                        retval.json.put((col!=null?col.text:null)); 
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop21;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 14, or_column_name_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "or_column_name_list"


    public static class group_by_clause_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "group_by_clause"
    // com/senseidb/bql/parsers/BQL.g:1636:1: group_by_clause returns [JSONObject json] : GROUP BY comma_column_name_list ( TOP top= INTEGER )? ;
    public final BQLParser.group_by_clause_return group_by_clause() throws RecognitionException {
        BQLParser.group_by_clause_return retval = new BQLParser.group_by_clause_return();
        retval.start = input.LT(1);

        int group_by_clause_StartIndex = input.index();

        Object root_0 = null;

        Token top=null;
        Token GROUP28=null;
        Token BY29=null;
        Token TOP31=null;
        BQLParser.comma_column_name_list_return comma_column_name_list30 =null;


        Object top_tree=null;
        Object GROUP28_tree=null;
        Object BY29_tree=null;
        Object TOP31_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 15) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1637:5: ( GROUP BY comma_column_name_list ( TOP top= INTEGER )? )
            // com/senseidb/bql/parsers/BQL.g:1637:9: GROUP BY comma_column_name_list ( TOP top= INTEGER )?
            {
            root_0 = (Object)adaptor.nil();


            GROUP28=(Token)match(input,GROUP,FOLLOW_GROUP_in_group_by_clause4856); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            GROUP28_tree = 
            (Object)adaptor.create(GROUP28)
            ;
            adaptor.addChild(root_0, GROUP28_tree);
            }

            BY29=(Token)match(input,BY,FOLLOW_BY_in_group_by_clause4858); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            BY29_tree = 
            (Object)adaptor.create(BY29)
            ;
            adaptor.addChild(root_0, BY29_tree);
            }

            pushFollow(FOLLOW_comma_column_name_list_in_group_by_clause4860);
            comma_column_name_list30=comma_column_name_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, comma_column_name_list30.getTree());

            // com/senseidb/bql/parsers/BQL.g:1637:41: ( TOP top= INTEGER )?
            int alt22=2;
            switch ( input.LA(1) ) {
                case TOP:
                    {
                    alt22=1;
                    }
                    break;
            }

            switch (alt22) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1637:42: TOP top= INTEGER
                    {
                    TOP31=(Token)match(input,TOP,FOLLOW_TOP_in_group_by_clause4863); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    TOP31_tree = 
                    (Object)adaptor.create(TOP31)
                    ;
                    adaptor.addChild(root_0, TOP31_tree);
                    }

                    top=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_group_by_clause4867); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    top_tree = 
                    (Object)adaptor.create(top)
                    ;
                    adaptor.addChild(root_0, top_tree);
                    }

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        retval.json = new FastJSONObject();
                        try {
                            JSONArray cols = (comma_column_name_list30!=null?comma_column_name_list30.json:null);
                            /*for (int i = 0; i < cols.length(); ++i) {
                                String col = cols.getString(i);
                                String[] facetInfo = _facetInfoMap.get(col);
                                if (facetInfo != null && (facetInfo[0].equals("range") ||
                                                          facetInfo[0].equals("multi") ||
                                                          facetInfo[0].equals("path"))) {
                                    throw new FailedPredicateException(input, 
                                                                       "group_by_clause",
                                                                       "Range/multi/path facet, \"" + col + "\", cannot be used in the GROUP BY clause.");
                                }
                            }*/
                            retval.json.put("columns", cols);
                            if (top != null) {
                                retval.json.put("top", Integer.parseInt(top.getText()));
                            }
                            else {
                                retval.json.put("top", DEFAULT_REQUEST_MAX_PER_GROUP);
                            }                    
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "group_by_clause", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 15, group_by_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "group_by_clause"


    public static class distinct_clause_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "distinct_clause"
    // com/senseidb/bql/parsers/BQL.g:1667:1: distinct_clause returns [JSONObject json] : DISTINCT or_column_name_list ;
    public final BQLParser.distinct_clause_return distinct_clause() throws RecognitionException {
        BQLParser.distinct_clause_return retval = new BQLParser.distinct_clause_return();
        retval.start = input.LT(1);

        int distinct_clause_StartIndex = input.index();

        Object root_0 = null;

        Token DISTINCT32=null;
        BQLParser.or_column_name_list_return or_column_name_list33 =null;


        Object DISTINCT32_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 16) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1668:5: ( DISTINCT or_column_name_list )
            // com/senseidb/bql/parsers/BQL.g:1668:9: DISTINCT or_column_name_list
            {
            root_0 = (Object)adaptor.nil();


            DISTINCT32=(Token)match(input,DISTINCT,FOLLOW_DISTINCT_in_distinct_clause4902); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            DISTINCT32_tree = 
            (Object)adaptor.create(DISTINCT32)
            ;
            adaptor.addChild(root_0, DISTINCT32_tree);
            }

            pushFollow(FOLLOW_or_column_name_list_in_distinct_clause4904);
            or_column_name_list33=or_column_name_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, or_column_name_list33.getTree());

            if ( state.backtracking==0 ) {
                        retval.json = new FastJSONObject();
                        try {
                            JSONArray cols = (or_column_name_list33!=null?or_column_name_list33.json:null); 
                            if (cols.length() > 1) {
                              throw new FailedPredicateException(input, 
                                                                 "distinct_clause",
                                                                 "DISTINCT only support a single column now.");
                            }
                            for (int i = 0; i < cols.length(); ++i) {
                                String col = cols.getString(i);
                                String[] facetInfo = _facetInfoMap.get(col);
                                if (facetInfo != null && (facetInfo[0].equals("range") ||
                                                          facetInfo[0].equals("multi") ||
                                                          facetInfo[0].equals("path"))) {
                                    throw new FailedPredicateException(input, 
                                                                       "distinct_clause",
                                                                       "Range/multi/path facet, \"" + col + "\", cannot be used in the DISTINCT clause.");
                                }
                            }
                            retval.json.put("columns", cols);
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "distinct_clause", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 16, distinct_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "distinct_clause"


    public static class browse_by_clause_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "browse_by_clause"
    // com/senseidb/bql/parsers/BQL.g:1697:1: browse_by_clause returns [JSONObject json] : BROWSE BY f= facet_spec ( COMMA f= facet_spec )* ;
    public final BQLParser.browse_by_clause_return browse_by_clause() throws RecognitionException {
        BQLParser.browse_by_clause_return retval = new BQLParser.browse_by_clause_return();
        retval.start = input.LT(1);

        int browse_by_clause_StartIndex = input.index();

        Object root_0 = null;

        Token BROWSE34=null;
        Token BY35=null;
        Token COMMA36=null;
        BQLParser.facet_spec_return f =null;


        Object BROWSE34_tree=null;
        Object BY35_tree=null;
        Object COMMA36_tree=null;


            retval.json = new FastJSONObject();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 17) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1701:5: ( BROWSE BY f= facet_spec ( COMMA f= facet_spec )* )
            // com/senseidb/bql/parsers/BQL.g:1701:9: BROWSE BY f= facet_spec ( COMMA f= facet_spec )*
            {
            root_0 = (Object)adaptor.nil();


            BROWSE34=(Token)match(input,BROWSE,FOLLOW_BROWSE_in_browse_by_clause4942); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            BROWSE34_tree = 
            (Object)adaptor.create(BROWSE34)
            ;
            adaptor.addChild(root_0, BROWSE34_tree);
            }

            BY35=(Token)match(input,BY,FOLLOW_BY_in_browse_by_clause4944); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            BY35_tree = 
            (Object)adaptor.create(BY35)
            ;
            adaptor.addChild(root_0, BY35_tree);
            }

            pushFollow(FOLLOW_facet_spec_in_browse_by_clause4948);
            f=facet_spec();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, f.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            retval.json.put((f!=null?f.column:null), (f!=null?f.spec:null));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "browse_by_clause", "JSONException: " + err.getMessage());
                        }                    
                    }

            // com/senseidb/bql/parsers/BQL.g:1710:9: ( COMMA f= facet_spec )*
            loop23:
            do {
                int alt23=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt23=1;
                    }
                    break;

                }

                switch (alt23) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1710:10: COMMA f= facet_spec
            	    {
            	    COMMA36=(Token)match(input,COMMA,FOLLOW_COMMA_in_browse_by_clause4969); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA36_tree = 
            	    (Object)adaptor.create(COMMA36)
            	    ;
            	    adaptor.addChild(root_0, COMMA36_tree);
            	    }

            	    pushFollow(FOLLOW_facet_spec_in_browse_by_clause4973);
            	    f=facet_spec();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, f.getTree());

            	    if ( state.backtracking==0 ) {
            	                    try {
            	                        retval.json.put((f!=null?f.column:null), (f!=null?f.spec:null));
            	                    }
            	                    catch (JSONException err) {
            	                        throw new FailedPredicateException(input, "browse_by_clause", "JSONException: " + err.getMessage());
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop23;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 17, browse_by_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "browse_by_clause"


    public static class execute_clause_return extends ParserRuleReturnScope {
        public String functionName;
        public JSONObject properties;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "execute_clause"
    // com/senseidb/bql/parsers/BQL.g:1721:1: execute_clause returns [String functionName, JSONObject properties] : EXECUTE LPAR funName= function_name ( ( COMMA map= python_style_dict ) | ( ( COMMA p= key_value_pair[KeyType.STRING_LITERAL] )* ) ) RPAR ;
    public final BQLParser.execute_clause_return execute_clause() throws RecognitionException {
        BQLParser.execute_clause_return retval = new BQLParser.execute_clause_return();
        retval.start = input.LT(1);

        int execute_clause_StartIndex = input.index();

        Object root_0 = null;

        Token EXECUTE37=null;
        Token LPAR38=null;
        Token COMMA39=null;
        Token COMMA40=null;
        Token RPAR41=null;
        BQLParser.function_name_return funName =null;

        BQLParser.python_style_dict_return map =null;

        BQLParser.key_value_pair_return p =null;


        Object EXECUTE37_tree=null;
        Object LPAR38_tree=null;
        Object COMMA39_tree=null;
        Object COMMA40_tree=null;
        Object RPAR41_tree=null;


            retval.properties = new FastJSONObject();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 18) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1725:5: ( EXECUTE LPAR funName= function_name ( ( COMMA map= python_style_dict ) | ( ( COMMA p= key_value_pair[KeyType.STRING_LITERAL] )* ) ) RPAR )
            // com/senseidb/bql/parsers/BQL.g:1725:9: EXECUTE LPAR funName= function_name ( ( COMMA map= python_style_dict ) | ( ( COMMA p= key_value_pair[KeyType.STRING_LITERAL] )* ) ) RPAR
            {
            root_0 = (Object)adaptor.nil();


            EXECUTE37=(Token)match(input,EXECUTE,FOLLOW_EXECUTE_in_execute_clause5025); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            EXECUTE37_tree = 
            (Object)adaptor.create(EXECUTE37)
            ;
            adaptor.addChild(root_0, EXECUTE37_tree);
            }

            LPAR38=(Token)match(input,LPAR,FOLLOW_LPAR_in_execute_clause5027); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR38_tree = 
            (Object)adaptor.create(LPAR38)
            ;
            adaptor.addChild(root_0, LPAR38_tree);
            }

            pushFollow(FOLLOW_function_name_in_execute_clause5031);
            funName=function_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, funName.getTree());

            if ( state.backtracking==0 ) {
                        retval.functionName = (funName!=null?funName.text:null);
                    }

            // com/senseidb/bql/parsers/BQL.g:1729:9: ( ( COMMA map= python_style_dict ) | ( ( COMMA p= key_value_pair[KeyType.STRING_LITERAL] )* ) )
            int alt25=2;
            switch ( input.LA(1) ) {
            case COMMA:
                {
                switch ( input.LA(2) ) {
                case 165:
                    {
                    alt25=1;
                    }
                    break;
                case IDENT:
                case STRING_LITERAL:
                    {
                    alt25=2;
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 25, 1, input);

                    throw nvae;

                }

                }
                break;
            case RPAR:
                {
                alt25=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 25, 0, input);

                throw nvae;

            }

            switch (alt25) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1729:10: ( COMMA map= python_style_dict )
                    {
                    // com/senseidb/bql/parsers/BQL.g:1729:10: ( COMMA map= python_style_dict )
                    // com/senseidb/bql/parsers/BQL.g:1729:11: COMMA map= python_style_dict
                    {
                    COMMA39=(Token)match(input,COMMA,FOLLOW_COMMA_in_execute_clause5053); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    COMMA39_tree = 
                    (Object)adaptor.create(COMMA39)
                    ;
                    adaptor.addChild(root_0, COMMA39_tree);
                    }

                    pushFollow(FOLLOW_python_style_dict_in_execute_clause5057);
                    map=python_style_dict();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, map.getTree());

                    if ( state.backtracking==0 ) {
                                   retval.properties = (map!=null?map.json:null);
                                   
                                }

                    }


                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1734:13: ( ( COMMA p= key_value_pair[KeyType.STRING_LITERAL] )* )
                    {
                    // com/senseidb/bql/parsers/BQL.g:1734:13: ( ( COMMA p= key_value_pair[KeyType.STRING_LITERAL] )* )
                    // com/senseidb/bql/parsers/BQL.g:1735:9: ( COMMA p= key_value_pair[KeyType.STRING_LITERAL] )*
                    {
                    // com/senseidb/bql/parsers/BQL.g:1735:9: ( COMMA p= key_value_pair[KeyType.STRING_LITERAL] )*
                    loop24:
                    do {
                        int alt24=2;
                        switch ( input.LA(1) ) {
                        case COMMA:
                            {
                            alt24=1;
                            }
                            break;

                        }

                        switch (alt24) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:1735:10: COMMA p= key_value_pair[KeyType.STRING_LITERAL]
                    	    {
                    	    COMMA40=(Token)match(input,COMMA,FOLLOW_COMMA_in_execute_clause5096); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    COMMA40_tree = 
                    	    (Object)adaptor.create(COMMA40)
                    	    ;
                    	    adaptor.addChild(root_0, COMMA40_tree);
                    	    }

                    	    pushFollow(FOLLOW_key_value_pair_in_execute_clause5100);
                    	    p=key_value_pair(KeyType.STRING_LITERAL);

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

                    	    if ( state.backtracking==0 ) {
                    	                    try {
                    	                        retval.properties.put((p!=null?p.key:null), (p!=null?p.val:null));
                    	                    }
                    	                    catch (JSONException err) {
                    	                        throw new FailedPredicateException(input, "python_style_dict", "JSONException: " + err.getMessage());
                    	                    }
                    	                }

                    	    }
                    	    break;

                    	default :
                    	    break loop24;
                        }
                    } while (true);


                    }


                    }
                    break;

            }


            RPAR41=(Token)match(input,RPAR,FOLLOW_RPAR_in_execute_clause5147); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR41_tree = 
            (Object)adaptor.create(RPAR41)
            ;
            adaptor.addChild(root_0, RPAR41_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 18, execute_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "execute_clause"


    public static class facet_spec_return extends ParserRuleReturnScope {
        public String column;
        public JSONObject spec;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "facet_spec"
    // com/senseidb/bql/parsers/BQL.g:1749:1: facet_spec returns [String column, JSONObject spec] : column_name ( LPAR ( TRUE | FALSE ) COMMA n1= INTEGER COMMA n2= INTEGER COMMA ( HITS | VALUE ) RPAR )* ;
    public final BQLParser.facet_spec_return facet_spec() throws RecognitionException {
        BQLParser.facet_spec_return retval = new BQLParser.facet_spec_return();
        retval.start = input.LT(1);

        int facet_spec_StartIndex = input.index();

        Object root_0 = null;

        Token n1=null;
        Token n2=null;
        Token LPAR43=null;
        Token TRUE44=null;
        Token FALSE45=null;
        Token COMMA46=null;
        Token COMMA47=null;
        Token COMMA48=null;
        Token HITS49=null;
        Token VALUE50=null;
        Token RPAR51=null;
        BQLParser.column_name_return column_name42 =null;


        Object n1_tree=null;
        Object n2_tree=null;
        Object LPAR43_tree=null;
        Object TRUE44_tree=null;
        Object FALSE45_tree=null;
        Object COMMA46_tree=null;
        Object COMMA47_tree=null;
        Object COMMA48_tree=null;
        Object HITS49_tree=null;
        Object VALUE50_tree=null;
        Object RPAR51_tree=null;


            boolean expand = false;
            int minhit = DEFAULT_FACET_MINHIT;
            int max = DEFAULT_FACET_MAXHIT;
            String orderBy = "hits";

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 19) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1756:5: ( column_name ( LPAR ( TRUE | FALSE ) COMMA n1= INTEGER COMMA n2= INTEGER COMMA ( HITS | VALUE ) RPAR )* )
            // com/senseidb/bql/parsers/BQL.g:1756:9: column_name ( LPAR ( TRUE | FALSE ) COMMA n1= INTEGER COMMA n2= INTEGER COMMA ( HITS | VALUE ) RPAR )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_column_name_in_facet_spec5180);
            column_name42=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name42.getTree());

            // com/senseidb/bql/parsers/BQL.g:1757:9: ( LPAR ( TRUE | FALSE ) COMMA n1= INTEGER COMMA n2= INTEGER COMMA ( HITS | VALUE ) RPAR )*
            loop28:
            do {
                int alt28=2;
                switch ( input.LA(1) ) {
                case LPAR:
                    {
                    alt28=1;
                    }
                    break;

                }

                switch (alt28) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1758:13: LPAR ( TRUE | FALSE ) COMMA n1= INTEGER COMMA n2= INTEGER COMMA ( HITS | VALUE ) RPAR
            	    {
            	    LPAR43=(Token)match(input,LPAR,FOLLOW_LPAR_in_facet_spec5204); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    LPAR43_tree = 
            	    (Object)adaptor.create(LPAR43)
            	    ;
            	    adaptor.addChild(root_0, LPAR43_tree);
            	    }

            	    // com/senseidb/bql/parsers/BQL.g:1759:13: ( TRUE | FALSE )
            	    int alt26=2;
            	    switch ( input.LA(1) ) {
            	    case TRUE:
            	        {
            	        alt26=1;
            	        }
            	        break;
            	    case FALSE:
            	        {
            	        alt26=2;
            	        }
            	        break;
            	    default:
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        NoViableAltException nvae =
            	            new NoViableAltException("", 26, 0, input);

            	        throw nvae;

            	    }

            	    switch (alt26) {
            	        case 1 :
            	            // com/senseidb/bql/parsers/BQL.g:1759:14: TRUE
            	            {
            	            TRUE44=(Token)match(input,TRUE,FOLLOW_TRUE_in_facet_spec5220); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            TRUE44_tree = 
            	            (Object)adaptor.create(TRUE44)
            	            ;
            	            adaptor.addChild(root_0, TRUE44_tree);
            	            }

            	            if ( state.backtracking==0 ) {expand = true;}

            	            }
            	            break;
            	        case 2 :
            	            // com/senseidb/bql/parsers/BQL.g:1759:38: FALSE
            	            {
            	            FALSE45=(Token)match(input,FALSE,FOLLOW_FALSE_in_facet_spec5226); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            FALSE45_tree = 
            	            (Object)adaptor.create(FALSE45)
            	            ;
            	            adaptor.addChild(root_0, FALSE45_tree);
            	            }

            	            }
            	            break;

            	    }


            	    COMMA46=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_spec5229); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA46_tree = 
            	    (Object)adaptor.create(COMMA46)
            	    ;
            	    adaptor.addChild(root_0, COMMA46_tree);
            	    }

            	    n1=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_facet_spec5245); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    n1_tree = 
            	    (Object)adaptor.create(n1)
            	    ;
            	    adaptor.addChild(root_0, n1_tree);
            	    }

            	    COMMA47=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_spec5247); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA47_tree = 
            	    (Object)adaptor.create(COMMA47)
            	    ;
            	    adaptor.addChild(root_0, COMMA47_tree);
            	    }

            	    n2=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_facet_spec5263); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    n2_tree = 
            	    (Object)adaptor.create(n2)
            	    ;
            	    adaptor.addChild(root_0, n2_tree);
            	    }

            	    COMMA48=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_spec5265); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA48_tree = 
            	    (Object)adaptor.create(COMMA48)
            	    ;
            	    adaptor.addChild(root_0, COMMA48_tree);
            	    }

            	    // com/senseidb/bql/parsers/BQL.g:1762:13: ( HITS | VALUE )
            	    int alt27=2;
            	    switch ( input.LA(1) ) {
            	    case HITS:
            	        {
            	        alt27=1;
            	        }
            	        break;
            	    case VALUE:
            	        {
            	        alt27=2;
            	        }
            	        break;
            	    default:
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        NoViableAltException nvae =
            	            new NoViableAltException("", 27, 0, input);

            	        throw nvae;

            	    }

            	    switch (alt27) {
            	        case 1 :
            	            // com/senseidb/bql/parsers/BQL.g:1762:14: HITS
            	            {
            	            HITS49=(Token)match(input,HITS,FOLLOW_HITS_in_facet_spec5280); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            HITS49_tree = 
            	            (Object)adaptor.create(HITS49)
            	            ;
            	            adaptor.addChild(root_0, HITS49_tree);
            	            }

            	            }
            	            break;
            	        case 2 :
            	            // com/senseidb/bql/parsers/BQL.g:1762:21: VALUE
            	            {
            	            VALUE50=(Token)match(input,VALUE,FOLLOW_VALUE_in_facet_spec5284); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            VALUE50_tree = 
            	            (Object)adaptor.create(VALUE50)
            	            ;
            	            adaptor.addChild(root_0, VALUE50_tree);
            	            }

            	            if ( state.backtracking==0 ) {orderBy = "val";}

            	            }
            	            break;

            	    }


            	    RPAR51=(Token)match(input,RPAR,FOLLOW_RPAR_in_facet_spec5301); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    RPAR51_tree = 
            	    (Object)adaptor.create(RPAR51)
            	    ;
            	    adaptor.addChild(root_0, RPAR51_tree);
            	    }

            	    }
            	    break;

            	default :
            	    break loop28;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        retval.column = (column_name42!=null?column_name42.text:null);
                        if (n1 != null) {
                            minhit = Integer.parseInt((n1!=null?n1.getText():null));
                        }
                        if (n2 != null) {
                            max = Integer.parseInt((n2!=null?n2.getText():null));
                        }
                        try {
                            retval.spec = new FastJSONObject().put("expand", expand)
                                                        .put("minhit", minhit)
                                                        .put("max", max)
                                                        .put("order", orderBy);
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "facet_spec", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 19, facet_spec_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "facet_spec"


    public static class fetching_stored_clause_return extends ParserRuleReturnScope {
        public boolean val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "fetching_stored_clause"
    // com/senseidb/bql/parsers/BQL.g:1785:1: fetching_stored_clause returns [boolean val] : FETCHING STORED ( ( TRUE | FALSE ) )* ;
    public final BQLParser.fetching_stored_clause_return fetching_stored_clause() throws RecognitionException {
        BQLParser.fetching_stored_clause_return retval = new BQLParser.fetching_stored_clause_return();
        retval.start = input.LT(1);

        int fetching_stored_clause_StartIndex = input.index();

        Object root_0 = null;

        Token FETCHING52=null;
        Token STORED53=null;
        Token TRUE54=null;
        Token FALSE55=null;

        Object FETCHING52_tree=null;
        Object STORED53_tree=null;
        Object TRUE54_tree=null;
        Object FALSE55_tree=null;


            retval.val = true;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 20) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1789:5: ( FETCHING STORED ( ( TRUE | FALSE ) )* )
            // com/senseidb/bql/parsers/BQL.g:1789:9: FETCHING STORED ( ( TRUE | FALSE ) )*
            {
            root_0 = (Object)adaptor.nil();


            FETCHING52=(Token)match(input,FETCHING,FOLLOW_FETCHING_in_fetching_stored_clause5350); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            FETCHING52_tree = 
            (Object)adaptor.create(FETCHING52)
            ;
            adaptor.addChild(root_0, FETCHING52_tree);
            }

            STORED53=(Token)match(input,STORED,FOLLOW_STORED_in_fetching_stored_clause5352); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            STORED53_tree = 
            (Object)adaptor.create(STORED53)
            ;
            adaptor.addChild(root_0, STORED53_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:1790:9: ( ( TRUE | FALSE ) )*
            loop30:
            do {
                int alt30=2;
                switch ( input.LA(1) ) {
                case FALSE:
                case TRUE:
                    {
                    alt30=1;
                    }
                    break;

                }

                switch (alt30) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1790:10: ( TRUE | FALSE )
            	    {
            	    // com/senseidb/bql/parsers/BQL.g:1790:10: ( TRUE | FALSE )
            	    int alt29=2;
            	    switch ( input.LA(1) ) {
            	    case TRUE:
            	        {
            	        alt29=1;
            	        }
            	        break;
            	    case FALSE:
            	        {
            	        alt29=2;
            	        }
            	        break;
            	    default:
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        NoViableAltException nvae =
            	            new NoViableAltException("", 29, 0, input);

            	        throw nvae;

            	    }

            	    switch (alt29) {
            	        case 1 :
            	            // com/senseidb/bql/parsers/BQL.g:1790:11: TRUE
            	            {
            	            TRUE54=(Token)match(input,TRUE,FOLLOW_TRUE_in_fetching_stored_clause5364); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            TRUE54_tree = 
            	            (Object)adaptor.create(TRUE54)
            	            ;
            	            adaptor.addChild(root_0, TRUE54_tree);
            	            }

            	            }
            	            break;
            	        case 2 :
            	            // com/senseidb/bql/parsers/BQL.g:1790:18: FALSE
            	            {
            	            FALSE55=(Token)match(input,FALSE,FOLLOW_FALSE_in_fetching_stored_clause5368); if (state.failed) return retval;
            	            if ( state.backtracking==0 ) {
            	            FALSE55_tree = 
            	            (Object)adaptor.create(FALSE55)
            	            ;
            	            adaptor.addChild(root_0, FALSE55_tree);
            	            }

            	            if ( state.backtracking==0 ) {retval.val = false;}

            	            }
            	            break;

            	    }


            	    }
            	    break;

            	default :
            	    break loop30;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 20, fetching_stored_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "fetching_stored_clause"


    public static class route_by_clause_return extends ParserRuleReturnScope {
        public String val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "route_by_clause"
    // com/senseidb/bql/parsers/BQL.g:1794:1: route_by_clause returns [String val] : ROUTE BY STRING_LITERAL ;
    public final BQLParser.route_by_clause_return route_by_clause() throws RecognitionException {
        BQLParser.route_by_clause_return retval = new BQLParser.route_by_clause_return();
        retval.start = input.LT(1);

        int route_by_clause_StartIndex = input.index();

        Object root_0 = null;

        Token ROUTE56=null;
        Token BY57=null;
        Token STRING_LITERAL58=null;

        Object ROUTE56_tree=null;
        Object BY57_tree=null;
        Object STRING_LITERAL58_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 21) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1795:5: ( ROUTE BY STRING_LITERAL )
            // com/senseidb/bql/parsers/BQL.g:1795:9: ROUTE BY STRING_LITERAL
            {
            root_0 = (Object)adaptor.nil();


            ROUTE56=(Token)match(input,ROUTE,FOLLOW_ROUTE_in_route_by_clause5405); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            ROUTE56_tree = 
            (Object)adaptor.create(ROUTE56)
            ;
            adaptor.addChild(root_0, ROUTE56_tree);
            }

            BY57=(Token)match(input,BY,FOLLOW_BY_in_route_by_clause5407); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            BY57_tree = 
            (Object)adaptor.create(BY57)
            ;
            adaptor.addChild(root_0, BY57_tree);
            }

            STRING_LITERAL58=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_route_by_clause5409); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            STRING_LITERAL58_tree = 
            (Object)adaptor.create(STRING_LITERAL58)
            ;
            adaptor.addChild(root_0, STRING_LITERAL58_tree);
            }

            if ( state.backtracking==0 ) { 
                        String orig = (STRING_LITERAL58!=null?STRING_LITERAL58.getText():null);
                        retval.val = orig.substring(1, orig.length() - 1);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 21, route_by_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "route_by_clause"


    public static class search_expr_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "search_expr"
    // com/senseidb/bql/parsers/BQL.g:1802:1: search_expr returns [Object json] : t= term_expr ( OR t= term_expr )* -> {array.length() > 1}? ^( OR_PRED ( term_expr )+ ) -> term_expr ;
    public final BQLParser.search_expr_return search_expr() throws RecognitionException {
        BQLParser.search_expr_return retval = new BQLParser.search_expr_return();
        retval.start = input.LT(1);

        int search_expr_StartIndex = input.index();

        Object root_0 = null;

        Token OR59=null;
        BQLParser.term_expr_return t =null;


        Object OR59_tree=null;
        RewriteRuleTokenStream stream_OR=new RewriteRuleTokenStream(adaptor,"token OR");
        RewriteRuleSubtreeStream stream_term_expr=new RewriteRuleSubtreeStream(adaptor,"rule term_expr");

            JSONArray array = new FastJSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 22) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1806:5: (t= term_expr ( OR t= term_expr )* -> {array.length() > 1}? ^( OR_PRED ( term_expr )+ ) -> term_expr )
            // com/senseidb/bql/parsers/BQL.g:1806:9: t= term_expr ( OR t= term_expr )*
            {
            pushFollow(FOLLOW_term_expr_in_search_expr5450);
            t=term_expr();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_term_expr.add(t.getTree());

            if ( state.backtracking==0 ) { array.put((t!=null?t.json:null)); }

            // com/senseidb/bql/parsers/BQL.g:1807:9: ( OR t= term_expr )*
            loop31:
            do {
                int alt31=2;
                switch ( input.LA(1) ) {
                case OR:
                    {
                    alt31=1;
                    }
                    break;

                }

                switch (alt31) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1807:10: OR t= term_expr
            	    {
            	    OR59=(Token)match(input,OR,FOLLOW_OR_in_search_expr5463); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_OR.add(OR59);


            	    pushFollow(FOLLOW_term_expr_in_search_expr5467);
            	    t=term_expr();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_term_expr.add(t.getTree());

            	    if ( state.backtracking==0 ) { array.put((t!=null?t.json:null)); }

            	    }
            	    break;

            	default :
            	    break loop31;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        try {
                            if (array.length() == 1) {
                                retval.json = array.get(0);
                            }
                            else {
                                retval.json = new FastJSONObject().put("or", array);
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "search_expr", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: term_expr, term_expr
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 1821:9: -> {array.length() > 1}? ^( OR_PRED ( term_expr )+ )
            if (array.length() > 1) {
                // com/senseidb/bql/parsers/BQL.g:1821:34: ^( OR_PRED ( term_expr )+ )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                (Object)adaptor.create(OR_PRED, "OR_PRED")
                , root_1);

                if ( !(stream_term_expr.hasNext()) ) {
                    throw new RewriteEarlyExitException();
                }
                while ( stream_term_expr.hasNext() ) {
                    adaptor.addChild(root_1, stream_term_expr.nextTree());

                }
                stream_term_expr.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            else // 1822:9: -> term_expr
            {
                adaptor.addChild(root_0, stream_term_expr.nextTree());

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 22, search_expr_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "search_expr"


    public static class term_expr_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "term_expr"
    // com/senseidb/bql/parsers/BQL.g:1825:1: term_expr returns [Object json] : f= factor_expr ( AND f= factor_expr )* -> { array.length() > 1}? ^( AND_PRED ( factor_expr )+ ) -> factor_expr ;
    public final BQLParser.term_expr_return term_expr() throws RecognitionException {
        BQLParser.term_expr_return retval = new BQLParser.term_expr_return();
        retval.start = input.LT(1);

        int term_expr_StartIndex = input.index();

        Object root_0 = null;

        Token AND60=null;
        BQLParser.factor_expr_return f =null;


        Object AND60_tree=null;
        RewriteRuleTokenStream stream_AND=new RewriteRuleTokenStream(adaptor,"token AND");
        RewriteRuleSubtreeStream stream_factor_expr=new RewriteRuleSubtreeStream(adaptor,"rule factor_expr");

            JSONArray array = new FastJSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 23) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1829:5: (f= factor_expr ( AND f= factor_expr )* -> { array.length() > 1}? ^( AND_PRED ( factor_expr )+ ) -> factor_expr )
            // com/senseidb/bql/parsers/BQL.g:1829:9: f= factor_expr ( AND f= factor_expr )*
            {
            pushFollow(FOLLOW_factor_expr_in_term_expr5543);
            f=factor_expr();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_factor_expr.add(f.getTree());

            if ( state.backtracking==0 ) { array.put((f!=null?f.json:null)); }

            // com/senseidb/bql/parsers/BQL.g:1830:9: ( AND f= factor_expr )*
            loop32:
            do {
                int alt32=2;
                switch ( input.LA(1) ) {
                case AND:
                    {
                    alt32=1;
                    }
                    break;

                }

                switch (alt32) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:1830:10: AND f= factor_expr
            	    {
            	    AND60=(Token)match(input,AND,FOLLOW_AND_in_term_expr5556); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_AND.add(AND60);


            	    pushFollow(FOLLOW_factor_expr_in_term_expr5560);
            	    f=factor_expr();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_factor_expr.add(f.getTree());

            	    if ( state.backtracking==0 ) { array.put((f!=null?f.json:null)); }

            	    }
            	    break;

            	default :
            	    break loop32;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        try {
                            JSONArray newArray = new FastJSONArray();
                            JSONObject fieldMap = new FastJSONObject();
                            for (int i = 0; i < array.length(); ++i) {
                                JSONObject pred = (JSONObject) array.get(i);
                                if (!"range".equals(predType(pred))) {
                                    newArray.put(pred);
                                }
                                else {
                                    accumulateRangePred(input, fieldMap, pred);
                                }
                            }
                            Iterator<String> itr = fieldMap.keys();
                            while (itr.hasNext()) {
                                newArray.put(fieldMap.get(itr.next()));
                            }
                            if (newArray.length() == 1) {
                                retval.json = newArray.get(0);
                            }
                            else {
                                retval.json = new FastJSONObject().put("and", newArray);
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "term_expr", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: factor_expr, factor_expr
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 1859:9: -> { array.length() > 1}? ^( AND_PRED ( factor_expr )+ )
            if ( array.length() > 1) {
                // com/senseidb/bql/parsers/BQL.g:1859:35: ^( AND_PRED ( factor_expr )+ )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                (Object)adaptor.create(AND_PRED, "AND_PRED")
                , root_1);

                if ( !(stream_factor_expr.hasNext()) ) {
                    throw new RewriteEarlyExitException();
                }
                while ( stream_factor_expr.hasNext() ) {
                    adaptor.addChild(root_1, stream_factor_expr.nextTree());

                }
                stream_factor_expr.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            else // 1860:9: -> factor_expr
            {
                adaptor.addChild(root_0, stream_factor_expr.nextTree());

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 23, term_expr_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "term_expr"


    public static class factor_expr_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "factor_expr"
    // com/senseidb/bql/parsers/BQL.g:1863:1: factor_expr returns [Object json] : ( predicate | LPAR search_expr RPAR -> search_expr );
    public final BQLParser.factor_expr_return factor_expr() throws RecognitionException {
        BQLParser.factor_expr_return retval = new BQLParser.factor_expr_return();
        retval.start = input.LT(1);

        int factor_expr_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR62=null;
        Token RPAR64=null;
        BQLParser.predicate_return predicate61 =null;

        BQLParser.search_expr_return search_expr63 =null;


        Object LPAR62_tree=null;
        Object RPAR64_tree=null;
        RewriteRuleTokenStream stream_RPAR=new RewriteRuleTokenStream(adaptor,"token RPAR");
        RewriteRuleTokenStream stream_LPAR=new RewriteRuleTokenStream(adaptor,"token LPAR");
        RewriteRuleSubtreeStream stream_search_expr=new RewriteRuleSubtreeStream(adaptor,"rule search_expr");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 24) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1864:5: ( predicate | LPAR search_expr RPAR -> search_expr )
            int alt33=2;
            switch ( input.LA(1) ) {
            case IDENT:
            case MATCH:
            case NOT:
            case QUERY:
            case STRING_LITERAL:
            case VARIABLE:
                {
                alt33=1;
                }
                break;
            case LPAR:
                {
                switch ( input.LA(2) ) {
                case AGO:
                case DATE:
                case FALSE:
                case INTEGER:
                case NOW:
                case REAL:
                case RPAR:
                case TRUE:
                    {
                    alt33=1;
                    }
                    break;
                case STRING_LITERAL:
                    {
                    switch ( input.LA(3) ) {
                    case COMMA:
                    case RPAR:
                        {
                        alt33=1;
                        }
                        break;
                    case AFTER:
                    case BEFORE:
                    case BETWEEN:
                    case CONTAINS:
                    case EQUAL:
                    case GT:
                    case GTE:
                    case IN:
                    case IS:
                    case LIKE:
                    case LT:
                    case LTE:
                    case NOT:
                    case NOT_EQUAL:
                    case SINCE:
                    case 129:
                        {
                        alt33=2;
                        }
                        break;
                    default:
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 33, 3, input);

                        throw nvae;

                    }

                    }
                    break;
                case VARIABLE:
                    {
                    switch ( input.LA(3) ) {
                    case COMMA:
                    case RPAR:
                        {
                        alt33=1;
                        }
                        break;
                    case IS:
                        {
                        alt33=2;
                        }
                        break;
                    default:
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 33, 4, input);

                        throw nvae;

                    }

                    }
                    break;
                case IDENT:
                case LPAR:
                case MATCH:
                case NOT:
                case QUERY:
                    {
                    alt33=2;
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 33, 2, input);

                    throw nvae;

                }

                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 33, 0, input);

                throw nvae;

            }

            switch (alt33) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1864:9: predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_predicate_in_factor_expr5629);
                    predicate61=predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, predicate61.getTree());

                    if ( state.backtracking==0 ) { retval.json = (predicate61!=null?predicate61.json:null); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1865:9: LPAR search_expr RPAR
                    {
                    LPAR62=(Token)match(input,LPAR,FOLLOW_LPAR_in_factor_expr5641); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_LPAR.add(LPAR62);


                    pushFollow(FOLLOW_search_expr_in_factor_expr5643);
                    search_expr63=search_expr();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_search_expr.add(search_expr63.getTree());

                    RPAR64=(Token)match(input,RPAR,FOLLOW_RPAR_in_factor_expr5645); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_RPAR.add(RPAR64);


                    if ( state.backtracking==0 ) {retval.json = (search_expr63!=null?search_expr63.json:null);}

                    // AST REWRITE
                    // elements: search_expr
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {

                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Object)adaptor.nil();
                    // 1867:9: -> search_expr
                    {
                        adaptor.addChild(root_0, stream_search_expr.nextTree());

                    }


                    retval.tree = root_0;
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 24, factor_expr_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "factor_expr"


    public static class predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "predicate"
    // com/senseidb/bql/parsers/BQL.g:1870:1: predicate returns [JSONObject json] : ( in_predicate | contains_all_predicate | equal_predicate | not_equal_predicate | query_predicate | between_predicate | range_predicate | time_predicate | match_predicate | like_predicate | null_predicate | empty_predicate );
    public final BQLParser.predicate_return predicate() throws RecognitionException {
        BQLParser.predicate_return retval = new BQLParser.predicate_return();
        retval.start = input.LT(1);

        int predicate_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.in_predicate_return in_predicate65 =null;

        BQLParser.contains_all_predicate_return contains_all_predicate66 =null;

        BQLParser.equal_predicate_return equal_predicate67 =null;

        BQLParser.not_equal_predicate_return not_equal_predicate68 =null;

        BQLParser.query_predicate_return query_predicate69 =null;

        BQLParser.between_predicate_return between_predicate70 =null;

        BQLParser.range_predicate_return range_predicate71 =null;

        BQLParser.time_predicate_return time_predicate72 =null;

        BQLParser.match_predicate_return match_predicate73 =null;

        BQLParser.like_predicate_return like_predicate74 =null;

        BQLParser.null_predicate_return null_predicate75 =null;

        BQLParser.empty_predicate_return empty_predicate76 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 25) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1871:5: ( in_predicate | contains_all_predicate | equal_predicate | not_equal_predicate | query_predicate | between_predicate | range_predicate | time_predicate | match_predicate | like_predicate | null_predicate | empty_predicate )
            int alt34=12;
            alt34 = dfa34.predict(input);
            switch (alt34) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1871:9: in_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_in_predicate_in_predicate5690);
                    in_predicate65=in_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, in_predicate65.getTree());

                    if ( state.backtracking==0 ) { retval.json = (in_predicate65!=null?in_predicate65.json:null); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:1872:9: contains_all_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_contains_all_predicate_in_predicate5702);
                    contains_all_predicate66=contains_all_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, contains_all_predicate66.getTree());

                    if ( state.backtracking==0 ) { retval.json = (contains_all_predicate66!=null?contains_all_predicate66.json:null); }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:1873:9: equal_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_equal_predicate_in_predicate5714);
                    equal_predicate67=equal_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, equal_predicate67.getTree());

                    if ( state.backtracking==0 ) { retval.json = (equal_predicate67!=null?equal_predicate67.json:null); }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:1874:9: not_equal_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_not_equal_predicate_in_predicate5726);
                    not_equal_predicate68=not_equal_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, not_equal_predicate68.getTree());

                    if ( state.backtracking==0 ) { retval.json = (not_equal_predicate68!=null?not_equal_predicate68.json:null); }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:1875:9: query_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_query_predicate_in_predicate5738);
                    query_predicate69=query_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, query_predicate69.getTree());

                    if ( state.backtracking==0 ) { retval.json = (query_predicate69!=null?query_predicate69.json:null); }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:1876:9: between_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_between_predicate_in_predicate5750);
                    between_predicate70=between_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, between_predicate70.getTree());

                    if ( state.backtracking==0 ) { retval.json = (between_predicate70!=null?between_predicate70.json:null); }

                    }
                    break;
                case 7 :
                    // com/senseidb/bql/parsers/BQL.g:1877:9: range_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_range_predicate_in_predicate5762);
                    range_predicate71=range_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, range_predicate71.getTree());

                    if ( state.backtracking==0 ) { retval.json = (range_predicate71!=null?range_predicate71.json:null); }

                    }
                    break;
                case 8 :
                    // com/senseidb/bql/parsers/BQL.g:1878:9: time_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_time_predicate_in_predicate5774);
                    time_predicate72=time_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, time_predicate72.getTree());

                    if ( state.backtracking==0 ) { retval.json = (time_predicate72!=null?time_predicate72.json:null); }

                    }
                    break;
                case 9 :
                    // com/senseidb/bql/parsers/BQL.g:1879:9: match_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_match_predicate_in_predicate5786);
                    match_predicate73=match_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, match_predicate73.getTree());

                    if ( state.backtracking==0 ) { retval.json = (match_predicate73!=null?match_predicate73.json:null); }

                    }
                    break;
                case 10 :
                    // com/senseidb/bql/parsers/BQL.g:1880:9: like_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_like_predicate_in_predicate5798);
                    like_predicate74=like_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, like_predicate74.getTree());

                    if ( state.backtracking==0 ) { retval.json = (like_predicate74!=null?like_predicate74.json:null); }

                    }
                    break;
                case 11 :
                    // com/senseidb/bql/parsers/BQL.g:1881:9: null_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_null_predicate_in_predicate5810);
                    null_predicate75=null_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, null_predicate75.getTree());

                    if ( state.backtracking==0 ) { retval.json = (null_predicate75!=null?null_predicate75.json:null); }

                    }
                    break;
                case 12 :
                    // com/senseidb/bql/parsers/BQL.g:1882:9: empty_predicate
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_empty_predicate_in_predicate5822);
                    empty_predicate76=empty_predicate();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, empty_predicate76.getTree());

                    if ( state.backtracking==0 ) { retval.json = (empty_predicate76!=null?empty_predicate76.json:null); }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 25, predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "predicate"


    public static class in_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "in_predicate"
    // com/senseidb/bql/parsers/BQL.g:1885:1: in_predicate returns [JSONObject json] : column_name (not= NOT )? IN value_list (except= except_clause )? ( predicate_props )? -> ^( IN ( NOT )? ^( column_name value_list ) ( except_clause )? ( predicate_props )? ) ;
    public final BQLParser.in_predicate_return in_predicate() throws RecognitionException {
        BQLParser.in_predicate_return retval = new BQLParser.in_predicate_return();
        retval.start = input.LT(1);

        int in_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token not=null;
        Token IN78=null;
        BQLParser.except_clause_return except =null;

        BQLParser.column_name_return column_name77 =null;

        BQLParser.value_list_return value_list79 =null;

        BQLParser.predicate_props_return predicate_props80 =null;


        Object not_tree=null;
        Object IN78_tree=null;
        RewriteRuleTokenStream stream_IN=new RewriteRuleTokenStream(adaptor,"token IN");
        RewriteRuleTokenStream stream_NOT=new RewriteRuleTokenStream(adaptor,"token NOT");
        RewriteRuleSubtreeStream stream_predicate_props=new RewriteRuleSubtreeStream(adaptor,"rule predicate_props");
        RewriteRuleSubtreeStream stream_except_clause=new RewriteRuleSubtreeStream(adaptor,"rule except_clause");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");
        RewriteRuleSubtreeStream stream_value_list=new RewriteRuleSubtreeStream(adaptor,"rule value_list");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 26) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1886:5: ( column_name (not= NOT )? IN value_list (except= except_clause )? ( predicate_props )? -> ^( IN ( NOT )? ^( column_name value_list ) ( except_clause )? ( predicate_props )? ) )
            // com/senseidb/bql/parsers/BQL.g:1886:9: column_name (not= NOT )? IN value_list (except= except_clause )? ( predicate_props )?
            {
            pushFollow(FOLLOW_column_name_in_in_predicate5847);
            column_name77=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(column_name77.getTree());

            // com/senseidb/bql/parsers/BQL.g:1886:24: (not= NOT )?
            int alt35=2;
            switch ( input.LA(1) ) {
                case NOT:
                    {
                    alt35=1;
                    }
                    break;
            }

            switch (alt35) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1886:24: not= NOT
                    {
                    not=(Token)match(input,NOT,FOLLOW_NOT_in_in_predicate5851); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_NOT.add(not);


                    }
                    break;

            }


            IN78=(Token)match(input,IN,FOLLOW_IN_in_in_predicate5854); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_IN.add(IN78);


            pushFollow(FOLLOW_value_list_in_in_predicate5856);
            value_list79=value_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value_list.add(value_list79.getTree());

            // com/senseidb/bql/parsers/BQL.g:1886:50: (except= except_clause )?
            int alt36=2;
            switch ( input.LA(1) ) {
                case EXCEPT:
                    {
                    alt36=1;
                    }
                    break;
            }

            switch (alt36) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1886:50: except= except_clause
                    {
                    pushFollow(FOLLOW_except_clause_in_in_predicate5860);
                    except=except_clause();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_except_clause.add(except.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1886:66: ( predicate_props )?
            int alt37=2;
            switch ( input.LA(1) ) {
                case WITH:
                    {
                    alt37=1;
                    }
                    break;
            }

            switch (alt37) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1886:66: predicate_props
                    {
                    pushFollow(FOLLOW_predicate_props_in_in_predicate5863);
                    predicate_props80=predicate_props();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_predicate_props.add(predicate_props80.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        String col = (column_name77!=null?column_name77.text:null);
                        String[] facetInfo = _facetInfoMap.get(col);

                        if (facetInfo != null && facetInfo[0].equals("range")) {
                            throw new FailedPredicateException(input, 
                                                               "in_predicate",
                                                               "Range facet \"" + col + "\" cannot be used in IN predicates.");
                        }
                        if (!verifyFieldDataType(col, (value_list79!=null?value_list79.json:null))) {
                            throw new FailedPredicateException(input,
                                                               "in_predicate",
                                                               "Value list for IN predicate of facet \"" + col + "\" contains incompatible value(s).");
                        }

                        if (except != null && !verifyFieldDataType(col, (except!=null?except.json:null))) {
                            throw new FailedPredicateException(input,
                                                               "in_predicate",
                                                               "EXCEPT value list for IN predicate of facet \"" + col + "\" contains incompatible value(s).");
                        }

                        try {
                            JSONObject dict = new FastJSONObject();
                            dict.put("operator", "or");
                            if (not == null) {
                                dict.put("values", (value_list79!=null?value_list79.json:null));
                                if (except != null) {
                                    dict.put("excludes", (except!=null?except.json:null));
                                }
                                else {
                                    dict.put("excludes", new FastJSONArray());
                                }
                            }
                            else {
                                dict.put("excludes", (value_list79!=null?value_list79.json:null));
                                if (except != null) {
                                    dict.put("values", (except!=null?except.json:null));
                                }
                                else {
                                    dict.put("values", new FastJSONArray());
                                }
                            }
                            if (_facetInfoMap.get(col) == null) {
                                dict.put("_noOptimize", true);
                            }
                            retval.json = new FastJSONObject().put("terms",
                                                             new FastJSONObject().put(col, dict));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "in_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: value_list, IN, column_name, NOT, predicate_props, except_clause
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 1939:9: -> ^( IN ( NOT )? ^( column_name value_list ) ( except_clause )? ( predicate_props )? )
            {
                // com/senseidb/bql/parsers/BQL.g:1939:12: ^( IN ( NOT )? ^( column_name value_list ) ( except_clause )? ( predicate_props )? )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                stream_IN.nextNode()
                , root_1);

                // com/senseidb/bql/parsers/BQL.g:1939:17: ( NOT )?
                if ( stream_NOT.hasNext() ) {
                    adaptor.addChild(root_1, 
                    stream_NOT.nextNode()
                    );

                }
                stream_NOT.reset();

                // com/senseidb/bql/parsers/BQL.g:1939:22: ^( column_name value_list )
                {
                Object root_2 = (Object)adaptor.nil();
                root_2 = (Object)adaptor.becomeRoot(stream_column_name.nextNode(), root_2);

                adaptor.addChild(root_2, stream_value_list.nextTree());

                adaptor.addChild(root_1, root_2);
                }

                // com/senseidb/bql/parsers/BQL.g:1939:48: ( except_clause )?
                if ( stream_except_clause.hasNext() ) {
                    adaptor.addChild(root_1, stream_except_clause.nextTree());

                }
                stream_except_clause.reset();

                // com/senseidb/bql/parsers/BQL.g:1939:63: ( predicate_props )?
                if ( stream_predicate_props.hasNext() ) {
                    adaptor.addChild(root_1, stream_predicate_props.nextTree());

                }
                stream_predicate_props.reset();

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 26, in_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "in_predicate"


    public static class empty_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "empty_predicate"
    // com/senseidb/bql/parsers/BQL.g:1942:1: empty_predicate returns [JSONObject json] : value_list IS ( NOT )? EMPTY ;
    public final BQLParser.empty_predicate_return empty_predicate() throws RecognitionException {
        BQLParser.empty_predicate_return retval = new BQLParser.empty_predicate_return();
        retval.start = input.LT(1);

        int empty_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token IS82=null;
        Token NOT83=null;
        Token EMPTY84=null;
        BQLParser.value_list_return value_list81 =null;


        Object IS82_tree=null;
        Object NOT83_tree=null;
        Object EMPTY84_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 27) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1943:5: ( value_list IS ( NOT )? EMPTY )
            // com/senseidb/bql/parsers/BQL.g:1943:9: value_list IS ( NOT )? EMPTY
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_value_list_in_empty_predicate5926);
            value_list81=value_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, value_list81.getTree());

            IS82=(Token)match(input,IS,FOLLOW_IS_in_empty_predicate5928); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            IS82_tree = 
            (Object)adaptor.create(IS82)
            ;
            adaptor.addChild(root_0, IS82_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:1943:23: ( NOT )?
            int alt38=2;
            switch ( input.LA(1) ) {
                case NOT:
                    {
                    alt38=1;
                    }
                    break;
            }

            switch (alt38) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1943:24: NOT
                    {
                    NOT83=(Token)match(input,NOT,FOLLOW_NOT_in_empty_predicate5931); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    NOT83_tree = 
                    (Object)adaptor.create(NOT83)
                    ;
                    adaptor.addChild(root_0, NOT83_tree);
                    }

                    }
                    break;

            }


            EMPTY84=(Token)match(input,EMPTY,FOLLOW_EMPTY_in_empty_predicate5935); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            EMPTY84_tree = 
            (Object)adaptor.create(EMPTY84)
            ;
            adaptor.addChild(root_0, EMPTY84_tree);
            }

            if ( state.backtracking==0 ) {   
                        try {
                            JSONObject exp = new FastJSONObject();
                            if (NOT83 != null) {
                                JSONObject functionJSON = new FastJSONObject();
                                JSONArray params = new FastJSONArray();
                                params.put((value_list81!=null?value_list81.json:null));
                                functionJSON.put("function", "length");
                                functionJSON.put("params", params);
                                exp.put("lvalue", functionJSON);
                                exp.put("operator", ">");
                                exp.put("rvalue", 0);
                                retval.json = new FastJSONObject().put("const_exp", exp);
                            }
                            else{
                                JSONObject functionJSON = new FastJSONObject();
                                JSONArray params = new FastJSONArray();
                                params.put((value_list81!=null?value_list81.json:null));
                                functionJSON.put("function", "length");
                                functionJSON.put("params", params);
                                exp.put("lvalue", functionJSON);
                                exp.put("operator", "==");
                                exp.put("rvalue", 0);
                                retval.json = new FastJSONObject().put("const_exp", exp);
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "empty_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 27, empty_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "empty_predicate"


    public static class contains_all_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "contains_all_predicate"
    // com/senseidb/bql/parsers/BQL.g:1976:1: contains_all_predicate returns [JSONObject json] : column_name CONTAINS ALL value_list (except= except_clause )? ( predicate_props )? -> ^( CONTAINS ^( column_name value_list ) ( except_clause )? ( predicate_props )? ) ;
    public final BQLParser.contains_all_predicate_return contains_all_predicate() throws RecognitionException {
        BQLParser.contains_all_predicate_return retval = new BQLParser.contains_all_predicate_return();
        retval.start = input.LT(1);

        int contains_all_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token CONTAINS86=null;
        Token ALL87=null;
        BQLParser.except_clause_return except =null;

        BQLParser.column_name_return column_name85 =null;

        BQLParser.value_list_return value_list88 =null;

        BQLParser.predicate_props_return predicate_props89 =null;


        Object CONTAINS86_tree=null;
        Object ALL87_tree=null;
        RewriteRuleTokenStream stream_CONTAINS=new RewriteRuleTokenStream(adaptor,"token CONTAINS");
        RewriteRuleTokenStream stream_ALL=new RewriteRuleTokenStream(adaptor,"token ALL");
        RewriteRuleSubtreeStream stream_predicate_props=new RewriteRuleSubtreeStream(adaptor,"rule predicate_props");
        RewriteRuleSubtreeStream stream_except_clause=new RewriteRuleSubtreeStream(adaptor,"rule except_clause");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");
        RewriteRuleSubtreeStream stream_value_list=new RewriteRuleSubtreeStream(adaptor,"rule value_list");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 28) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:1977:5: ( column_name CONTAINS ALL value_list (except= except_clause )? ( predicate_props )? -> ^( CONTAINS ^( column_name value_list ) ( except_clause )? ( predicate_props )? ) )
            // com/senseidb/bql/parsers/BQL.g:1977:9: column_name CONTAINS ALL value_list (except= except_clause )? ( predicate_props )?
            {
            pushFollow(FOLLOW_column_name_in_contains_all_predicate5972);
            column_name85=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(column_name85.getTree());

            CONTAINS86=(Token)match(input,CONTAINS,FOLLOW_CONTAINS_in_contains_all_predicate5974); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_CONTAINS.add(CONTAINS86);


            ALL87=(Token)match(input,ALL,FOLLOW_ALL_in_contains_all_predicate5976); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_ALL.add(ALL87);


            pushFollow(FOLLOW_value_list_in_contains_all_predicate5978);
            value_list88=value_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value_list.add(value_list88.getTree());

            // com/senseidb/bql/parsers/BQL.g:1977:51: (except= except_clause )?
            int alt39=2;
            switch ( input.LA(1) ) {
                case EXCEPT:
                    {
                    alt39=1;
                    }
                    break;
            }

            switch (alt39) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1977:51: except= except_clause
                    {
                    pushFollow(FOLLOW_except_clause_in_contains_all_predicate5982);
                    except=except_clause();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_except_clause.add(except.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:1977:67: ( predicate_props )?
            int alt40=2;
            switch ( input.LA(1) ) {
                case WITH:
                    {
                    alt40=1;
                    }
                    break;
            }

            switch (alt40) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:1977:67: predicate_props
                    {
                    pushFollow(FOLLOW_predicate_props_in_contains_all_predicate5985);
                    predicate_props89=predicate_props();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_predicate_props.add(predicate_props89.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        String col = (column_name85!=null?column_name85.text:null);
                        String[] facetInfo = _facetInfoMap.get(col);
                        if (facetInfo != null && facetInfo[0].equals("range")) {
                            throw new FailedPredicateException(input, 
                                                               "contains_all_predicate", 
                                                               "Range facet column \"" + col + "\" cannot be used in CONTAINS ALL predicates.");
                        }
                        if (!verifyFieldDataType(col, (value_list88!=null?value_list88.json:null))) {
                            throw new FailedPredicateException(input,
                                                               "contains_all_predicate",
                                                               "Value list for CONTAINS ALL predicate of facet \"" + col + "\" contains incompatible value(s).");
                        }

                        if (except != null && !verifyFieldDataType(col, (except!=null?except.json:null))) {
                            throw new FailedPredicateException(input,
                                                               "contains_all_predicate",
                                                               "EXCEPT value list for CONTAINS ALL predicate of facet \"" + col + "\" contains incompatible value(s).");
                        }

                        try {
                            JSONObject dict = new FastJSONObject();
                            dict.put("operator", "and");
                            dict.put("values", (value_list88!=null?value_list88.json:null));
                            if (except != null) {
                                dict.put("excludes", (except!=null?except.json:null));
                            }
                            else {
                                dict.put("excludes", new FastJSONArray());
                            }
                            if (_facetInfoMap.get(col) == null) {
                                dict.put("_noOptimize", true);
                            }
                            retval.json = new FastJSONObject().put("terms",
                                                             new FastJSONObject().put(col, dict));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "contains_all_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: CONTAINS, value_list, predicate_props, except_clause, column_name
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 2018:9: -> ^( CONTAINS ^( column_name value_list ) ( except_clause )? ( predicate_props )? )
            {
                // com/senseidb/bql/parsers/BQL.g:2018:12: ^( CONTAINS ^( column_name value_list ) ( except_clause )? ( predicate_props )? )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                stream_CONTAINS.nextNode()
                , root_1);

                // com/senseidb/bql/parsers/BQL.g:2018:23: ^( column_name value_list )
                {
                Object root_2 = (Object)adaptor.nil();
                root_2 = (Object)adaptor.becomeRoot(stream_column_name.nextNode(), root_2);

                adaptor.addChild(root_2, stream_value_list.nextTree());

                adaptor.addChild(root_1, root_2);
                }

                // com/senseidb/bql/parsers/BQL.g:2018:49: ( except_clause )?
                if ( stream_except_clause.hasNext() ) {
                    adaptor.addChild(root_1, stream_except_clause.nextTree());

                }
                stream_except_clause.reset();

                // com/senseidb/bql/parsers/BQL.g:2018:64: ( predicate_props )?
                if ( stream_predicate_props.hasNext() ) {
                    adaptor.addChild(root_1, stream_predicate_props.nextTree());

                }
                stream_predicate_props.reset();

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 28, contains_all_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "contains_all_predicate"


    public static class equal_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "equal_predicate"
    // com/senseidb/bql/parsers/BQL.g:2021:1: equal_predicate returns [JSONObject json] : column_name EQUAL value (props= predicate_props )? -> ^( EQUAL column_name value ( predicate_props )? ) ;
    public final BQLParser.equal_predicate_return equal_predicate() throws RecognitionException {
        BQLParser.equal_predicate_return retval = new BQLParser.equal_predicate_return();
        retval.start = input.LT(1);

        int equal_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token EQUAL91=null;
        BQLParser.predicate_props_return props =null;

        BQLParser.column_name_return column_name90 =null;

        BQLParser.value_return value92 =null;


        Object EQUAL91_tree=null;
        RewriteRuleTokenStream stream_EQUAL=new RewriteRuleTokenStream(adaptor,"token EQUAL");
        RewriteRuleSubtreeStream stream_predicate_props=new RewriteRuleSubtreeStream(adaptor,"rule predicate_props");
        RewriteRuleSubtreeStream stream_value=new RewriteRuleSubtreeStream(adaptor,"rule value");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 29) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2022:5: ( column_name EQUAL value (props= predicate_props )? -> ^( EQUAL column_name value ( predicate_props )? ) )
            // com/senseidb/bql/parsers/BQL.g:2022:9: column_name EQUAL value (props= predicate_props )?
            {
            pushFollow(FOLLOW_column_name_in_equal_predicate6046);
            column_name90=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(column_name90.getTree());

            EQUAL91=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_equal_predicate6048); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_EQUAL.add(EQUAL91);


            pushFollow(FOLLOW_value_in_equal_predicate6050);
            value92=value();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value.add(value92.getTree());

            // com/senseidb/bql/parsers/BQL.g:2022:38: (props= predicate_props )?
            int alt41=2;
            switch ( input.LA(1) ) {
                case WITH:
                    {
                    alt41=1;
                    }
                    break;
            }

            switch (alt41) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2022:38: props= predicate_props
                    {
                    pushFollow(FOLLOW_predicate_props_in_equal_predicate6054);
                    props=predicate_props();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_predicate_props.add(props.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        String col = (column_name90!=null?column_name90.text:null);
                        if (!verifyFieldDataType(col, (value92!=null?value92.val:null))) {
                            throw new FailedPredicateException(input, 
                                                               "equal_predicate", 
                                                               "Incompatible data type was found in an EQUAL predicate for column \"" + col + "\".");
                        }
                        try {
                            String[] facetInfo = _facetInfoMap.get(col);
                            if (facetInfo != null && facetInfo[0].equals("range")) {
                                retval.json = new FastJSONObject().put("range",
                                           new FastJSONObject().put(col,
                                               new FastJSONObject().put("from", (value92!=null?value92.val:null))
                                                                   .put("to", (value92!=null?value92.val:null))
                                                                   .put("include_lower", true)
                                                                   .put("include_upper", true)));
                            }
                            else if (facetInfo != null && facetInfo[0].equals("path")) {
                                JSONObject valObj = new FastJSONObject();
                                valObj.put("value", (value92!=null?value92.val:null));
                                if (props != null) {
                                    JSONObject propsJson = (props!=null?props.json:null);
                                    Iterator<String> itr = propsJson.keys();
                                    while (itr.hasNext()) {
                                        String key = itr.next();
                                        if (key.equals("strict") || key.equals("depth")) {
                                            valObj.put(key, propsJson.get(key));
                                        }
                                        else {
                                            throw new FailedPredicateException(input,
                                                                               "equal_predicate", 
                                                                               "Unsupported property was found in an EQUAL predicate for path facet column \"" + col + "\": " + key + ".");
                                        }
                                    }
                                }
                                retval.json = new FastJSONObject().put("path", new FastJSONObject().put(col, valObj));
                            }
                            else {
                                JSONObject valSpec = new FastJSONObject().put("value", (value92!=null?value92.val:null));
                                if (_facetInfoMap.get(col) == null) {
                                    valSpec.put("_noOptimize", true);
                                }
                                retval.json = new FastJSONObject().put("term", 
                                                                 new FastJSONObject().put(col, valSpec));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "equal_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: predicate_props, column_name, EQUAL, value
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 2073:9: -> ^( EQUAL column_name value ( predicate_props )? )
            {
                // com/senseidb/bql/parsers/BQL.g:2073:12: ^( EQUAL column_name value ( predicate_props )? )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                stream_EQUAL.nextNode()
                , root_1);

                adaptor.addChild(root_1, stream_column_name.nextTree());

                adaptor.addChild(root_1, stream_value.nextTree());

                // com/senseidb/bql/parsers/BQL.g:2073:38: ( predicate_props )?
                if ( stream_predicate_props.hasNext() ) {
                    adaptor.addChild(root_1, stream_predicate_props.nextTree());

                }
                stream_predicate_props.reset();

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 29, equal_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "equal_predicate"


    public static class not_equal_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "not_equal_predicate"
    // com/senseidb/bql/parsers/BQL.g:2076:1: not_equal_predicate returns [JSONObject json] : column_name NOT_EQUAL value ( predicate_props )? -> ^( NOT_EQUAL column_name value ( predicate_props )? ) ;
    public final BQLParser.not_equal_predicate_return not_equal_predicate() throws RecognitionException {
        BQLParser.not_equal_predicate_return retval = new BQLParser.not_equal_predicate_return();
        retval.start = input.LT(1);

        int not_equal_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token NOT_EQUAL94=null;
        BQLParser.column_name_return column_name93 =null;

        BQLParser.value_return value95 =null;

        BQLParser.predicate_props_return predicate_props96 =null;


        Object NOT_EQUAL94_tree=null;
        RewriteRuleTokenStream stream_NOT_EQUAL=new RewriteRuleTokenStream(adaptor,"token NOT_EQUAL");
        RewriteRuleSubtreeStream stream_predicate_props=new RewriteRuleSubtreeStream(adaptor,"rule predicate_props");
        RewriteRuleSubtreeStream stream_value=new RewriteRuleSubtreeStream(adaptor,"rule value");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 30) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2077:5: ( column_name NOT_EQUAL value ( predicate_props )? -> ^( NOT_EQUAL column_name value ( predicate_props )? ) )
            // com/senseidb/bql/parsers/BQL.g:2077:9: column_name NOT_EQUAL value ( predicate_props )?
            {
            pushFollow(FOLLOW_column_name_in_not_equal_predicate6109);
            column_name93=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(column_name93.getTree());

            NOT_EQUAL94=(Token)match(input,NOT_EQUAL,FOLLOW_NOT_EQUAL_in_not_equal_predicate6111); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_NOT_EQUAL.add(NOT_EQUAL94);


            pushFollow(FOLLOW_value_in_not_equal_predicate6113);
            value95=value();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value.add(value95.getTree());

            // com/senseidb/bql/parsers/BQL.g:2077:37: ( predicate_props )?
            int alt42=2;
            switch ( input.LA(1) ) {
                case WITH:
                    {
                    alt42=1;
                    }
                    break;
            }

            switch (alt42) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2077:37: predicate_props
                    {
                    pushFollow(FOLLOW_predicate_props_in_not_equal_predicate6115);
                    predicate_props96=predicate_props();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_predicate_props.add(predicate_props96.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        String col = (column_name93!=null?column_name93.text:null);
                        if (!verifyFieldDataType(col, (value95!=null?value95.val:null))) {
                            throw new FailedPredicateException(input, 
                                                               "not_equal_predicate", 
                                                               "Incompatible data type was found in a NOT EQUAL predicate for column \"" + col + "\".");
                        }
                        try {
                            String[] facetInfo = _facetInfoMap.get(col);
                            if (facetInfo != null && facetInfo[0].equals("range")) {
                                JSONObject left = new FastJSONObject().put("range",
                                                                           new FastJSONObject().put(col,
                                                                                                    new FastJSONObject().put("to", (value95!=null?value95.val:null))
                                                                                                                        .put("include_upper", false)));
                                JSONObject right = new FastJSONObject().put("range",
                                                                            new FastJSONObject().put(col,
                                                                                                     new FastJSONObject().put("from", (value95!=null?value95.val:null))
                                                                                                                         .put("include_lower", false)));
                                retval.json = new FastJSONObject().put("or", new FastJSONArray().put(left).put(right));
                            }
                            else if (facetInfo != null && facetInfo[0].equals("path")) {
                                throw new FailedPredicateException(input, 
                                                                   "not_equal_predicate",
                                                                   "NOT EQUAL predicate is not supported for path facets (column \"" + col + "\").");
                            }
                            else {
                                JSONObject valObj = new FastJSONObject();
                                valObj.put("operator", "or");
                                valObj.put("values", new FastJSONArray());
                                valObj.put("excludes", new FastJSONArray().put((value95!=null?value95.val:null)));
                                if (_facetInfoMap.get(col) == null) {
                                    valObj.put("_noOptimize", true);
                                }
                                retval.json = new FastJSONObject().put("terms",
                                                                 new FastJSONObject().put(col, valObj));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "not_equal_predicate", "JSONException: " + err.getMessage());
                        }                                         
                    }

            // AST REWRITE
            // elements: column_name, value, predicate_props, NOT_EQUAL
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 2119:9: -> ^( NOT_EQUAL column_name value ( predicate_props )? )
            {
                // com/senseidb/bql/parsers/BQL.g:2119:12: ^( NOT_EQUAL column_name value ( predicate_props )? )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                stream_NOT_EQUAL.nextNode()
                , root_1);

                adaptor.addChild(root_1, stream_column_name.nextTree());

                adaptor.addChild(root_1, stream_value.nextTree());

                // com/senseidb/bql/parsers/BQL.g:2119:42: ( predicate_props )?
                if ( stream_predicate_props.hasNext() ) {
                    adaptor.addChild(root_1, stream_predicate_props.nextTree());

                }
                stream_predicate_props.reset();

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 30, not_equal_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "not_equal_predicate"


    public static class query_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "query_predicate"
    // com/senseidb/bql/parsers/BQL.g:2122:1: query_predicate returns [JSONObject json] : QUERY IS STRING_LITERAL -> ^( QUERY STRING_LITERAL ) ;
    public final BQLParser.query_predicate_return query_predicate() throws RecognitionException {
        BQLParser.query_predicate_return retval = new BQLParser.query_predicate_return();
        retval.start = input.LT(1);

        int query_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token QUERY97=null;
        Token IS98=null;
        Token STRING_LITERAL99=null;

        Object QUERY97_tree=null;
        Object IS98_tree=null;
        Object STRING_LITERAL99_tree=null;
        RewriteRuleTokenStream stream_QUERY=new RewriteRuleTokenStream(adaptor,"token QUERY");
        RewriteRuleTokenStream stream_STRING_LITERAL=new RewriteRuleTokenStream(adaptor,"token STRING_LITERAL");
        RewriteRuleTokenStream stream_IS=new RewriteRuleTokenStream(adaptor,"token IS");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 31) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2123:5: ( QUERY IS STRING_LITERAL -> ^( QUERY STRING_LITERAL ) )
            // com/senseidb/bql/parsers/BQL.g:2123:9: QUERY IS STRING_LITERAL
            {
            QUERY97=(Token)match(input,QUERY,FOLLOW_QUERY_in_query_predicate6170); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_QUERY.add(QUERY97);


            IS98=(Token)match(input,IS,FOLLOW_IS_in_query_predicate6172); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_IS.add(IS98);


            STRING_LITERAL99=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_query_predicate6174); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_STRING_LITERAL.add(STRING_LITERAL99);


            if ( state.backtracking==0 ) {
                        try {
                            String orig = (STRING_LITERAL99!=null?STRING_LITERAL99.getText():null);
                            orig = orig.substring(1, orig.length() - 1);
                            retval.json = new FastJSONObject().put("query",
                                                             new FastJSONObject().put("query_string",
                                                                                      new FastJSONObject().put("query", orig)));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "query_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: QUERY, STRING_LITERAL
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 2136:9: -> ^( QUERY STRING_LITERAL )
            {
                // com/senseidb/bql/parsers/BQL.g:2136:12: ^( QUERY STRING_LITERAL )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                stream_QUERY.nextNode()
                , root_1);

                adaptor.addChild(root_1, 
                stream_STRING_LITERAL.nextNode()
                );

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 31, query_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "query_predicate"


    public static class between_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "between_predicate"
    // com/senseidb/bql/parsers/BQL.g:2139:1: between_predicate returns [JSONObject json] : column_name (not= NOT )? BETWEEN val1= value AND val2= value -> ^( BETWEEN ( NOT )? $val1 $val2) ;
    public final BQLParser.between_predicate_return between_predicate() throws RecognitionException {
        BQLParser.between_predicate_return retval = new BQLParser.between_predicate_return();
        retval.start = input.LT(1);

        int between_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token not=null;
        Token BETWEEN101=null;
        Token AND102=null;
        BQLParser.value_return val1 =null;

        BQLParser.value_return val2 =null;

        BQLParser.column_name_return column_name100 =null;


        Object not_tree=null;
        Object BETWEEN101_tree=null;
        Object AND102_tree=null;
        RewriteRuleTokenStream stream_NOT=new RewriteRuleTokenStream(adaptor,"token NOT");
        RewriteRuleTokenStream stream_AND=new RewriteRuleTokenStream(adaptor,"token AND");
        RewriteRuleTokenStream stream_BETWEEN=new RewriteRuleTokenStream(adaptor,"token BETWEEN");
        RewriteRuleSubtreeStream stream_value=new RewriteRuleSubtreeStream(adaptor,"rule value");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 32) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2140:5: ( column_name (not= NOT )? BETWEEN val1= value AND val2= value -> ^( BETWEEN ( NOT )? $val1 $val2) )
            // com/senseidb/bql/parsers/BQL.g:2140:9: column_name (not= NOT )? BETWEEN val1= value AND val2= value
            {
            pushFollow(FOLLOW_column_name_in_between_predicate6223);
            column_name100=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(column_name100.getTree());

            // com/senseidb/bql/parsers/BQL.g:2140:24: (not= NOT )?
            int alt43=2;
            switch ( input.LA(1) ) {
                case NOT:
                    {
                    alt43=1;
                    }
                    break;
            }

            switch (alt43) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2140:24: not= NOT
                    {
                    not=(Token)match(input,NOT,FOLLOW_NOT_in_between_predicate6227); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_NOT.add(not);


                    }
                    break;

            }


            BETWEEN101=(Token)match(input,BETWEEN,FOLLOW_BETWEEN_in_between_predicate6230); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_BETWEEN.add(BETWEEN101);


            pushFollow(FOLLOW_value_in_between_predicate6234);
            val1=value();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value.add(val1.getTree());

            AND102=(Token)match(input,AND,FOLLOW_AND_in_between_predicate6236); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_AND.add(AND102);


            pushFollow(FOLLOW_value_in_between_predicate6240);
            val2=value();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value.add(val2.getTree());

            if ( state.backtracking==0 ) {
                        String col = (column_name100!=null?column_name100.text:null);
                        if (!verifyFacetType(col, "range")) {
                            throw new FailedPredicateException(input, 
                                                               "between_predicate",
                                                               "Non-rangable facet column \"" + col + "\" cannot be used in BETWEEN predicates.");
                        }

                        if (!verifyFieldDataType(col, new Object[]{(val1!=null?val1.val:null), (val2!=null?val2.val:null)})) {
                            throw new FailedPredicateException(input,
                                                               "between_predicate", 
                                                               "Incompatible data type was found in a BETWEEN predicate for column \"" + col + "\".");
                        }

                        try {
                            if (not == null) {
                                retval.json = new FastJSONObject().put("range",
                                                                 new FastJSONObject().put(col,
                                                                                          new FastJSONObject().put("from", (val1!=null?val1.val:null))
                                                                                                              .put("to", (val2!=null?val2.val:null))
                                                                                                              .put("include_lower", true)
                                                                                                              .put("include_upper", true)));
                            }
                            else {
                                JSONObject range1 = 
                                    new FastJSONObject().put("range",
                                                             new FastJSONObject().put(col,
                                                                                      new FastJSONObject().put("to", (val1!=null?val1.val:null))
                                                                                                          .put("include_upper", false)));
                                JSONObject range2 = 
                                    new FastJSONObject().put("range",
                                                             new FastJSONObject().put(col,
                                                                                      new FastJSONObject().put("from", (val2!=null?val2.val:null))
                                                                                                          .put("include_lower", false)));

                                retval.json = new FastJSONObject().put("or", new FastJSONArray().put(range1).put(range2));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "between_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: val2, val1, NOT, BETWEEN
            // token labels: 
            // rule labels: retval, val1, val2
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            RewriteRuleSubtreeStream stream_val1=new RewriteRuleSubtreeStream(adaptor,"rule val1",val1!=null?val1.tree:null);
            RewriteRuleSubtreeStream stream_val2=new RewriteRuleSubtreeStream(adaptor,"rule val2",val2!=null?val2.tree:null);

            root_0 = (Object)adaptor.nil();
            // 2183:9: -> ^( BETWEEN ( NOT )? $val1 $val2)
            {
                // com/senseidb/bql/parsers/BQL.g:2183:12: ^( BETWEEN ( NOT )? $val1 $val2)
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(
                stream_BETWEEN.nextNode()
                , root_1);

                // com/senseidb/bql/parsers/BQL.g:2183:22: ( NOT )?
                if ( stream_NOT.hasNext() ) {
                    adaptor.addChild(root_1, 
                    stream_NOT.nextNode()
                    );

                }
                stream_NOT.reset();

                adaptor.addChild(root_1, stream_val1.nextTree());

                adaptor.addChild(root_1, stream_val2.nextTree());

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 32, between_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "between_predicate"


    public static class range_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "range_predicate"
    // com/senseidb/bql/parsers/BQL.g:2186:1: range_predicate returns [JSONObject json] : column_name (op= GT |op= GTE |op= LT |op= LTE ) val= value -> ^( $op column_name value ) ;
    public final BQLParser.range_predicate_return range_predicate() throws RecognitionException {
        BQLParser.range_predicate_return retval = new BQLParser.range_predicate_return();
        retval.start = input.LT(1);

        int range_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token op=null;
        BQLParser.value_return val =null;

        BQLParser.column_name_return column_name103 =null;


        Object op_tree=null;
        RewriteRuleTokenStream stream_GT=new RewriteRuleTokenStream(adaptor,"token GT");
        RewriteRuleTokenStream stream_LT=new RewriteRuleTokenStream(adaptor,"token LT");
        RewriteRuleTokenStream stream_LTE=new RewriteRuleTokenStream(adaptor,"token LTE");
        RewriteRuleTokenStream stream_GTE=new RewriteRuleTokenStream(adaptor,"token GTE");
        RewriteRuleSubtreeStream stream_value=new RewriteRuleSubtreeStream(adaptor,"rule value");
        RewriteRuleSubtreeStream stream_column_name=new RewriteRuleSubtreeStream(adaptor,"rule column_name");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 33) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2187:5: ( column_name (op= GT |op= GTE |op= LT |op= LTE ) val= value -> ^( $op column_name value ) )
            // com/senseidb/bql/parsers/BQL.g:2187:9: column_name (op= GT |op= GTE |op= LT |op= LTE ) val= value
            {
            pushFollow(FOLLOW_column_name_in_range_predicate6296);
            column_name103=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_column_name.add(column_name103.getTree());

            // com/senseidb/bql/parsers/BQL.g:2187:21: (op= GT |op= GTE |op= LT |op= LTE )
            int alt44=4;
            switch ( input.LA(1) ) {
            case GT:
                {
                alt44=1;
                }
                break;
            case GTE:
                {
                alt44=2;
                }
                break;
            case LT:
                {
                alt44=3;
                }
                break;
            case LTE:
                {
                alt44=4;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 44, 0, input);

                throw nvae;

            }

            switch (alt44) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2187:22: op= GT
                    {
                    op=(Token)match(input,GT,FOLLOW_GT_in_range_predicate6301); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_GT.add(op);


                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2187:30: op= GTE
                    {
                    op=(Token)match(input,GTE,FOLLOW_GTE_in_range_predicate6307); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_GTE.add(op);


                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2187:39: op= LT
                    {
                    op=(Token)match(input,LT,FOLLOW_LT_in_range_predicate6313); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_LT.add(op);


                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2187:47: op= LTE
                    {
                    op=(Token)match(input,LTE,FOLLOW_LTE_in_range_predicate6319); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_LTE.add(op);


                    }
                    break;

            }


            pushFollow(FOLLOW_value_in_range_predicate6324);
            val=value();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_value.add(val.getTree());

            if ( state.backtracking==0 ) {
                        String col = (column_name103!=null?column_name103.text:null);
                        if (!verifyFacetType(col, "range")) {
                            throw new FailedPredicateException(input, 
                                                               "range_predicate",
                                                               "Non-rangable facet column \"" + col + "\" cannot be used in RANGE predicates.");
                        }

                        if (!verifyFieldDataType(col, (val!=null?val.val:null))) {
                            throw new FailedPredicateException(input,
                                                               "range_predicate", 
                                                               "Incompatible data type was found in a RANGE predicate for column \"" + col + "\".");
                        }

                        try {
                            if ((op!=null?op.getText():null).charAt(0) == '>') {
                                retval.json = new FastJSONObject().put("range",
                                                                 new FastJSONObject().put(col,
                                                                                          new FastJSONObject().put("from", (val!=null?val.val:null))
                                                                                                              .put("include_lower", ">=".equals((op!=null?op.getText():null)))));
                            }
                            else {
                                retval.json = new FastJSONObject().put("range",
                                                                 new FastJSONObject().put(col,
                                                                                          new FastJSONObject().put("to", (val!=null?val.val:null))
                                                                                                              .put("include_upper", "<=".equals((op!=null?op.getText():null)))));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "range_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            // AST REWRITE
            // elements: column_name, value, op
            // token labels: op
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {

            retval.tree = root_0;
            RewriteRuleTokenStream stream_op=new RewriteRuleTokenStream(adaptor,"token op",op);
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Object)adaptor.nil();
            // 2220:9: -> ^( $op column_name value )
            {
                // com/senseidb/bql/parsers/BQL.g:2220:12: ^( $op column_name value )
                {
                Object root_1 = (Object)adaptor.nil();
                root_1 = (Object)adaptor.becomeRoot(stream_op.nextNode(), root_1);

                adaptor.addChild(root_1, stream_column_name.nextTree());

                adaptor.addChild(root_1, stream_value.nextTree());

                adaptor.addChild(root_0, root_1);
                }

            }


            retval.tree = root_0;
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 33, range_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "range_predicate"


    public static class time_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_predicate"
    // com/senseidb/bql/parsers/BQL.g:2223:1: time_predicate returns [JSONObject json] : ( column_name ( NOT )? IN LAST time_span | column_name ( NOT )? (since= SINCE |since= AFTER |before= BEFORE ) time_expr );
    public final BQLParser.time_predicate_return time_predicate() throws RecognitionException {
        BQLParser.time_predicate_return retval = new BQLParser.time_predicate_return();
        retval.start = input.LT(1);

        int time_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token since=null;
        Token before=null;
        Token NOT105=null;
        Token IN106=null;
        Token LAST107=null;
        Token NOT110=null;
        BQLParser.column_name_return column_name104 =null;

        BQLParser.time_span_return time_span108 =null;

        BQLParser.column_name_return column_name109 =null;

        BQLParser.time_expr_return time_expr111 =null;


        Object since_tree=null;
        Object before_tree=null;
        Object NOT105_tree=null;
        Object IN106_tree=null;
        Object LAST107_tree=null;
        Object NOT110_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 34) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2224:5: ( column_name ( NOT )? IN LAST time_span | column_name ( NOT )? (since= SINCE |since= AFTER |before= BEFORE ) time_expr )
            int alt48=2;
            alt48 = dfa48.predict(input);
            switch (alt48) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2224:9: column_name ( NOT )? IN LAST time_span
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_column_name_in_time_predicate6376);
                    column_name104=column_name();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name104.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2224:21: ( NOT )?
                    int alt45=2;
                    switch ( input.LA(1) ) {
                        case NOT:
                            {
                            alt45=1;
                            }
                            break;
                    }

                    switch (alt45) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2224:22: NOT
                            {
                            NOT105=(Token)match(input,NOT,FOLLOW_NOT_in_time_predicate6379); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            NOT105_tree = 
                            (Object)adaptor.create(NOT105)
                            ;
                            adaptor.addChild(root_0, NOT105_tree);
                            }

                            }
                            break;

                    }


                    IN106=(Token)match(input,IN,FOLLOW_IN_in_time_predicate6383); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    IN106_tree = 
                    (Object)adaptor.create(IN106)
                    ;
                    adaptor.addChild(root_0, IN106_tree);
                    }

                    LAST107=(Token)match(input,LAST,FOLLOW_LAST_in_time_predicate6385); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    LAST107_tree = 
                    (Object)adaptor.create(LAST107)
                    ;
                    adaptor.addChild(root_0, LAST107_tree);
                    }

                    pushFollow(FOLLOW_time_span_in_time_predicate6387);
                    time_span108=time_span();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, time_span108.getTree());

                    if ( state.backtracking==0 ) {
                                String col = (column_name104!=null?column_name104.text:null);
                                if (!verifyFacetType(col, "range")) {
                                    throw new FailedPredicateException(input, 
                                                                       "range_predicate",
                                                                       "Non-rangable facet column \"" + col + "\" cannot be used in TIME predicates.");
                                }

                                try {
                                    if (NOT105 == null) {
                                        retval.json = new FastJSONObject().put("range",
                                                                         new FastJSONObject().put(col,
                                                                                                  new FastJSONObject().put("from", (time_span108!=null?time_span108.val:0))
                                                                                                                      .put("include_lower", false)));
                                    }
                                    else {
                                        retval.json = new FastJSONObject().put("range",
                                                                         new FastJSONObject().put(col,
                                                                                                  new FastJSONObject().put("to", (time_span108!=null?time_span108.val:0))
                                                                                                                      .put("include_upper", true)));
                                    }
                                }
                                catch (JSONException err) {
                                    throw new FailedPredicateException(input, "time_predicate", "JSONException: " + err.getMessage());
                                }
                            }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2251:9: column_name ( NOT )? (since= SINCE |since= AFTER |before= BEFORE ) time_expr
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_column_name_in_time_predicate6407);
                    column_name109=column_name();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name109.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2251:21: ( NOT )?
                    int alt46=2;
                    switch ( input.LA(1) ) {
                        case NOT:
                            {
                            alt46=1;
                            }
                            break;
                    }

                    switch (alt46) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2251:22: NOT
                            {
                            NOT110=(Token)match(input,NOT,FOLLOW_NOT_in_time_predicate6410); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            NOT110_tree = 
                            (Object)adaptor.create(NOT110)
                            ;
                            adaptor.addChild(root_0, NOT110_tree);
                            }

                            }
                            break;

                    }


                    // com/senseidb/bql/parsers/BQL.g:2251:28: (since= SINCE |since= AFTER |before= BEFORE )
                    int alt47=3;
                    switch ( input.LA(1) ) {
                    case SINCE:
                        {
                        alt47=1;
                        }
                        break;
                    case AFTER:
                        {
                        alt47=2;
                        }
                        break;
                    case BEFORE:
                        {
                        alt47=3;
                        }
                        break;
                    default:
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 47, 0, input);

                        throw nvae;

                    }

                    switch (alt47) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2251:29: since= SINCE
                            {
                            since=(Token)match(input,SINCE,FOLLOW_SINCE_in_time_predicate6417); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            since_tree = 
                            (Object)adaptor.create(since)
                            ;
                            adaptor.addChild(root_0, since_tree);
                            }

                            }
                            break;
                        case 2 :
                            // com/senseidb/bql/parsers/BQL.g:2251:43: since= AFTER
                            {
                            since=(Token)match(input,AFTER,FOLLOW_AFTER_in_time_predicate6423); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            since_tree = 
                            (Object)adaptor.create(since)
                            ;
                            adaptor.addChild(root_0, since_tree);
                            }

                            }
                            break;
                        case 3 :
                            // com/senseidb/bql/parsers/BQL.g:2251:57: before= BEFORE
                            {
                            before=(Token)match(input,BEFORE,FOLLOW_BEFORE_in_time_predicate6429); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            before_tree = 
                            (Object)adaptor.create(before)
                            ;
                            adaptor.addChild(root_0, before_tree);
                            }

                            }
                            break;

                    }


                    pushFollow(FOLLOW_time_expr_in_time_predicate6432);
                    time_expr111=time_expr();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, time_expr111.getTree());

                    if ( state.backtracking==0 ) {
                                String col = (column_name109!=null?column_name109.text:null);
                                if (!verifyFacetType(col, "range")) {
                                    throw new FailedPredicateException(input, 
                                                                       "range_predicate",
                                                                       "Non-rangable facet column \"" + col + "\" cannot be used in TIME predicates.");
                                }

                                try {
                                    if (since != null && NOT110 == null ||
                                        since == null && NOT110 != null) {
                                        retval.json = new FastJSONObject().put("range",
                                                                         new FastJSONObject().put(col,
                                                                                                  new FastJSONObject().put("from", (time_expr111!=null?time_expr111.val:0))
                                                                                                                      .put("include_lower", false)));
                                    }
                                    else {
                                        retval.json = new FastJSONObject().put("range",
                                                                         new FastJSONObject().put(col,
                                                                                                  new FastJSONObject().put("to", (time_expr111!=null?time_expr111.val:0))
                                                                                                                      .put("include_upper", false)));
                                    }
                                }
                                catch (JSONException err) {
                                    throw new FailedPredicateException(input, "time_predicate", "JSONException: " + err.getMessage());
                                }
                            }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 34, time_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_predicate"


    public static class time_span_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_span"
    // com/senseidb/bql/parsers/BQL.g:2281:1: time_span returns [long val] : (week= time_week_part )? (day= time_day_part )? (hour= time_hour_part )? (minute= time_minute_part )? (second= time_second_part )? (msec= time_millisecond_part )? ;
    public final BQLParser.time_span_return time_span() throws RecognitionException {
        BQLParser.time_span_return retval = new BQLParser.time_span_return();
        retval.start = input.LT(1);

        int time_span_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.time_week_part_return week =null;

        BQLParser.time_day_part_return day =null;

        BQLParser.time_hour_part_return hour =null;

        BQLParser.time_minute_part_return minute =null;

        BQLParser.time_second_part_return second =null;

        BQLParser.time_millisecond_part_return msec =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 35) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2282:5: ( (week= time_week_part )? (day= time_day_part )? (hour= time_hour_part )? (minute= time_minute_part )? (second= time_second_part )? (msec= time_millisecond_part )? )
            // com/senseidb/bql/parsers/BQL.g:2282:9: (week= time_week_part )? (day= time_day_part )? (hour= time_hour_part )? (minute= time_minute_part )? (second= time_second_part )? (msec= time_millisecond_part )?
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2282:13: (week= time_week_part )?
            int alt49=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    switch ( input.LA(2) ) {
                        case WEEKS:
                            {
                            alt49=1;
                            }
                            break;
                    }

                    }
                    break;
            }

            switch (alt49) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2282:13: week= time_week_part
                    {
                    pushFollow(FOLLOW_time_week_part_in_time_span6467);
                    week=time_week_part();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, week.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:2282:33: (day= time_day_part )?
            int alt50=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    switch ( input.LA(2) ) {
                        case DAYS:
                            {
                            alt50=1;
                            }
                            break;
                    }

                    }
                    break;
            }

            switch (alt50) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2282:33: day= time_day_part
                    {
                    pushFollow(FOLLOW_time_day_part_in_time_span6472);
                    day=time_day_part();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, day.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:2282:53: (hour= time_hour_part )?
            int alt51=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    switch ( input.LA(2) ) {
                        case HOURS:
                            {
                            alt51=1;
                            }
                            break;
                    }

                    }
                    break;
            }

            switch (alt51) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2282:53: hour= time_hour_part
                    {
                    pushFollow(FOLLOW_time_hour_part_in_time_span6477);
                    hour=time_hour_part();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, hour.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:2283:15: (minute= time_minute_part )?
            int alt52=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    switch ( input.LA(2) ) {
                        case MINS:
                        case MINUTES:
                            {
                            alt52=1;
                            }
                            break;
                    }

                    }
                    break;
            }

            switch (alt52) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2283:15: minute= time_minute_part
                    {
                    pushFollow(FOLLOW_time_minute_part_in_time_span6491);
                    minute=time_minute_part();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, minute.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:2283:40: (second= time_second_part )?
            int alt53=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    switch ( input.LA(2) ) {
                        case SECONDS:
                        case SECS:
                            {
                            alt53=1;
                            }
                            break;
                    }

                    }
                    break;
            }

            switch (alt53) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2283:40: second= time_second_part
                    {
                    pushFollow(FOLLOW_time_second_part_in_time_span6496);
                    second=time_second_part();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, second.getTree());

                    }
                    break;

            }


            // com/senseidb/bql/parsers/BQL.g:2283:63: (msec= time_millisecond_part )?
            int alt54=2;
            switch ( input.LA(1) ) {
                case INTEGER:
                    {
                    alt54=1;
                    }
                    break;
            }

            switch (alt54) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2283:63: msec= time_millisecond_part
                    {
                    pushFollow(FOLLOW_time_millisecond_part_in_time_span6501);
                    msec=time_millisecond_part();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, msec.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        retval.val = 0;
                        if (week != null) retval.val += (week!=null?week.val:0);
                        if (day != null) retval.val += (day!=null?day.val:0);
                        if (hour != null) retval.val += (hour!=null?hour.val:0);
                        if (minute != null) retval.val += (minute!=null?minute.val:0);
                        if (second != null) retval.val += (second!=null?second.val:0);
                        if (msec != null) retval.val += (msec!=null?msec.val:0);
                        retval.val = _now - retval.val;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 35, time_span_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_span"


    public static class time_week_part_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_week_part"
    // com/senseidb/bql/parsers/BQL.g:2296:1: time_week_part returns [long val] : INTEGER WEEKS ;
    public final BQLParser.time_week_part_return time_week_part() throws RecognitionException {
        BQLParser.time_week_part_return retval = new BQLParser.time_week_part_return();
        retval.start = input.LT(1);

        int time_week_part_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER112=null;
        Token WEEKS113=null;

        Object INTEGER112_tree=null;
        Object WEEKS113_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 36) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2297:5: ( INTEGER WEEKS )
            // com/senseidb/bql/parsers/BQL.g:2297:9: INTEGER WEEKS
            {
            root_0 = (Object)adaptor.nil();


            INTEGER112=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_time_week_part6535); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            INTEGER112_tree = 
            (Object)adaptor.create(INTEGER112)
            ;
            adaptor.addChild(root_0, INTEGER112_tree);
            }

            WEEKS113=(Token)match(input,WEEKS,FOLLOW_WEEKS_in_time_week_part6537); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            WEEKS113_tree = 
            (Object)adaptor.create(WEEKS113)
            ;
            adaptor.addChild(root_0, WEEKS113_tree);
            }

            if ( state.backtracking==0 ) {
                        retval.val = Integer.parseInt((INTEGER112!=null?INTEGER112.getText():null)) * 7 * 24 * 60 * 60 * 1000L;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 36, time_week_part_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_week_part"


    public static class time_day_part_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_day_part"
    // com/senseidb/bql/parsers/BQL.g:2303:1: time_day_part returns [long val] : INTEGER DAYS ;
    public final BQLParser.time_day_part_return time_day_part() throws RecognitionException {
        BQLParser.time_day_part_return retval = new BQLParser.time_day_part_return();
        retval.start = input.LT(1);

        int time_day_part_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER114=null;
        Token DAYS115=null;

        Object INTEGER114_tree=null;
        Object DAYS115_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 37) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2304:5: ( INTEGER DAYS )
            // com/senseidb/bql/parsers/BQL.g:2304:9: INTEGER DAYS
            {
            root_0 = (Object)adaptor.nil();


            INTEGER114=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_time_day_part6570); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            INTEGER114_tree = 
            (Object)adaptor.create(INTEGER114)
            ;
            adaptor.addChild(root_0, INTEGER114_tree);
            }

            DAYS115=(Token)match(input,DAYS,FOLLOW_DAYS_in_time_day_part6572); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            DAYS115_tree = 
            (Object)adaptor.create(DAYS115)
            ;
            adaptor.addChild(root_0, DAYS115_tree);
            }

            if ( state.backtracking==0 ) {
                      retval.val = Integer.parseInt((INTEGER114!=null?INTEGER114.getText():null)) * 24 * 60 * 60 * 1000L;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 37, time_day_part_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_day_part"


    public static class time_hour_part_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_hour_part"
    // com/senseidb/bql/parsers/BQL.g:2310:1: time_hour_part returns [long val] : INTEGER HOURS ;
    public final BQLParser.time_hour_part_return time_hour_part() throws RecognitionException {
        BQLParser.time_hour_part_return retval = new BQLParser.time_hour_part_return();
        retval.start = input.LT(1);

        int time_hour_part_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER116=null;
        Token HOURS117=null;

        Object INTEGER116_tree=null;
        Object HOURS117_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 38) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2311:5: ( INTEGER HOURS )
            // com/senseidb/bql/parsers/BQL.g:2311:9: INTEGER HOURS
            {
            root_0 = (Object)adaptor.nil();


            INTEGER116=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_time_hour_part6605); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            INTEGER116_tree = 
            (Object)adaptor.create(INTEGER116)
            ;
            adaptor.addChild(root_0, INTEGER116_tree);
            }

            HOURS117=(Token)match(input,HOURS,FOLLOW_HOURS_in_time_hour_part6607); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            HOURS117_tree = 
            (Object)adaptor.create(HOURS117)
            ;
            adaptor.addChild(root_0, HOURS117_tree);
            }

            if ( state.backtracking==0 ) {
                      retval.val = Integer.parseInt((INTEGER116!=null?INTEGER116.getText():null)) * 60 * 60 * 1000L;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 38, time_hour_part_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_hour_part"


    public static class time_minute_part_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_minute_part"
    // com/senseidb/bql/parsers/BQL.g:2317:1: time_minute_part returns [long val] : INTEGER ( MINUTES | MINS ) ;
    public final BQLParser.time_minute_part_return time_minute_part() throws RecognitionException {
        BQLParser.time_minute_part_return retval = new BQLParser.time_minute_part_return();
        retval.start = input.LT(1);

        int time_minute_part_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER118=null;
        Token set119=null;

        Object INTEGER118_tree=null;
        Object set119_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 39) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2318:5: ( INTEGER ( MINUTES | MINS ) )
            // com/senseidb/bql/parsers/BQL.g:2318:9: INTEGER ( MINUTES | MINS )
            {
            root_0 = (Object)adaptor.nil();


            INTEGER118=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_time_minute_part6640); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            INTEGER118_tree = 
            (Object)adaptor.create(INTEGER118)
            ;
            adaptor.addChild(root_0, INTEGER118_tree);
            }

            set119=(Token)input.LT(1);

            if ( (input.LA(1) >= MINS && input.LA(1) <= MINUTES) ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                (Object)adaptor.create(set119)
                );
                state.errorRecovery=false;
                state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            if ( state.backtracking==0 ) {
                      retval.val = Integer.parseInt((INTEGER118!=null?INTEGER118.getText():null)) * 60 * 1000L;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 39, time_minute_part_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_minute_part"


    public static class time_second_part_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_second_part"
    // com/senseidb/bql/parsers/BQL.g:2324:1: time_second_part returns [long val] : INTEGER ( SECONDS | SECS ) ;
    public final BQLParser.time_second_part_return time_second_part() throws RecognitionException {
        BQLParser.time_second_part_return retval = new BQLParser.time_second_part_return();
        retval.start = input.LT(1);

        int time_second_part_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER120=null;
        Token set121=null;

        Object INTEGER120_tree=null;
        Object set121_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 40) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2325:5: ( INTEGER ( SECONDS | SECS ) )
            // com/senseidb/bql/parsers/BQL.g:2325:9: INTEGER ( SECONDS | SECS )
            {
            root_0 = (Object)adaptor.nil();


            INTEGER120=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_time_second_part6681); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            INTEGER120_tree = 
            (Object)adaptor.create(INTEGER120)
            ;
            adaptor.addChild(root_0, INTEGER120_tree);
            }

            set121=(Token)input.LT(1);

            if ( (input.LA(1) >= SECONDS && input.LA(1) <= SECS) ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                (Object)adaptor.create(set121)
                );
                state.errorRecovery=false;
                state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            if ( state.backtracking==0 ) {
                      retval.val = Integer.parseInt((INTEGER120!=null?INTEGER120.getText():null)) * 1000L;
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 40, time_second_part_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_second_part"


    public static class time_millisecond_part_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_millisecond_part"
    // com/senseidb/bql/parsers/BQL.g:2331:1: time_millisecond_part returns [long val] : INTEGER ( MILLISECONDS | MSECS ) ;
    public final BQLParser.time_millisecond_part_return time_millisecond_part() throws RecognitionException {
        BQLParser.time_millisecond_part_return retval = new BQLParser.time_millisecond_part_return();
        retval.start = input.LT(1);

        int time_millisecond_part_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER122=null;
        Token set123=null;

        Object INTEGER122_tree=null;
        Object set123_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 41) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2332:5: ( INTEGER ( MILLISECONDS | MSECS ) )
            // com/senseidb/bql/parsers/BQL.g:2332:9: INTEGER ( MILLISECONDS | MSECS )
            {
            root_0 = (Object)adaptor.nil();


            INTEGER122=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_time_millisecond_part6723); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            INTEGER122_tree = 
            (Object)adaptor.create(INTEGER122)
            ;
            adaptor.addChild(root_0, INTEGER122_tree);
            }

            set123=(Token)input.LT(1);

            if ( input.LA(1)==MILLISECONDS||input.LA(1)==MSECS ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                (Object)adaptor.create(set123)
                );
                state.errorRecovery=false;
                state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            if ( state.backtracking==0 ) {
                      retval.val = Integer.parseInt((INTEGER122!=null?INTEGER122.getText():null));
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 41, time_millisecond_part_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_millisecond_part"


    public static class time_expr_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "time_expr"
    // com/senseidb/bql/parsers/BQL.g:2338:1: time_expr returns [long val] : ( time_span AGO | date_time_string | NOW );
    public final BQLParser.time_expr_return time_expr() throws RecognitionException {
        BQLParser.time_expr_return retval = new BQLParser.time_expr_return();
        retval.start = input.LT(1);

        int time_expr_StartIndex = input.index();

        Object root_0 = null;

        Token AGO125=null;
        Token NOW127=null;
        BQLParser.time_span_return time_span124 =null;

        BQLParser.date_time_string_return date_time_string126 =null;


        Object AGO125_tree=null;
        Object NOW127_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 42) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2339:5: ( time_span AGO | date_time_string | NOW )
            int alt55=3;
            switch ( input.LA(1) ) {
            case EOF:
            case AGO:
            case AND:
            case BROWSE:
            case DISTINCT:
            case EXECUTE:
            case FETCHING:
            case GIVEN:
            case GROUP:
            case INTEGER:
            case LIMIT:
            case OR:
            case ORDER:
            case ROUTE:
            case RPAR:
            case SEMI:
            case USING:
                {
                alt55=1;
                }
                break;
            case DATE:
                {
                alt55=2;
                }
                break;
            case NOW:
                {
                alt55=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 55, 0, input);

                throw nvae;

            }

            switch (alt55) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2339:9: time_span AGO
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_time_span_in_time_expr6764);
                    time_span124=time_span();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, time_span124.getTree());

                    AGO125=(Token)match(input,AGO,FOLLOW_AGO_in_time_expr6766); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    AGO125_tree = 
                    (Object)adaptor.create(AGO125)
                    ;
                    adaptor.addChild(root_0, AGO125_tree);
                    }

                    if ( state.backtracking==0 ) {
                                retval.val = (time_span124!=null?time_span124.val:0);
                            }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2343:9: date_time_string
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_date_time_string_in_time_expr6786);
                    date_time_string126=date_time_string();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, date_time_string126.getTree());

                    if ( state.backtracking==0 ) {
                                retval.val = (date_time_string126!=null?date_time_string126.val:0);
                            }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2347:9: NOW
                    {
                    root_0 = (Object)adaptor.nil();


                    NOW127=(Token)match(input,NOW,FOLLOW_NOW_in_time_expr6806); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    NOW127_tree = 
                    (Object)adaptor.create(NOW127)
                    ;
                    adaptor.addChild(root_0, NOW127_tree);
                    }

                    if ( state.backtracking==0 ) {
                                retval.val = _now;
                            }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 42, time_expr_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "time_expr"


    public static class date_time_string_return extends ParserRuleReturnScope {
        public long val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "date_time_string"
    // com/senseidb/bql/parsers/BQL.g:2353:1: date_time_string returns [long val] : DATE ( TIME )? ;
    public final BQLParser.date_time_string_return date_time_string() throws RecognitionException {
        BQLParser.date_time_string_return retval = new BQLParser.date_time_string_return();
        retval.start = input.LT(1);

        int date_time_string_StartIndex = input.index();

        Object root_0 = null;

        Token DATE128=null;
        Token TIME129=null;

        Object DATE128_tree=null;
        Object TIME129_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 43) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2354:5: ( DATE ( TIME )? )
            // com/senseidb/bql/parsers/BQL.g:2354:9: DATE ( TIME )?
            {
            root_0 = (Object)adaptor.nil();


            DATE128=(Token)match(input,DATE,FOLLOW_DATE_in_date_time_string6839); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            DATE128_tree = 
            (Object)adaptor.create(DATE128)
            ;
            adaptor.addChild(root_0, DATE128_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:2354:14: ( TIME )?
            int alt56=2;
            switch ( input.LA(1) ) {
                case TIME:
                    {
                    alt56=1;
                    }
                    break;
            }

            switch (alt56) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2354:14: TIME
                    {
                    TIME129=(Token)match(input,TIME,FOLLOW_TIME_in_date_time_string6841); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    TIME129_tree = 
                    (Object)adaptor.create(TIME129)
                    ;
                    adaptor.addChild(root_0, TIME129_tree);
                    }

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        SimpleDateFormat format;
                        String dateTimeStr = (DATE128!=null?DATE128.getText():null);
                        char separator = dateTimeStr.charAt(4);
                        if (TIME129 != null) {
                            dateTimeStr = dateTimeStr + " " + (TIME129!=null?TIME129.getText():null);
                        }
                        int formatIdx = (separator == '-' ? 0 : 1);

                        if (TIME129 == null) {
                            if (_format1[formatIdx] != null) {
                                format = _format1[formatIdx];
                            }
                            else {
                                format = _format1[formatIdx] = new SimpleDateFormat("yyyy" + separator + "MM" + separator + "dd");
                            }
                        }
                        else {
                            if (_format2[formatIdx] != null) {
                                format = _format2[formatIdx];
                            }
                            else {
                                format = _format2[formatIdx] = new SimpleDateFormat("yyyy" + separator + "MM" + separator + "dd HH:mm:ss");
                            }
                        }
                        try {
                            retval.val = format.parse(dateTimeStr).getTime();
                            if (!dateTimeStr.equals(format.format(retval.val))) {
                                throw new FailedPredicateException(input,
                                                                   "date_time_string", 
                                                                   "Date string contains invalid date/time: \"" + dateTimeStr + "\".");
                            }
                        }
                        catch (ParseException err) {
                            throw new FailedPredicateException(input,
                                                               "date_time_string", 
                                                               "ParseException happened for \"" + dateTimeStr + "\": " + 
                                                               err.getMessage() + ".");
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 43, date_time_string_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "date_time_string"


    public static class match_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "match_predicate"
    // com/senseidb/bql/parsers/BQL.g:2397:1: match_predicate returns [JSONObject json] : ( NOT )? MATCH LPAR selection_list RPAR AGAINST LPAR STRING_LITERAL RPAR ;
    public final BQLParser.match_predicate_return match_predicate() throws RecognitionException {
        BQLParser.match_predicate_return retval = new BQLParser.match_predicate_return();
        retval.start = input.LT(1);

        int match_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token NOT130=null;
        Token MATCH131=null;
        Token LPAR132=null;
        Token RPAR134=null;
        Token AGAINST135=null;
        Token LPAR136=null;
        Token STRING_LITERAL137=null;
        Token RPAR138=null;
        BQLParser.selection_list_return selection_list133 =null;


        Object NOT130_tree=null;
        Object MATCH131_tree=null;
        Object LPAR132_tree=null;
        Object RPAR134_tree=null;
        Object AGAINST135_tree=null;
        Object LPAR136_tree=null;
        Object STRING_LITERAL137_tree=null;
        Object RPAR138_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 44) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2398:5: ( ( NOT )? MATCH LPAR selection_list RPAR AGAINST LPAR STRING_LITERAL RPAR )
            // com/senseidb/bql/parsers/BQL.g:2398:9: ( NOT )? MATCH LPAR selection_list RPAR AGAINST LPAR STRING_LITERAL RPAR
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2398:9: ( NOT )?
            int alt57=2;
            switch ( input.LA(1) ) {
                case NOT:
                    {
                    alt57=1;
                    }
                    break;
            }

            switch (alt57) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2398:10: NOT
                    {
                    NOT130=(Token)match(input,NOT,FOLLOW_NOT_in_match_predicate6876); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    NOT130_tree = 
                    (Object)adaptor.create(NOT130)
                    ;
                    adaptor.addChild(root_0, NOT130_tree);
                    }

                    }
                    break;

            }


            MATCH131=(Token)match(input,MATCH,FOLLOW_MATCH_in_match_predicate6880); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            MATCH131_tree = 
            (Object)adaptor.create(MATCH131)
            ;
            adaptor.addChild(root_0, MATCH131_tree);
            }

            LPAR132=(Token)match(input,LPAR,FOLLOW_LPAR_in_match_predicate6882); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR132_tree = 
            (Object)adaptor.create(LPAR132)
            ;
            adaptor.addChild(root_0, LPAR132_tree);
            }

            pushFollow(FOLLOW_selection_list_in_match_predicate6884);
            selection_list133=selection_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, selection_list133.getTree());

            RPAR134=(Token)match(input,RPAR,FOLLOW_RPAR_in_match_predicate6886); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR134_tree = 
            (Object)adaptor.create(RPAR134)
            ;
            adaptor.addChild(root_0, RPAR134_tree);
            }

            AGAINST135=(Token)match(input,AGAINST,FOLLOW_AGAINST_in_match_predicate6888); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            AGAINST135_tree = 
            (Object)adaptor.create(AGAINST135)
            ;
            adaptor.addChild(root_0, AGAINST135_tree);
            }

            LPAR136=(Token)match(input,LPAR,FOLLOW_LPAR_in_match_predicate6890); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR136_tree = 
            (Object)adaptor.create(LPAR136)
            ;
            adaptor.addChild(root_0, LPAR136_tree);
            }

            STRING_LITERAL137=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_match_predicate6892); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            STRING_LITERAL137_tree = 
            (Object)adaptor.create(STRING_LITERAL137)
            ;
            adaptor.addChild(root_0, STRING_LITERAL137_tree);
            }

            RPAR138=(Token)match(input,RPAR,FOLLOW_RPAR_in_match_predicate6894); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR138_tree = 
            (Object)adaptor.create(RPAR138)
            ;
            adaptor.addChild(root_0, RPAR138_tree);
            }

            if ( state.backtracking==0 ) {
                        try {
                            JSONArray cols = (selection_list133!=null?selection_list133.json:null);
                            for (int i = 0; i < cols.length(); ++i) {
                                String col = cols.getString(i);
                                String[] facetInfo = _facetInfoMap.get(col);
                                if (facetInfo != null && !facetInfo[1].equals("string")) {
                                    throw new FailedPredicateException(input, 
                                                                       "match_predicate", 
                                                                       "Non-string type column \"" + col + "\" cannot be used in MATCH AGAINST predicates.");
                                }
                            }

                            String orig = (STRING_LITERAL137!=null?STRING_LITERAL137.getText():null);
                            orig = orig.substring(1, orig.length() - 1);
                            retval.json = new FastJSONObject().put("query",
                                                             new FastJSONObject().put("query_string",
                                                                                      new FastJSONObject().put("fields", cols)
                                                                                                          .put("query", orig)));
                            if (NOT130 != null) {
                                retval.json = new FastJSONObject().put("bool",
                                                                 new FastJSONObject().put("must_not", retval.json));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "match_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 44, match_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "match_predicate"


    public static class like_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "like_predicate"
    // com/senseidb/bql/parsers/BQL.g:2429:1: like_predicate returns [JSONObject json] : column_name ( NOT )? LIKE STRING_LITERAL ;
    public final BQLParser.like_predicate_return like_predicate() throws RecognitionException {
        BQLParser.like_predicate_return retval = new BQLParser.like_predicate_return();
        retval.start = input.LT(1);

        int like_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token NOT140=null;
        Token LIKE141=null;
        Token STRING_LITERAL142=null;
        BQLParser.column_name_return column_name139 =null;


        Object NOT140_tree=null;
        Object LIKE141_tree=null;
        Object STRING_LITERAL142_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 45) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2430:5: ( column_name ( NOT )? LIKE STRING_LITERAL )
            // com/senseidb/bql/parsers/BQL.g:2430:9: column_name ( NOT )? LIKE STRING_LITERAL
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_column_name_in_like_predicate6927);
            column_name139=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name139.getTree());

            // com/senseidb/bql/parsers/BQL.g:2430:21: ( NOT )?
            int alt58=2;
            switch ( input.LA(1) ) {
                case NOT:
                    {
                    alt58=1;
                    }
                    break;
            }

            switch (alt58) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2430:22: NOT
                    {
                    NOT140=(Token)match(input,NOT,FOLLOW_NOT_in_like_predicate6930); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    NOT140_tree = 
                    (Object)adaptor.create(NOT140)
                    ;
                    adaptor.addChild(root_0, NOT140_tree);
                    }

                    }
                    break;

            }


            LIKE141=(Token)match(input,LIKE,FOLLOW_LIKE_in_like_predicate6934); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LIKE141_tree = 
            (Object)adaptor.create(LIKE141)
            ;
            adaptor.addChild(root_0, LIKE141_tree);
            }

            STRING_LITERAL142=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_like_predicate6936); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            STRING_LITERAL142_tree = 
            (Object)adaptor.create(STRING_LITERAL142)
            ;
            adaptor.addChild(root_0, STRING_LITERAL142_tree);
            }

            if ( state.backtracking==0 ) {
                        String col = (column_name139!=null?column_name139.text:null);
                        String[] facetInfo = _facetInfoMap.get(col);
                        if (facetInfo != null && !facetInfo[1].equals("string")) {
                            throw new FailedPredicateException(input, 
                                                               "match_predicate", 
                                                               "Non-string type column \"" + col + "\" cannot be used in LIKE predicates.");
                        }
                        String orig = (STRING_LITERAL142!=null?STRING_LITERAL142.getText():null);
                        orig = orig.substring(1, orig.length() - 1);
                        String likeString = orig.replace('%', '*').replace('_', '?');
                        try {
                            retval.json = new FastJSONObject().put("query",
                                                             new FastJSONObject().put("wildcard",
                                                                                      new FastJSONObject().put(col, likeString)));
                            if (NOT140 != null) {
                                retval.json = new FastJSONObject().put("bool",
                                                                 new FastJSONObject().put("must_not", retval.json));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "like_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 45, like_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "like_predicate"


    public static class null_predicate_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "null_predicate"
    // com/senseidb/bql/parsers/BQL.g:2457:1: null_predicate returns [JSONObject json] : column_name IS ( NOT )? NULL ;
    public final BQLParser.null_predicate_return null_predicate() throws RecognitionException {
        BQLParser.null_predicate_return retval = new BQLParser.null_predicate_return();
        retval.start = input.LT(1);

        int null_predicate_StartIndex = input.index();

        Object root_0 = null;

        Token IS144=null;
        Token NOT145=null;
        Token NULL146=null;
        BQLParser.column_name_return column_name143 =null;


        Object IS144_tree=null;
        Object NOT145_tree=null;
        Object NULL146_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 46) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2458:5: ( column_name IS ( NOT )? NULL )
            // com/senseidb/bql/parsers/BQL.g:2458:9: column_name IS ( NOT )? NULL
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_column_name_in_null_predicate6969);
            column_name143=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name143.getTree());

            IS144=(Token)match(input,IS,FOLLOW_IS_in_null_predicate6971); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            IS144_tree = 
            (Object)adaptor.create(IS144)
            ;
            adaptor.addChild(root_0, IS144_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:2458:24: ( NOT )?
            int alt59=2;
            switch ( input.LA(1) ) {
                case NOT:
                    {
                    alt59=1;
                    }
                    break;
            }

            switch (alt59) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2458:25: NOT
                    {
                    NOT145=(Token)match(input,NOT,FOLLOW_NOT_in_null_predicate6974); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    NOT145_tree = 
                    (Object)adaptor.create(NOT145)
                    ;
                    adaptor.addChild(root_0, NOT145_tree);
                    }

                    }
                    break;

            }


            NULL146=(Token)match(input,NULL,FOLLOW_NULL_in_null_predicate6978); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            NULL146_tree = 
            (Object)adaptor.create(NULL146)
            ;
            adaptor.addChild(root_0, NULL146_tree);
            }

            if ( state.backtracking==0 ) {
                        String col = (column_name143!=null?column_name143.text:null);
                        try {
                            retval.json = new FastJSONObject().put("isNull", col);
                            if (NOT145 != null) {
                                retval.json = new FastJSONObject().put("bool",
                                                                 new FastJSONObject().put("must_not", retval.json));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "null_predicate", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 46, null_predicate_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "null_predicate"


    public static class non_variable_value_list_return extends ParserRuleReturnScope {
        public JSONArray json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "non_variable_value_list"
    // com/senseidb/bql/parsers/BQL.g:2474:1: non_variable_value_list returns [JSONArray json] : ( LPAR v= value ( COMMA v= value )* RPAR | LPAR RPAR );
    public final BQLParser.non_variable_value_list_return non_variable_value_list() throws RecognitionException {
        BQLParser.non_variable_value_list_return retval = new BQLParser.non_variable_value_list_return();
        retval.start = input.LT(1);

        int non_variable_value_list_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR147=null;
        Token COMMA148=null;
        Token RPAR149=null;
        Token LPAR150=null;
        Token RPAR151=null;
        BQLParser.value_return v =null;


        Object LPAR147_tree=null;
        Object COMMA148_tree=null;
        Object RPAR149_tree=null;
        Object LPAR150_tree=null;
        Object RPAR151_tree=null;


            retval.json = new FastJSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 47) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2478:5: ( LPAR v= value ( COMMA v= value )* RPAR | LPAR RPAR )
            int alt61=2;
            switch ( input.LA(1) ) {
            case LPAR:
                {
                switch ( input.LA(2) ) {
                case RPAR:
                    {
                    alt61=2;
                    }
                    break;
                case AGO:
                case DATE:
                case FALSE:
                case INTEGER:
                case NOW:
                case REAL:
                case STRING_LITERAL:
                case TRUE:
                case VARIABLE:
                    {
                    alt61=1;
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 61, 1, input);

                    throw nvae;

                }

                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 61, 0, input);

                throw nvae;

            }

            switch (alt61) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2478:9: LPAR v= value ( COMMA v= value )* RPAR
                    {
                    root_0 = (Object)adaptor.nil();


                    LPAR147=(Token)match(input,LPAR,FOLLOW_LPAR_in_non_variable_value_list7016); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    LPAR147_tree = 
                    (Object)adaptor.create(LPAR147)
                    ;
                    adaptor.addChild(root_0, LPAR147_tree);
                    }

                    pushFollow(FOLLOW_value_in_non_variable_value_list7020);
                    v=value();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, v.getTree());

                    if ( state.backtracking==0 ) {
                                retval.json.put((v!=null?v.val:null));
                            }

                    // com/senseidb/bql/parsers/BQL.g:2482:9: ( COMMA v= value )*
                    loop60:
                    do {
                        int alt60=2;
                        switch ( input.LA(1) ) {
                        case COMMA:
                            {
                            alt60=1;
                            }
                            break;

                        }

                        switch (alt60) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2482:10: COMMA v= value
                    	    {
                    	    COMMA148=(Token)match(input,COMMA,FOLLOW_COMMA_in_non_variable_value_list7041); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    COMMA148_tree = 
                    	    (Object)adaptor.create(COMMA148)
                    	    ;
                    	    adaptor.addChild(root_0, COMMA148_tree);
                    	    }

                    	    pushFollow(FOLLOW_value_in_non_variable_value_list7045);
                    	    v=value();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) adaptor.addChild(root_0, v.getTree());

                    	    if ( state.backtracking==0 ) {
                    	                    retval.json.put((v!=null?v.val:null));
                    	                }

                    	    }
                    	    break;

                    	default :
                    	    break loop60;
                        }
                    } while (true);


                    RPAR149=(Token)match(input,RPAR,FOLLOW_RPAR_in_non_variable_value_list7072); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    RPAR149_tree = 
                    (Object)adaptor.create(RPAR149)
                    ;
                    adaptor.addChild(root_0, RPAR149_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2487:9: LPAR RPAR
                    {
                    root_0 = (Object)adaptor.nil();


                    LPAR150=(Token)match(input,LPAR,FOLLOW_LPAR_in_non_variable_value_list7082); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    LPAR150_tree = 
                    (Object)adaptor.create(LPAR150)
                    ;
                    adaptor.addChild(root_0, LPAR150_tree);
                    }

                    RPAR151=(Token)match(input,RPAR,FOLLOW_RPAR_in_non_variable_value_list7084); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    RPAR151_tree = 
                    (Object)adaptor.create(RPAR151)
                    ;
                    adaptor.addChild(root_0, RPAR151_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 47, non_variable_value_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "non_variable_value_list"


    public static class python_style_list_return extends ParserRuleReturnScope {
        public JSONArray json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "python_style_list"
    // com/senseidb/bql/parsers/BQL.g:2490:1: python_style_list returns [JSONArray json] : '[' (v= python_style_value )? ( COMMA v= python_style_value )* ']' ;
    public final BQLParser.python_style_list_return python_style_list() throws RecognitionException {
        BQLParser.python_style_list_return retval = new BQLParser.python_style_list_return();
        retval.start = input.LT(1);

        int python_style_list_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal152=null;
        Token COMMA153=null;
        Token char_literal154=null;
        BQLParser.python_style_value_return v =null;


        Object char_literal152_tree=null;
        Object COMMA153_tree=null;
        Object char_literal154_tree=null;


            retval.json = new FastJSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 48) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2494:5: ( '[' (v= python_style_value )? ( COMMA v= python_style_value )* ']' )
            // com/senseidb/bql/parsers/BQL.g:2494:9: '[' (v= python_style_value )? ( COMMA v= python_style_value )* ']'
            {
            root_0 = (Object)adaptor.nil();


            char_literal152=(Token)match(input,141,FOLLOW_141_in_python_style_list7112); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal152_tree = 
            (Object)adaptor.create(char_literal152)
            ;
            adaptor.addChild(root_0, char_literal152_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:2494:14: (v= python_style_value )?
            int alt62=2;
            switch ( input.LA(1) ) {
                case EOF:
                case AGO:
                case AND:
                case BROWSE:
                case DATE:
                case DISTINCT:
                case EXECUTE:
                case FALSE:
                case FETCHING:
                case GIVEN:
                case GROUP:
                case INTEGER:
                case LIMIT:
                case NOW:
                case OR:
                case ORDER:
                case REAL:
                case ROUTE:
                case RPAR:
                case SEMI:
                case STRING_LITERAL:
                case TRUE:
                case USING:
                case VARIABLE:
                case 141:
                case 165:
                    {
                    alt62=1;
                    }
                    break;
            }

            switch (alt62) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2494:14: v= python_style_value
                    {
                    pushFollow(FOLLOW_python_style_value_in_python_style_list7116);
                    v=python_style_value();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, v.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        retval.json.put((v!=null?v.val:null));
                    }

            // com/senseidb/bql/parsers/BQL.g:2498:9: ( COMMA v= python_style_value )*
            loop63:
            do {
                int alt63=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt63=1;
                    }
                    break;

                }

                switch (alt63) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2498:10: COMMA v= python_style_value
            	    {
            	    COMMA153=(Token)match(input,COMMA,FOLLOW_COMMA_in_python_style_list7138); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA153_tree = 
            	    (Object)adaptor.create(COMMA153)
            	    ;
            	    adaptor.addChild(root_0, COMMA153_tree);
            	    }

            	    pushFollow(FOLLOW_python_style_value_in_python_style_list7142);
            	    v=python_style_value();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, v.getTree());

            	    if ( state.backtracking==0 ) {
            	                    retval.json.put((v!=null?v.val:null));
            	                }

            	    }
            	    break;

            	default :
            	    break loop63;
                }
            } while (true);


            char_literal154=(Token)match(input,142,FOLLOW_142_in_python_style_list7169); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal154_tree = 
            (Object)adaptor.create(char_literal154)
            ;
            adaptor.addChild(root_0, char_literal154_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 48, python_style_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "python_style_list"


    public static class python_style_dict_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "python_style_dict"
    // com/senseidb/bql/parsers/BQL.g:2505:1: python_style_dict returns [JSONObject json] : ( '{' '}' | '{' p= key_value_pair[KeyType.STRING_LITERAL] ( COMMA p= key_value_pair[KeyType.STRING_LITERAL] )* '}' );
    public final BQLParser.python_style_dict_return python_style_dict() throws RecognitionException {
        BQLParser.python_style_dict_return retval = new BQLParser.python_style_dict_return();
        retval.start = input.LT(1);

        int python_style_dict_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal155=null;
        Token char_literal156=null;
        Token char_literal157=null;
        Token COMMA158=null;
        Token char_literal159=null;
        BQLParser.key_value_pair_return p =null;


        Object char_literal155_tree=null;
        Object char_literal156_tree=null;
        Object char_literal157_tree=null;
        Object COMMA158_tree=null;
        Object char_literal159_tree=null;


            retval.json = new FastJSONObject();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 49) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2509:5: ( '{' '}' | '{' p= key_value_pair[KeyType.STRING_LITERAL] ( COMMA p= key_value_pair[KeyType.STRING_LITERAL] )* '}' )
            int alt65=2;
            switch ( input.LA(1) ) {
            case 165:
                {
                switch ( input.LA(2) ) {
                case 169:
                    {
                    alt65=1;
                    }
                    break;
                case IDENT:
                case STRING_LITERAL:
                    {
                    alt65=2;
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 65, 1, input);

                    throw nvae;

                }

                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 65, 0, input);

                throw nvae;

            }

            switch (alt65) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2509:9: '{' '}'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal155=(Token)match(input,165,FOLLOW_165_in_python_style_dict7197); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal155_tree = 
                    (Object)adaptor.create(char_literal155)
                    ;
                    adaptor.addChild(root_0, char_literal155_tree);
                    }

                    char_literal156=(Token)match(input,169,FOLLOW_169_in_python_style_dict7198); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal156_tree = 
                    (Object)adaptor.create(char_literal156)
                    ;
                    adaptor.addChild(root_0, char_literal156_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2510:9: '{' p= key_value_pair[KeyType.STRING_LITERAL] ( COMMA p= key_value_pair[KeyType.STRING_LITERAL] )* '}'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal157=(Token)match(input,165,FOLLOW_165_in_python_style_dict7217); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal157_tree = 
                    (Object)adaptor.create(char_literal157)
                    ;
                    adaptor.addChild(root_0, char_literal157_tree);
                    }

                    pushFollow(FOLLOW_key_value_pair_in_python_style_dict7221);
                    p=key_value_pair(KeyType.STRING_LITERAL);

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

                    if ( state.backtracking==0 ) {
                                try {
                                    retval.json.put((p!=null?p.key:null), (p!=null?p.val:null));
                                }
                                catch (JSONException err) {
                                    throw new FailedPredicateException(input, "python_style_dict", "JSONException: " + err.getMessage());
                                }
                            }

                    // com/senseidb/bql/parsers/BQL.g:2519:9: ( COMMA p= key_value_pair[KeyType.STRING_LITERAL] )*
                    loop64:
                    do {
                        int alt64=2;
                        switch ( input.LA(1) ) {
                        case COMMA:
                            {
                            alt64=1;
                            }
                            break;

                        }

                        switch (alt64) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2519:10: COMMA p= key_value_pair[KeyType.STRING_LITERAL]
                    	    {
                    	    COMMA158=(Token)match(input,COMMA,FOLLOW_COMMA_in_python_style_dict7243); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    COMMA158_tree = 
                    	    (Object)adaptor.create(COMMA158)
                    	    ;
                    	    adaptor.addChild(root_0, COMMA158_tree);
                    	    }

                    	    pushFollow(FOLLOW_key_value_pair_in_python_style_dict7247);
                    	    p=key_value_pair(KeyType.STRING_LITERAL);

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

                    	    if ( state.backtracking==0 ) {
                    	                    try {
                    	                        retval.json.put((p!=null?p.key:null), (p!=null?p.val:null));
                    	                    }
                    	                    catch (JSONException err) {
                    	                        throw new FailedPredicateException(input, "python_style_dict", "JSONException: " + err.getMessage());
                    	                    }
                    	                }

                    	    }
                    	    break;

                    	default :
                    	    break loop64;
                        }
                    } while (true);


                    char_literal159=(Token)match(input,169,FOLLOW_169_in_python_style_dict7275); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal159_tree = 
                    (Object)adaptor.create(char_literal159)
                    ;
                    adaptor.addChild(root_0, char_literal159_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 49, python_style_dict_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "python_style_dict"


    public static class python_style_value_return extends ParserRuleReturnScope {
        public Object val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "python_style_value"
    // com/senseidb/bql/parsers/BQL.g:2531:1: python_style_value returns [Object val] : ( value | python_style_list | python_style_dict );
    public final BQLParser.python_style_value_return python_style_value() throws RecognitionException {
        BQLParser.python_style_value_return retval = new BQLParser.python_style_value_return();
        retval.start = input.LT(1);

        int python_style_value_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.value_return value160 =null;

        BQLParser.python_style_list_return python_style_list161 =null;

        BQLParser.python_style_dict_return python_style_dict162 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 50) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2532:5: ( value | python_style_list | python_style_dict )
            int alt66=3;
            switch ( input.LA(1) ) {
            case EOF:
            case AGO:
            case AND:
            case BROWSE:
            case DATE:
            case DISTINCT:
            case EXECUTE:
            case FALSE:
            case FETCHING:
            case GIVEN:
            case GROUP:
            case INTEGER:
            case LIMIT:
            case NOW:
            case OR:
            case ORDER:
            case REAL:
            case ROUTE:
            case RPAR:
            case SEMI:
            case STRING_LITERAL:
            case TRUE:
            case USING:
            case VARIABLE:
                {
                alt66=1;
                }
                break;
            case 141:
                {
                alt66=2;
                }
                break;
            case 165:
                {
                alt66=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 66, 0, input);

                throw nvae;

            }

            switch (alt66) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2532:9: value
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_value_in_python_style_value7298);
                    value160=value();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, value160.getTree());

                    if ( state.backtracking==0 ) { retval.val = (value160!=null?value160.val:null); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2533:9: python_style_list
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_python_style_list_in_python_style_value7310);
                    python_style_list161=python_style_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, python_style_list161.getTree());

                    if ( state.backtracking==0 ) { retval.val = (python_style_list161!=null?python_style_list161.json:null); }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2534:9: python_style_dict
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_python_style_dict_in_python_style_value7322);
                    python_style_dict162=python_style_dict();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, python_style_dict162.getTree());

                    if ( state.backtracking==0 ) { retval.val = (python_style_dict162!=null?python_style_dict162.json:null); }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 50, python_style_value_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "python_style_value"


    public static class value_list_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "value_list"
    // com/senseidb/bql/parsers/BQL.g:2537:1: value_list returns [Object json] : ( non_variable_value_list | VARIABLE );
    public final BQLParser.value_list_return value_list() throws RecognitionException {
        BQLParser.value_list_return retval = new BQLParser.value_list_return();
        retval.start = input.LT(1);

        int value_list_StartIndex = input.index();

        Object root_0 = null;

        Token VARIABLE164=null;
        BQLParser.non_variable_value_list_return non_variable_value_list163 =null;


        Object VARIABLE164_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 51) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2538:5: ( non_variable_value_list | VARIABLE )
            int alt67=2;
            switch ( input.LA(1) ) {
            case LPAR:
                {
                alt67=1;
                }
                break;
            case VARIABLE:
                {
                alt67=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 67, 0, input);

                throw nvae;

            }

            switch (alt67) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2538:9: non_variable_value_list
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_non_variable_value_list_in_value_list7347);
                    non_variable_value_list163=non_variable_value_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, non_variable_value_list163.getTree());

                    if ( state.backtracking==0 ) {
                                retval.json = (non_variable_value_list163!=null?non_variable_value_list163.json:null);
                            }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2542:9: VARIABLE
                    {
                    root_0 = (Object)adaptor.nil();


                    VARIABLE164=(Token)match(input,VARIABLE,FOLLOW_VARIABLE_in_value_list7367); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    VARIABLE164_tree = 
                    (Object)adaptor.create(VARIABLE164)
                    ;
                    adaptor.addChild(root_0, VARIABLE164_tree);
                    }

                    if ( state.backtracking==0 ) {
                                retval.json = (VARIABLE164!=null?VARIABLE164.getText():null);
                                _variables.add(((VARIABLE164!=null?VARIABLE164.getText():null)).substring(1));
                            }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 51, value_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "value_list"


    public static class value_return extends ParserRuleReturnScope {
        public Object val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "value"
    // com/senseidb/bql/parsers/BQL.g:2549:1: value returns [Object val] : ( numeric | STRING_LITERAL | TRUE | FALSE | VARIABLE );
    public final BQLParser.value_return value() throws RecognitionException {
        BQLParser.value_return retval = new BQLParser.value_return();
        retval.start = input.LT(1);

        int value_StartIndex = input.index();

        Object root_0 = null;

        Token STRING_LITERAL166=null;
        Token TRUE167=null;
        Token FALSE168=null;
        Token VARIABLE169=null;
        BQLParser.numeric_return numeric165 =null;


        Object STRING_LITERAL166_tree=null;
        Object TRUE167_tree=null;
        Object FALSE168_tree=null;
        Object VARIABLE169_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 52) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2550:5: ( numeric | STRING_LITERAL | TRUE | FALSE | VARIABLE )
            int alt68=5;
            switch ( input.LA(1) ) {
            case EOF:
            case AGO:
            case AND:
            case BROWSE:
            case DATE:
            case DISTINCT:
            case EXECUTE:
            case FETCHING:
            case GIVEN:
            case GROUP:
            case INTEGER:
            case LIMIT:
            case NOW:
            case OR:
            case ORDER:
            case REAL:
            case ROUTE:
            case RPAR:
            case SEMI:
            case USING:
                {
                alt68=1;
                }
                break;
            case STRING_LITERAL:
                {
                alt68=2;
                }
                break;
            case TRUE:
                {
                alt68=3;
                }
                break;
            case FALSE:
                {
                alt68=4;
                }
                break;
            case VARIABLE:
                {
                alt68=5;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 68, 0, input);

                throw nvae;

            }

            switch (alt68) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2550:9: numeric
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_numeric_in_value7400);
                    numeric165=numeric();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, numeric165.getTree());

                    if ( state.backtracking==0 ) { retval.val = (numeric165!=null?numeric165.val:null); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2551:9: STRING_LITERAL
                    {
                    root_0 = (Object)adaptor.nil();


                    STRING_LITERAL166=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_value7412); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    STRING_LITERAL166_tree = 
                    (Object)adaptor.create(STRING_LITERAL166)
                    ;
                    adaptor.addChild(root_0, STRING_LITERAL166_tree);
                    }

                    if ( state.backtracking==0 ) {
                                String orig = (STRING_LITERAL166!=null?STRING_LITERAL166.getText():null);
                                orig = orig.substring(1, orig.length() - 1);
                                retval.val = orig;
                            }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2557:9: TRUE
                    {
                    root_0 = (Object)adaptor.nil();


                    TRUE167=(Token)match(input,TRUE,FOLLOW_TRUE_in_value7432); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    TRUE167_tree = 
                    (Object)adaptor.create(TRUE167)
                    ;
                    adaptor.addChild(root_0, TRUE167_tree);
                    }

                    if ( state.backtracking==0 ) { retval.val = true; }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2558:9: FALSE
                    {
                    root_0 = (Object)adaptor.nil();


                    FALSE168=(Token)match(input,FALSE,FOLLOW_FALSE_in_value7444); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    FALSE168_tree = 
                    (Object)adaptor.create(FALSE168)
                    ;
                    adaptor.addChild(root_0, FALSE168_tree);
                    }

                    if ( state.backtracking==0 ) { retval.val = false; }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:2559:9: VARIABLE
                    {
                    root_0 = (Object)adaptor.nil();


                    VARIABLE169=(Token)match(input,VARIABLE,FOLLOW_VARIABLE_in_value7456); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    VARIABLE169_tree = 
                    (Object)adaptor.create(VARIABLE169)
                    ;
                    adaptor.addChild(root_0, VARIABLE169_tree);
                    }

                    if ( state.backtracking==0 ) {
                                retval.val = (VARIABLE169!=null?VARIABLE169.getText():null);
                                _variables.add(((VARIABLE169!=null?VARIABLE169.getText():null)).substring(1));
                            }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 52, value_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "value"


    public static class numeric_return extends ParserRuleReturnScope {
        public Object val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "numeric"
    // com/senseidb/bql/parsers/BQL.g:2566:1: numeric returns [Object val] : ( time_expr | INTEGER | REAL );
    public final BQLParser.numeric_return numeric() throws RecognitionException {
        BQLParser.numeric_return retval = new BQLParser.numeric_return();
        retval.start = input.LT(1);

        int numeric_StartIndex = input.index();

        Object root_0 = null;

        Token INTEGER171=null;
        Token REAL172=null;
        BQLParser.time_expr_return time_expr170 =null;


        Object INTEGER171_tree=null;
        Object REAL172_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 53) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2567:5: ( time_expr | INTEGER | REAL )
            int alt69=3;
            switch ( input.LA(1) ) {
            case INTEGER:
                {
                switch ( input.LA(2) ) {
                case DAYS:
                case HOURS:
                case MILLISECONDS:
                case MINS:
                case MINUTES:
                case MSECS:
                case SECONDS:
                case SECS:
                case WEEKS:
                    {
                    alt69=1;
                    }
                    break;
                case EOF:
                case AND:
                case BROWSE:
                case COMMA:
                case DISTINCT:
                case EXECUTE:
                case FETCHING:
                case GIVEN:
                case GROUP:
                case LIMIT:
                case OR:
                case ORDER:
                case ROUTE:
                case RPAR:
                case SEMI:
                case USING:
                case WITH:
                case 142:
                case 169:
                    {
                    alt69=2;
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 69, 1, input);

                    throw nvae;

                }

                }
                break;
            case AGO:
            case DATE:
            case NOW:
                {
                alt69=1;
                }
                break;
            case REAL:
                {
                alt69=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 69, 0, input);

                throw nvae;

            }

            switch (alt69) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2567:9: time_expr
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_time_expr_in_numeric7489);
                    time_expr170=time_expr();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, time_expr170.getTree());

                    if ( state.backtracking==0 ) { retval.val = (time_expr170!=null?time_expr170.val:0); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2568:9: INTEGER
                    {
                    root_0 = (Object)adaptor.nil();


                    INTEGER171=(Token)match(input,INTEGER,FOLLOW_INTEGER_in_numeric7501); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    INTEGER171_tree = 
                    (Object)adaptor.create(INTEGER171)
                    ;
                    adaptor.addChild(root_0, INTEGER171_tree);
                    }

                    if ( state.backtracking==0 ) {
                                try {
                                    retval.val = Long.parseLong((INTEGER171!=null?INTEGER171.getText():null));
                                }
                                catch (NumberFormatException err) {
                                    throw new FailedPredicateException(input, "numeric", "Hit NumberFormatException: " + err.getMessage());
                                }
                            }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2576:9: REAL
                    {
                    root_0 = (Object)adaptor.nil();


                    REAL172=(Token)match(input,REAL,FOLLOW_REAL_in_numeric7513); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    REAL172_tree = 
                    (Object)adaptor.create(REAL172)
                    ;
                    adaptor.addChild(root_0, REAL172_tree);
                    }

                    if ( state.backtracking==0 ) {
                                try {
                                    retval.val = Float.parseFloat((REAL172!=null?REAL172.getText():null));
                                }
                                catch (NumberFormatException err) {
                                    throw new FailedPredicateException(input, "numeric", "Hit NumberFormatException: " + err.getMessage());
                                }
                            }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 53, numeric_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "numeric"


    public static class except_clause_return extends ParserRuleReturnScope {
        public Object json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "except_clause"
    // com/senseidb/bql/parsers/BQL.g:2586:1: except_clause returns [Object json] : EXCEPT ^ value_list ;
    public final BQLParser.except_clause_return except_clause() throws RecognitionException {
        BQLParser.except_clause_return retval = new BQLParser.except_clause_return();
        retval.start = input.LT(1);

        int except_clause_StartIndex = input.index();

        Object root_0 = null;

        Token EXCEPT173=null;
        BQLParser.value_list_return value_list174 =null;


        Object EXCEPT173_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 54) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2587:5: ( EXCEPT ^ value_list )
            // com/senseidb/bql/parsers/BQL.g:2587:9: EXCEPT ^ value_list
            {
            root_0 = (Object)adaptor.nil();


            EXCEPT173=(Token)match(input,EXCEPT,FOLLOW_EXCEPT_in_except_clause7538); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            EXCEPT173_tree = 
            (Object)adaptor.create(EXCEPT173)
            ;
            root_0 = (Object)adaptor.becomeRoot(EXCEPT173_tree, root_0);
            }

            pushFollow(FOLLOW_value_list_in_except_clause7541);
            value_list174=value_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, value_list174.getTree());

            if ( state.backtracking==0 ) {
                        retval.json = (value_list174!=null?value_list174.json:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 54, except_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "except_clause"


    public static class predicate_props_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "predicate_props"
    // com/senseidb/bql/parsers/BQL.g:2593:1: predicate_props returns [JSONObject json] : WITH ^ prop_list[KeyType.STRING_LITERAL] ;
    public final BQLParser.predicate_props_return predicate_props() throws RecognitionException {
        BQLParser.predicate_props_return retval = new BQLParser.predicate_props_return();
        retval.start = input.LT(1);

        int predicate_props_StartIndex = input.index();

        Object root_0 = null;

        Token WITH175=null;
        BQLParser.prop_list_return prop_list176 =null;


        Object WITH175_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 55) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2594:5: ( WITH ^ prop_list[KeyType.STRING_LITERAL] )
            // com/senseidb/bql/parsers/BQL.g:2594:9: WITH ^ prop_list[KeyType.STRING_LITERAL]
            {
            root_0 = (Object)adaptor.nil();


            WITH175=(Token)match(input,WITH,FOLLOW_WITH_in_predicate_props7576); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            WITH175_tree = 
            (Object)adaptor.create(WITH175)
            ;
            root_0 = (Object)adaptor.becomeRoot(WITH175_tree, root_0);
            }

            pushFollow(FOLLOW_prop_list_in_predicate_props7579);
            prop_list176=prop_list(KeyType.STRING_LITERAL);

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, prop_list176.getTree());

            if ( state.backtracking==0 ) {
                        retval.json = (prop_list176!=null?prop_list176.json:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 55, predicate_props_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "predicate_props"


    public static class prop_list_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "prop_list"
    // com/senseidb/bql/parsers/BQL.g:2600:1: prop_list[KeyType keyType] returns [JSONObject json] : LPAR p= key_value_pair[keyType] ( COMMA p= key_value_pair[keyType] )* RPAR ;
    public final BQLParser.prop_list_return prop_list(KeyType keyType) throws RecognitionException {
        BQLParser.prop_list_return retval = new BQLParser.prop_list_return();
        retval.start = input.LT(1);

        int prop_list_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR177=null;
        Token COMMA178=null;
        Token RPAR179=null;
        BQLParser.key_value_pair_return p =null;


        Object LPAR177_tree=null;
        Object COMMA178_tree=null;
        Object RPAR179_tree=null;


            retval.json = new FastJSONObject();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 56) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2604:5: ( LPAR p= key_value_pair[keyType] ( COMMA p= key_value_pair[keyType] )* RPAR )
            // com/senseidb/bql/parsers/BQL.g:2604:9: LPAR p= key_value_pair[keyType] ( COMMA p= key_value_pair[keyType] )* RPAR
            {
            root_0 = (Object)adaptor.nil();


            LPAR177=(Token)match(input,LPAR,FOLLOW_LPAR_in_prop_list7619); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR177_tree = 
            (Object)adaptor.create(LPAR177)
            ;
            adaptor.addChild(root_0, LPAR177_tree);
            }

            pushFollow(FOLLOW_key_value_pair_in_prop_list7623);
            p=key_value_pair(keyType);

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            retval.json.put((p!=null?p.key:null), (p!=null?p.val:null));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "prop_list", "JSONException: " + err.getMessage());
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:2613:9: ( COMMA p= key_value_pair[keyType] )*
            loop70:
            do {
                int alt70=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt70=1;
                    }
                    break;

                }

                switch (alt70) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2613:10: COMMA p= key_value_pair[keyType]
            	    {
            	    COMMA178=(Token)match(input,COMMA,FOLLOW_COMMA_in_prop_list7645); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA178_tree = 
            	    (Object)adaptor.create(COMMA178)
            	    ;
            	    adaptor.addChild(root_0, COMMA178_tree);
            	    }

            	    pushFollow(FOLLOW_key_value_pair_in_prop_list7649);
            	    p=key_value_pair(keyType);

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

            	    if ( state.backtracking==0 ) {
            	                    try {
            	                        retval.json.put((p!=null?p.key:null), (p!=null?p.val:null));
            	                    }
            	                    catch (JSONException err) {
            	                        throw new FailedPredicateException(input, "prop_list", "JSONException: " + err.getMessage());
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop70;
                }
            } while (true);


            RPAR179=(Token)match(input,RPAR,FOLLOW_RPAR_in_prop_list7677); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR179_tree = 
            (Object)adaptor.create(RPAR179)
            ;
            adaptor.addChild(root_0, RPAR179_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 56, prop_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "prop_list"


    protected static class key_value_pair_scope {
        KeyType type;
    }
    protected Stack key_value_pair_stack = new Stack();


    public static class key_value_pair_return extends ParserRuleReturnScope {
        public String key;
        public Object val;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "key_value_pair"
    // com/senseidb/bql/parsers/BQL.g:2625:1: key_value_pair[KeyType keyType] returns [String key, Object val] : ({...}? => STRING_LITERAL |{...}? => IDENT ) COLON (v= value |vs= python_style_list |vd= python_style_dict ) ;
    public final BQLParser.key_value_pair_return key_value_pair(KeyType keyType) throws RecognitionException {
        key_value_pair_stack.push(new key_value_pair_scope());
        BQLParser.key_value_pair_return retval = new BQLParser.key_value_pair_return();
        retval.start = input.LT(1);

        int key_value_pair_StartIndex = input.index();

        Object root_0 = null;

        Token STRING_LITERAL180=null;
        Token IDENT181=null;
        Token COLON182=null;
        BQLParser.value_return v =null;

        BQLParser.python_style_list_return vs =null;

        BQLParser.python_style_dict_return vd =null;


        Object STRING_LITERAL180_tree=null;
        Object IDENT181_tree=null;
        Object COLON182_tree=null;


            ((key_value_pair_scope)key_value_pair_stack.peek()).type = keyType;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 57) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2632:5: ( ({...}? => STRING_LITERAL |{...}? => IDENT ) COLON (v= value |vs= python_style_list |vd= python_style_dict ) )
            // com/senseidb/bql/parsers/BQL.g:2632:9: ({...}? => STRING_LITERAL |{...}? => IDENT ) COLON (v= value |vs= python_style_list |vd= python_style_dict )
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2632:9: ({...}? => STRING_LITERAL |{...}? => IDENT )
            int alt71=2;
            int LA71_0 = input.LA(1);

            if ( (LA71_0==STRING_LITERAL) && (( ((key_value_pair_scope)key_value_pair_stack.peek()).type == KeyType.STRING_LITERAL ||
                        ((key_value_pair_scope)key_value_pair_stack.peek()).type == KeyType.STRING_LITERAL_AND_IDENT))) {
                alt71=1;
            }
            else if ( (LA71_0==IDENT) && (( ((key_value_pair_scope)key_value_pair_stack.peek()).type == KeyType.IDENT ||
                        ((key_value_pair_scope)key_value_pair_stack.peek()).type == KeyType.STRING_LITERAL_AND_IDENT))) {
                alt71=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 71, 0, input);

                throw nvae;

            }
            switch (alt71) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2632:11: {...}? => STRING_LITERAL
                    {
                    if ( !(( ((key_value_pair_scope)key_value_pair_stack.peek()).type == KeyType.STRING_LITERAL ||
                                ((key_value_pair_scope)key_value_pair_stack.peek()).type == KeyType.STRING_LITERAL_AND_IDENT)) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "key_value_pair", " $key_value_pair::type == KeyType.STRING_LITERAL ||\n            $key_value_pair::type == KeyType.STRING_LITERAL_AND_IDENT");
                    }

                    STRING_LITERAL180=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_key_value_pair7715); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    STRING_LITERAL180_tree = 
                    (Object)adaptor.create(STRING_LITERAL180)
                    ;
                    adaptor.addChild(root_0, STRING_LITERAL180_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2634:11: {...}? => IDENT
                    {
                    if ( !(( ((key_value_pair_scope)key_value_pair_stack.peek()).type == KeyType.IDENT ||
                                ((key_value_pair_scope)key_value_pair_stack.peek()).type == KeyType.STRING_LITERAL_AND_IDENT)) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "key_value_pair", " $key_value_pair::type == KeyType.IDENT ||\n            $key_value_pair::type == KeyType.STRING_LITERAL_AND_IDENT");
                    }

                    IDENT181=(Token)match(input,IDENT,FOLLOW_IDENT_in_key_value_pair7730); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    IDENT181_tree = 
                    (Object)adaptor.create(IDENT181)
                    ;
                    adaptor.addChild(root_0, IDENT181_tree);
                    }

                    }
                    break;

            }


            COLON182=(Token)match(input,COLON,FOLLOW_COLON_in_key_value_pair7750); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            COLON182_tree = 
            (Object)adaptor.create(COLON182)
            ;
            adaptor.addChild(root_0, COLON182_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:2637:15: (v= value |vs= python_style_list |vd= python_style_dict )
            int alt72=3;
            switch ( input.LA(1) ) {
            case EOF:
            case AGO:
            case AND:
            case BROWSE:
            case DATE:
            case DISTINCT:
            case EXECUTE:
            case FALSE:
            case FETCHING:
            case GIVEN:
            case GROUP:
            case INTEGER:
            case LIMIT:
            case NOW:
            case OR:
            case ORDER:
            case REAL:
            case ROUTE:
            case RPAR:
            case SEMI:
            case STRING_LITERAL:
            case TRUE:
            case USING:
            case VARIABLE:
                {
                alt72=1;
                }
                break;
            case 141:
                {
                alt72=2;
                }
                break;
            case 165:
                {
                alt72=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 72, 0, input);

                throw nvae;

            }

            switch (alt72) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2637:16: v= value
                    {
                    pushFollow(FOLLOW_value_in_key_value_pair7755);
                    v=value();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, v.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2637:26: vs= python_style_list
                    {
                    pushFollow(FOLLOW_python_style_list_in_key_value_pair7761);
                    vs=python_style_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, vs.getTree());

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2637:49: vd= python_style_dict
                    {
                    pushFollow(FOLLOW_python_style_dict_in_key_value_pair7767);
                    vd=python_style_dict();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, vd.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        if (STRING_LITERAL180 != null) {
                            String orig = (STRING_LITERAL180!=null?STRING_LITERAL180.getText():null);
                            retval.key = orig.substring(1, orig.length() - 1);
                        }
                        else {
                            retval.key = (IDENT181!=null?IDENT181.getText():null);
                        }
                        if (v != null) {
                            retval.val = (v!=null?v.val:null);
                        }
                        else if (vs != null) {
                            retval.val = (vs!=null?vs.json:null);
                        }
                        else {
                            retval.val = (vd!=null?vd.json:null);
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 57, key_value_pair_StartIndex); }

            key_value_pair_stack.pop();
        }
        return retval;
    }
    // $ANTLR end "key_value_pair"


    public static class given_clause_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "given_clause"
    // com/senseidb/bql/parsers/BQL.g:2658:1: given_clause returns [JSONObject json] : GIVEN FACET PARAM facet_param_list ;
    public final BQLParser.given_clause_return given_clause() throws RecognitionException {
        BQLParser.given_clause_return retval = new BQLParser.given_clause_return();
        retval.start = input.LT(1);

        int given_clause_StartIndex = input.index();

        Object root_0 = null;

        Token GIVEN183=null;
        Token FACET184=null;
        Token PARAM185=null;
        BQLParser.facet_param_list_return facet_param_list186 =null;


        Object GIVEN183_tree=null;
        Object FACET184_tree=null;
        Object PARAM185_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 58) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2659:5: ( GIVEN FACET PARAM facet_param_list )
            // com/senseidb/bql/parsers/BQL.g:2659:9: GIVEN FACET PARAM facet_param_list
            {
            root_0 = (Object)adaptor.nil();


            GIVEN183=(Token)match(input,GIVEN,FOLLOW_GIVEN_in_given_clause7801); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            GIVEN183_tree = 
            (Object)adaptor.create(GIVEN183)
            ;
            adaptor.addChild(root_0, GIVEN183_tree);
            }

            FACET184=(Token)match(input,FACET,FOLLOW_FACET_in_given_clause7803); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            FACET184_tree = 
            (Object)adaptor.create(FACET184)
            ;
            adaptor.addChild(root_0, FACET184_tree);
            }

            PARAM185=(Token)match(input,PARAM,FOLLOW_PARAM_in_given_clause7805); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            PARAM185_tree = 
            (Object)adaptor.create(PARAM185)
            ;
            adaptor.addChild(root_0, PARAM185_tree);
            }

            pushFollow(FOLLOW_facet_param_list_in_given_clause7807);
            facet_param_list186=facet_param_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, facet_param_list186.getTree());

            if ( state.backtracking==0 ) {
                        retval.json = (facet_param_list186!=null?facet_param_list186.json:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 58, given_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "given_clause"


    public static class variable_declarators_return extends ParserRuleReturnScope {
        public JSONArray json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "variable_declarators"
    // com/senseidb/bql/parsers/BQL.g:2670:1: variable_declarators returns [JSONArray json] : var1= variable_declarator ( COMMA var2= variable_declarator )* ;
    public final BQLParser.variable_declarators_return variable_declarators() throws RecognitionException {
        BQLParser.variable_declarators_return retval = new BQLParser.variable_declarators_return();
        retval.start = input.LT(1);

        int variable_declarators_StartIndex = input.index();

        Object root_0 = null;

        Token COMMA187=null;
        BQLParser.variable_declarator_return var1 =null;

        BQLParser.variable_declarator_return var2 =null;


        Object COMMA187_tree=null;


            retval.json = new FastJSONArray();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 59) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2674:5: (var1= variable_declarator ( COMMA var2= variable_declarator )* )
            // com/senseidb/bql/parsers/BQL.g:2674:9: var1= variable_declarator ( COMMA var2= variable_declarator )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_variable_declarator_in_variable_declarators7852);
            var1=variable_declarator();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, var1.getTree());

            if ( state.backtracking==0 ) {
                        retval.json.put((var1!=null?var1.varName:null));
                    }

            // com/senseidb/bql/parsers/BQL.g:2678:9: ( COMMA var2= variable_declarator )*
            loop73:
            do {
                int alt73=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt73=1;
                    }
                    break;

                }

                switch (alt73) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2678:10: COMMA var2= variable_declarator
            	    {
            	    COMMA187=(Token)match(input,COMMA,FOLLOW_COMMA_in_variable_declarators7873); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA187_tree = 
            	    (Object)adaptor.create(COMMA187)
            	    ;
            	    adaptor.addChild(root_0, COMMA187_tree);
            	    }

            	    pushFollow(FOLLOW_variable_declarator_in_variable_declarators7877);
            	    var2=variable_declarator();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, var2.getTree());

            	    if ( state.backtracking==0 ) {
            	                    retval.json.put((var2!=null?var2.varName:null));
            	                }

            	    }
            	    break;

            	default :
            	    break loop73;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 59, variable_declarators_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "variable_declarators"


    public static class variable_declarator_return extends ParserRuleReturnScope {
        public String varName;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "variable_declarator"
    // com/senseidb/bql/parsers/BQL.g:2685:1: variable_declarator returns [String varName] : variable_declarator_id ( '=' variable_initializer )? ;
    public final BQLParser.variable_declarator_return variable_declarator() throws RecognitionException {
        BQLParser.variable_declarator_return retval = new BQLParser.variable_declarator_return();
        retval.start = input.LT(1);

        int variable_declarator_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal189=null;
        BQLParser.variable_declarator_id_return variable_declarator_id188 =null;

        BQLParser.variable_initializer_return variable_initializer190 =null;


        Object char_literal189_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 60) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2686:5: ( variable_declarator_id ( '=' variable_initializer )? )
            // com/senseidb/bql/parsers/BQL.g:2686:9: variable_declarator_id ( '=' variable_initializer )?
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_variable_declarator_id_in_variable_declarator7925);
            variable_declarator_id188=variable_declarator_id();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_declarator_id188.getTree());

            // com/senseidb/bql/parsers/BQL.g:2686:32: ( '=' variable_initializer )?
            int alt74=2;
            switch ( input.LA(1) ) {
                case EQUAL:
                    {
                    alt74=1;
                    }
                    break;
            }

            switch (alt74) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2686:33: '=' variable_initializer
                    {
                    char_literal189=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_variable_declarator7928); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal189_tree = 
                    (Object)adaptor.create(char_literal189)
                    ;
                    adaptor.addChild(root_0, char_literal189_tree);
                    }

                    pushFollow(FOLLOW_variable_initializer_in_variable_declarator7930);
                    variable_initializer190=variable_initializer();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_initializer190.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        retval.varName = (variable_declarator_id188!=null?variable_declarator_id188.varName:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 60, variable_declarator_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "variable_declarator"


    public static class variable_declarator_id_return extends ParserRuleReturnScope {
        public String varName;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "variable_declarator_id"
    // com/senseidb/bql/parsers/BQL.g:2692:1: variable_declarator_id returns [String varName] : IDENT ( '[' ']' )* ;
    public final BQLParser.variable_declarator_id_return variable_declarator_id() throws RecognitionException {
        BQLParser.variable_declarator_id_return retval = new BQLParser.variable_declarator_id_return();
        retval.start = input.LT(1);

        int variable_declarator_id_StartIndex = input.index();

        Object root_0 = null;

        Token IDENT191=null;
        Token char_literal192=null;
        Token char_literal193=null;

        Object IDENT191_tree=null;
        Object char_literal192_tree=null;
        Object char_literal193_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 61) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2693:5: ( IDENT ( '[' ']' )* )
            // com/senseidb/bql/parsers/BQL.g:2693:9: IDENT ( '[' ']' )*
            {
            root_0 = (Object)adaptor.nil();


            IDENT191=(Token)match(input,IDENT,FOLLOW_IDENT_in_variable_declarator_id7965); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            IDENT191_tree = 
            (Object)adaptor.create(IDENT191)
            ;
            adaptor.addChild(root_0, IDENT191_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:2693:15: ( '[' ']' )*
            loop75:
            do {
                int alt75=2;
                switch ( input.LA(1) ) {
                case 141:
                    {
                    alt75=1;
                    }
                    break;

                }

                switch (alt75) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2693:16: '[' ']'
            	    {
            	    char_literal192=(Token)match(input,141,FOLLOW_141_in_variable_declarator_id7968); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal192_tree = 
            	    (Object)adaptor.create(char_literal192)
            	    ;
            	    adaptor.addChild(root_0, char_literal192_tree);
            	    }

            	    char_literal193=(Token)match(input,142,FOLLOW_142_in_variable_declarator_id7970); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal193_tree = 
            	    (Object)adaptor.create(char_literal193)
            	    ;
            	    adaptor.addChild(root_0, char_literal193_tree);
            	    }

            	    }
            	    break;

            	default :
            	    break loop75;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        retval.varName = (IDENT191!=null?IDENT191.getText():null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 61, variable_declarator_id_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "variable_declarator_id"


    public static class variable_initializer_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "variable_initializer"
    // com/senseidb/bql/parsers/BQL.g:2699:1: variable_initializer : ( array_initializer | expression );
    public final BQLParser.variable_initializer_return variable_initializer() throws RecognitionException {
        BQLParser.variable_initializer_return retval = new BQLParser.variable_initializer_return();
        retval.start = input.LT(1);

        int variable_initializer_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.array_initializer_return array_initializer194 =null;

        BQLParser.expression_return expression195 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 62) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2700:5: ( array_initializer | expression )
            int alt76=2;
            switch ( input.LA(1) ) {
            case 165:
                {
                alt76=1;
                }
                break;
            case BOOLEAN:
            case BYTE:
            case CHARACTER_LITERAL:
            case CONTAINS:
            case DOUBLE:
            case FALSE:
            case FLOATING_POINT_LITERAL:
            case HEX_LITERAL:
            case IDENT:
            case INTEGER:
            case LONG:
            case LPAR:
            case NULL:
            case OCTAL_LITERAL:
            case REAL:
            case STRING_LITERAL:
            case TRUE:
            case 114:
            case 123:
            case 124:
            case 126:
            case 127:
            case 134:
            case 135:
            case 136:
            case 137:
            case 138:
            case 139:
            case 140:
            case 170:
                {
                alt76=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 76, 0, input);

                throw nvae;

            }

            switch (alt76) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2700:9: array_initializer
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_array_initializer_in_variable_initializer8001);
                    array_initializer194=array_initializer();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, array_initializer194.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2701:9: expression
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_expression_in_variable_initializer8011);
                    expression195=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression195.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 62, variable_initializer_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "variable_initializer"


    public static class array_initializer_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "array_initializer"
    // com/senseidb/bql/parsers/BQL.g:2704:1: array_initializer : '{' ( variable_initializer ( ',' variable_initializer )* ( ',' )? )? '}' ;
    public final BQLParser.array_initializer_return array_initializer() throws RecognitionException {
        BQLParser.array_initializer_return retval = new BQLParser.array_initializer_return();
        retval.start = input.LT(1);

        int array_initializer_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal196=null;
        Token char_literal198=null;
        Token char_literal200=null;
        Token char_literal201=null;
        BQLParser.variable_initializer_return variable_initializer197 =null;

        BQLParser.variable_initializer_return variable_initializer199 =null;


        Object char_literal196_tree=null;
        Object char_literal198_tree=null;
        Object char_literal200_tree=null;
        Object char_literal201_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 63) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2705:5: ( '{' ( variable_initializer ( ',' variable_initializer )* ( ',' )? )? '}' )
            // com/senseidb/bql/parsers/BQL.g:2705:9: '{' ( variable_initializer ( ',' variable_initializer )* ( ',' )? )? '}'
            {
            root_0 = (Object)adaptor.nil();


            char_literal196=(Token)match(input,165,FOLLOW_165_in_array_initializer8030); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal196_tree = 
            (Object)adaptor.create(char_literal196)
            ;
            adaptor.addChild(root_0, char_literal196_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:2705:13: ( variable_initializer ( ',' variable_initializer )* ( ',' )? )?
            int alt79=2;
            switch ( input.LA(1) ) {
                case BOOLEAN:
                case BYTE:
                case CHARACTER_LITERAL:
                case CONTAINS:
                case DOUBLE:
                case FALSE:
                case FLOATING_POINT_LITERAL:
                case HEX_LITERAL:
                case IDENT:
                case INTEGER:
                case LONG:
                case LPAR:
                case NULL:
                case OCTAL_LITERAL:
                case REAL:
                case STRING_LITERAL:
                case TRUE:
                case 114:
                case 123:
                case 124:
                case 126:
                case 127:
                case 134:
                case 135:
                case 136:
                case 137:
                case 138:
                case 139:
                case 140:
                case 165:
                case 170:
                    {
                    alt79=1;
                    }
                    break;
            }

            switch (alt79) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2705:14: variable_initializer ( ',' variable_initializer )* ( ',' )?
                    {
                    pushFollow(FOLLOW_variable_initializer_in_array_initializer8033);
                    variable_initializer197=variable_initializer();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_initializer197.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2705:35: ( ',' variable_initializer )*
                    loop77:
                    do {
                        int alt77=2;
                        switch ( input.LA(1) ) {
                        case COMMA:
                            {
                            switch ( input.LA(2) ) {
                            case BOOLEAN:
                            case BYTE:
                            case CHARACTER_LITERAL:
                            case CONTAINS:
                            case DOUBLE:
                            case FALSE:
                            case FLOATING_POINT_LITERAL:
                            case HEX_LITERAL:
                            case IDENT:
                            case INTEGER:
                            case LONG:
                            case LPAR:
                            case NULL:
                            case OCTAL_LITERAL:
                            case REAL:
                            case STRING_LITERAL:
                            case TRUE:
                            case 114:
                            case 123:
                            case 124:
                            case 126:
                            case 127:
                            case 134:
                            case 135:
                            case 136:
                            case 137:
                            case 138:
                            case 139:
                            case 140:
                            case 165:
                            case 170:
                                {
                                alt77=1;
                                }
                                break;

                            }

                            }
                            break;

                        }

                        switch (alt77) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2705:36: ',' variable_initializer
                    	    {
                    	    char_literal198=(Token)match(input,COMMA,FOLLOW_COMMA_in_array_initializer8036); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal198_tree = 
                    	    (Object)adaptor.create(char_literal198)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal198_tree);
                    	    }

                    	    pushFollow(FOLLOW_variable_initializer_in_array_initializer8038);
                    	    variable_initializer199=variable_initializer();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_initializer199.getTree());

                    	    }
                    	    break;

                    	default :
                    	    break loop77;
                        }
                    } while (true);


                    // com/senseidb/bql/parsers/BQL.g:2705:63: ( ',' )?
                    int alt78=2;
                    switch ( input.LA(1) ) {
                        case COMMA:
                            {
                            alt78=1;
                            }
                            break;
                    }

                    switch (alt78) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2705:64: ','
                            {
                            char_literal200=(Token)match(input,COMMA,FOLLOW_COMMA_in_array_initializer8043); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            char_literal200_tree = 
                            (Object)adaptor.create(char_literal200)
                            ;
                            adaptor.addChild(root_0, char_literal200_tree);
                            }

                            }
                            break;

                    }


                    }
                    break;

            }


            char_literal201=(Token)match(input,169,FOLLOW_169_in_array_initializer8049); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal201_tree = 
            (Object)adaptor.create(char_literal201)
            ;
            adaptor.addChild(root_0, char_literal201_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 63, array_initializer_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "array_initializer"


    public static class type_return extends ParserRuleReturnScope {
        public String typeName;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "type"
    // com/senseidb/bql/parsers/BQL.g:2708:1: type returns [String typeName] : ( class_or_interface_type ( '[' ']' )* | primitive_type ( '[' ']' )* | boxed_type ( '[' ']' )* | limited_type ( '[' ']' )* );
    public final BQLParser.type_return type() throws RecognitionException {
        BQLParser.type_return retval = new BQLParser.type_return();
        retval.start = input.LT(1);

        int type_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal203=null;
        Token char_literal204=null;
        Token char_literal206=null;
        Token char_literal207=null;
        Token char_literal209=null;
        Token char_literal210=null;
        Token char_literal212=null;
        Token char_literal213=null;
        BQLParser.class_or_interface_type_return class_or_interface_type202 =null;

        BQLParser.primitive_type_return primitive_type205 =null;

        BQLParser.boxed_type_return boxed_type208 =null;

        BQLParser.limited_type_return limited_type211 =null;


        Object char_literal203_tree=null;
        Object char_literal204_tree=null;
        Object char_literal206_tree=null;
        Object char_literal207_tree=null;
        Object char_literal209_tree=null;
        Object char_literal210_tree=null;
        Object char_literal212_tree=null;
        Object char_literal213_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 64) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2709:5: ( class_or_interface_type ( '[' ']' )* | primitive_type ( '[' ']' )* | boxed_type ( '[' ']' )* | limited_type ( '[' ']' )* )
            int alt84=4;
            switch ( input.LA(1) ) {
            case FAST_UTIL_DATA_TYPE:
                {
                alt84=1;
                }
                break;
            case BOOLEAN:
                {
                int LA84_2 = input.LA(2);

                if ( (((synpred118_BQL()&&synpred118_BQL())&&( "boolean".equals(input.LT(1).getText()) ))) ) {
                    alt84=2;
                }
                else if ( (((synpred120_BQL()&&synpred120_BQL())&&( "Boolean".equals(input.LT(1).getText()) ))) ) {
                    alt84=3;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 84, 2, input);

                    throw nvae;

                }
                }
                break;
            case INT:
            case 147:
            case 154:
            case 160:
                {
                alt84=2;
                }
                break;
            case BYTE:
                {
                int LA84_4 = input.LA(2);

                if ( (((synpred118_BQL()&&synpred118_BQL())&&( "byte".equals(input.LT(1).getText()) ))) ) {
                    alt84=2;
                }
                else if ( (((synpred120_BQL()&&synpred120_BQL())&&( "Byte".equals(input.LT(1).getText()) ))) ) {
                    alt84=3;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 84, 4, input);

                    throw nvae;

                }
                }
                break;
            case LONG:
                {
                int LA84_5 = input.LA(2);

                if ( (((synpred118_BQL()&&synpred118_BQL())&&( "long".equals(input.LT(1).getText()) ))) ) {
                    alt84=2;
                }
                else if ( (((synpred120_BQL()&&synpred120_BQL())&&( "Long".equals(input.LT(1).getText()) ))) ) {
                    alt84=3;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 84, 5, input);

                    throw nvae;

                }
                }
                break;
            case DOUBLE:
                {
                int LA84_6 = input.LA(2);

                if ( (((synpred118_BQL()&&synpred118_BQL())&&( "double".equals(input.LT(1).getText()) ))) ) {
                    alt84=2;
                }
                else if ( (((synpred120_BQL()&&synpred120_BQL())&&( "Double".equals(input.LT(1).getText()) ))) ) {
                    alt84=3;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 84, 6, input);

                    throw nvae;

                }
                }
                break;
            case 134:
            case 135:
            case 136:
            case 138:
                {
                alt84=3;
                }
                break;
            case 137:
            case 139:
            case 140:
                {
                alt84=4;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 84, 0, input);

                throw nvae;

            }

            switch (alt84) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2709:9: class_or_interface_type ( '[' ']' )*
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_class_or_interface_type_in_type8072);
                    class_or_interface_type202=class_or_interface_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, class_or_interface_type202.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2709:33: ( '[' ']' )*
                    loop80:
                    do {
                        int alt80=2;
                        switch ( input.LA(1) ) {
                        case 141:
                            {
                            alt80=1;
                            }
                            break;

                        }

                        switch (alt80) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2709:34: '[' ']'
                    	    {
                    	    char_literal203=(Token)match(input,141,FOLLOW_141_in_type8075); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal203_tree = 
                    	    (Object)adaptor.create(char_literal203)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal203_tree);
                    	    }

                    	    char_literal204=(Token)match(input,142,FOLLOW_142_in_type8077); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal204_tree = 
                    	    (Object)adaptor.create(char_literal204)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal204_tree);
                    	    }

                    	    }
                    	    break;

                    	default :
                    	    break loop80;
                        }
                    } while (true);


                    if ( state.backtracking==0 ) { retval.typeName = (class_or_interface_type202!=null?class_or_interface_type202.typeName:null); }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2711:9: primitive_type ( '[' ']' )*
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_primitive_type_in_type8099);
                    primitive_type205=primitive_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, primitive_type205.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2711:24: ( '[' ']' )*
                    loop81:
                    do {
                        int alt81=2;
                        switch ( input.LA(1) ) {
                        case 141:
                            {
                            alt81=1;
                            }
                            break;

                        }

                        switch (alt81) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2711:25: '[' ']'
                    	    {
                    	    char_literal206=(Token)match(input,141,FOLLOW_141_in_type8102); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal206_tree = 
                    	    (Object)adaptor.create(char_literal206)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal206_tree);
                    	    }

                    	    char_literal207=(Token)match(input,142,FOLLOW_142_in_type8104); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal207_tree = 
                    	    (Object)adaptor.create(char_literal207)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal207_tree);
                    	    }

                    	    }
                    	    break;

                    	default :
                    	    break loop81;
                        }
                    } while (true);


                    if ( state.backtracking==0 ) { retval.typeName = (primitive_type205!=null?input.toString(primitive_type205.start,primitive_type205.stop):null);}

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2713:9: boxed_type ( '[' ']' )*
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_boxed_type_in_type8126);
                    boxed_type208=boxed_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, boxed_type208.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2713:20: ( '[' ']' )*
                    loop82:
                    do {
                        int alt82=2;
                        switch ( input.LA(1) ) {
                        case 141:
                            {
                            alt82=1;
                            }
                            break;

                        }

                        switch (alt82) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2713:21: '[' ']'
                    	    {
                    	    char_literal209=(Token)match(input,141,FOLLOW_141_in_type8129); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal209_tree = 
                    	    (Object)adaptor.create(char_literal209)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal209_tree);
                    	    }

                    	    char_literal210=(Token)match(input,142,FOLLOW_142_in_type8131); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal210_tree = 
                    	    (Object)adaptor.create(char_literal210)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal210_tree);
                    	    }

                    	    }
                    	    break;

                    	default :
                    	    break loop82;
                        }
                    } while (true);


                    if ( state.backtracking==0 ) { retval.typeName = (boxed_type208!=null?input.toString(boxed_type208.start,boxed_type208.stop):null);}

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2715:9: limited_type ( '[' ']' )*
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_limited_type_in_type8153);
                    limited_type211=limited_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, limited_type211.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2715:22: ( '[' ']' )*
                    loop83:
                    do {
                        int alt83=2;
                        switch ( input.LA(1) ) {
                        case 141:
                            {
                            alt83=1;
                            }
                            break;

                        }

                        switch (alt83) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:2715:23: '[' ']'
                    	    {
                    	    char_literal212=(Token)match(input,141,FOLLOW_141_in_type8156); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal212_tree = 
                    	    (Object)adaptor.create(char_literal212)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal212_tree);
                    	    }

                    	    char_literal213=(Token)match(input,142,FOLLOW_142_in_type8158); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal213_tree = 
                    	    (Object)adaptor.create(char_literal213)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal213_tree);
                    	    }

                    	    }
                    	    break;

                    	default :
                    	    break loop83;
                        }
                    } while (true);


                    if ( state.backtracking==0 ) { retval.typeName = (limited_type211!=null?input.toString(limited_type211.start,limited_type211.stop):null);}

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 64, type_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "type"


    public static class class_or_interface_type_return extends ParserRuleReturnScope {
        public String typeName;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "class_or_interface_type"
    // com/senseidb/bql/parsers/BQL.g:2719:1: class_or_interface_type returns [String typeName] : FAST_UTIL_DATA_TYPE ;
    public final BQLParser.class_or_interface_type_return class_or_interface_type() throws RecognitionException {
        BQLParser.class_or_interface_type_return retval = new BQLParser.class_or_interface_type_return();
        retval.start = input.LT(1);

        int class_or_interface_type_StartIndex = input.index();

        Object root_0 = null;

        Token FAST_UTIL_DATA_TYPE214=null;

        Object FAST_UTIL_DATA_TYPE214_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 65) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2720:5: ( FAST_UTIL_DATA_TYPE )
            // com/senseidb/bql/parsers/BQL.g:2720:9: FAST_UTIL_DATA_TYPE
            {
            root_0 = (Object)adaptor.nil();


            FAST_UTIL_DATA_TYPE214=(Token)match(input,FAST_UTIL_DATA_TYPE,FOLLOW_FAST_UTIL_DATA_TYPE_in_class_or_interface_type8193); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            FAST_UTIL_DATA_TYPE214_tree = 
            (Object)adaptor.create(FAST_UTIL_DATA_TYPE214)
            ;
            adaptor.addChild(root_0, FAST_UTIL_DATA_TYPE214_tree);
            }

            if ( state.backtracking==0 ) { retval.typeName = _fastutilTypeMap.get((FAST_UTIL_DATA_TYPE214!=null?FAST_UTIL_DATA_TYPE214.getText():null)); }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 65, class_or_interface_type_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "class_or_interface_type"


    public static class type_arguments_return extends ParserRuleReturnScope {
        public String typeArgs;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "type_arguments"
    // com/senseidb/bql/parsers/BQL.g:2724:1: type_arguments returns [String typeArgs] : '<' ta1= type_argument ( COMMA ta2= type_argument )* '>' ;
    public final BQLParser.type_arguments_return type_arguments() throws RecognitionException {
        BQLParser.type_arguments_return retval = new BQLParser.type_arguments_return();
        retval.start = input.LT(1);

        int type_arguments_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal215=null;
        Token COMMA216=null;
        Token char_literal217=null;
        BQLParser.type_argument_return ta1 =null;

        BQLParser.type_argument_return ta2 =null;


        Object char_literal215_tree=null;
        Object COMMA216_tree=null;
        Object char_literal217_tree=null;


            StringBuilder builder = new StringBuilder();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 66) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2728:5: ( '<' ta1= type_argument ( COMMA ta2= type_argument )* '>' )
            // com/senseidb/bql/parsers/BQL.g:2728:9: '<' ta1= type_argument ( COMMA ta2= type_argument )* '>'
            {
            root_0 = (Object)adaptor.nil();


            char_literal215=(Token)match(input,LT,FOLLOW_LT_in_type_arguments8231); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal215_tree = 
            (Object)adaptor.create(char_literal215)
            ;
            adaptor.addChild(root_0, char_literal215_tree);
            }

            pushFollow(FOLLOW_type_argument_in_type_arguments8243);
            ta1=type_argument();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, ta1.getTree());

            if ( state.backtracking==0 ) { builder.append((ta1!=null?input.toString(ta1.start,ta1.stop):null)); }

            // com/senseidb/bql/parsers/BQL.g:2731:9: ( COMMA ta2= type_argument )*
            loop85:
            do {
                int alt85=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt85=1;
                    }
                    break;

                }

                switch (alt85) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2731:10: COMMA ta2= type_argument
            	    {
            	    COMMA216=(Token)match(input,COMMA,FOLLOW_COMMA_in_type_arguments8264); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA216_tree = 
            	    (Object)adaptor.create(COMMA216)
            	    ;
            	    adaptor.addChild(root_0, COMMA216_tree);
            	    }

            	    pushFollow(FOLLOW_type_argument_in_type_arguments8268);
            	    ta2=type_argument();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, ta2.getTree());

            	    if ( state.backtracking==0 ) { builder.append("_").append((ta2!=null?input.toString(ta2.start,ta2.stop):null)); }

            	    }
            	    break;

            	default :
            	    break loop85;
                }
            } while (true);


            char_literal217=(Token)match(input,GT,FOLLOW_GT_in_type_arguments8295); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal217_tree = 
            (Object)adaptor.create(char_literal217)
            ;
            adaptor.addChild(root_0, char_literal217_tree);
            }

            if ( state.backtracking==0 ) {
                        retval.typeArgs = builder.toString();
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 66, type_arguments_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "type_arguments"


    public static class type_argument_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "type_argument"
    // com/senseidb/bql/parsers/BQL.g:2739:1: type_argument : ( type | '?' ( ( 'extends' | 'super' ) type )? );
    public final BQLParser.type_argument_return type_argument() throws RecognitionException {
        BQLParser.type_argument_return retval = new BQLParser.type_argument_return();
        retval.start = input.LT(1);

        int type_argument_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal219=null;
        Token set220=null;
        BQLParser.type_return type218 =null;

        BQLParser.type_return type221 =null;


        Object char_literal219_tree=null;
        Object set220_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 67) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2740:5: ( type | '?' ( ( 'extends' | 'super' ) type )? )
            int alt87=2;
            switch ( input.LA(1) ) {
            case BOOLEAN:
            case BYTE:
            case DOUBLE:
            case FAST_UTIL_DATA_TYPE:
            case INT:
            case LONG:
            case 134:
            case 135:
            case 136:
            case 137:
            case 138:
            case 139:
            case 140:
            case 147:
            case 154:
            case 160:
                {
                alt87=1;
                }
                break;
            case 133:
                {
                alt87=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 87, 0, input);

                throw nvae;

            }

            switch (alt87) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2740:9: type
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_type_in_type_argument8324);
                    type218=type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, type218.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2741:9: '?' ( ( 'extends' | 'super' ) type )?
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal219=(Token)match(input,133,FOLLOW_133_in_type_argument8334); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal219_tree = 
                    (Object)adaptor.create(char_literal219)
                    ;
                    adaptor.addChild(root_0, char_literal219_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:2741:13: ( ( 'extends' | 'super' ) type )?
                    int alt86=2;
                    switch ( input.LA(1) ) {
                        case 152:
                        case 161:
                            {
                            alt86=1;
                            }
                            break;
                    }

                    switch (alt86) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2741:14: ( 'extends' | 'super' ) type
                            {
                            set220=(Token)input.LT(1);

                            if ( input.LA(1)==152||input.LA(1)==161 ) {
                                input.consume();
                                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                                (Object)adaptor.create(set220)
                                );
                                state.errorRecovery=false;
                                state.failed=false;
                            }
                            else {
                                if (state.backtracking>0) {state.failed=true; return retval;}
                                MismatchedSetException mse = new MismatchedSetException(null,input);
                                throw mse;
                            }


                            pushFollow(FOLLOW_type_in_type_argument8345);
                            type221=type();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, type221.getTree());

                            }
                            break;

                    }


                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 67, type_argument_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "type_argument"


    public static class formal_parameters_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "formal_parameters"
    // com/senseidb/bql/parsers/BQL.g:2744:1: formal_parameters returns [JSONObject json] : LPAR formal_parameter_decls RPAR ;
    public final BQLParser.formal_parameters_return formal_parameters() throws RecognitionException {
        BQLParser.formal_parameters_return retval = new BQLParser.formal_parameters_return();
        retval.start = input.LT(1);

        int formal_parameters_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR222=null;
        Token RPAR224=null;
        BQLParser.formal_parameter_decls_return formal_parameter_decls223 =null;


        Object LPAR222_tree=null;
        Object RPAR224_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 68) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2745:5: ( LPAR formal_parameter_decls RPAR )
            // com/senseidb/bql/parsers/BQL.g:2745:9: LPAR formal_parameter_decls RPAR
            {
            root_0 = (Object)adaptor.nil();


            LPAR222=(Token)match(input,LPAR,FOLLOW_LPAR_in_formal_parameters8370); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR222_tree = 
            (Object)adaptor.create(LPAR222)
            ;
            adaptor.addChild(root_0, LPAR222_tree);
            }

            pushFollow(FOLLOW_formal_parameter_decls_in_formal_parameters8372);
            formal_parameter_decls223=formal_parameter_decls();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, formal_parameter_decls223.getTree());

            RPAR224=(Token)match(input,RPAR,FOLLOW_RPAR_in_formal_parameters8374); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR224_tree = 
            (Object)adaptor.create(RPAR224)
            ;
            adaptor.addChild(root_0, RPAR224_tree);
            }

            if ( state.backtracking==0 ) {
                        retval.json = (formal_parameter_decls223!=null?formal_parameter_decls223.json:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 68, formal_parameters_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "formal_parameters"


    public static class formal_parameter_decls_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "formal_parameter_decls"
    // com/senseidb/bql/parsers/BQL.g:2751:1: formal_parameter_decls returns [JSONObject json] : decl= formal_parameter_decl ( COMMA decl= formal_parameter_decl )* ;
    public final BQLParser.formal_parameter_decls_return formal_parameter_decls() throws RecognitionException {
        BQLParser.formal_parameter_decls_return retval = new BQLParser.formal_parameter_decls_return();
        retval.start = input.LT(1);

        int formal_parameter_decls_StartIndex = input.index();

        Object root_0 = null;

        Token COMMA225=null;
        BQLParser.formal_parameter_decl_return decl =null;


        Object COMMA225_tree=null;


            retval.json = new FastJSONObject();
            Set<String> params = new HashSet<String>();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 69) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2756:5: (decl= formal_parameter_decl ( COMMA decl= formal_parameter_decl )* )
            // com/senseidb/bql/parsers/BQL.g:2756:9: decl= formal_parameter_decl ( COMMA decl= formal_parameter_decl )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_formal_parameter_decl_in_formal_parameter_decls8414);
            decl=formal_parameter_decl();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, decl.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            processRelevanceModelParam(input, retval.json, params, (decl!=null?decl.typeName:null), (decl!=null?decl.varName:null));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input,
                                                               "formal_parameter_decls",
                                                               "JSONException: " + err.getMessage());
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:2767:9: ( COMMA decl= formal_parameter_decl )*
            loop88:
            do {
                int alt88=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt88=1;
                    }
                    break;

                }

                switch (alt88) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2767:10: COMMA decl= formal_parameter_decl
            	    {
            	    COMMA225=(Token)match(input,COMMA,FOLLOW_COMMA_in_formal_parameter_decls8435); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA225_tree = 
            	    (Object)adaptor.create(COMMA225)
            	    ;
            	    adaptor.addChild(root_0, COMMA225_tree);
            	    }

            	    pushFollow(FOLLOW_formal_parameter_decl_in_formal_parameter_decls8439);
            	    decl=formal_parameter_decl();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, decl.getTree());

            	    if ( state.backtracking==0 ) {
            	                    try {
            	                        processRelevanceModelParam(input, retval.json, params, (decl!=null?decl.typeName:null), (decl!=null?decl.varName:null));
            	                    }
            	                    catch (JSONException err) {
            	                        throw new FailedPredicateException(input,
            	                                                           "formal_parameter_decl",
            	                                                           "JSONException: " + err.getMessage());
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop88;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 69, formal_parameter_decls_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "formal_parameter_decls"


    public static class formal_parameter_decl_return extends ParserRuleReturnScope {
        public String typeName;
        public String varName;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "formal_parameter_decl"
    // com/senseidb/bql/parsers/BQL.g:2781:1: formal_parameter_decl returns [String typeName, String varName] : variable_modifiers type variable_declarator_id ;
    public final BQLParser.formal_parameter_decl_return formal_parameter_decl() throws RecognitionException {
        BQLParser.formal_parameter_decl_return retval = new BQLParser.formal_parameter_decl_return();
        retval.start = input.LT(1);

        int formal_parameter_decl_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.variable_modifiers_return variable_modifiers226 =null;

        BQLParser.type_return type227 =null;

        BQLParser.variable_declarator_id_return variable_declarator_id228 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 70) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2782:5: ( variable_modifiers type variable_declarator_id )
            // com/senseidb/bql/parsers/BQL.g:2782:9: variable_modifiers type variable_declarator_id
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_variable_modifiers_in_formal_parameter_decl8491);
            variable_modifiers226=variable_modifiers();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_modifiers226.getTree());

            pushFollow(FOLLOW_type_in_formal_parameter_decl8493);
            type227=type();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, type227.getTree());

            pushFollow(FOLLOW_variable_declarator_id_in_formal_parameter_decl8495);
            variable_declarator_id228=variable_declarator_id();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_declarator_id228.getTree());

            if ( state.backtracking==0 ) {
                        retval.typeName = (type227!=null?type227.typeName:null);
                        retval.varName = (variable_declarator_id228!=null?variable_declarator_id228.varName:null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 70, formal_parameter_decl_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "formal_parameter_decl"


    public static class primitive_type_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "primitive_type"
    // com/senseidb/bql/parsers/BQL.g:2789:1: primitive_type : ({...}? BOOLEAN | 'char' |{...}? BYTE | 'short' |{...}? INT |{...}? LONG | 'float' |{...}? DOUBLE );
    public final BQLParser.primitive_type_return primitive_type() throws RecognitionException {
        BQLParser.primitive_type_return retval = new BQLParser.primitive_type_return();
        retval.start = input.LT(1);

        int primitive_type_StartIndex = input.index();

        Object root_0 = null;

        Token BOOLEAN229=null;
        Token string_literal230=null;
        Token BYTE231=null;
        Token string_literal232=null;
        Token INT233=null;
        Token LONG234=null;
        Token string_literal235=null;
        Token DOUBLE236=null;

        Object BOOLEAN229_tree=null;
        Object string_literal230_tree=null;
        Object BYTE231_tree=null;
        Object string_literal232_tree=null;
        Object INT233_tree=null;
        Object LONG234_tree=null;
        Object string_literal235_tree=null;
        Object DOUBLE236_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 71) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2790:5: ({...}? BOOLEAN | 'char' |{...}? BYTE | 'short' |{...}? INT |{...}? LONG | 'float' |{...}? DOUBLE )
            int alt89=8;
            switch ( input.LA(1) ) {
            case BOOLEAN:
                {
                alt89=1;
                }
                break;
            case 147:
                {
                alt89=2;
                }
                break;
            case BYTE:
                {
                alt89=3;
                }
                break;
            case 160:
                {
                alt89=4;
                }
                break;
            case INT:
                {
                alt89=5;
                }
                break;
            case LONG:
                {
                alt89=6;
                }
                break;
            case 154:
                {
                alt89=7;
                }
                break;
            case DOUBLE:
                {
                alt89=8;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 89, 0, input);

                throw nvae;

            }

            switch (alt89) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2790:9: {...}? BOOLEAN
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "boolean".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "primitive_type", " \"boolean\".equals(input.LT(1).getText()) ");
                    }

                    BOOLEAN229=(Token)match(input,BOOLEAN,FOLLOW_BOOLEAN_in_primitive_type8526); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    BOOLEAN229_tree = 
                    (Object)adaptor.create(BOOLEAN229)
                    ;
                    adaptor.addChild(root_0, BOOLEAN229_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2791:9: 'char'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal230=(Token)match(input,147,FOLLOW_147_in_primitive_type8536); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal230_tree = 
                    (Object)adaptor.create(string_literal230)
                    ;
                    adaptor.addChild(root_0, string_literal230_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2792:9: {...}? BYTE
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "byte".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "primitive_type", " \"byte\".equals(input.LT(1).getText()) ");
                    }

                    BYTE231=(Token)match(input,BYTE,FOLLOW_BYTE_in_primitive_type8548); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    BYTE231_tree = 
                    (Object)adaptor.create(BYTE231)
                    ;
                    adaptor.addChild(root_0, BYTE231_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2793:9: 'short'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal232=(Token)match(input,160,FOLLOW_160_in_primitive_type8558); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal232_tree = 
                    (Object)adaptor.create(string_literal232)
                    ;
                    adaptor.addChild(root_0, string_literal232_tree);
                    }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:2794:9: {...}? INT
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "int".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "primitive_type", " \"int\".equals(input.LT(1).getText()) ");
                    }

                    INT233=(Token)match(input,INT,FOLLOW_INT_in_primitive_type8570); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    INT233_tree = 
                    (Object)adaptor.create(INT233)
                    ;
                    adaptor.addChild(root_0, INT233_tree);
                    }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:2795:9: {...}? LONG
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "long".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "primitive_type", " \"long\".equals(input.LT(1).getText()) ");
                    }

                    LONG234=(Token)match(input,LONG,FOLLOW_LONG_in_primitive_type8582); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    LONG234_tree = 
                    (Object)adaptor.create(LONG234)
                    ;
                    adaptor.addChild(root_0, LONG234_tree);
                    }

                    }
                    break;
                case 7 :
                    // com/senseidb/bql/parsers/BQL.g:2796:9: 'float'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal235=(Token)match(input,154,FOLLOW_154_in_primitive_type8592); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal235_tree = 
                    (Object)adaptor.create(string_literal235)
                    ;
                    adaptor.addChild(root_0, string_literal235_tree);
                    }

                    }
                    break;
                case 8 :
                    // com/senseidb/bql/parsers/BQL.g:2797:9: {...}? DOUBLE
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "double".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "primitive_type", " \"double\".equals(input.LT(1).getText()) ");
                    }

                    DOUBLE236=(Token)match(input,DOUBLE,FOLLOW_DOUBLE_in_primitive_type8604); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    DOUBLE236_tree = 
                    (Object)adaptor.create(DOUBLE236)
                    ;
                    adaptor.addChild(root_0, DOUBLE236_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 71, primitive_type_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "primitive_type"


    public static class boxed_type_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "boxed_type"
    // com/senseidb/bql/parsers/BQL.g:2800:1: boxed_type : ({...}? BOOLEAN | 'Character' |{...}? BYTE | 'Short' | 'Integer' |{...}? LONG | 'Float' |{...}? DOUBLE );
    public final BQLParser.boxed_type_return boxed_type() throws RecognitionException {
        BQLParser.boxed_type_return retval = new BQLParser.boxed_type_return();
        retval.start = input.LT(1);

        int boxed_type_StartIndex = input.index();

        Object root_0 = null;

        Token BOOLEAN237=null;
        Token string_literal238=null;
        Token BYTE239=null;
        Token string_literal240=null;
        Token string_literal241=null;
        Token LONG242=null;
        Token string_literal243=null;
        Token DOUBLE244=null;

        Object BOOLEAN237_tree=null;
        Object string_literal238_tree=null;
        Object BYTE239_tree=null;
        Object string_literal240_tree=null;
        Object string_literal241_tree=null;
        Object LONG242_tree=null;
        Object string_literal243_tree=null;
        Object DOUBLE244_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 72) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2801:5: ({...}? BOOLEAN | 'Character' |{...}? BYTE | 'Short' | 'Integer' |{...}? LONG | 'Float' |{...}? DOUBLE )
            int alt90=8;
            switch ( input.LA(1) ) {
            case BOOLEAN:
                {
                alt90=1;
                }
                break;
            case 134:
                {
                alt90=2;
                }
                break;
            case BYTE:
                {
                alt90=3;
                }
                break;
            case 138:
                {
                alt90=4;
                }
                break;
            case 136:
                {
                alt90=5;
                }
                break;
            case LONG:
                {
                alt90=6;
                }
                break;
            case 135:
                {
                alt90=7;
                }
                break;
            case DOUBLE:
                {
                alt90=8;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 90, 0, input);

                throw nvae;

            }

            switch (alt90) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2801:9: {...}? BOOLEAN
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "Boolean".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "boxed_type", " \"Boolean\".equals(input.LT(1).getText()) ");
                    }

                    BOOLEAN237=(Token)match(input,BOOLEAN,FOLLOW_BOOLEAN_in_boxed_type8625); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    BOOLEAN237_tree = 
                    (Object)adaptor.create(BOOLEAN237)
                    ;
                    adaptor.addChild(root_0, BOOLEAN237_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2802:9: 'Character'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal238=(Token)match(input,134,FOLLOW_134_in_boxed_type8635); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal238_tree = 
                    (Object)adaptor.create(string_literal238)
                    ;
                    adaptor.addChild(root_0, string_literal238_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2803:9: {...}? BYTE
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "Byte".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "boxed_type", " \"Byte\".equals(input.LT(1).getText()) ");
                    }

                    BYTE239=(Token)match(input,BYTE,FOLLOW_BYTE_in_boxed_type8647); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    BYTE239_tree = 
                    (Object)adaptor.create(BYTE239)
                    ;
                    adaptor.addChild(root_0, BYTE239_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2804:9: 'Short'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal240=(Token)match(input,138,FOLLOW_138_in_boxed_type8657); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal240_tree = 
                    (Object)adaptor.create(string_literal240)
                    ;
                    adaptor.addChild(root_0, string_literal240_tree);
                    }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:2805:9: 'Integer'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal241=(Token)match(input,136,FOLLOW_136_in_boxed_type8667); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal241_tree = 
                    (Object)adaptor.create(string_literal241)
                    ;
                    adaptor.addChild(root_0, string_literal241_tree);
                    }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:2806:9: {...}? LONG
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "Long".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "boxed_type", " \"Long\".equals(input.LT(1).getText()) ");
                    }

                    LONG242=(Token)match(input,LONG,FOLLOW_LONG_in_boxed_type8679); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    LONG242_tree = 
                    (Object)adaptor.create(LONG242)
                    ;
                    adaptor.addChild(root_0, LONG242_tree);
                    }

                    }
                    break;
                case 7 :
                    // com/senseidb/bql/parsers/BQL.g:2807:9: 'Float'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal243=(Token)match(input,135,FOLLOW_135_in_boxed_type8689); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal243_tree = 
                    (Object)adaptor.create(string_literal243)
                    ;
                    adaptor.addChild(root_0, string_literal243_tree);
                    }

                    }
                    break;
                case 8 :
                    // com/senseidb/bql/parsers/BQL.g:2808:9: {...}? DOUBLE
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "Double".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "boxed_type", " \"Double\".equals(input.LT(1).getText()) ");
                    }

                    DOUBLE244=(Token)match(input,DOUBLE,FOLLOW_DOUBLE_in_boxed_type8701); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    DOUBLE244_tree = 
                    (Object)adaptor.create(DOUBLE244)
                    ;
                    adaptor.addChild(root_0, DOUBLE244_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 72, boxed_type_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "boxed_type"


    public static class limited_type_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "limited_type"
    // com/senseidb/bql/parsers/BQL.g:2811:1: limited_type : ( 'String' | 'System' | 'Math' );
    public final BQLParser.limited_type_return limited_type() throws RecognitionException {
        BQLParser.limited_type_return retval = new BQLParser.limited_type_return();
        retval.start = input.LT(1);

        int limited_type_StartIndex = input.index();

        Object root_0 = null;

        Token set245=null;

        Object set245_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 73) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2812:5: ( 'String' | 'System' | 'Math' )
            // com/senseidb/bql/parsers/BQL.g:
            {
            root_0 = (Object)adaptor.nil();


            set245=(Token)input.LT(1);

            if ( input.LA(1)==137||(input.LA(1) >= 139 && input.LA(1) <= 140) ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                (Object)adaptor.create(set245)
                );
                state.errorRecovery=false;
                state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 73, limited_type_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "limited_type"


    public static class variable_modifier_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "variable_modifier"
    // com/senseidb/bql/parsers/BQL.g:2817:1: variable_modifier : 'final' ;
    public final BQLParser.variable_modifier_return variable_modifier() throws RecognitionException {
        BQLParser.variable_modifier_return retval = new BQLParser.variable_modifier_return();
        retval.start = input.LT(1);

        int variable_modifier_StartIndex = input.index();

        Object root_0 = null;

        Token string_literal246=null;

        Object string_literal246_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 74) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2818:5: ( 'final' )
            // com/senseidb/bql/parsers/BQL.g:2818:9: 'final'
            {
            root_0 = (Object)adaptor.nil();


            string_literal246=(Token)match(input,153,FOLLOW_153_in_variable_modifier8764); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            string_literal246_tree = 
            (Object)adaptor.create(string_literal246)
            ;
            adaptor.addChild(root_0, string_literal246_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 74, variable_modifier_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "variable_modifier"


    public static class relevance_model_return extends ParserRuleReturnScope {
        public String functionBody;
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "relevance_model"
    // com/senseidb/bql/parsers/BQL.g:2821:1: relevance_model returns [String functionBody, JSONObject json] : DEFINED AS params= formal_parameters BEGIN model_block END ;
    public final BQLParser.relevance_model_return relevance_model() throws RecognitionException {
        BQLParser.relevance_model_return retval = new BQLParser.relevance_model_return();
        retval.start = input.LT(1);

        int relevance_model_StartIndex = input.index();

        Object root_0 = null;

        Token DEFINED247=null;
        Token AS248=null;
        Token BEGIN249=null;
        Token END251=null;
        BQLParser.formal_parameters_return params =null;

        BQLParser.model_block_return model_block250 =null;


        Object DEFINED247_tree=null;
        Object AS248_tree=null;
        Object BEGIN249_tree=null;
        Object END251_tree=null;


            _usedFacets = new HashSet<String>();
            _usedInternalVars = new HashSet<String>();
            _symbolTable = new LinkedList<Map<String, String>>();
            _currentScope = new HashMap<String, String>();
            _symbolTable.offerLast(_currentScope);

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 75) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2829:5: ( DEFINED AS params= formal_parameters BEGIN model_block END )
            // com/senseidb/bql/parsers/BQL.g:2829:9: DEFINED AS params= formal_parameters BEGIN model_block END
            {
            root_0 = (Object)adaptor.nil();


            DEFINED247=(Token)match(input,DEFINED,FOLLOW_DEFINED_in_relevance_model8792); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            DEFINED247_tree = 
            (Object)adaptor.create(DEFINED247)
            ;
            adaptor.addChild(root_0, DEFINED247_tree);
            }

            AS248=(Token)match(input,AS,FOLLOW_AS_in_relevance_model8794); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            AS248_tree = 
            (Object)adaptor.create(AS248)
            ;
            adaptor.addChild(root_0, AS248_tree);
            }

            pushFollow(FOLLOW_formal_parameters_in_relevance_model8798);
            params=formal_parameters();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, params.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            JSONObject varParams = (params!=null?params.json:null).optJSONObject("variables");
                            if (varParams != null) {
                                Iterator<String> itr = varParams.keys();
                                while (itr.hasNext()) {
                                    String key = (String) itr.next();
                                    JSONArray vars = varParams.getJSONArray(key);
                                    for (int i = 0; i < vars.length(); ++i) {
                                        _currentScope.put(vars.getString(i), key);
                                    }
                                }
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "relevance_model", "JSONException: " + err.getMessage());
                        }
                    }

            BEGIN249=(Token)match(input,BEGIN,FOLLOW_BEGIN_in_relevance_model8818); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            BEGIN249_tree = 
            (Object)adaptor.create(BEGIN249)
            ;
            adaptor.addChild(root_0, BEGIN249_tree);
            }

            pushFollow(FOLLOW_model_block_in_relevance_model8820);
            model_block250=model_block();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, model_block250.getTree());

            END251=(Token)match(input,END,FOLLOW_END_in_relevance_model8822); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            END251_tree = 
            (Object)adaptor.create(END251)
            ;
            adaptor.addChild(root_0, END251_tree);
            }

            if ( state.backtracking==0 ) {
                        retval.functionBody = (model_block250!=null?input.toString(model_block250.start,model_block250.stop):null);
                        retval.json = (params!=null?params.json:null);

                        // Append facets and internal variable to "function_params".
                        try {
                            JSONArray funcParams = retval.json.getJSONArray("function_params");

                            JSONObject facets = new FastJSONObject();
                            retval.json.put("facets", facets);

                            for (String facet: _usedFacets) {
                                funcParams.put(facet);
                                String[] facetInfo = _facetInfoMap.get(facet);
                                String typeName = (facetInfo[0].equals("multi") ? "m" :
                                                   (facetInfo[0].equals("weighted-multi") ? "wm" : "")
                                                  )
                                                  + _facetInfoMap.get(facet)[1];
                                JSONArray facetsWithSameType = facets.optJSONArray(typeName);
                                if (facetsWithSameType == null) {
                                    facetsWithSameType = new FastJSONArray();
                                    facets.put(typeName, facetsWithSameType);
                                }
                                facetsWithSameType.put(facet);
                            }

                            // Internal variables, like _NOW, do not need to be
                            // included in "variables".
                            for (String varName: _usedInternalVars) {
                                if( ! _internalStaticVarMap.containsKey(varName))
                                    funcParams.put(varName);
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input,
                                                               "formal_parameter_decl",
                                                               "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 75, relevance_model_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "relevance_model"


    public static class model_block_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "model_block"
    // com/senseidb/bql/parsers/BQL.g:2890:1: model_block : ( block_statement )+ ;
    public final BQLParser.model_block_return model_block() throws RecognitionException {
        BQLParser.model_block_return retval = new BQLParser.model_block_return();
        retval.start = input.LT(1);

        int model_block_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.block_statement_return block_statement252 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 76) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2891:5: ( ( block_statement )+ )
            // com/senseidb/bql/parsers/BQL.g:2891:9: ( block_statement )+
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2891:9: ( block_statement )+
            int cnt91=0;
            loop91:
            do {
                int alt91=2;
                switch ( input.LA(1) ) {
                case BOOLEAN:
                case BYTE:
                case CHARACTER_LITERAL:
                case CONTAINS:
                case DOUBLE:
                case FALSE:
                case FAST_UTIL_DATA_TYPE:
                case FLOATING_POINT_LITERAL:
                case HEX_LITERAL:
                case IDENT:
                case INT:
                case INTEGER:
                case LONG:
                case LPAR:
                case NULL:
                case OCTAL_LITERAL:
                case REAL:
                case SEMI:
                case STRING_LITERAL:
                case TRUE:
                case 114:
                case 123:
                case 124:
                case 126:
                case 127:
                case 134:
                case 135:
                case 136:
                case 137:
                case 138:
                case 139:
                case 140:
                case 145:
                case 147:
                case 149:
                case 151:
                case 153:
                case 154:
                case 155:
                case 156:
                case 159:
                case 160:
                case 162:
                case 164:
                case 165:
                case 170:
                    {
                    alt91=1;
                    }
                    break;

                }

                switch (alt91) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2891:9: block_statement
            	    {
            	    pushFollow(FOLLOW_block_statement_in_model_block8851);
            	    block_statement252=block_statement();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, block_statement252.getTree());

            	    }
            	    break;

            	default :
            	    if ( cnt91 >= 1 ) break loop91;
            	    if (state.backtracking>0) {state.failed=true; return retval;}
                        EarlyExitException eee =
                            new EarlyExitException(91, input);
                        throw eee;
                }
                cnt91++;
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 76, model_block_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "model_block"


    public static class block_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "block"
    // com/senseidb/bql/parsers/BQL.g:2894:1: block : '{' ( block_statement )* '}' ;
    public final BQLParser.block_return block() throws RecognitionException {
        BQLParser.block_return retval = new BQLParser.block_return();
        retval.start = input.LT(1);

        int block_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal253=null;
        Token char_literal255=null;
        BQLParser.block_statement_return block_statement254 =null;


        Object char_literal253_tree=null;
        Object char_literal255_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 77) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2895:5: ( '{' ( block_statement )* '}' )
            // com/senseidb/bql/parsers/BQL.g:2895:9: '{' ( block_statement )* '}'
            {
            root_0 = (Object)adaptor.nil();


            char_literal253=(Token)match(input,165,FOLLOW_165_in_block8871); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal253_tree = 
            (Object)adaptor.create(char_literal253)
            ;
            adaptor.addChild(root_0, char_literal253_tree);
            }

            if ( state.backtracking==0 ) {
                        _currentScope = new HashMap<String, String>();
                        _symbolTable.offerLast(_currentScope);
                    }

            // com/senseidb/bql/parsers/BQL.g:2900:9: ( block_statement )*
            loop92:
            do {
                int alt92=2;
                switch ( input.LA(1) ) {
                case BOOLEAN:
                case BYTE:
                case CHARACTER_LITERAL:
                case CONTAINS:
                case DOUBLE:
                case FALSE:
                case FAST_UTIL_DATA_TYPE:
                case FLOATING_POINT_LITERAL:
                case HEX_LITERAL:
                case IDENT:
                case INT:
                case INTEGER:
                case LONG:
                case LPAR:
                case NULL:
                case OCTAL_LITERAL:
                case REAL:
                case SEMI:
                case STRING_LITERAL:
                case TRUE:
                case 114:
                case 123:
                case 124:
                case 126:
                case 127:
                case 134:
                case 135:
                case 136:
                case 137:
                case 138:
                case 139:
                case 140:
                case 145:
                case 147:
                case 149:
                case 151:
                case 153:
                case 154:
                case 155:
                case 156:
                case 159:
                case 160:
                case 162:
                case 164:
                case 165:
                case 170:
                    {
                    alt92=1;
                    }
                    break;

                }

                switch (alt92) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2900:9: block_statement
            	    {
            	    pushFollow(FOLLOW_block_statement_in_block8892);
            	    block_statement254=block_statement();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, block_statement254.getTree());

            	    }
            	    break;

            	default :
            	    break loop92;
                }
            } while (true);


            if ( state.backtracking==0 ) {
                        _symbolTable.pollLast();
                        _currentScope = _symbolTable.peekLast();
                    }

            char_literal255=(Token)match(input,169,FOLLOW_169_in_block8914); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal255_tree = 
            (Object)adaptor.create(char_literal255)
            ;
            adaptor.addChild(root_0, char_literal255_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 77, block_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "block"


    public static class block_statement_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "block_statement"
    // com/senseidb/bql/parsers/BQL.g:2908:1: block_statement : ( local_variable_declaration_stmt | java_statement );
    public final BQLParser.block_statement_return block_statement() throws RecognitionException {
        BQLParser.block_statement_return retval = new BQLParser.block_statement_return();
        retval.start = input.LT(1);

        int block_statement_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.local_variable_declaration_stmt_return local_variable_declaration_stmt256 =null;

        BQLParser.java_statement_return java_statement257 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 78) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2909:5: ( local_variable_declaration_stmt | java_statement )
            int alt93=2;
            alt93 = dfa93.predict(input);
            switch (alt93) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2909:9: local_variable_declaration_stmt
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_local_variable_declaration_stmt_in_block_statement8933);
                    local_variable_declaration_stmt256=local_variable_declaration_stmt();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, local_variable_declaration_stmt256.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2910:9: java_statement
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_java_statement_in_block_statement8943);
                    java_statement257=java_statement();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_statement257.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 78, block_statement_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "block_statement"


    public static class local_variable_declaration_stmt_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "local_variable_declaration_stmt"
    // com/senseidb/bql/parsers/BQL.g:2913:1: local_variable_declaration_stmt : local_variable_declaration SEMI ;
    public final BQLParser.local_variable_declaration_stmt_return local_variable_declaration_stmt() throws RecognitionException {
        BQLParser.local_variable_declaration_stmt_return retval = new BQLParser.local_variable_declaration_stmt_return();
        retval.start = input.LT(1);

        int local_variable_declaration_stmt_StartIndex = input.index();

        Object root_0 = null;

        Token SEMI259=null;
        BQLParser.local_variable_declaration_return local_variable_declaration258 =null;


        Object SEMI259_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 79) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2914:5: ( local_variable_declaration SEMI )
            // com/senseidb/bql/parsers/BQL.g:2914:9: local_variable_declaration SEMI
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_local_variable_declaration_in_local_variable_declaration_stmt8962);
            local_variable_declaration258=local_variable_declaration();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, local_variable_declaration258.getTree());

            SEMI259=(Token)match(input,SEMI,FOLLOW_SEMI_in_local_variable_declaration_stmt8964); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            SEMI259_tree = 
            (Object)adaptor.create(SEMI259)
            ;
            adaptor.addChild(root_0, SEMI259_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 79, local_variable_declaration_stmt_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "local_variable_declaration_stmt"


    public static class local_variable_declaration_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "local_variable_declaration"
    // com/senseidb/bql/parsers/BQL.g:2917:1: local_variable_declaration : variable_modifiers type variable_declarators ;
    public final BQLParser.local_variable_declaration_return local_variable_declaration() throws RecognitionException {
        BQLParser.local_variable_declaration_return retval = new BQLParser.local_variable_declaration_return();
        retval.start = input.LT(1);

        int local_variable_declaration_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.variable_modifiers_return variable_modifiers260 =null;

        BQLParser.type_return type261 =null;

        BQLParser.variable_declarators_return variable_declarators262 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 80) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2918:5: ( variable_modifiers type variable_declarators )
            // com/senseidb/bql/parsers/BQL.g:2918:9: variable_modifiers type variable_declarators
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_variable_modifiers_in_local_variable_declaration8983);
            variable_modifiers260=variable_modifiers();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_modifiers260.getTree());

            pushFollow(FOLLOW_type_in_local_variable_declaration8985);
            type261=type();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, type261.getTree());

            pushFollow(FOLLOW_variable_declarators_in_local_variable_declaration8987);
            variable_declarators262=variable_declarators();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_declarators262.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            JSONArray vars = (variable_declarators262!=null?variable_declarators262.json:null);
                            for (int i = 0; i < vars.length(); ++i) {
                                String var = vars.getString(i);
                                if (_facetInfoMap.containsKey(var)) {
                                    throw new FailedPredicateException(input,
                                                                       "local_variable_declaration",
                                                                       "Facet name \"" + var + "\" cannot be used to declare a variable.");
                                }
                                else if (_internalVarMap.containsKey(var)) {
                                    throw new FailedPredicateException(input,
                                                                       "local_variable_declaration",
                                                                       "Internal variable \"" + var + "\" cannot be re-used to declare another variable.");
                                }
                                else if (verifyVariable(var)) {
                                    throw new FailedPredicateException(input,
                                                                       "local_variable_declaration",
                                                                       "Variable \"" + var + "\" is already defined.");
                                }
                                else {
                                    _currentScope.put(var, (type261!=null?type261.typeName:null));
                                }
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "local_variable_declaration", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 80, local_variable_declaration_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "local_variable_declaration"


    public static class variable_modifiers_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "variable_modifiers"
    // com/senseidb/bql/parsers/BQL.g:2950:1: variable_modifiers : ( variable_modifier )* ;
    public final BQLParser.variable_modifiers_return variable_modifiers() throws RecognitionException {
        BQLParser.variable_modifiers_return retval = new BQLParser.variable_modifiers_return();
        retval.start = input.LT(1);

        int variable_modifiers_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.variable_modifier_return variable_modifier263 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 81) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2951:5: ( ( variable_modifier )* )
            // com/senseidb/bql/parsers/BQL.g:2951:9: ( variable_modifier )*
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2951:9: ( variable_modifier )*
            loop94:
            do {
                int alt94=2;
                switch ( input.LA(1) ) {
                case 153:
                    {
                    alt94=1;
                    }
                    break;

                }

                switch (alt94) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2951:9: variable_modifier
            	    {
            	    pushFollow(FOLLOW_variable_modifier_in_variable_modifiers9016);
            	    variable_modifier263=variable_modifier();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_modifier263.getTree());

            	    }
            	    break;

            	default :
            	    break loop94;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 81, variable_modifiers_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "variable_modifiers"


    public static class java_statement_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "java_statement"
    // com/senseidb/bql/parsers/BQL.g:2954:1: java_statement : ( block | 'if' par_expression java_statement ( else_statement )? | 'for' LPAR for_control RPAR java_statement | 'while' par_expression java_statement | 'do' java_statement 'while' par_expression SEMI | 'switch' par_expression '{' switch_block_statement_groups '}' | 'return' expression SEMI | 'break' ( IDENT )? SEMI | 'continue' ( IDENT )? SEMI | SEMI | statement_expression SEMI );
    public final BQLParser.java_statement_return java_statement() throws RecognitionException {
        BQLParser.java_statement_return retval = new BQLParser.java_statement_return();
        retval.start = input.LT(1);

        int java_statement_StartIndex = input.index();

        Object root_0 = null;

        Token string_literal265=null;
        Token string_literal269=null;
        Token LPAR270=null;
        Token RPAR272=null;
        Token string_literal274=null;
        Token string_literal277=null;
        Token string_literal279=null;
        Token SEMI281=null;
        Token string_literal282=null;
        Token char_literal284=null;
        Token char_literal286=null;
        Token string_literal287=null;
        Token SEMI289=null;
        Token string_literal290=null;
        Token IDENT291=null;
        Token SEMI292=null;
        Token string_literal293=null;
        Token IDENT294=null;
        Token SEMI295=null;
        Token SEMI296=null;
        Token SEMI298=null;
        BQLParser.block_return block264 =null;

        BQLParser.par_expression_return par_expression266 =null;

        BQLParser.java_statement_return java_statement267 =null;

        BQLParser.else_statement_return else_statement268 =null;

        BQLParser.for_control_return for_control271 =null;

        BQLParser.java_statement_return java_statement273 =null;

        BQLParser.par_expression_return par_expression275 =null;

        BQLParser.java_statement_return java_statement276 =null;

        BQLParser.java_statement_return java_statement278 =null;

        BQLParser.par_expression_return par_expression280 =null;

        BQLParser.par_expression_return par_expression283 =null;

        BQLParser.switch_block_statement_groups_return switch_block_statement_groups285 =null;

        BQLParser.expression_return expression288 =null;

        BQLParser.statement_expression_return statement_expression297 =null;


        Object string_literal265_tree=null;
        Object string_literal269_tree=null;
        Object LPAR270_tree=null;
        Object RPAR272_tree=null;
        Object string_literal274_tree=null;
        Object string_literal277_tree=null;
        Object string_literal279_tree=null;
        Object SEMI281_tree=null;
        Object string_literal282_tree=null;
        Object char_literal284_tree=null;
        Object char_literal286_tree=null;
        Object string_literal287_tree=null;
        Object SEMI289_tree=null;
        Object string_literal290_tree=null;
        Object IDENT291_tree=null;
        Object SEMI292_tree=null;
        Object string_literal293_tree=null;
        Object IDENT294_tree=null;
        Object SEMI295_tree=null;
        Object SEMI296_tree=null;
        Object SEMI298_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 82) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2955:5: ( block | 'if' par_expression java_statement ( else_statement )? | 'for' LPAR for_control RPAR java_statement | 'while' par_expression java_statement | 'do' java_statement 'while' par_expression SEMI | 'switch' par_expression '{' switch_block_statement_groups '}' | 'return' expression SEMI | 'break' ( IDENT )? SEMI | 'continue' ( IDENT )? SEMI | SEMI | statement_expression SEMI )
            int alt98=11;
            switch ( input.LA(1) ) {
            case 165:
                {
                alt98=1;
                }
                break;
            case 156:
                {
                alt98=2;
                }
                break;
            case 155:
                {
                alt98=3;
                }
                break;
            case 164:
                {
                alt98=4;
                }
                break;
            case 151:
                {
                alt98=5;
                }
                break;
            case 162:
                {
                alt98=6;
                }
                break;
            case 159:
                {
                alt98=7;
                }
                break;
            case 145:
                {
                alt98=8;
                }
                break;
            case 149:
                {
                alt98=9;
                }
                break;
            case SEMI:
                {
                alt98=10;
                }
                break;
            case BOOLEAN:
            case BYTE:
            case CHARACTER_LITERAL:
            case CONTAINS:
            case DOUBLE:
            case FALSE:
            case FLOATING_POINT_LITERAL:
            case HEX_LITERAL:
            case IDENT:
            case INTEGER:
            case LONG:
            case LPAR:
            case NULL:
            case OCTAL_LITERAL:
            case REAL:
            case STRING_LITERAL:
            case TRUE:
            case 114:
            case 123:
            case 124:
            case 126:
            case 127:
            case 134:
            case 135:
            case 136:
            case 137:
            case 138:
            case 139:
            case 140:
            case 170:
                {
                alt98=11;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 98, 0, input);

                throw nvae;

            }

            switch (alt98) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2955:9: block
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_block_in_java_statement9036);
                    block264=block();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, block264.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2956:9: 'if' par_expression java_statement ( else_statement )?
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal265=(Token)match(input,156,FOLLOW_156_in_java_statement9046); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal265_tree = 
                    (Object)adaptor.create(string_literal265)
                    ;
                    adaptor.addChild(root_0, string_literal265_tree);
                    }

                    pushFollow(FOLLOW_par_expression_in_java_statement9048);
                    par_expression266=par_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, par_expression266.getTree());

                    pushFollow(FOLLOW_java_statement_in_java_statement9050);
                    java_statement267=java_statement();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_statement267.getTree());

                    // com/senseidb/bql/parsers/BQL.g:2956:44: ( else_statement )?
                    int alt95=2;
                    switch ( input.LA(1) ) {
                        case ELSE:
                            {
                            int LA95_1 = input.LA(2);

                            if ( (((( "else".equals(input.LT(1).getText()) )&&( "else".equals(input.LT(1).getText()) ))&&synpred148_BQL())) ) {
                                alt95=1;
                            }
                            }
                            break;
                    }

                    switch (alt95) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2956:45: else_statement
                            {
                            pushFollow(FOLLOW_else_statement_in_java_statement9053);
                            else_statement268=else_statement();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, else_statement268.getTree());

                            }
                            break;

                    }


                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2957:9: 'for' LPAR for_control RPAR java_statement
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal269=(Token)match(input,155,FOLLOW_155_in_java_statement9065); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal269_tree = 
                    (Object)adaptor.create(string_literal269)
                    ;
                    adaptor.addChild(root_0, string_literal269_tree);
                    }

                    LPAR270=(Token)match(input,LPAR,FOLLOW_LPAR_in_java_statement9067); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    LPAR270_tree = 
                    (Object)adaptor.create(LPAR270)
                    ;
                    adaptor.addChild(root_0, LPAR270_tree);
                    }

                    if ( state.backtracking==0 ) {
                                _currentScope = new HashMap<String, String>();
                                _symbolTable.offerLast(_currentScope);
                            }

                    pushFollow(FOLLOW_for_control_in_java_statement9087);
                    for_control271=for_control();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, for_control271.getTree());

                    RPAR272=(Token)match(input,RPAR,FOLLOW_RPAR_in_java_statement9089); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    RPAR272_tree = 
                    (Object)adaptor.create(RPAR272)
                    ;
                    adaptor.addChild(root_0, RPAR272_tree);
                    }

                    pushFollow(FOLLOW_java_statement_in_java_statement9091);
                    java_statement273=java_statement();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_statement273.getTree());

                    if ( state.backtracking==0 ) {
                                _symbolTable.pollLast();
                                _currentScope = _symbolTable.peekLast();
                            }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:2967:9: 'while' par_expression java_statement
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal274=(Token)match(input,164,FOLLOW_164_in_java_statement9111); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal274_tree = 
                    (Object)adaptor.create(string_literal274)
                    ;
                    adaptor.addChild(root_0, string_literal274_tree);
                    }

                    pushFollow(FOLLOW_par_expression_in_java_statement9113);
                    par_expression275=par_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, par_expression275.getTree());

                    pushFollow(FOLLOW_java_statement_in_java_statement9115);
                    java_statement276=java_statement();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_statement276.getTree());

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:2968:9: 'do' java_statement 'while' par_expression SEMI
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal277=(Token)match(input,151,FOLLOW_151_in_java_statement9125); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal277_tree = 
                    (Object)adaptor.create(string_literal277)
                    ;
                    adaptor.addChild(root_0, string_literal277_tree);
                    }

                    pushFollow(FOLLOW_java_statement_in_java_statement9127);
                    java_statement278=java_statement();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_statement278.getTree());

                    string_literal279=(Token)match(input,164,FOLLOW_164_in_java_statement9129); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal279_tree = 
                    (Object)adaptor.create(string_literal279)
                    ;
                    adaptor.addChild(root_0, string_literal279_tree);
                    }

                    pushFollow(FOLLOW_par_expression_in_java_statement9131);
                    par_expression280=par_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, par_expression280.getTree());

                    SEMI281=(Token)match(input,SEMI,FOLLOW_SEMI_in_java_statement9133); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI281_tree = 
                    (Object)adaptor.create(SEMI281)
                    ;
                    adaptor.addChild(root_0, SEMI281_tree);
                    }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:2969:9: 'switch' par_expression '{' switch_block_statement_groups '}'
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal282=(Token)match(input,162,FOLLOW_162_in_java_statement9143); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal282_tree = 
                    (Object)adaptor.create(string_literal282)
                    ;
                    adaptor.addChild(root_0, string_literal282_tree);
                    }

                    pushFollow(FOLLOW_par_expression_in_java_statement9145);
                    par_expression283=par_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, par_expression283.getTree());

                    char_literal284=(Token)match(input,165,FOLLOW_165_in_java_statement9147); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal284_tree = 
                    (Object)adaptor.create(char_literal284)
                    ;
                    adaptor.addChild(root_0, char_literal284_tree);
                    }

                    pushFollow(FOLLOW_switch_block_statement_groups_in_java_statement9149);
                    switch_block_statement_groups285=switch_block_statement_groups();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, switch_block_statement_groups285.getTree());

                    char_literal286=(Token)match(input,169,FOLLOW_169_in_java_statement9151); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal286_tree = 
                    (Object)adaptor.create(char_literal286)
                    ;
                    adaptor.addChild(root_0, char_literal286_tree);
                    }

                    }
                    break;
                case 7 :
                    // com/senseidb/bql/parsers/BQL.g:2970:9: 'return' expression SEMI
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal287=(Token)match(input,159,FOLLOW_159_in_java_statement9161); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal287_tree = 
                    (Object)adaptor.create(string_literal287)
                    ;
                    adaptor.addChild(root_0, string_literal287_tree);
                    }

                    pushFollow(FOLLOW_expression_in_java_statement9163);
                    expression288=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression288.getTree());

                    SEMI289=(Token)match(input,SEMI,FOLLOW_SEMI_in_java_statement9165); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI289_tree = 
                    (Object)adaptor.create(SEMI289)
                    ;
                    adaptor.addChild(root_0, SEMI289_tree);
                    }

                    }
                    break;
                case 8 :
                    // com/senseidb/bql/parsers/BQL.g:2971:9: 'break' ( IDENT )? SEMI
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal290=(Token)match(input,145,FOLLOW_145_in_java_statement9175); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal290_tree = 
                    (Object)adaptor.create(string_literal290)
                    ;
                    adaptor.addChild(root_0, string_literal290_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:2971:17: ( IDENT )?
                    int alt96=2;
                    switch ( input.LA(1) ) {
                        case IDENT:
                            {
                            alt96=1;
                            }
                            break;
                    }

                    switch (alt96) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2971:17: IDENT
                            {
                            IDENT291=(Token)match(input,IDENT,FOLLOW_IDENT_in_java_statement9177); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            IDENT291_tree = 
                            (Object)adaptor.create(IDENT291)
                            ;
                            adaptor.addChild(root_0, IDENT291_tree);
                            }

                            }
                            break;

                    }


                    SEMI292=(Token)match(input,SEMI,FOLLOW_SEMI_in_java_statement9180); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI292_tree = 
                    (Object)adaptor.create(SEMI292)
                    ;
                    adaptor.addChild(root_0, SEMI292_tree);
                    }

                    }
                    break;
                case 9 :
                    // com/senseidb/bql/parsers/BQL.g:2972:9: 'continue' ( IDENT )? SEMI
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal293=(Token)match(input,149,FOLLOW_149_in_java_statement9190); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal293_tree = 
                    (Object)adaptor.create(string_literal293)
                    ;
                    adaptor.addChild(root_0, string_literal293_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:2972:20: ( IDENT )?
                    int alt97=2;
                    switch ( input.LA(1) ) {
                        case IDENT:
                            {
                            alt97=1;
                            }
                            break;
                    }

                    switch (alt97) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2972:20: IDENT
                            {
                            IDENT294=(Token)match(input,IDENT,FOLLOW_IDENT_in_java_statement9192); if (state.failed) return retval;
                            if ( state.backtracking==0 ) {
                            IDENT294_tree = 
                            (Object)adaptor.create(IDENT294)
                            ;
                            adaptor.addChild(root_0, IDENT294_tree);
                            }

                            }
                            break;

                    }


                    SEMI295=(Token)match(input,SEMI,FOLLOW_SEMI_in_java_statement9195); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI295_tree = 
                    (Object)adaptor.create(SEMI295)
                    ;
                    adaptor.addChild(root_0, SEMI295_tree);
                    }

                    }
                    break;
                case 10 :
                    // com/senseidb/bql/parsers/BQL.g:2973:9: SEMI
                    {
                    root_0 = (Object)adaptor.nil();


                    SEMI296=(Token)match(input,SEMI,FOLLOW_SEMI_in_java_statement9205); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI296_tree = 
                    (Object)adaptor.create(SEMI296)
                    ;
                    adaptor.addChild(root_0, SEMI296_tree);
                    }

                    }
                    break;
                case 11 :
                    // com/senseidb/bql/parsers/BQL.g:2974:9: statement_expression SEMI
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_statement_expression_in_java_statement9215);
                    statement_expression297=statement_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, statement_expression297.getTree());

                    SEMI298=(Token)match(input,SEMI,FOLLOW_SEMI_in_java_statement9217); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI298_tree = 
                    (Object)adaptor.create(SEMI298)
                    ;
                    adaptor.addChild(root_0, SEMI298_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 82, java_statement_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "java_statement"


    public static class else_statement_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "else_statement"
    // com/senseidb/bql/parsers/BQL.g:2977:1: else_statement :{...}? ELSE java_statement ;
    public final BQLParser.else_statement_return else_statement() throws RecognitionException {
        BQLParser.else_statement_return retval = new BQLParser.else_statement_return();
        retval.start = input.LT(1);

        int else_statement_StartIndex = input.index();

        Object root_0 = null;

        Token ELSE299=null;
        BQLParser.java_statement_return java_statement300 =null;


        Object ELSE299_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 83) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2978:5: ({...}? ELSE java_statement )
            // com/senseidb/bql/parsers/BQL.g:2978:9: {...}? ELSE java_statement
            {
            root_0 = (Object)adaptor.nil();


            if ( !(( "else".equals(input.LT(1).getText()) )) ) {
                if (state.backtracking>0) {state.failed=true; return retval;}
                throw new FailedPredicateException(input, "else_statement", " \"else\".equals(input.LT(1).getText()) ");
            }

            ELSE299=(Token)match(input,ELSE,FOLLOW_ELSE_in_else_statement9238); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            ELSE299_tree = 
            (Object)adaptor.create(ELSE299)
            ;
            adaptor.addChild(root_0, ELSE299_tree);
            }

            pushFollow(FOLLOW_java_statement_in_else_statement9240);
            java_statement300=java_statement();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, java_statement300.getTree());

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 83, else_statement_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "else_statement"


    public static class switch_block_statement_groups_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "switch_block_statement_groups"
    // com/senseidb/bql/parsers/BQL.g:2981:1: switch_block_statement_groups : ( switch_block_statement_group )* ;
    public final BQLParser.switch_block_statement_groups_return switch_block_statement_groups() throws RecognitionException {
        BQLParser.switch_block_statement_groups_return retval = new BQLParser.switch_block_statement_groups_return();
        retval.start = input.LT(1);

        int switch_block_statement_groups_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.switch_block_statement_group_return switch_block_statement_group301 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 84) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2982:5: ( ( switch_block_statement_group )* )
            // com/senseidb/bql/parsers/BQL.g:2982:9: ( switch_block_statement_group )*
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2982:9: ( switch_block_statement_group )*
            loop99:
            do {
                int alt99=2;
                switch ( input.LA(1) ) {
                case 146:
                case 150:
                    {
                    alt99=1;
                    }
                    break;

                }

                switch (alt99) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2982:10: switch_block_statement_group
            	    {
            	    pushFollow(FOLLOW_switch_block_statement_group_in_switch_block_statement_groups9260);
            	    switch_block_statement_group301=switch_block_statement_group();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, switch_block_statement_group301.getTree());

            	    }
            	    break;

            	default :
            	    break loop99;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 84, switch_block_statement_groups_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "switch_block_statement_groups"


    public static class switch_block_statement_group_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "switch_block_statement_group"
    // com/senseidb/bql/parsers/BQL.g:2985:1: switch_block_statement_group : ( switch_label )+ ( block_statement )* ;
    public final BQLParser.switch_block_statement_group_return switch_block_statement_group() throws RecognitionException {
        BQLParser.switch_block_statement_group_return retval = new BQLParser.switch_block_statement_group_return();
        retval.start = input.LT(1);

        int switch_block_statement_group_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.switch_label_return switch_label302 =null;

        BQLParser.block_statement_return block_statement303 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 85) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2986:5: ( ( switch_label )+ ( block_statement )* )
            // com/senseidb/bql/parsers/BQL.g:2986:9: ( switch_label )+ ( block_statement )*
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:2986:9: ( switch_label )+
            int cnt100=0;
            loop100:
            do {
                int alt100=2;
                switch ( input.LA(1) ) {
                case 146:
                    {
                    int LA100_2 = input.LA(2);

                    if ( (synpred161_BQL()) ) {
                        alt100=1;
                    }


                    }
                    break;
                case 150:
                    {
                    int LA100_3 = input.LA(2);

                    if ( (synpred161_BQL()) ) {
                        alt100=1;
                    }


                    }
                    break;

                }

                switch (alt100) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2986:9: switch_label
            	    {
            	    pushFollow(FOLLOW_switch_label_in_switch_block_statement_group9281);
            	    switch_label302=switch_label();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, switch_label302.getTree());

            	    }
            	    break;

            	default :
            	    if ( cnt100 >= 1 ) break loop100;
            	    if (state.backtracking>0) {state.failed=true; return retval;}
                        EarlyExitException eee =
                            new EarlyExitException(100, input);
                        throw eee;
                }
                cnt100++;
            } while (true);


            // com/senseidb/bql/parsers/BQL.g:2986:23: ( block_statement )*
            loop101:
            do {
                int alt101=2;
                switch ( input.LA(1) ) {
                case BOOLEAN:
                case BYTE:
                case CHARACTER_LITERAL:
                case CONTAINS:
                case DOUBLE:
                case FALSE:
                case FAST_UTIL_DATA_TYPE:
                case FLOATING_POINT_LITERAL:
                case HEX_LITERAL:
                case IDENT:
                case INT:
                case INTEGER:
                case LONG:
                case LPAR:
                case NULL:
                case OCTAL_LITERAL:
                case REAL:
                case SEMI:
                case STRING_LITERAL:
                case TRUE:
                case 114:
                case 123:
                case 124:
                case 126:
                case 127:
                case 134:
                case 135:
                case 136:
                case 137:
                case 138:
                case 139:
                case 140:
                case 145:
                case 147:
                case 149:
                case 151:
                case 153:
                case 154:
                case 155:
                case 156:
                case 159:
                case 160:
                case 162:
                case 164:
                case 165:
                case 170:
                    {
                    alt101=1;
                    }
                    break;

                }

                switch (alt101) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:2986:23: block_statement
            	    {
            	    pushFollow(FOLLOW_block_statement_in_switch_block_statement_group9284);
            	    block_statement303=block_statement();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, block_statement303.getTree());

            	    }
            	    break;

            	default :
            	    break loop101;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 85, switch_block_statement_group_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "switch_block_statement_group"


    public static class switch_label_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "switch_label"
    // com/senseidb/bql/parsers/BQL.g:2989:1: switch_label : ( 'case' constant_expression COLON | 'case' enum_constant_name COLON | 'default' COLON );
    public final BQLParser.switch_label_return switch_label() throws RecognitionException {
        BQLParser.switch_label_return retval = new BQLParser.switch_label_return();
        retval.start = input.LT(1);

        int switch_label_StartIndex = input.index();

        Object root_0 = null;

        Token string_literal304=null;
        Token COLON306=null;
        Token string_literal307=null;
        Token COLON309=null;
        Token string_literal310=null;
        Token COLON311=null;
        BQLParser.constant_expression_return constant_expression305 =null;

        BQLParser.enum_constant_name_return enum_constant_name308 =null;


        Object string_literal304_tree=null;
        Object COLON306_tree=null;
        Object string_literal307_tree=null;
        Object COLON309_tree=null;
        Object string_literal310_tree=null;
        Object COLON311_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 86) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2990:5: ( 'case' constant_expression COLON | 'case' enum_constant_name COLON | 'default' COLON )
            int alt102=3;
            switch ( input.LA(1) ) {
            case 146:
                {
                switch ( input.LA(2) ) {
                case BOOLEAN:
                case BYTE:
                case CHARACTER_LITERAL:
                case CONTAINS:
                case DOUBLE:
                case FALSE:
                case FLOATING_POINT_LITERAL:
                case HEX_LITERAL:
                case INTEGER:
                case LONG:
                case LPAR:
                case NULL:
                case OCTAL_LITERAL:
                case REAL:
                case STRING_LITERAL:
                case TRUE:
                case 114:
                case 123:
                case 124:
                case 126:
                case 127:
                case 134:
                case 135:
                case 136:
                case 137:
                case 138:
                case 139:
                case 140:
                case 170:
                    {
                    alt102=1;
                    }
                    break;
                case IDENT:
                    {
                    switch ( input.LA(3) ) {
                    case EQUAL:
                    case GT:
                    case LPAR:
                    case LT:
                    case 115:
                    case 116:
                    case 117:
                    case 118:
                    case 119:
                    case 120:
                    case 121:
                    case 122:
                    case 123:
                    case 124:
                    case 125:
                    case 126:
                    case 127:
                    case 128:
                    case 129:
                    case 130:
                    case 131:
                    case 132:
                    case 133:
                    case 141:
                    case 143:
                    case 144:
                    case 157:
                    case 166:
                    case 167:
                    case 168:
                        {
                        alt102=1;
                        }
                        break;
                    case COLON:
                        {
                        int LA102_5 = input.LA(4);

                        if ( (synpred163_BQL()) ) {
                            alt102=1;
                        }
                        else if ( (synpred164_BQL()) ) {
                            alt102=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 102, 5, input);

                            throw nvae;

                        }
                        }
                        break;
                    default:
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 102, 4, input);

                        throw nvae;

                    }

                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 102, 1, input);

                    throw nvae;

                }

                }
                break;
            case 150:
                {
                alt102=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 102, 0, input);

                throw nvae;

            }

            switch (alt102) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2990:9: 'case' constant_expression COLON
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal304=(Token)match(input,146,FOLLOW_146_in_switch_label9304); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal304_tree = 
                    (Object)adaptor.create(string_literal304)
                    ;
                    adaptor.addChild(root_0, string_literal304_tree);
                    }

                    pushFollow(FOLLOW_constant_expression_in_switch_label9306);
                    constant_expression305=constant_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, constant_expression305.getTree());

                    COLON306=(Token)match(input,COLON,FOLLOW_COLON_in_switch_label9308); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    COLON306_tree = 
                    (Object)adaptor.create(COLON306)
                    ;
                    adaptor.addChild(root_0, COLON306_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2991:9: 'case' enum_constant_name COLON
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal307=(Token)match(input,146,FOLLOW_146_in_switch_label9318); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal307_tree = 
                    (Object)adaptor.create(string_literal307)
                    ;
                    adaptor.addChild(root_0, string_literal307_tree);
                    }

                    pushFollow(FOLLOW_enum_constant_name_in_switch_label9320);
                    enum_constant_name308=enum_constant_name();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, enum_constant_name308.getTree());

                    COLON309=(Token)match(input,COLON,FOLLOW_COLON_in_switch_label9322); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    COLON309_tree = 
                    (Object)adaptor.create(COLON309)
                    ;
                    adaptor.addChild(root_0, COLON309_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:2992:9: 'default' COLON
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal310=(Token)match(input,150,FOLLOW_150_in_switch_label9332); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal310_tree = 
                    (Object)adaptor.create(string_literal310)
                    ;
                    adaptor.addChild(root_0, string_literal310_tree);
                    }

                    COLON311=(Token)match(input,COLON,FOLLOW_COLON_in_switch_label9334); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    COLON311_tree = 
                    (Object)adaptor.create(COLON311)
                    ;
                    adaptor.addChild(root_0, COLON311_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 86, switch_label_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "switch_label"


    public static class for_control_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "for_control"
    // com/senseidb/bql/parsers/BQL.g:2995:1: for_control options {k=3; } : ( enhanced_for_control | ( for_init )? SEMI ( expression )? SEMI ( for_update )? );
    public final BQLParser.for_control_return for_control() throws RecognitionException {
        BQLParser.for_control_return retval = new BQLParser.for_control_return();
        retval.start = input.LT(1);

        int for_control_StartIndex = input.index();

        Object root_0 = null;

        Token SEMI314=null;
        Token SEMI316=null;
        BQLParser.enhanced_for_control_return enhanced_for_control312 =null;

        BQLParser.for_init_return for_init313 =null;

        BQLParser.expression_return expression315 =null;

        BQLParser.for_update_return for_update317 =null;


        Object SEMI314_tree=null;
        Object SEMI316_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 87) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:2997:5: ( enhanced_for_control | ( for_init )? SEMI ( expression )? SEMI ( for_update )? )
            int alt106=2;
            alt106 = dfa106.predict(input);
            switch (alt106) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:2997:9: enhanced_for_control
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_enhanced_for_control_in_for_control9360);
                    enhanced_for_control312=enhanced_for_control();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, enhanced_for_control312.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:2998:9: ( for_init )? SEMI ( expression )? SEMI ( for_update )?
                    {
                    root_0 = (Object)adaptor.nil();


                    // com/senseidb/bql/parsers/BQL.g:2998:9: ( for_init )?
                    int alt103=2;
                    switch ( input.LA(1) ) {
                        case BOOLEAN:
                        case BYTE:
                        case CHARACTER_LITERAL:
                        case CONTAINS:
                        case DOUBLE:
                        case FALSE:
                        case FAST_UTIL_DATA_TYPE:
                        case FLOATING_POINT_LITERAL:
                        case HEX_LITERAL:
                        case IDENT:
                        case INT:
                        case INTEGER:
                        case LONG:
                        case LPAR:
                        case NULL:
                        case OCTAL_LITERAL:
                        case REAL:
                        case STRING_LITERAL:
                        case TRUE:
                        case 114:
                        case 123:
                        case 124:
                        case 126:
                        case 127:
                        case 134:
                        case 135:
                        case 136:
                        case 137:
                        case 138:
                        case 139:
                        case 140:
                        case 147:
                        case 153:
                        case 154:
                        case 160:
                        case 170:
                            {
                            alt103=1;
                            }
                            break;
                    }

                    switch (alt103) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2998:9: for_init
                            {
                            pushFollow(FOLLOW_for_init_in_for_control9370);
                            for_init313=for_init();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, for_init313.getTree());

                            }
                            break;

                    }


                    SEMI314=(Token)match(input,SEMI,FOLLOW_SEMI_in_for_control9373); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI314_tree = 
                    (Object)adaptor.create(SEMI314)
                    ;
                    adaptor.addChild(root_0, SEMI314_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:2998:24: ( expression )?
                    int alt104=2;
                    switch ( input.LA(1) ) {
                        case BOOLEAN:
                        case BYTE:
                        case CHARACTER_LITERAL:
                        case CONTAINS:
                        case DOUBLE:
                        case FALSE:
                        case FLOATING_POINT_LITERAL:
                        case HEX_LITERAL:
                        case IDENT:
                        case INTEGER:
                        case LONG:
                        case LPAR:
                        case NULL:
                        case OCTAL_LITERAL:
                        case REAL:
                        case STRING_LITERAL:
                        case TRUE:
                        case 114:
                        case 123:
                        case 124:
                        case 126:
                        case 127:
                        case 134:
                        case 135:
                        case 136:
                        case 137:
                        case 138:
                        case 139:
                        case 140:
                        case 170:
                            {
                            alt104=1;
                            }
                            break;
                    }

                    switch (alt104) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2998:24: expression
                            {
                            pushFollow(FOLLOW_expression_in_for_control9375);
                            expression315=expression();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression315.getTree());

                            }
                            break;

                    }


                    SEMI316=(Token)match(input,SEMI,FOLLOW_SEMI_in_for_control9378); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    SEMI316_tree = 
                    (Object)adaptor.create(SEMI316)
                    ;
                    adaptor.addChild(root_0, SEMI316_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:2998:41: ( for_update )?
                    int alt105=2;
                    switch ( input.LA(1) ) {
                        case BOOLEAN:
                        case BYTE:
                        case CHARACTER_LITERAL:
                        case CONTAINS:
                        case DOUBLE:
                        case FALSE:
                        case FLOATING_POINT_LITERAL:
                        case HEX_LITERAL:
                        case IDENT:
                        case INTEGER:
                        case LONG:
                        case LPAR:
                        case NULL:
                        case OCTAL_LITERAL:
                        case REAL:
                        case STRING_LITERAL:
                        case TRUE:
                        case 114:
                        case 123:
                        case 124:
                        case 126:
                        case 127:
                        case 134:
                        case 135:
                        case 136:
                        case 137:
                        case 138:
                        case 139:
                        case 140:
                        case 170:
                            {
                            alt105=1;
                            }
                            break;
                    }

                    switch (alt105) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:2998:41: for_update
                            {
                            pushFollow(FOLLOW_for_update_in_for_control9380);
                            for_update317=for_update();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, for_update317.getTree());

                            }
                            break;

                    }


                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 87, for_control_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "for_control"


    public static class for_init_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "for_init"
    // com/senseidb/bql/parsers/BQL.g:3001:1: for_init : ( local_variable_declaration | expression_list );
    public final BQLParser.for_init_return for_init() throws RecognitionException {
        BQLParser.for_init_return retval = new BQLParser.for_init_return();
        retval.start = input.LT(1);

        int for_init_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.local_variable_declaration_return local_variable_declaration318 =null;

        BQLParser.expression_list_return expression_list319 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 88) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3002:5: ( local_variable_declaration | expression_list )
            int alt107=2;
            alt107 = dfa107.predict(input);
            switch (alt107) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3002:9: local_variable_declaration
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_local_variable_declaration_in_for_init9400);
                    local_variable_declaration318=local_variable_declaration();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, local_variable_declaration318.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3003:9: expression_list
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_expression_list_in_for_init9410);
                    expression_list319=expression_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression_list319.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 88, for_init_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "for_init"


    public static class enhanced_for_control_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "enhanced_for_control"
    // com/senseidb/bql/parsers/BQL.g:3006:1: enhanced_for_control : variable_modifiers type IDENT COLON expression ;
    public final BQLParser.enhanced_for_control_return enhanced_for_control() throws RecognitionException {
        BQLParser.enhanced_for_control_return retval = new BQLParser.enhanced_for_control_return();
        retval.start = input.LT(1);

        int enhanced_for_control_StartIndex = input.index();

        Object root_0 = null;

        Token IDENT322=null;
        Token COLON323=null;
        BQLParser.variable_modifiers_return variable_modifiers320 =null;

        BQLParser.type_return type321 =null;

        BQLParser.expression_return expression324 =null;


        Object IDENT322_tree=null;
        Object COLON323_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 89) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3007:5: ( variable_modifiers type IDENT COLON expression )
            // com/senseidb/bql/parsers/BQL.g:3007:9: variable_modifiers type IDENT COLON expression
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_variable_modifiers_in_enhanced_for_control9429);
            variable_modifiers320=variable_modifiers();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, variable_modifiers320.getTree());

            pushFollow(FOLLOW_type_in_enhanced_for_control9431);
            type321=type();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, type321.getTree());

            IDENT322=(Token)match(input,IDENT,FOLLOW_IDENT_in_enhanced_for_control9433); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            IDENT322_tree = 
            (Object)adaptor.create(IDENT322)
            ;
            adaptor.addChild(root_0, IDENT322_tree);
            }

            COLON323=(Token)match(input,COLON,FOLLOW_COLON_in_enhanced_for_control9435); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            COLON323_tree = 
            (Object)adaptor.create(COLON323)
            ;
            adaptor.addChild(root_0, COLON323_tree);
            }

            pushFollow(FOLLOW_expression_in_enhanced_for_control9437);
            expression324=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression324.getTree());

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 89, enhanced_for_control_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "enhanced_for_control"


    public static class for_update_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "for_update"
    // com/senseidb/bql/parsers/BQL.g:3010:1: for_update : expression_list ;
    public final BQLParser.for_update_return for_update() throws RecognitionException {
        BQLParser.for_update_return retval = new BQLParser.for_update_return();
        retval.start = input.LT(1);

        int for_update_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.expression_list_return expression_list325 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 90) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3011:5: ( expression_list )
            // com/senseidb/bql/parsers/BQL.g:3011:9: expression_list
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_expression_list_in_for_update9456);
            expression_list325=expression_list();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression_list325.getTree());

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 90, for_update_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "for_update"


    public static class par_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "par_expression"
    // com/senseidb/bql/parsers/BQL.g:3014:1: par_expression : LPAR expression RPAR ;
    public final BQLParser.par_expression_return par_expression() throws RecognitionException {
        BQLParser.par_expression_return retval = new BQLParser.par_expression_return();
        retval.start = input.LT(1);

        int par_expression_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR326=null;
        Token RPAR328=null;
        BQLParser.expression_return expression327 =null;


        Object LPAR326_tree=null;
        Object RPAR328_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 91) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3015:5: ( LPAR expression RPAR )
            // com/senseidb/bql/parsers/BQL.g:3015:9: LPAR expression RPAR
            {
            root_0 = (Object)adaptor.nil();


            LPAR326=(Token)match(input,LPAR,FOLLOW_LPAR_in_par_expression9475); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR326_tree = 
            (Object)adaptor.create(LPAR326)
            ;
            adaptor.addChild(root_0, LPAR326_tree);
            }

            pushFollow(FOLLOW_expression_in_par_expression9477);
            expression327=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression327.getTree());

            RPAR328=(Token)match(input,RPAR,FOLLOW_RPAR_in_par_expression9479); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR328_tree = 
            (Object)adaptor.create(RPAR328)
            ;
            adaptor.addChild(root_0, RPAR328_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 91, par_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "par_expression"


    public static class expression_list_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "expression_list"
    // com/senseidb/bql/parsers/BQL.g:3018:1: expression_list : expression ( ',' expression )* ;
    public final BQLParser.expression_list_return expression_list() throws RecognitionException {
        BQLParser.expression_list_return retval = new BQLParser.expression_list_return();
        retval.start = input.LT(1);

        int expression_list_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal330=null;
        BQLParser.expression_return expression329 =null;

        BQLParser.expression_return expression331 =null;


        Object char_literal330_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 92) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3019:5: ( expression ( ',' expression )* )
            // com/senseidb/bql/parsers/BQL.g:3019:9: expression ( ',' expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_expression_in_expression_list9498);
            expression329=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression329.getTree());

            // com/senseidb/bql/parsers/BQL.g:3019:20: ( ',' expression )*
            loop108:
            do {
                int alt108=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt108=1;
                    }
                    break;

                }

                switch (alt108) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3019:21: ',' expression
            	    {
            	    char_literal330=(Token)match(input,COMMA,FOLLOW_COMMA_in_expression_list9501); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal330_tree = 
            	    (Object)adaptor.create(char_literal330)
            	    ;
            	    adaptor.addChild(root_0, char_literal330_tree);
            	    }

            	    pushFollow(FOLLOW_expression_in_expression_list9503);
            	    expression331=expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression331.getTree());

            	    }
            	    break;

            	default :
            	    break loop108;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 92, expression_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "expression_list"


    public static class statement_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "statement_expression"
    // com/senseidb/bql/parsers/BQL.g:3022:1: statement_expression : expression ;
    public final BQLParser.statement_expression_return statement_expression() throws RecognitionException {
        BQLParser.statement_expression_return retval = new BQLParser.statement_expression_return();
        retval.start = input.LT(1);

        int statement_expression_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.expression_return expression332 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 93) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3023:5: ( expression )
            // com/senseidb/bql/parsers/BQL.g:3023:9: expression
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_expression_in_statement_expression9524);
            expression332=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression332.getTree());

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 93, statement_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "statement_expression"


    public static class constant_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "constant_expression"
    // com/senseidb/bql/parsers/BQL.g:3026:1: constant_expression : expression ;
    public final BQLParser.constant_expression_return constant_expression() throws RecognitionException {
        BQLParser.constant_expression_return retval = new BQLParser.constant_expression_return();
        retval.start = input.LT(1);

        int constant_expression_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.expression_return expression333 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 94) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3027:5: ( expression )
            // com/senseidb/bql/parsers/BQL.g:3027:9: expression
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_expression_in_constant_expression9543);
            expression333=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression333.getTree());

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 94, constant_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "constant_expression"


    public static class enum_constant_name_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "enum_constant_name"
    // com/senseidb/bql/parsers/BQL.g:3030:1: enum_constant_name : IDENT ;
    public final BQLParser.enum_constant_name_return enum_constant_name() throws RecognitionException {
        BQLParser.enum_constant_name_return retval = new BQLParser.enum_constant_name_return();
        retval.start = input.LT(1);

        int enum_constant_name_StartIndex = input.index();

        Object root_0 = null;

        Token IDENT334=null;

        Object IDENT334_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 95) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3031:5: ( IDENT )
            // com/senseidb/bql/parsers/BQL.g:3031:9: IDENT
            {
            root_0 = (Object)adaptor.nil();


            IDENT334=(Token)match(input,IDENT,FOLLOW_IDENT_in_enum_constant_name9562); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            IDENT334_tree = 
            (Object)adaptor.create(IDENT334)
            ;
            adaptor.addChild(root_0, IDENT334_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 95, enum_constant_name_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "enum_constant_name"


    public static class expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "expression"
    // com/senseidb/bql/parsers/BQL.g:3034:1: expression : conditional_expression ( assignment_operator expression )? ;
    public final BQLParser.expression_return expression() throws RecognitionException {
        BQLParser.expression_return retval = new BQLParser.expression_return();
        retval.start = input.LT(1);

        int expression_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.conditional_expression_return conditional_expression335 =null;

        BQLParser.assignment_operator_return assignment_operator336 =null;

        BQLParser.expression_return expression337 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 96) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3035:5: ( conditional_expression ( assignment_operator expression )? )
            // com/senseidb/bql/parsers/BQL.g:3035:9: conditional_expression ( assignment_operator expression )?
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_conditional_expression_in_expression9581);
            conditional_expression335=conditional_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, conditional_expression335.getTree());

            // com/senseidb/bql/parsers/BQL.g:3035:32: ( assignment_operator expression )?
            int alt109=2;
            switch ( input.LA(1) ) {
                case EQUAL:
                    {
                    int LA109_1 = input.LA(2);

                    if ( (synpred171_BQL()) ) {
                        alt109=1;
                    }
                    }
                    break;
                case 125:
                    {
                    int LA109_2 = input.LA(2);

                    if ( (synpred171_BQL()) ) {
                        alt109=1;
                    }
                    }
                    break;
                case 128:
                    {
                    int LA109_3 = input.LA(2);

                    if ( (synpred171_BQL()) ) {
                        alt109=1;
                    }
                    }
                    break;
                case 122:
                    {
                    int LA109_4 = input.LA(2);

                    if ( (synpred171_BQL()) ) {
                        alt109=1;
                    }
                    }
                    break;
                case 131:
                    {
                    int LA109_5 = input.LA(2);

                    if ( (synpred171_BQL()) ) {
                        alt109=1;
                    }
                    }
                    break;
                case 120:
                    {
                    int LA109_6 = input.LA(2);

                    if ( (synpred171_BQL()) ) {
                        alt109=1;
                    }
                    }
                    break;
                case 167:
                    {
                    int LA109_7 = input.LA(2);

                    if ( (synpred171_BQL()) ) {
                        alt109=1;
                    }
                    }
                    break;
                case 144:
                    {
                    int LA109_8 = input.LA(2);

                    if ( (synpred171_BQL()) ) {
                        alt109=1;
                    }
                    }
                    break;
                case 117:
                    {
                    int LA109_9 = input.LA(2);

                    if ( (synpred171_BQL()) ) {
                        alt109=1;
                    }
                    }
                    break;
                case LT:
                    {
                    int LA109_10 = input.LA(2);

                    if ( (synpred171_BQL()) ) {
                        alt109=1;
                    }
                    }
                    break;
                case GT:
                    {
                    int LA109_11 = input.LA(2);

                    if ( (synpred171_BQL()) ) {
                        alt109=1;
                    }
                    }
                    break;
            }

            switch (alt109) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3035:33: assignment_operator expression
                    {
                    pushFollow(FOLLOW_assignment_operator_in_expression9584);
                    assignment_operator336=assignment_operator();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, assignment_operator336.getTree());

                    pushFollow(FOLLOW_expression_in_expression9586);
                    expression337=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression337.getTree());

                    }
                    break;

            }


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 96, expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "expression"


    public static class assignment_operator_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "assignment_operator"
    // com/senseidb/bql/parsers/BQL.g:3038:1: assignment_operator : ( '=' | '+=' | '-=' | '*=' | '/=' | '&=' | '|=' | '^=' | '%=' | ( '<' '<' '=' )=>t1= '<' t2= '<' t3= '=' {...}?| ( '>' '>' '>' '=' )=>t1= '>' t2= '>' t3= '>' t4= '=' {...}?| ( '>' '>' '=' )=>t1= '>' t2= '>' t3= '=' {...}?);
    public final BQLParser.assignment_operator_return assignment_operator() throws RecognitionException {
        BQLParser.assignment_operator_return retval = new BQLParser.assignment_operator_return();
        retval.start = input.LT(1);

        int assignment_operator_StartIndex = input.index();

        Object root_0 = null;

        Token t1=null;
        Token t2=null;
        Token t3=null;
        Token t4=null;
        Token char_literal338=null;
        Token string_literal339=null;
        Token string_literal340=null;
        Token string_literal341=null;
        Token string_literal342=null;
        Token string_literal343=null;
        Token string_literal344=null;
        Token string_literal345=null;
        Token string_literal346=null;

        Object t1_tree=null;
        Object t2_tree=null;
        Object t3_tree=null;
        Object t4_tree=null;
        Object char_literal338_tree=null;
        Object string_literal339_tree=null;
        Object string_literal340_tree=null;
        Object string_literal341_tree=null;
        Object string_literal342_tree=null;
        Object string_literal343_tree=null;
        Object string_literal344_tree=null;
        Object string_literal345_tree=null;
        Object string_literal346_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 97) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3039:5: ( '=' | '+=' | '-=' | '*=' | '/=' | '&=' | '|=' | '^=' | '%=' | ( '<' '<' '=' )=>t1= '<' t2= '<' t3= '=' {...}?| ( '>' '>' '>' '=' )=>t1= '>' t2= '>' t3= '>' t4= '=' {...}?| ( '>' '>' '=' )=>t1= '>' t2= '>' t3= '=' {...}?)
            int alt110=12;
            int LA110_0 = input.LA(1);

            if ( (LA110_0==EQUAL) ) {
                alt110=1;
            }
            else if ( (LA110_0==125) ) {
                alt110=2;
            }
            else if ( (LA110_0==128) ) {
                alt110=3;
            }
            else if ( (LA110_0==122) ) {
                alt110=4;
            }
            else if ( (LA110_0==131) ) {
                alt110=5;
            }
            else if ( (LA110_0==120) ) {
                alt110=6;
            }
            else if ( (LA110_0==167) ) {
                alt110=7;
            }
            else if ( (LA110_0==144) ) {
                alt110=8;
            }
            else if ( (LA110_0==117) ) {
                alt110=9;
            }
            else if ( (LA110_0==LT) && (synpred181_BQL())) {
                alt110=10;
            }
            else if ( (LA110_0==GT) ) {
                switch ( input.LA(2) ) {
                case GT:
                    {
                    int LA110_12 = input.LA(3);

                    if ( (LA110_12==GT) && (synpred182_BQL())) {
                        alt110=11;
                    }
                    else if ( (LA110_12==EQUAL) && (synpred183_BQL())) {
                        alt110=12;
                    }
                    else {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 110, 12, input);

                        throw nvae;

                    }
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 110, 11, input);

                    throw nvae;

                }

            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 110, 0, input);

                throw nvae;

            }
            switch (alt110) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3039:9: '='
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal338=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_assignment_operator9607); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal338_tree = 
                    (Object)adaptor.create(char_literal338)
                    ;
                    adaptor.addChild(root_0, char_literal338_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3040:9: '+='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal339=(Token)match(input,125,FOLLOW_125_in_assignment_operator9617); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal339_tree = 
                    (Object)adaptor.create(string_literal339)
                    ;
                    adaptor.addChild(root_0, string_literal339_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3041:9: '-='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal340=(Token)match(input,128,FOLLOW_128_in_assignment_operator9627); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal340_tree = 
                    (Object)adaptor.create(string_literal340)
                    ;
                    adaptor.addChild(root_0, string_literal340_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:3042:9: '*='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal341=(Token)match(input,122,FOLLOW_122_in_assignment_operator9637); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal341_tree = 
                    (Object)adaptor.create(string_literal341)
                    ;
                    adaptor.addChild(root_0, string_literal341_tree);
                    }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:3043:9: '/='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal342=(Token)match(input,131,FOLLOW_131_in_assignment_operator9647); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal342_tree = 
                    (Object)adaptor.create(string_literal342)
                    ;
                    adaptor.addChild(root_0, string_literal342_tree);
                    }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:3044:9: '&='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal343=(Token)match(input,120,FOLLOW_120_in_assignment_operator9657); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal343_tree = 
                    (Object)adaptor.create(string_literal343)
                    ;
                    adaptor.addChild(root_0, string_literal343_tree);
                    }

                    }
                    break;
                case 7 :
                    // com/senseidb/bql/parsers/BQL.g:3045:9: '|='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal344=(Token)match(input,167,FOLLOW_167_in_assignment_operator9667); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal344_tree = 
                    (Object)adaptor.create(string_literal344)
                    ;
                    adaptor.addChild(root_0, string_literal344_tree);
                    }

                    }
                    break;
                case 8 :
                    // com/senseidb/bql/parsers/BQL.g:3046:9: '^='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal345=(Token)match(input,144,FOLLOW_144_in_assignment_operator9677); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal345_tree = 
                    (Object)adaptor.create(string_literal345)
                    ;
                    adaptor.addChild(root_0, string_literal345_tree);
                    }

                    }
                    break;
                case 9 :
                    // com/senseidb/bql/parsers/BQL.g:3047:9: '%='
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal346=(Token)match(input,117,FOLLOW_117_in_assignment_operator9687); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal346_tree = 
                    (Object)adaptor.create(string_literal346)
                    ;
                    adaptor.addChild(root_0, string_literal346_tree);
                    }

                    }
                    break;
                case 10 :
                    // com/senseidb/bql/parsers/BQL.g:3048:9: ( '<' '<' '=' )=>t1= '<' t2= '<' t3= '=' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,LT,FOLLOW_LT_in_assignment_operator9708); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,LT,FOLLOW_LT_in_assignment_operator9712); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    t3=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_assignment_operator9716); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t3_tree = 
                    (Object)adaptor.create(t3)
                    ;
                    adaptor.addChild(root_0, t3_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() &&
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() && 
                              t2.getLine() == t3.getLine() && 
                              t2.getCharPositionInLine() + 1 == t3.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "assignment_operator", " $t1.getLine() == $t2.getLine() &&\n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() && \n          $t2.getLine() == $t3.getLine() && \n          $t2.getCharPositionInLine() + 1 == $t3.getCharPositionInLine() ");
                    }

                    }
                    break;
                case 11 :
                    // com/senseidb/bql/parsers/BQL.g:3053:9: ( '>' '>' '>' '=' )=>t1= '>' t2= '>' t3= '>' t4= '=' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,GT,FOLLOW_GT_in_assignment_operator9750); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,GT,FOLLOW_GT_in_assignment_operator9754); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    t3=(Token)match(input,GT,FOLLOW_GT_in_assignment_operator9758); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t3_tree = 
                    (Object)adaptor.create(t3)
                    ;
                    adaptor.addChild(root_0, t3_tree);
                    }

                    t4=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_assignment_operator9762); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t4_tree = 
                    (Object)adaptor.create(t4)
                    ;
                    adaptor.addChild(root_0, t4_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() &&
                              t2.getLine() == t3.getLine() && 
                              t2.getCharPositionInLine() + 1 == t3.getCharPositionInLine() &&
                              t3.getLine() == t4.getLine() && 
                              t3.getCharPositionInLine() + 1 == t4.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "assignment_operator", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() &&\n          $t2.getLine() == $t3.getLine() && \n          $t2.getCharPositionInLine() + 1 == $t3.getCharPositionInLine() &&\n          $t3.getLine() == $t4.getLine() && \n          $t3.getCharPositionInLine() + 1 == $t4.getCharPositionInLine() ");
                    }

                    }
                    break;
                case 12 :
                    // com/senseidb/bql/parsers/BQL.g:3060:9: ( '>' '>' '=' )=>t1= '>' t2= '>' t3= '=' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,GT,FOLLOW_GT_in_assignment_operator9793); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,GT,FOLLOW_GT_in_assignment_operator9797); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    t3=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_assignment_operator9801); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t3_tree = 
                    (Object)adaptor.create(t3)
                    ;
                    adaptor.addChild(root_0, t3_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() && 
                              t2.getLine() == t3.getLine() && 
                              t2.getCharPositionInLine() + 1 == t3.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "assignment_operator", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() && \n          $t2.getLine() == $t3.getLine() && \n          $t2.getCharPositionInLine() + 1 == $t3.getCharPositionInLine() ");
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 97, assignment_operator_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "assignment_operator"


    public static class conditional_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "conditional_expression"
    // com/senseidb/bql/parsers/BQL.g:3067:1: conditional_expression : conditional_or_expression ( '?' expression ':' expression )? ;
    public final BQLParser.conditional_expression_return conditional_expression() throws RecognitionException {
        BQLParser.conditional_expression_return retval = new BQLParser.conditional_expression_return();
        retval.start = input.LT(1);

        int conditional_expression_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal348=null;
        Token char_literal350=null;
        BQLParser.conditional_or_expression_return conditional_or_expression347 =null;

        BQLParser.expression_return expression349 =null;

        BQLParser.expression_return expression351 =null;


        Object char_literal348_tree=null;
        Object char_literal350_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 98) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3068:5: ( conditional_or_expression ( '?' expression ':' expression )? )
            // com/senseidb/bql/parsers/BQL.g:3068:9: conditional_or_expression ( '?' expression ':' expression )?
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_conditional_or_expression_in_conditional_expression9830);
            conditional_or_expression347=conditional_or_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, conditional_or_expression347.getTree());

            // com/senseidb/bql/parsers/BQL.g:3068:35: ( '?' expression ':' expression )?
            int alt111=2;
            switch ( input.LA(1) ) {
                case 133:
                    {
                    alt111=1;
                    }
                    break;
            }

            switch (alt111) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3068:37: '?' expression ':' expression
                    {
                    char_literal348=(Token)match(input,133,FOLLOW_133_in_conditional_expression9834); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal348_tree = 
                    (Object)adaptor.create(char_literal348)
                    ;
                    adaptor.addChild(root_0, char_literal348_tree);
                    }

                    pushFollow(FOLLOW_expression_in_conditional_expression9836);
                    expression349=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression349.getTree());

                    char_literal350=(Token)match(input,COLON,FOLLOW_COLON_in_conditional_expression9838); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal350_tree = 
                    (Object)adaptor.create(char_literal350)
                    ;
                    adaptor.addChild(root_0, char_literal350_tree);
                    }

                    pushFollow(FOLLOW_expression_in_conditional_expression9840);
                    expression351=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression351.getTree());

                    }
                    break;

            }


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 98, conditional_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "conditional_expression"


    public static class conditional_or_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "conditional_or_expression"
    // com/senseidb/bql/parsers/BQL.g:3071:1: conditional_or_expression : conditional_and_expression ( '||' conditional_and_expression )* ;
    public final BQLParser.conditional_or_expression_return conditional_or_expression() throws RecognitionException {
        BQLParser.conditional_or_expression_return retval = new BQLParser.conditional_or_expression_return();
        retval.start = input.LT(1);

        int conditional_or_expression_StartIndex = input.index();

        Object root_0 = null;

        Token string_literal353=null;
        BQLParser.conditional_and_expression_return conditional_and_expression352 =null;

        BQLParser.conditional_and_expression_return conditional_and_expression354 =null;


        Object string_literal353_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 99) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3072:5: ( conditional_and_expression ( '||' conditional_and_expression )* )
            // com/senseidb/bql/parsers/BQL.g:3072:9: conditional_and_expression ( '||' conditional_and_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_conditional_and_expression_in_conditional_or_expression9862);
            conditional_and_expression352=conditional_and_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, conditional_and_expression352.getTree());

            // com/senseidb/bql/parsers/BQL.g:3072:36: ( '||' conditional_and_expression )*
            loop112:
            do {
                int alt112=2;
                switch ( input.LA(1) ) {
                case 168:
                    {
                    alt112=1;
                    }
                    break;

                }

                switch (alt112) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3072:38: '||' conditional_and_expression
            	    {
            	    string_literal353=(Token)match(input,168,FOLLOW_168_in_conditional_or_expression9866); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    string_literal353_tree = 
            	    (Object)adaptor.create(string_literal353)
            	    ;
            	    adaptor.addChild(root_0, string_literal353_tree);
            	    }

            	    pushFollow(FOLLOW_conditional_and_expression_in_conditional_or_expression9868);
            	    conditional_and_expression354=conditional_and_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, conditional_and_expression354.getTree());

            	    }
            	    break;

            	default :
            	    break loop112;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 99, conditional_or_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "conditional_or_expression"


    public static class conditional_and_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "conditional_and_expression"
    // com/senseidb/bql/parsers/BQL.g:3075:1: conditional_and_expression : inclusive_or_expression ( '&&' inclusive_or_expression )* ;
    public final BQLParser.conditional_and_expression_return conditional_and_expression() throws RecognitionException {
        BQLParser.conditional_and_expression_return retval = new BQLParser.conditional_and_expression_return();
        retval.start = input.LT(1);

        int conditional_and_expression_StartIndex = input.index();

        Object root_0 = null;

        Token string_literal356=null;
        BQLParser.inclusive_or_expression_return inclusive_or_expression355 =null;

        BQLParser.inclusive_or_expression_return inclusive_or_expression357 =null;


        Object string_literal356_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 100) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3076:5: ( inclusive_or_expression ( '&&' inclusive_or_expression )* )
            // com/senseidb/bql/parsers/BQL.g:3076:9: inclusive_or_expression ( '&&' inclusive_or_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_inclusive_or_expression_in_conditional_and_expression9890);
            inclusive_or_expression355=inclusive_or_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, inclusive_or_expression355.getTree());

            // com/senseidb/bql/parsers/BQL.g:3076:33: ( '&&' inclusive_or_expression )*
            loop113:
            do {
                int alt113=2;
                switch ( input.LA(1) ) {
                case 118:
                    {
                    alt113=1;
                    }
                    break;

                }

                switch (alt113) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3076:34: '&&' inclusive_or_expression
            	    {
            	    string_literal356=(Token)match(input,118,FOLLOW_118_in_conditional_and_expression9893); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    string_literal356_tree = 
            	    (Object)adaptor.create(string_literal356)
            	    ;
            	    adaptor.addChild(root_0, string_literal356_tree);
            	    }

            	    pushFollow(FOLLOW_inclusive_or_expression_in_conditional_and_expression9895);
            	    inclusive_or_expression357=inclusive_or_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, inclusive_or_expression357.getTree());

            	    }
            	    break;

            	default :
            	    break loop113;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 100, conditional_and_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "conditional_and_expression"


    public static class inclusive_or_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "inclusive_or_expression"
    // com/senseidb/bql/parsers/BQL.g:3079:1: inclusive_or_expression : exclusive_or_expression ( '|' exclusive_or_expression )* ;
    public final BQLParser.inclusive_or_expression_return inclusive_or_expression() throws RecognitionException {
        BQLParser.inclusive_or_expression_return retval = new BQLParser.inclusive_or_expression_return();
        retval.start = input.LT(1);

        int inclusive_or_expression_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal359=null;
        BQLParser.exclusive_or_expression_return exclusive_or_expression358 =null;

        BQLParser.exclusive_or_expression_return exclusive_or_expression360 =null;


        Object char_literal359_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 101) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3080:5: ( exclusive_or_expression ( '|' exclusive_or_expression )* )
            // com/senseidb/bql/parsers/BQL.g:3080:9: exclusive_or_expression ( '|' exclusive_or_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_exclusive_or_expression_in_inclusive_or_expression9917);
            exclusive_or_expression358=exclusive_or_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, exclusive_or_expression358.getTree());

            // com/senseidb/bql/parsers/BQL.g:3080:33: ( '|' exclusive_or_expression )*
            loop114:
            do {
                int alt114=2;
                switch ( input.LA(1) ) {
                case 166:
                    {
                    alt114=1;
                    }
                    break;

                }

                switch (alt114) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3080:34: '|' exclusive_or_expression
            	    {
            	    char_literal359=(Token)match(input,166,FOLLOW_166_in_inclusive_or_expression9920); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal359_tree = 
            	    (Object)adaptor.create(char_literal359)
            	    ;
            	    adaptor.addChild(root_0, char_literal359_tree);
            	    }

            	    pushFollow(FOLLOW_exclusive_or_expression_in_inclusive_or_expression9922);
            	    exclusive_or_expression360=exclusive_or_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, exclusive_or_expression360.getTree());

            	    }
            	    break;

            	default :
            	    break loop114;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 101, inclusive_or_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "inclusive_or_expression"


    public static class exclusive_or_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "exclusive_or_expression"
    // com/senseidb/bql/parsers/BQL.g:3083:1: exclusive_or_expression : and_expression ( '^' and_expression )* ;
    public final BQLParser.exclusive_or_expression_return exclusive_or_expression() throws RecognitionException {
        BQLParser.exclusive_or_expression_return retval = new BQLParser.exclusive_or_expression_return();
        retval.start = input.LT(1);

        int exclusive_or_expression_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal362=null;
        BQLParser.and_expression_return and_expression361 =null;

        BQLParser.and_expression_return and_expression363 =null;


        Object char_literal362_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 102) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3084:5: ( and_expression ( '^' and_expression )* )
            // com/senseidb/bql/parsers/BQL.g:3084:9: and_expression ( '^' and_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_and_expression_in_exclusive_or_expression9944);
            and_expression361=and_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, and_expression361.getTree());

            // com/senseidb/bql/parsers/BQL.g:3084:24: ( '^' and_expression )*
            loop115:
            do {
                int alt115=2;
                switch ( input.LA(1) ) {
                case 143:
                    {
                    alt115=1;
                    }
                    break;

                }

                switch (alt115) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3084:25: '^' and_expression
            	    {
            	    char_literal362=(Token)match(input,143,FOLLOW_143_in_exclusive_or_expression9947); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal362_tree = 
            	    (Object)adaptor.create(char_literal362)
            	    ;
            	    adaptor.addChild(root_0, char_literal362_tree);
            	    }

            	    pushFollow(FOLLOW_and_expression_in_exclusive_or_expression9949);
            	    and_expression363=and_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, and_expression363.getTree());

            	    }
            	    break;

            	default :
            	    break loop115;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 102, exclusive_or_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "exclusive_or_expression"


    public static class and_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "and_expression"
    // com/senseidb/bql/parsers/BQL.g:3087:1: and_expression : equality_expression ( '&' equality_expression )* ;
    public final BQLParser.and_expression_return and_expression() throws RecognitionException {
        BQLParser.and_expression_return retval = new BQLParser.and_expression_return();
        retval.start = input.LT(1);

        int and_expression_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal365=null;
        BQLParser.equality_expression_return equality_expression364 =null;

        BQLParser.equality_expression_return equality_expression366 =null;


        Object char_literal365_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 103) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3088:5: ( equality_expression ( '&' equality_expression )* )
            // com/senseidb/bql/parsers/BQL.g:3088:9: equality_expression ( '&' equality_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_equality_expression_in_and_expression9971);
            equality_expression364=equality_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, equality_expression364.getTree());

            // com/senseidb/bql/parsers/BQL.g:3088:29: ( '&' equality_expression )*
            loop116:
            do {
                int alt116=2;
                switch ( input.LA(1) ) {
                case 119:
                    {
                    alt116=1;
                    }
                    break;

                }

                switch (alt116) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3088:31: '&' equality_expression
            	    {
            	    char_literal365=(Token)match(input,119,FOLLOW_119_in_and_expression9975); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    char_literal365_tree = 
            	    (Object)adaptor.create(char_literal365)
            	    ;
            	    adaptor.addChild(root_0, char_literal365_tree);
            	    }

            	    pushFollow(FOLLOW_equality_expression_in_and_expression9977);
            	    equality_expression366=equality_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, equality_expression366.getTree());

            	    }
            	    break;

            	default :
            	    break loop116;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 103, and_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "and_expression"


    public static class equality_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "equality_expression"
    // com/senseidb/bql/parsers/BQL.g:3091:1: equality_expression : instanceof_expression ( ( '==' | '!=' ) instanceof_expression )* ;
    public final BQLParser.equality_expression_return equality_expression() throws RecognitionException {
        BQLParser.equality_expression_return retval = new BQLParser.equality_expression_return();
        retval.start = input.LT(1);

        int equality_expression_StartIndex = input.index();

        Object root_0 = null;

        Token set368=null;
        BQLParser.instanceof_expression_return instanceof_expression367 =null;

        BQLParser.instanceof_expression_return instanceof_expression369 =null;


        Object set368_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 104) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3092:5: ( instanceof_expression ( ( '==' | '!=' ) instanceof_expression )* )
            // com/senseidb/bql/parsers/BQL.g:3092:9: instanceof_expression ( ( '==' | '!=' ) instanceof_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_instanceof_expression_in_equality_expression9999);
            instanceof_expression367=instanceof_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, instanceof_expression367.getTree());

            // com/senseidb/bql/parsers/BQL.g:3092:31: ( ( '==' | '!=' ) instanceof_expression )*
            loop117:
            do {
                int alt117=2;
                switch ( input.LA(1) ) {
                case 115:
                case 132:
                    {
                    alt117=1;
                    }
                    break;

                }

                switch (alt117) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3092:33: ( '==' | '!=' ) instanceof_expression
            	    {
            	    set368=(Token)input.LT(1);

            	    if ( input.LA(1)==115||input.LA(1)==132 ) {
            	        input.consume();
            	        if ( state.backtracking==0 ) adaptor.addChild(root_0, 
            	        (Object)adaptor.create(set368)
            	        );
            	        state.errorRecovery=false;
            	        state.failed=false;
            	    }
            	    else {
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        MismatchedSetException mse = new MismatchedSetException(null,input);
            	        throw mse;
            	    }


            	    pushFollow(FOLLOW_instanceof_expression_in_equality_expression10011);
            	    instanceof_expression369=instanceof_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, instanceof_expression369.getTree());

            	    }
            	    break;

            	default :
            	    break loop117;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 104, equality_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "equality_expression"


    public static class instanceof_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "instanceof_expression"
    // com/senseidb/bql/parsers/BQL.g:3095:1: instanceof_expression : relational_expression ( 'instanceof' type )? ;
    public final BQLParser.instanceof_expression_return instanceof_expression() throws RecognitionException {
        BQLParser.instanceof_expression_return retval = new BQLParser.instanceof_expression_return();
        retval.start = input.LT(1);

        int instanceof_expression_StartIndex = input.index();

        Object root_0 = null;

        Token string_literal371=null;
        BQLParser.relational_expression_return relational_expression370 =null;

        BQLParser.type_return type372 =null;


        Object string_literal371_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 105) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3096:5: ( relational_expression ( 'instanceof' type )? )
            // com/senseidb/bql/parsers/BQL.g:3096:9: relational_expression ( 'instanceof' type )?
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_relational_expression_in_instanceof_expression10033);
            relational_expression370=relational_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, relational_expression370.getTree());

            // com/senseidb/bql/parsers/BQL.g:3096:31: ( 'instanceof' type )?
            int alt118=2;
            switch ( input.LA(1) ) {
                case 157:
                    {
                    alt118=1;
                    }
                    break;
            }

            switch (alt118) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3096:32: 'instanceof' type
                    {
                    string_literal371=(Token)match(input,157,FOLLOW_157_in_instanceof_expression10036); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal371_tree = 
                    (Object)adaptor.create(string_literal371)
                    ;
                    adaptor.addChild(root_0, string_literal371_tree);
                    }

                    pushFollow(FOLLOW_type_in_instanceof_expression10038);
                    type372=type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, type372.getTree());

                    }
                    break;

            }


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 105, instanceof_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "instanceof_expression"


    public static class relational_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "relational_expression"
    // com/senseidb/bql/parsers/BQL.g:3099:1: relational_expression : shift_expression ( relational_op shift_expression )* ;
    public final BQLParser.relational_expression_return relational_expression() throws RecognitionException {
        BQLParser.relational_expression_return retval = new BQLParser.relational_expression_return();
        retval.start = input.LT(1);

        int relational_expression_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.shift_expression_return shift_expression373 =null;

        BQLParser.relational_op_return relational_op374 =null;

        BQLParser.shift_expression_return shift_expression375 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 106) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3100:5: ( shift_expression ( relational_op shift_expression )* )
            // com/senseidb/bql/parsers/BQL.g:3100:9: shift_expression ( relational_op shift_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_shift_expression_in_relational_expression10059);
            shift_expression373=shift_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, shift_expression373.getTree());

            // com/senseidb/bql/parsers/BQL.g:3100:26: ( relational_op shift_expression )*
            loop119:
            do {
                int alt119=2;
                switch ( input.LA(1) ) {
                case LT:
                    {
                    switch ( input.LA(2) ) {
                    case BOOLEAN:
                    case BYTE:
                    case CHARACTER_LITERAL:
                    case CONTAINS:
                    case DOUBLE:
                    case EQUAL:
                    case FALSE:
                    case FLOATING_POINT_LITERAL:
                    case HEX_LITERAL:
                    case IDENT:
                    case INTEGER:
                    case LONG:
                    case LPAR:
                    case NULL:
                    case OCTAL_LITERAL:
                    case REAL:
                    case STRING_LITERAL:
                    case TRUE:
                    case 114:
                    case 123:
                    case 124:
                    case 126:
                    case 127:
                    case 134:
                    case 135:
                    case 136:
                    case 137:
                    case 138:
                    case 139:
                    case 140:
                    case 170:
                        {
                        alt119=1;
                        }
                        break;

                    }

                    }
                    break;
                case GT:
                    {
                    switch ( input.LA(2) ) {
                    case BOOLEAN:
                    case BYTE:
                    case CHARACTER_LITERAL:
                    case CONTAINS:
                    case DOUBLE:
                    case EQUAL:
                    case FALSE:
                    case FLOATING_POINT_LITERAL:
                    case HEX_LITERAL:
                    case IDENT:
                    case INTEGER:
                    case LONG:
                    case LPAR:
                    case NULL:
                    case OCTAL_LITERAL:
                    case REAL:
                    case STRING_LITERAL:
                    case TRUE:
                    case 114:
                    case 123:
                    case 124:
                    case 126:
                    case 127:
                    case 134:
                    case 135:
                    case 136:
                    case 137:
                    case 138:
                    case 139:
                    case 140:
                    case 170:
                        {
                        alt119=1;
                        }
                        break;

                    }

                    }
                    break;

                }

                switch (alt119) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3100:28: relational_op shift_expression
            	    {
            	    pushFollow(FOLLOW_relational_op_in_relational_expression10063);
            	    relational_op374=relational_op();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, relational_op374.getTree());

            	    pushFollow(FOLLOW_shift_expression_in_relational_expression10065);
            	    shift_expression375=shift_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, shift_expression375.getTree());

            	    }
            	    break;

            	default :
            	    break loop119;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 106, relational_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "relational_expression"


    public static class relational_op_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "relational_op"
    // com/senseidb/bql/parsers/BQL.g:3103:1: relational_op : ( ( '<' '=' )=>t1= '<' t2= '=' {...}?| ( '>' '=' )=>t1= '>' t2= '=' {...}?| '<' | '>' );
    public final BQLParser.relational_op_return relational_op() throws RecognitionException {
        BQLParser.relational_op_return retval = new BQLParser.relational_op_return();
        retval.start = input.LT(1);

        int relational_op_StartIndex = input.index();

        Object root_0 = null;

        Token t1=null;
        Token t2=null;
        Token char_literal376=null;
        Token char_literal377=null;

        Object t1_tree=null;
        Object t2_tree=null;
        Object char_literal376_tree=null;
        Object char_literal377_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 107) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3104:5: ( ( '<' '=' )=>t1= '<' t2= '=' {...}?| ( '>' '=' )=>t1= '>' t2= '=' {...}?| '<' | '>' )
            int alt120=4;
            switch ( input.LA(1) ) {
            case LT:
                {
                int LA120_1 = input.LA(2);

                if ( (LA120_1==EQUAL) && (synpred194_BQL())) {
                    alt120=1;
                }
                else if ( (LA120_1==BOOLEAN||LA120_1==BYTE||LA120_1==CHARACTER_LITERAL||LA120_1==CONTAINS||LA120_1==DOUBLE||LA120_1==FALSE||LA120_1==FLOATING_POINT_LITERAL||LA120_1==HEX_LITERAL||LA120_1==IDENT||LA120_1==INTEGER||(LA120_1 >= LONG && LA120_1 <= LPAR)||LA120_1==NULL||LA120_1==OCTAL_LITERAL||LA120_1==REAL||LA120_1==STRING_LITERAL||LA120_1==TRUE||LA120_1==114||(LA120_1 >= 123 && LA120_1 <= 124)||(LA120_1 >= 126 && LA120_1 <= 127)||(LA120_1 >= 134 && LA120_1 <= 140)||LA120_1==170) ) {
                    alt120=3;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 120, 1, input);

                    throw nvae;

                }
                }
                break;
            case GT:
                {
                int LA120_2 = input.LA(2);

                if ( (LA120_2==EQUAL) && (synpred195_BQL())) {
                    alt120=2;
                }
                else if ( (LA120_2==BOOLEAN||LA120_2==BYTE||LA120_2==CHARACTER_LITERAL||LA120_2==CONTAINS||LA120_2==DOUBLE||LA120_2==FALSE||LA120_2==FLOATING_POINT_LITERAL||LA120_2==HEX_LITERAL||LA120_2==IDENT||LA120_2==INTEGER||(LA120_2 >= LONG && LA120_2 <= LPAR)||LA120_2==NULL||LA120_2==OCTAL_LITERAL||LA120_2==REAL||LA120_2==STRING_LITERAL||LA120_2==TRUE||LA120_2==114||(LA120_2 >= 123 && LA120_2 <= 124)||(LA120_2 >= 126 && LA120_2 <= 127)||(LA120_2 >= 134 && LA120_2 <= 140)||LA120_2==170) ) {
                    alt120=4;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 120, 2, input);

                    throw nvae;

                }
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 120, 0, input);

                throw nvae;

            }

            switch (alt120) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3104:9: ( '<' '=' )=>t1= '<' t2= '=' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,LT,FOLLOW_LT_in_relational_op10096); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_relational_op10100); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "relational_op", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() ");
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3107:9: ( '>' '=' )=>t1= '>' t2= '=' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,GT,FOLLOW_GT_in_relational_op10130); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,EQUAL,FOLLOW_EQUAL_in_relational_op10134); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "relational_op", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() ");
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3110:9: '<'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal376=(Token)match(input,LT,FOLLOW_LT_in_relational_op10155); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal376_tree = 
                    (Object)adaptor.create(char_literal376)
                    ;
                    adaptor.addChild(root_0, char_literal376_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:3111:9: '>'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal377=(Token)match(input,GT,FOLLOW_GT_in_relational_op10165); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal377_tree = 
                    (Object)adaptor.create(char_literal377)
                    ;
                    adaptor.addChild(root_0, char_literal377_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 107, relational_op_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "relational_op"


    public static class shift_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "shift_expression"
    // com/senseidb/bql/parsers/BQL.g:3114:1: shift_expression : additive_expression ( shift_op additive_expression )* ;
    public final BQLParser.shift_expression_return shift_expression() throws RecognitionException {
        BQLParser.shift_expression_return retval = new BQLParser.shift_expression_return();
        retval.start = input.LT(1);

        int shift_expression_StartIndex = input.index();

        Object root_0 = null;

        BQLParser.additive_expression_return additive_expression378 =null;

        BQLParser.shift_op_return shift_op379 =null;

        BQLParser.additive_expression_return additive_expression380 =null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 108) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3115:5: ( additive_expression ( shift_op additive_expression )* )
            // com/senseidb/bql/parsers/BQL.g:3115:9: additive_expression ( shift_op additive_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_additive_expression_in_shift_expression10184);
            additive_expression378=additive_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, additive_expression378.getTree());

            // com/senseidb/bql/parsers/BQL.g:3115:29: ( shift_op additive_expression )*
            loop121:
            do {
                int alt121=2;
                switch ( input.LA(1) ) {
                case LT:
                    {
                    switch ( input.LA(2) ) {
                    case LT:
                        {
                        switch ( input.LA(3) ) {
                        case BOOLEAN:
                        case BYTE:
                        case CHARACTER_LITERAL:
                        case CONTAINS:
                        case DOUBLE:
                        case FALSE:
                        case FLOATING_POINT_LITERAL:
                        case HEX_LITERAL:
                        case IDENT:
                        case INTEGER:
                        case LONG:
                        case LPAR:
                        case NULL:
                        case OCTAL_LITERAL:
                        case REAL:
                        case STRING_LITERAL:
                        case TRUE:
                        case 114:
                        case 123:
                        case 124:
                        case 126:
                        case 127:
                        case 134:
                        case 135:
                        case 136:
                        case 137:
                        case 138:
                        case 139:
                        case 140:
                        case 170:
                            {
                            alt121=1;
                            }
                            break;

                        }

                        }
                        break;

                    }

                    }
                    break;
                case GT:
                    {
                    switch ( input.LA(2) ) {
                    case GT:
                        {
                        switch ( input.LA(3) ) {
                        case GT:
                            {
                            switch ( input.LA(4) ) {
                            case BOOLEAN:
                            case BYTE:
                            case CHARACTER_LITERAL:
                            case CONTAINS:
                            case DOUBLE:
                            case FALSE:
                            case FLOATING_POINT_LITERAL:
                            case HEX_LITERAL:
                            case IDENT:
                            case INTEGER:
                            case LONG:
                            case LPAR:
                            case NULL:
                            case OCTAL_LITERAL:
                            case REAL:
                            case STRING_LITERAL:
                            case TRUE:
                            case 114:
                            case 123:
                            case 124:
                            case 126:
                            case 127:
                            case 134:
                            case 135:
                            case 136:
                            case 137:
                            case 138:
                            case 139:
                            case 140:
                            case 170:
                                {
                                alt121=1;
                                }
                                break;

                            }

                            }
                            break;
                        case BOOLEAN:
                        case BYTE:
                        case CHARACTER_LITERAL:
                        case CONTAINS:
                        case DOUBLE:
                        case FALSE:
                        case FLOATING_POINT_LITERAL:
                        case HEX_LITERAL:
                        case IDENT:
                        case INTEGER:
                        case LONG:
                        case LPAR:
                        case NULL:
                        case OCTAL_LITERAL:
                        case REAL:
                        case STRING_LITERAL:
                        case TRUE:
                        case 114:
                        case 123:
                        case 124:
                        case 126:
                        case 127:
                        case 134:
                        case 135:
                        case 136:
                        case 137:
                        case 138:
                        case 139:
                        case 140:
                        case 170:
                            {
                            alt121=1;
                            }
                            break;

                        }

                        }
                        break;

                    }

                    }
                    break;

                }

                switch (alt121) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3115:31: shift_op additive_expression
            	    {
            	    pushFollow(FOLLOW_shift_op_in_shift_expression10188);
            	    shift_op379=shift_op();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, shift_op379.getTree());

            	    pushFollow(FOLLOW_additive_expression_in_shift_expression10190);
            	    additive_expression380=additive_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, additive_expression380.getTree());

            	    }
            	    break;

            	default :
            	    break loop121;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 108, shift_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "shift_expression"


    public static class shift_op_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "shift_op"
    // com/senseidb/bql/parsers/BQL.g:3118:1: shift_op : ( ( '<' '<' )=>t1= '<' t2= '<' {...}?| ( '>' '>' '>' )=>t1= '>' t2= '>' t3= '>' {...}?| ( '>' '>' )=>t1= '>' t2= '>' {...}?);
    public final BQLParser.shift_op_return shift_op() throws RecognitionException {
        BQLParser.shift_op_return retval = new BQLParser.shift_op_return();
        retval.start = input.LT(1);

        int shift_op_StartIndex = input.index();

        Object root_0 = null;

        Token t1=null;
        Token t2=null;
        Token t3=null;

        Object t1_tree=null;
        Object t2_tree=null;
        Object t3_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 109) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3119:5: ( ( '<' '<' )=>t1= '<' t2= '<' {...}?| ( '>' '>' '>' )=>t1= '>' t2= '>' t3= '>' {...}?| ( '>' '>' )=>t1= '>' t2= '>' {...}?)
            int alt122=3;
            int LA122_0 = input.LA(1);

            if ( (LA122_0==LT) && (synpred198_BQL())) {
                alt122=1;
            }
            else if ( (LA122_0==GT) ) {
                switch ( input.LA(2) ) {
                case GT:
                    {
                    int LA122_3 = input.LA(3);

                    if ( (LA122_3==GT) && (synpred199_BQL())) {
                        alt122=2;
                    }
                    else if ( (LA122_3==123) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==126) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==124) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==127) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==170) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==114) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==LPAR) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==HEX_LITERAL||LA122_3==INTEGER||LA122_3==OCTAL_LITERAL) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==REAL) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==FLOATING_POINT_LITERAL) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==CHARACTER_LITERAL) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==STRING_LITERAL) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==TRUE) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==FALSE) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==NULL) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==CONTAINS) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==IDENT) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==BOOLEAN) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==134) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==BYTE) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==138) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==136) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==LONG) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==135) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==DOUBLE) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else if ( (LA122_3==137||(LA122_3 >= 139 && LA122_3 <= 140)) && (synpred200_BQL())) {
                        alt122=3;
                    }
                    else {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 122, 3, input);

                        throw nvae;

                    }
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 122, 2, input);

                    throw nvae;

                }

            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 122, 0, input);

                throw nvae;

            }
            switch (alt122) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3119:9: ( '<' '<' )=>t1= '<' t2= '<' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,LT,FOLLOW_LT_in_shift_op10221); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,LT,FOLLOW_LT_in_shift_op10225); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "shift_op", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() ");
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3122:9: ( '>' '>' '>' )=>t1= '>' t2= '>' t3= '>' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,GT,FOLLOW_GT_in_shift_op10257); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,GT,FOLLOW_GT_in_shift_op10261); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    t3=(Token)match(input,GT,FOLLOW_GT_in_shift_op10265); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t3_tree = 
                    (Object)adaptor.create(t3)
                    ;
                    adaptor.addChild(root_0, t3_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() &&
                              t2.getLine() == t3.getLine() && 
                              t2.getCharPositionInLine() + 1 == t3.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "shift_op", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() &&\n          $t2.getLine() == $t3.getLine() && \n          $t2.getCharPositionInLine() + 1 == $t3.getCharPositionInLine() ");
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3127:9: ( '>' '>' )=>t1= '>' t2= '>' {...}?
                    {
                    root_0 = (Object)adaptor.nil();


                    t1=(Token)match(input,GT,FOLLOW_GT_in_shift_op10295); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t1_tree = 
                    (Object)adaptor.create(t1)
                    ;
                    adaptor.addChild(root_0, t1_tree);
                    }

                    t2=(Token)match(input,GT,FOLLOW_GT_in_shift_op10299); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t2_tree = 
                    (Object)adaptor.create(t2)
                    ;
                    adaptor.addChild(root_0, t2_tree);
                    }

                    if ( !(( t1.getLine() == t2.getLine() && 
                              t1.getCharPositionInLine() + 1 == t2.getCharPositionInLine() )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "shift_op", " $t1.getLine() == $t2.getLine() && \n          $t1.getCharPositionInLine() + 1 == $t2.getCharPositionInLine() ");
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 109, shift_op_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "shift_op"


    public static class additive_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "additive_expression"
    // com/senseidb/bql/parsers/BQL.g:3132:1: additive_expression : multiplicative_expression ( ( '+' | '-' ) multiplicative_expression )* ;
    public final BQLParser.additive_expression_return additive_expression() throws RecognitionException {
        BQLParser.additive_expression_return retval = new BQLParser.additive_expression_return();
        retval.start = input.LT(1);

        int additive_expression_StartIndex = input.index();

        Object root_0 = null;

        Token set382=null;
        BQLParser.multiplicative_expression_return multiplicative_expression381 =null;

        BQLParser.multiplicative_expression_return multiplicative_expression383 =null;


        Object set382_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 110) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3133:5: ( multiplicative_expression ( ( '+' | '-' ) multiplicative_expression )* )
            // com/senseidb/bql/parsers/BQL.g:3133:9: multiplicative_expression ( ( '+' | '-' ) multiplicative_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_multiplicative_expression_in_additive_expression10328);
            multiplicative_expression381=multiplicative_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, multiplicative_expression381.getTree());

            // com/senseidb/bql/parsers/BQL.g:3133:35: ( ( '+' | '-' ) multiplicative_expression )*
            loop123:
            do {
                int alt123=2;
                switch ( input.LA(1) ) {
                case 123:
                case 126:
                    {
                    alt123=1;
                    }
                    break;

                }

                switch (alt123) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3133:37: ( '+' | '-' ) multiplicative_expression
            	    {
            	    set382=(Token)input.LT(1);

            	    if ( input.LA(1)==123||input.LA(1)==126 ) {
            	        input.consume();
            	        if ( state.backtracking==0 ) adaptor.addChild(root_0, 
            	        (Object)adaptor.create(set382)
            	        );
            	        state.errorRecovery=false;
            	        state.failed=false;
            	    }
            	    else {
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        MismatchedSetException mse = new MismatchedSetException(null,input);
            	        throw mse;
            	    }


            	    pushFollow(FOLLOW_multiplicative_expression_in_additive_expression10340);
            	    multiplicative_expression383=multiplicative_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, multiplicative_expression383.getTree());

            	    }
            	    break;

            	default :
            	    break loop123;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 110, additive_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "additive_expression"


    public static class multiplicative_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "multiplicative_expression"
    // com/senseidb/bql/parsers/BQL.g:3136:1: multiplicative_expression : unary_expression ( ( '*' | '/' | '%' ) unary_expression )* ;
    public final BQLParser.multiplicative_expression_return multiplicative_expression() throws RecognitionException {
        BQLParser.multiplicative_expression_return retval = new BQLParser.multiplicative_expression_return();
        retval.start = input.LT(1);

        int multiplicative_expression_StartIndex = input.index();

        Object root_0 = null;

        Token set385=null;
        BQLParser.unary_expression_return unary_expression384 =null;

        BQLParser.unary_expression_return unary_expression386 =null;


        Object set385_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 111) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3137:5: ( unary_expression ( ( '*' | '/' | '%' ) unary_expression )* )
            // com/senseidb/bql/parsers/BQL.g:3137:9: unary_expression ( ( '*' | '/' | '%' ) unary_expression )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_unary_expression_in_multiplicative_expression10362);
            unary_expression384=unary_expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression384.getTree());

            // com/senseidb/bql/parsers/BQL.g:3137:26: ( ( '*' | '/' | '%' ) unary_expression )*
            loop124:
            do {
                int alt124=2;
                switch ( input.LA(1) ) {
                case 116:
                case 121:
                case 130:
                    {
                    alt124=1;
                    }
                    break;

                }

                switch (alt124) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3137:28: ( '*' | '/' | '%' ) unary_expression
            	    {
            	    set385=(Token)input.LT(1);

            	    if ( input.LA(1)==116||input.LA(1)==121||input.LA(1)==130 ) {
            	        input.consume();
            	        if ( state.backtracking==0 ) adaptor.addChild(root_0, 
            	        (Object)adaptor.create(set385)
            	        );
            	        state.errorRecovery=false;
            	        state.failed=false;
            	    }
            	    else {
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        MismatchedSetException mse = new MismatchedSetException(null,input);
            	        throw mse;
            	    }


            	    pushFollow(FOLLOW_unary_expression_in_multiplicative_expression10380);
            	    unary_expression386=unary_expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression386.getTree());

            	    }
            	    break;

            	default :
            	    break loop124;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 111, multiplicative_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "multiplicative_expression"


    public static class unary_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "unary_expression"
    // com/senseidb/bql/parsers/BQL.g:3140:1: unary_expression : ( '+' unary_expression | '-' unary_expression | '++' unary_expression | '--' unary_expression | unary_expression_not_plus_minus );
    public final BQLParser.unary_expression_return unary_expression() throws RecognitionException {
        BQLParser.unary_expression_return retval = new BQLParser.unary_expression_return();
        retval.start = input.LT(1);

        int unary_expression_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal387=null;
        Token char_literal389=null;
        Token string_literal391=null;
        Token string_literal393=null;
        BQLParser.unary_expression_return unary_expression388 =null;

        BQLParser.unary_expression_return unary_expression390 =null;

        BQLParser.unary_expression_return unary_expression392 =null;

        BQLParser.unary_expression_return unary_expression394 =null;

        BQLParser.unary_expression_not_plus_minus_return unary_expression_not_plus_minus395 =null;


        Object char_literal387_tree=null;
        Object char_literal389_tree=null;
        Object string_literal391_tree=null;
        Object string_literal393_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 112) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3141:5: ( '+' unary_expression | '-' unary_expression | '++' unary_expression | '--' unary_expression | unary_expression_not_plus_minus )
            int alt125=5;
            switch ( input.LA(1) ) {
            case 123:
                {
                alt125=1;
                }
                break;
            case 126:
                {
                alt125=2;
                }
                break;
            case 124:
                {
                alt125=3;
                }
                break;
            case 127:
                {
                alt125=4;
                }
                break;
            case BOOLEAN:
            case BYTE:
            case CHARACTER_LITERAL:
            case CONTAINS:
            case DOUBLE:
            case FALSE:
            case FLOATING_POINT_LITERAL:
            case HEX_LITERAL:
            case IDENT:
            case INTEGER:
            case LONG:
            case LPAR:
            case NULL:
            case OCTAL_LITERAL:
            case REAL:
            case STRING_LITERAL:
            case TRUE:
            case 114:
            case 134:
            case 135:
            case 136:
            case 137:
            case 138:
            case 139:
            case 140:
            case 170:
                {
                alt125=5;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 125, 0, input);

                throw nvae;

            }

            switch (alt125) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3141:9: '+' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal387=(Token)match(input,123,FOLLOW_123_in_unary_expression10406); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal387_tree = 
                    (Object)adaptor.create(char_literal387)
                    ;
                    adaptor.addChild(root_0, char_literal387_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_unary_expression10408);
                    unary_expression388=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression388.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3142:9: '-' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal389=(Token)match(input,126,FOLLOW_126_in_unary_expression10418); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal389_tree = 
                    (Object)adaptor.create(char_literal389)
                    ;
                    adaptor.addChild(root_0, char_literal389_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_unary_expression10420);
                    unary_expression390=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression390.getTree());

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3143:9: '++' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal391=(Token)match(input,124,FOLLOW_124_in_unary_expression10430); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal391_tree = 
                    (Object)adaptor.create(string_literal391)
                    ;
                    adaptor.addChild(root_0, string_literal391_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_unary_expression10432);
                    unary_expression392=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression392.getTree());

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:3144:9: '--' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    string_literal393=(Token)match(input,127,FOLLOW_127_in_unary_expression10442); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal393_tree = 
                    (Object)adaptor.create(string_literal393)
                    ;
                    adaptor.addChild(root_0, string_literal393_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_unary_expression10444);
                    unary_expression394=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression394.getTree());

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:3145:9: unary_expression_not_plus_minus
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_unary_expression_not_plus_minus_in_unary_expression10454);
                    unary_expression_not_plus_minus395=unary_expression_not_plus_minus();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression_not_plus_minus395.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 112, unary_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "unary_expression"


    public static class unary_expression_not_plus_minus_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "unary_expression_not_plus_minus"
    // com/senseidb/bql/parsers/BQL.g:3148:1: unary_expression_not_plus_minus : ( '~' unary_expression | '!' unary_expression | cast_expression | primary ( selector )* ( '++' | '--' )? );
    public final BQLParser.unary_expression_not_plus_minus_return unary_expression_not_plus_minus() throws RecognitionException {
        BQLParser.unary_expression_not_plus_minus_return retval = new BQLParser.unary_expression_not_plus_minus_return();
        retval.start = input.LT(1);

        int unary_expression_not_plus_minus_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal396=null;
        Token char_literal398=null;
        Token set403=null;
        BQLParser.unary_expression_return unary_expression397 =null;

        BQLParser.unary_expression_return unary_expression399 =null;

        BQLParser.cast_expression_return cast_expression400 =null;

        BQLParser.primary_return primary401 =null;

        BQLParser.selector_return selector402 =null;


        Object char_literal396_tree=null;
        Object char_literal398_tree=null;
        Object set403_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 113) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3149:5: ( '~' unary_expression | '!' unary_expression | cast_expression | primary ( selector )* ( '++' | '--' )? )
            int alt128=4;
            switch ( input.LA(1) ) {
            case 170:
                {
                alt128=1;
                }
                break;
            case 114:
                {
                alt128=2;
                }
                break;
            case LPAR:
                {
                int LA128_3 = input.LA(2);

                if ( (synpred212_BQL()) ) {
                    alt128=3;
                }
                else if ( (true) ) {
                    alt128=4;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 128, 3, input);

                    throw nvae;

                }
                }
                break;
            case BOOLEAN:
            case BYTE:
            case CHARACTER_LITERAL:
            case CONTAINS:
            case DOUBLE:
            case FALSE:
            case FLOATING_POINT_LITERAL:
            case HEX_LITERAL:
            case IDENT:
            case INTEGER:
            case LONG:
            case NULL:
            case OCTAL_LITERAL:
            case REAL:
            case STRING_LITERAL:
            case TRUE:
            case 134:
            case 135:
            case 136:
            case 137:
            case 138:
            case 139:
            case 140:
                {
                alt128=4;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 128, 0, input);

                throw nvae;

            }

            switch (alt128) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3149:9: '~' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal396=(Token)match(input,170,FOLLOW_170_in_unary_expression_not_plus_minus10473); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal396_tree = 
                    (Object)adaptor.create(char_literal396)
                    ;
                    adaptor.addChild(root_0, char_literal396_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_unary_expression_not_plus_minus10475);
                    unary_expression397=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression397.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3150:9: '!' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal398=(Token)match(input,114,FOLLOW_114_in_unary_expression_not_plus_minus10485); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal398_tree = 
                    (Object)adaptor.create(char_literal398)
                    ;
                    adaptor.addChild(root_0, char_literal398_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_unary_expression_not_plus_minus10487);
                    unary_expression399=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression399.getTree());

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3151:9: cast_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_cast_expression_in_unary_expression_not_plus_minus10497);
                    cast_expression400=cast_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, cast_expression400.getTree());

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:3152:9: primary ( selector )* ( '++' | '--' )?
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_primary_in_unary_expression_not_plus_minus10507);
                    primary401=primary();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, primary401.getTree());

                    // com/senseidb/bql/parsers/BQL.g:3152:17: ( selector )*
                    loop126:
                    do {
                        int alt126=2;
                        switch ( input.LA(1) ) {
                        case 129:
                        case 141:
                            {
                            alt126=1;
                            }
                            break;

                        }

                        switch (alt126) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:3152:17: selector
                    	    {
                    	    pushFollow(FOLLOW_selector_in_unary_expression_not_plus_minus10509);
                    	    selector402=selector();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) adaptor.addChild(root_0, selector402.getTree());

                    	    }
                    	    break;

                    	default :
                    	    break loop126;
                        }
                    } while (true);


                    // com/senseidb/bql/parsers/BQL.g:3152:27: ( '++' | '--' )?
                    int alt127=2;
                    switch ( input.LA(1) ) {
                        case 124:
                        case 127:
                            {
                            alt127=1;
                            }
                            break;
                    }

                    switch (alt127) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:
                            {
                            set403=(Token)input.LT(1);

                            if ( input.LA(1)==124||input.LA(1)==127 ) {
                                input.consume();
                                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                                (Object)adaptor.create(set403)
                                );
                                state.errorRecovery=false;
                                state.failed=false;
                            }
                            else {
                                if (state.backtracking>0) {state.failed=true; return retval;}
                                MismatchedSetException mse = new MismatchedSetException(null,input);
                                throw mse;
                            }


                            }
                            break;

                    }


                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 113, unary_expression_not_plus_minus_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "unary_expression_not_plus_minus"


    public static class cast_expression_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "cast_expression"
    // com/senseidb/bql/parsers/BQL.g:3155:1: cast_expression : ( '(' primitive_type ')' unary_expression | '(' ( type | expression ) ')' unary_expression_not_plus_minus );
    public final BQLParser.cast_expression_return cast_expression() throws RecognitionException {
        BQLParser.cast_expression_return retval = new BQLParser.cast_expression_return();
        retval.start = input.LT(1);

        int cast_expression_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal404=null;
        Token char_literal406=null;
        Token char_literal408=null;
        Token char_literal411=null;
        BQLParser.primitive_type_return primitive_type405 =null;

        BQLParser.unary_expression_return unary_expression407 =null;

        BQLParser.type_return type409 =null;

        BQLParser.expression_return expression410 =null;

        BQLParser.unary_expression_not_plus_minus_return unary_expression_not_plus_minus412 =null;


        Object char_literal404_tree=null;
        Object char_literal406_tree=null;
        Object char_literal408_tree=null;
        Object char_literal411_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 114) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3156:5: ( '(' primitive_type ')' unary_expression | '(' ( type | expression ) ')' unary_expression_not_plus_minus )
            int alt130=2;
            switch ( input.LA(1) ) {
            case LPAR:
                {
                int LA130_1 = input.LA(2);

                if ( (synpred216_BQL()) ) {
                    alt130=1;
                }
                else if ( (true) ) {
                    alt130=2;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 130, 1, input);

                    throw nvae;

                }
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 130, 0, input);

                throw nvae;

            }

            switch (alt130) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3156:8: '(' primitive_type ')' unary_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal404=(Token)match(input,LPAR,FOLLOW_LPAR_in_cast_expression10535); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal404_tree = 
                    (Object)adaptor.create(char_literal404)
                    ;
                    adaptor.addChild(root_0, char_literal404_tree);
                    }

                    pushFollow(FOLLOW_primitive_type_in_cast_expression10537);
                    primitive_type405=primitive_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, primitive_type405.getTree());

                    char_literal406=(Token)match(input,RPAR,FOLLOW_RPAR_in_cast_expression10539); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal406_tree = 
                    (Object)adaptor.create(char_literal406)
                    ;
                    adaptor.addChild(root_0, char_literal406_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_in_cast_expression10541);
                    unary_expression407=unary_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression407.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3157:8: '(' ( type | expression ) ')' unary_expression_not_plus_minus
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal408=(Token)match(input,LPAR,FOLLOW_LPAR_in_cast_expression10550); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal408_tree = 
                    (Object)adaptor.create(char_literal408)
                    ;
                    adaptor.addChild(root_0, char_literal408_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:3157:12: ( type | expression )
                    int alt129=2;
                    switch ( input.LA(1) ) {
                    case FAST_UTIL_DATA_TYPE:
                    case INT:
                    case 147:
                    case 154:
                    case 160:
                        {
                        alt129=1;
                        }
                        break;
                    case BOOLEAN:
                        {
                        int LA129_2 = input.LA(2);

                        if ( ((((( "Boolean".equals(input.LT(1).getText()) )||( "boolean".equals(input.LT(1).getText()) ))&&(( "Boolean".equals(input.LT(1).getText()) )||( "boolean".equals(input.LT(1).getText()) )))&&synpred217_BQL())) ) {
                            alt129=1;
                        }
                        else if ( (( "Boolean".equals(input.LT(1).getText()) )) ) {
                            alt129=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 129, 2, input);

                            throw nvae;

                        }
                        }
                        break;
                    case BYTE:
                        {
                        int LA129_3 = input.LA(2);

                        if ( (((synpred217_BQL()&&synpred217_BQL())&&(( "Byte".equals(input.LT(1).getText()) )||( "byte".equals(input.LT(1).getText()) )))) ) {
                            alt129=1;
                        }
                        else if ( (( "Byte".equals(input.LT(1).getText()) )) ) {
                            alt129=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 129, 3, input);

                            throw nvae;

                        }
                        }
                        break;
                    case LONG:
                        {
                        int LA129_4 = input.LA(2);

                        if ( ((((( "long".equals(input.LT(1).getText()) )||( "Long".equals(input.LT(1).getText()) ))&&(( "long".equals(input.LT(1).getText()) )||( "Long".equals(input.LT(1).getText()) )))&&synpred217_BQL())) ) {
                            alt129=1;
                        }
                        else if ( (( "Long".equals(input.LT(1).getText()) )) ) {
                            alt129=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 129, 4, input);

                            throw nvae;

                        }
                        }
                        break;
                    case DOUBLE:
                        {
                        int LA129_5 = input.LA(2);

                        if ( ((((( "double".equals(input.LT(1).getText()) )||( "Double".equals(input.LT(1).getText()) ))&&(( "double".equals(input.LT(1).getText()) )||( "Double".equals(input.LT(1).getText()) )))&&synpred217_BQL())) ) {
                            alt129=1;
                        }
                        else if ( (( "Double".equals(input.LT(1).getText()) )) ) {
                            alt129=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 129, 5, input);

                            throw nvae;

                        }
                        }
                        break;
                    case 134:
                        {
                        int LA129_6 = input.LA(2);

                        if ( (synpred217_BQL()) ) {
                            alt129=1;
                        }
                        else if ( (true) ) {
                            alt129=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 129, 6, input);

                            throw nvae;

                        }
                        }
                        break;
                    case 138:
                        {
                        int LA129_7 = input.LA(2);

                        if ( (synpred217_BQL()) ) {
                            alt129=1;
                        }
                        else if ( (true) ) {
                            alt129=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 129, 7, input);

                            throw nvae;

                        }
                        }
                        break;
                    case 136:
                        {
                        int LA129_8 = input.LA(2);

                        if ( (synpred217_BQL()) ) {
                            alt129=1;
                        }
                        else if ( (true) ) {
                            alt129=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 129, 8, input);

                            throw nvae;

                        }
                        }
                        break;
                    case 135:
                        {
                        int LA129_9 = input.LA(2);

                        if ( (synpred217_BQL()) ) {
                            alt129=1;
                        }
                        else if ( (true) ) {
                            alt129=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 129, 9, input);

                            throw nvae;

                        }
                        }
                        break;
                    case 137:
                    case 139:
                    case 140:
                        {
                        int LA129_10 = input.LA(2);

                        if ( (synpred217_BQL()) ) {
                            alt129=1;
                        }
                        else if ( (true) ) {
                            alt129=2;
                        }
                        else {
                            if (state.backtracking>0) {state.failed=true; return retval;}
                            NoViableAltException nvae =
                                new NoViableAltException("", 129, 10, input);

                            throw nvae;

                        }
                        }
                        break;
                    case CHARACTER_LITERAL:
                    case CONTAINS:
                    case FALSE:
                    case FLOATING_POINT_LITERAL:
                    case HEX_LITERAL:
                    case IDENT:
                    case INTEGER:
                    case LPAR:
                    case NULL:
                    case OCTAL_LITERAL:
                    case REAL:
                    case STRING_LITERAL:
                    case TRUE:
                    case 114:
                    case 123:
                    case 124:
                    case 126:
                    case 127:
                    case 170:
                        {
                        alt129=2;
                        }
                        break;
                    default:
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 129, 0, input);

                        throw nvae;

                    }

                    switch (alt129) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:3157:13: type
                            {
                            pushFollow(FOLLOW_type_in_cast_expression10553);
                            type409=type();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, type409.getTree());

                            }
                            break;
                        case 2 :
                            // com/senseidb/bql/parsers/BQL.g:3157:20: expression
                            {
                            pushFollow(FOLLOW_expression_in_cast_expression10557);
                            expression410=expression();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression410.getTree());

                            }
                            break;

                    }


                    char_literal411=(Token)match(input,RPAR,FOLLOW_RPAR_in_cast_expression10560); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal411_tree = 
                    (Object)adaptor.create(char_literal411)
                    ;
                    adaptor.addChild(root_0, char_literal411_tree);
                    }

                    pushFollow(FOLLOW_unary_expression_not_plus_minus_in_cast_expression10562);
                    unary_expression_not_plus_minus412=unary_expression_not_plus_minus();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unary_expression_not_plus_minus412.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 114, cast_expression_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "cast_expression"


    public static class primary_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "primary"
    // com/senseidb/bql/parsers/BQL.g:3160:1: primary : ( par_expression | literal | java_method identifier_suffix | java_ident ( '.' java_method )* ( identifier_suffix )? );
    public final BQLParser.primary_return primary() throws RecognitionException {
        BQLParser.primary_return retval = new BQLParser.primary_return();
        retval.start = input.LT(1);

        int primary_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal418=null;
        BQLParser.par_expression_return par_expression413 =null;

        BQLParser.literal_return literal414 =null;

        BQLParser.java_method_return java_method415 =null;

        BQLParser.identifier_suffix_return identifier_suffix416 =null;

        BQLParser.java_ident_return java_ident417 =null;

        BQLParser.java_method_return java_method419 =null;

        BQLParser.identifier_suffix_return identifier_suffix420 =null;


        Object char_literal418_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 115) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3161:5: ( par_expression | literal | java_method identifier_suffix | java_ident ( '.' java_method )* ( identifier_suffix )? )
            int alt133=4;
            switch ( input.LA(1) ) {
            case LPAR:
                {
                alt133=1;
                }
                break;
            case CHARACTER_LITERAL:
            case FALSE:
            case FLOATING_POINT_LITERAL:
            case HEX_LITERAL:
            case INTEGER:
            case NULL:
            case OCTAL_LITERAL:
            case REAL:
            case STRING_LITERAL:
            case TRUE:
                {
                alt133=2;
                }
                break;
            case CONTAINS:
                {
                alt133=3;
                }
                break;
            case IDENT:
                {
                int LA133_11 = input.LA(2);

                if ( (synpred220_BQL()) ) {
                    alt133=3;
                }
                else if ( (true) ) {
                    alt133=4;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 133, 11, input);

                    throw nvae;

                }
                }
                break;
            case BOOLEAN:
            case BYTE:
            case DOUBLE:
            case LONG:
            case 134:
            case 135:
            case 136:
            case 137:
            case 138:
            case 139:
            case 140:
                {
                alt133=4;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 133, 0, input);

                throw nvae;

            }

            switch (alt133) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3161:9: par_expression
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_par_expression_in_primary10581);
                    par_expression413=par_expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, par_expression413.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3162:9: literal
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_literal_in_primary10591);
                    literal414=literal();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, literal414.getTree());

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3163:9: java_method identifier_suffix
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_java_method_in_primary10604);
                    java_method415=java_method();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_method415.getTree());

                    pushFollow(FOLLOW_identifier_suffix_in_primary10606);
                    identifier_suffix416=identifier_suffix();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, identifier_suffix416.getTree());

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:3164:9: java_ident ( '.' java_method )* ( identifier_suffix )?
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_java_ident_in_primary10616);
                    java_ident417=java_ident();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_ident417.getTree());

                    // com/senseidb/bql/parsers/BQL.g:3164:20: ( '.' java_method )*
                    loop131:
                    do {
                        int alt131=2;
                        switch ( input.LA(1) ) {
                        case 129:
                            {
                            switch ( input.LA(2) ) {
                            case IDENT:
                                {
                                int LA131_3 = input.LA(3);

                                if ( (synpred221_BQL()) ) {
                                    alt131=1;
                                }


                                }
                                break;
                            case CONTAINS:
                                {
                                alt131=1;
                                }
                                break;

                            }

                            }
                            break;

                        }

                        switch (alt131) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:3164:21: '.' java_method
                    	    {
                    	    char_literal418=(Token)match(input,129,FOLLOW_129_in_primary10619); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal418_tree = 
                    	    (Object)adaptor.create(char_literal418)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal418_tree);
                    	    }

                    	    pushFollow(FOLLOW_java_method_in_primary10621);
                    	    java_method419=java_method();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) adaptor.addChild(root_0, java_method419.getTree());

                    	    }
                    	    break;

                    	default :
                    	    break loop131;
                        }
                    } while (true);


                    // com/senseidb/bql/parsers/BQL.g:3164:39: ( identifier_suffix )?
                    int alt132=2;
                    switch ( input.LA(1) ) {
                        case 141:
                            {
                            switch ( input.LA(2) ) {
                                case 142:
                                    {
                                    alt132=1;
                                    }
                                    break;
                            }

                            }
                            break;
                        case LPAR:
                            {
                            alt132=1;
                            }
                            break;
                        case 129:
                            {
                            switch ( input.LA(2) ) {
                                case 148:
                                case 161:
                                    {
                                    alt132=1;
                                    }
                                    break;
                                case 163:
                                    {
                                    int LA132_5 = input.LA(3);

                                    if ( (synpred222_BQL()) ) {
                                        alt132=1;
                                    }
                                    }
                                    break;
                            }

                            }
                            break;
                    }

                    switch (alt132) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:3164:39: identifier_suffix
                            {
                            pushFollow(FOLLOW_identifier_suffix_in_primary10625);
                            identifier_suffix420=identifier_suffix();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, identifier_suffix420.getTree());

                            }
                            break;

                    }


                    if ( state.backtracking==0 ) {
                                String var = (java_ident417!=null?input.toString(java_ident417.start,java_ident417.stop):null);
                                if (_facetInfoMap.containsKey(var)) {
                                    _usedFacets.add(var);
                                }
                                else if (_internalVarMap.containsKey(var)) {
                                    _usedInternalVars.add(var);
                                }
                                else if (!_supportedClasses.contains(var) && !verifyVariable(var)) {
                                    throw new FailedPredicateException(input,
                                                                       "primary",
                                                                       "Variable or class \"" + var + "\" is not defined.");
                                }
                            }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 115, primary_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "primary"


    public static class java_ident_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "java_ident"
    // com/senseidb/bql/parsers/BQL.g:3181:1: java_ident : ( boxed_type | limited_type | IDENT );
    public final BQLParser.java_ident_return java_ident() throws RecognitionException {
        BQLParser.java_ident_return retval = new BQLParser.java_ident_return();
        retval.start = input.LT(1);

        int java_ident_StartIndex = input.index();

        Object root_0 = null;

        Token IDENT423=null;
        BQLParser.boxed_type_return boxed_type421 =null;

        BQLParser.limited_type_return limited_type422 =null;


        Object IDENT423_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 116) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3182:5: ( boxed_type | limited_type | IDENT )
            int alt134=3;
            switch ( input.LA(1) ) {
            case BOOLEAN:
            case BYTE:
            case DOUBLE:
            case LONG:
            case 134:
            case 135:
            case 136:
            case 138:
                {
                alt134=1;
                }
                break;
            case 137:
            case 139:
            case 140:
                {
                alt134=2;
                }
                break;
            case IDENT:
                {
                alt134=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 134, 0, input);

                throw nvae;

            }

            switch (alt134) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3182:9: boxed_type
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_boxed_type_in_java_ident10663);
                    boxed_type421=boxed_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, boxed_type421.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3183:9: limited_type
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_limited_type_in_java_ident10673);
                    limited_type422=limited_type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, limited_type422.getTree());

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3184:9: IDENT
                    {
                    root_0 = (Object)adaptor.nil();


                    IDENT423=(Token)match(input,IDENT,FOLLOW_IDENT_in_java_ident10683); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    IDENT423_tree = 
                    (Object)adaptor.create(IDENT423)
                    ;
                    adaptor.addChild(root_0, IDENT423_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 116, java_ident_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "java_ident"


    public static class java_method_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "java_method"
    // com/senseidb/bql/parsers/BQL.g:3189:1: java_method : ({...}? CONTAINS | IDENT );
    public final BQLParser.java_method_return java_method() throws RecognitionException {
        BQLParser.java_method_return retval = new BQLParser.java_method_return();
        retval.start = input.LT(1);

        int java_method_StartIndex = input.index();

        Object root_0 = null;

        Token CONTAINS424=null;
        Token IDENT425=null;

        Object CONTAINS424_tree=null;
        Object IDENT425_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 117) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3190:5: ({...}? CONTAINS | IDENT )
            int alt135=2;
            switch ( input.LA(1) ) {
            case CONTAINS:
                {
                alt135=1;
                }
                break;
            case IDENT:
                {
                alt135=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 135, 0, input);

                throw nvae;

            }

            switch (alt135) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3190:9: {...}? CONTAINS
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "contains".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "java_method", " \"contains\".equals(input.LT(1).getText()) ");
                    }

                    CONTAINS424=(Token)match(input,CONTAINS,FOLLOW_CONTAINS_in_java_method10706); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    CONTAINS424_tree = 
                    (Object)adaptor.create(CONTAINS424)
                    ;
                    adaptor.addChild(root_0, CONTAINS424_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3191:9: IDENT
                    {
                    root_0 = (Object)adaptor.nil();


                    IDENT425=(Token)match(input,IDENT,FOLLOW_IDENT_in_java_method10716); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    IDENT425_tree = 
                    (Object)adaptor.create(IDENT425)
                    ;
                    adaptor.addChild(root_0, IDENT425_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 117, java_method_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "java_method"


    public static class identifier_suffix_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "identifier_suffix"
    // com/senseidb/bql/parsers/BQL.g:3194:1: identifier_suffix : ( ( '[' ']' )+ '.' 'class' | arguments | '.' 'class' | '.' 'this' | '.' 'super' arguments );
    public final BQLParser.identifier_suffix_return identifier_suffix() throws RecognitionException {
        BQLParser.identifier_suffix_return retval = new BQLParser.identifier_suffix_return();
        retval.start = input.LT(1);

        int identifier_suffix_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal426=null;
        Token char_literal427=null;
        Token char_literal428=null;
        Token string_literal429=null;
        Token char_literal431=null;
        Token string_literal432=null;
        Token char_literal433=null;
        Token string_literal434=null;
        Token char_literal435=null;
        Token string_literal436=null;
        BQLParser.arguments_return arguments430 =null;

        BQLParser.arguments_return arguments437 =null;


        Object char_literal426_tree=null;
        Object char_literal427_tree=null;
        Object char_literal428_tree=null;
        Object string_literal429_tree=null;
        Object char_literal431_tree=null;
        Object string_literal432_tree=null;
        Object char_literal433_tree=null;
        Object string_literal434_tree=null;
        Object char_literal435_tree=null;
        Object string_literal436_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 118) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3195:5: ( ( '[' ']' )+ '.' 'class' | arguments | '.' 'class' | '.' 'this' | '.' 'super' arguments )
            int alt137=5;
            switch ( input.LA(1) ) {
            case 141:
                {
                alt137=1;
                }
                break;
            case LPAR:
                {
                alt137=2;
                }
                break;
            case 129:
                {
                switch ( input.LA(2) ) {
                case 148:
                    {
                    alt137=3;
                    }
                    break;
                case 163:
                    {
                    alt137=4;
                    }
                    break;
                case 161:
                    {
                    alt137=5;
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 137, 3, input);

                    throw nvae;

                }

                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 137, 0, input);

                throw nvae;

            }

            switch (alt137) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3195:9: ( '[' ']' )+ '.' 'class'
                    {
                    root_0 = (Object)adaptor.nil();


                    // com/senseidb/bql/parsers/BQL.g:3195:9: ( '[' ']' )+
                    int cnt136=0;
                    loop136:
                    do {
                        int alt136=2;
                        switch ( input.LA(1) ) {
                        case 141:
                            {
                            alt136=1;
                            }
                            break;

                        }

                        switch (alt136) {
                    	case 1 :
                    	    // com/senseidb/bql/parsers/BQL.g:3195:10: '[' ']'
                    	    {
                    	    char_literal426=(Token)match(input,141,FOLLOW_141_in_identifier_suffix10736); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal426_tree = 
                    	    (Object)adaptor.create(char_literal426)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal426_tree);
                    	    }

                    	    char_literal427=(Token)match(input,142,FOLLOW_142_in_identifier_suffix10738); if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) {
                    	    char_literal427_tree = 
                    	    (Object)adaptor.create(char_literal427)
                    	    ;
                    	    adaptor.addChild(root_0, char_literal427_tree);
                    	    }

                    	    }
                    	    break;

                    	default :
                    	    if ( cnt136 >= 1 ) break loop136;
                    	    if (state.backtracking>0) {state.failed=true; return retval;}
                                EarlyExitException eee =
                                    new EarlyExitException(136, input);
                                throw eee;
                        }
                        cnt136++;
                    } while (true);


                    char_literal428=(Token)match(input,129,FOLLOW_129_in_identifier_suffix10742); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal428_tree = 
                    (Object)adaptor.create(char_literal428)
                    ;
                    adaptor.addChild(root_0, char_literal428_tree);
                    }

                    string_literal429=(Token)match(input,148,FOLLOW_148_in_identifier_suffix10744); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal429_tree = 
                    (Object)adaptor.create(string_literal429)
                    ;
                    adaptor.addChild(root_0, string_literal429_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3196:9: arguments
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_arguments_in_identifier_suffix10754);
                    arguments430=arguments();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, arguments430.getTree());

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3197:9: '.' 'class'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal431=(Token)match(input,129,FOLLOW_129_in_identifier_suffix10764); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal431_tree = 
                    (Object)adaptor.create(char_literal431)
                    ;
                    adaptor.addChild(root_0, char_literal431_tree);
                    }

                    string_literal432=(Token)match(input,148,FOLLOW_148_in_identifier_suffix10766); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal432_tree = 
                    (Object)adaptor.create(string_literal432)
                    ;
                    adaptor.addChild(root_0, string_literal432_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:3198:9: '.' 'this'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal433=(Token)match(input,129,FOLLOW_129_in_identifier_suffix10776); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal433_tree = 
                    (Object)adaptor.create(char_literal433)
                    ;
                    adaptor.addChild(root_0, char_literal433_tree);
                    }

                    string_literal434=(Token)match(input,163,FOLLOW_163_in_identifier_suffix10778); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal434_tree = 
                    (Object)adaptor.create(string_literal434)
                    ;
                    adaptor.addChild(root_0, string_literal434_tree);
                    }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:3199:9: '.' 'super' arguments
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal435=(Token)match(input,129,FOLLOW_129_in_identifier_suffix10788); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal435_tree = 
                    (Object)adaptor.create(char_literal435)
                    ;
                    adaptor.addChild(root_0, char_literal435_tree);
                    }

                    string_literal436=(Token)match(input,161,FOLLOW_161_in_identifier_suffix10790); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal436_tree = 
                    (Object)adaptor.create(string_literal436)
                    ;
                    adaptor.addChild(root_0, string_literal436_tree);
                    }

                    pushFollow(FOLLOW_arguments_in_identifier_suffix10792);
                    arguments437=arguments();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, arguments437.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 118, identifier_suffix_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "identifier_suffix"


    public static class literal_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "literal"
    // com/senseidb/bql/parsers/BQL.g:3202:1: literal : ( integer_literal | REAL | FLOATING_POINT_LITERAL | CHARACTER_LITERAL | STRING_LITERAL | boolean_literal |{...}? NULL );
    public final BQLParser.literal_return literal() throws RecognitionException {
        BQLParser.literal_return retval = new BQLParser.literal_return();
        retval.start = input.LT(1);

        int literal_StartIndex = input.index();

        Object root_0 = null;

        Token REAL439=null;
        Token FLOATING_POINT_LITERAL440=null;
        Token CHARACTER_LITERAL441=null;
        Token STRING_LITERAL442=null;
        Token NULL444=null;
        BQLParser.integer_literal_return integer_literal438 =null;

        BQLParser.boolean_literal_return boolean_literal443 =null;


        Object REAL439_tree=null;
        Object FLOATING_POINT_LITERAL440_tree=null;
        Object CHARACTER_LITERAL441_tree=null;
        Object STRING_LITERAL442_tree=null;
        Object NULL444_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 119) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3203:5: ( integer_literal | REAL | FLOATING_POINT_LITERAL | CHARACTER_LITERAL | STRING_LITERAL | boolean_literal |{...}? NULL )
            int alt138=7;
            switch ( input.LA(1) ) {
            case HEX_LITERAL:
            case INTEGER:
            case OCTAL_LITERAL:
                {
                alt138=1;
                }
                break;
            case REAL:
                {
                alt138=2;
                }
                break;
            case FLOATING_POINT_LITERAL:
                {
                alt138=3;
                }
                break;
            case CHARACTER_LITERAL:
                {
                alt138=4;
                }
                break;
            case STRING_LITERAL:
                {
                alt138=5;
                }
                break;
            case FALSE:
            case TRUE:
                {
                alt138=6;
                }
                break;
            case NULL:
                {
                alt138=7;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 138, 0, input);

                throw nvae;

            }

            switch (alt138) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3203:9: integer_literal
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_integer_literal_in_literal10812);
                    integer_literal438=integer_literal();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, integer_literal438.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3204:9: REAL
                    {
                    root_0 = (Object)adaptor.nil();


                    REAL439=(Token)match(input,REAL,FOLLOW_REAL_in_literal10822); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    REAL439_tree = 
                    (Object)adaptor.create(REAL439)
                    ;
                    adaptor.addChild(root_0, REAL439_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3205:9: FLOATING_POINT_LITERAL
                    {
                    root_0 = (Object)adaptor.nil();


                    FLOATING_POINT_LITERAL440=(Token)match(input,FLOATING_POINT_LITERAL,FOLLOW_FLOATING_POINT_LITERAL_in_literal10832); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    FLOATING_POINT_LITERAL440_tree = 
                    (Object)adaptor.create(FLOATING_POINT_LITERAL440)
                    ;
                    adaptor.addChild(root_0, FLOATING_POINT_LITERAL440_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:3206:9: CHARACTER_LITERAL
                    {
                    root_0 = (Object)adaptor.nil();


                    CHARACTER_LITERAL441=(Token)match(input,CHARACTER_LITERAL,FOLLOW_CHARACTER_LITERAL_in_literal10842); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    CHARACTER_LITERAL441_tree = 
                    (Object)adaptor.create(CHARACTER_LITERAL441)
                    ;
                    adaptor.addChild(root_0, CHARACTER_LITERAL441_tree);
                    }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:3207:9: STRING_LITERAL
                    {
                    root_0 = (Object)adaptor.nil();


                    STRING_LITERAL442=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_literal10852); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    STRING_LITERAL442_tree = 
                    (Object)adaptor.create(STRING_LITERAL442)
                    ;
                    adaptor.addChild(root_0, STRING_LITERAL442_tree);
                    }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:3208:9: boolean_literal
                    {
                    root_0 = (Object)adaptor.nil();


                    pushFollow(FOLLOW_boolean_literal_in_literal10862);
                    boolean_literal443=boolean_literal();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, boolean_literal443.getTree());

                    }
                    break;
                case 7 :
                    // com/senseidb/bql/parsers/BQL.g:3209:9: {...}? NULL
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "null".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "literal", " \"null\".equals(input.LT(1).getText()) ");
                    }

                    NULL444=(Token)match(input,NULL,FOLLOW_NULL_in_literal10874); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    NULL444_tree = 
                    (Object)adaptor.create(NULL444)
                    ;
                    adaptor.addChild(root_0, NULL444_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 119, literal_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "literal"


    public static class integer_literal_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "integer_literal"
    // com/senseidb/bql/parsers/BQL.g:3212:1: integer_literal : ( HEX_LITERAL | OCTAL_LITERAL | INTEGER );
    public final BQLParser.integer_literal_return integer_literal() throws RecognitionException {
        BQLParser.integer_literal_return retval = new BQLParser.integer_literal_return();
        retval.start = input.LT(1);

        int integer_literal_StartIndex = input.index();

        Object root_0 = null;

        Token set445=null;

        Object set445_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 120) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3213:5: ( HEX_LITERAL | OCTAL_LITERAL | INTEGER )
            // com/senseidb/bql/parsers/BQL.g:
            {
            root_0 = (Object)adaptor.nil();


            set445=(Token)input.LT(1);

            if ( input.LA(1)==HEX_LITERAL||input.LA(1)==INTEGER||input.LA(1)==OCTAL_LITERAL ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, 
                (Object)adaptor.create(set445)
                );
                state.errorRecovery=false;
                state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 120, integer_literal_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "integer_literal"


    public static class boolean_literal_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "boolean_literal"
    // com/senseidb/bql/parsers/BQL.g:3218:1: boolean_literal : ({...}? TRUE |{...}? FALSE );
    public final BQLParser.boolean_literal_return boolean_literal() throws RecognitionException {
        BQLParser.boolean_literal_return retval = new BQLParser.boolean_literal_return();
        retval.start = input.LT(1);

        int boolean_literal_StartIndex = input.index();

        Object root_0 = null;

        Token TRUE446=null;
        Token FALSE447=null;

        Object TRUE446_tree=null;
        Object FALSE447_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 121) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3219:5: ({...}? TRUE |{...}? FALSE )
            int alt139=2;
            switch ( input.LA(1) ) {
            case TRUE:
                {
                alt139=1;
                }
                break;
            case FALSE:
                {
                alt139=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 139, 0, input);

                throw nvae;

            }

            switch (alt139) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3219:9: {...}? TRUE
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "true".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "boolean_literal", " \"true\".equals(input.LT(1).getText()) ");
                    }

                    TRUE446=(Token)match(input,TRUE,FOLLOW_TRUE_in_boolean_literal10934); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    TRUE446_tree = 
                    (Object)adaptor.create(TRUE446)
                    ;
                    adaptor.addChild(root_0, TRUE446_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3220:9: {...}? FALSE
                    {
                    root_0 = (Object)adaptor.nil();


                    if ( !(( "false".equals(input.LT(1).getText()) )) ) {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        throw new FailedPredicateException(input, "boolean_literal", " \"false\".equals(input.LT(1).getText()) ");
                    }

                    FALSE447=(Token)match(input,FALSE,FOLLOW_FALSE_in_boolean_literal10946); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    FALSE447_tree = 
                    (Object)adaptor.create(FALSE447)
                    ;
                    adaptor.addChild(root_0, FALSE447_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 121, boolean_literal_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "boolean_literal"


    public static class selector_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "selector"
    // com/senseidb/bql/parsers/BQL.g:3223:1: selector : ( '.' IDENT ( arguments )? | '.' 'this' | '[' expression ']' );
    public final BQLParser.selector_return selector() throws RecognitionException {
        BQLParser.selector_return retval = new BQLParser.selector_return();
        retval.start = input.LT(1);

        int selector_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal448=null;
        Token IDENT449=null;
        Token char_literal451=null;
        Token string_literal452=null;
        Token char_literal453=null;
        Token char_literal455=null;
        BQLParser.arguments_return arguments450 =null;

        BQLParser.expression_return expression454 =null;


        Object char_literal448_tree=null;
        Object IDENT449_tree=null;
        Object char_literal451_tree=null;
        Object string_literal452_tree=null;
        Object char_literal453_tree=null;
        Object char_literal455_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 122) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3224:5: ( '.' IDENT ( arguments )? | '.' 'this' | '[' expression ']' )
            int alt141=3;
            switch ( input.LA(1) ) {
            case 129:
                {
                switch ( input.LA(2) ) {
                case IDENT:
                    {
                    alt141=1;
                    }
                    break;
                case 163:
                    {
                    alt141=2;
                    }
                    break;
                default:
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 141, 1, input);

                    throw nvae;

                }

                }
                break;
            case 141:
                {
                alt141=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 141, 0, input);

                throw nvae;

            }

            switch (alt141) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3224:9: '.' IDENT ( arguments )?
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal448=(Token)match(input,129,FOLLOW_129_in_selector10965); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal448_tree = 
                    (Object)adaptor.create(char_literal448)
                    ;
                    adaptor.addChild(root_0, char_literal448_tree);
                    }

                    IDENT449=(Token)match(input,IDENT,FOLLOW_IDENT_in_selector10967); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    IDENT449_tree = 
                    (Object)adaptor.create(IDENT449)
                    ;
                    adaptor.addChild(root_0, IDENT449_tree);
                    }

                    // com/senseidb/bql/parsers/BQL.g:3224:19: ( arguments )?
                    int alt140=2;
                    switch ( input.LA(1) ) {
                        case LPAR:
                            {
                            alt140=1;
                            }
                            break;
                    }

                    switch (alt140) {
                        case 1 :
                            // com/senseidb/bql/parsers/BQL.g:3224:19: arguments
                            {
                            pushFollow(FOLLOW_arguments_in_selector10969);
                            arguments450=arguments();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) adaptor.addChild(root_0, arguments450.getTree());

                            }
                            break;

                    }


                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3225:9: '.' 'this'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal451=(Token)match(input,129,FOLLOW_129_in_selector10980); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal451_tree = 
                    (Object)adaptor.create(char_literal451)
                    ;
                    adaptor.addChild(root_0, char_literal451_tree);
                    }

                    string_literal452=(Token)match(input,163,FOLLOW_163_in_selector10982); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    string_literal452_tree = 
                    (Object)adaptor.create(string_literal452)
                    ;
                    adaptor.addChild(root_0, string_literal452_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3226:9: '[' expression ']'
                    {
                    root_0 = (Object)adaptor.nil();


                    char_literal453=(Token)match(input,141,FOLLOW_141_in_selector10992); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal453_tree = 
                    (Object)adaptor.create(char_literal453)
                    ;
                    adaptor.addChild(root_0, char_literal453_tree);
                    }

                    pushFollow(FOLLOW_expression_in_selector10994);
                    expression454=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression454.getTree());

                    char_literal455=(Token)match(input,142,FOLLOW_142_in_selector10996); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    char_literal455_tree = 
                    (Object)adaptor.create(char_literal455)
                    ;
                    adaptor.addChild(root_0, char_literal455_tree);
                    }

                    }
                    break;

            }
            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 122, selector_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "selector"


    public static class arguments_return extends ParserRuleReturnScope {
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "arguments"
    // com/senseidb/bql/parsers/BQL.g:3229:1: arguments : '(' ( expression_list )? ')' ;
    public final BQLParser.arguments_return arguments() throws RecognitionException {
        BQLParser.arguments_return retval = new BQLParser.arguments_return();
        retval.start = input.LT(1);

        int arguments_StartIndex = input.index();

        Object root_0 = null;

        Token char_literal456=null;
        Token char_literal458=null;
        BQLParser.expression_list_return expression_list457 =null;


        Object char_literal456_tree=null;
        Object char_literal458_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 123) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3230:5: ( '(' ( expression_list )? ')' )
            // com/senseidb/bql/parsers/BQL.g:3230:9: '(' ( expression_list )? ')'
            {
            root_0 = (Object)adaptor.nil();


            char_literal456=(Token)match(input,LPAR,FOLLOW_LPAR_in_arguments11015); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal456_tree = 
            (Object)adaptor.create(char_literal456)
            ;
            adaptor.addChild(root_0, char_literal456_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:3230:13: ( expression_list )?
            int alt142=2;
            switch ( input.LA(1) ) {
                case BOOLEAN:
                case BYTE:
                case CHARACTER_LITERAL:
                case CONTAINS:
                case DOUBLE:
                case FALSE:
                case FLOATING_POINT_LITERAL:
                case HEX_LITERAL:
                case IDENT:
                case INTEGER:
                case LONG:
                case LPAR:
                case NULL:
                case OCTAL_LITERAL:
                case REAL:
                case STRING_LITERAL:
                case TRUE:
                case 114:
                case 123:
                case 124:
                case 126:
                case 127:
                case 134:
                case 135:
                case 136:
                case 137:
                case 138:
                case 139:
                case 140:
                case 170:
                    {
                    alt142=1;
                    }
                    break;
            }

            switch (alt142) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3230:13: expression_list
                    {
                    pushFollow(FOLLOW_expression_list_in_arguments11017);
                    expression_list457=expression_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression_list457.getTree());

                    }
                    break;

            }


            char_literal458=(Token)match(input,RPAR,FOLLOW_RPAR_in_arguments11020); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            char_literal458_tree = 
            (Object)adaptor.create(char_literal458)
            ;
            adaptor.addChild(root_0, char_literal458_tree);
            }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 123, arguments_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "arguments"


    public static class relevance_model_clause_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "relevance_model_clause"
    // com/senseidb/bql/parsers/BQL.g:3233:1: relevance_model_clause returns [JSONObject json] : USING RELEVANCE MODEL IDENT prop_list[KeyType.STRING_LITERAL_AND_IDENT] (model= relevance_model )? ;
    public final BQLParser.relevance_model_clause_return relevance_model_clause() throws RecognitionException {
        BQLParser.relevance_model_clause_return retval = new BQLParser.relevance_model_clause_return();
        retval.start = input.LT(1);

        int relevance_model_clause_StartIndex = input.index();

        Object root_0 = null;

        Token USING459=null;
        Token RELEVANCE460=null;
        Token MODEL461=null;
        Token IDENT462=null;
        BQLParser.relevance_model_return model =null;

        BQLParser.prop_list_return prop_list463 =null;


        Object USING459_tree=null;
        Object RELEVANCE460_tree=null;
        Object MODEL461_tree=null;
        Object IDENT462_tree=null;


            retval.json = new FastJSONObject();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 124) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3237:5: ( USING RELEVANCE MODEL IDENT prop_list[KeyType.STRING_LITERAL_AND_IDENT] (model= relevance_model )? )
            // com/senseidb/bql/parsers/BQL.g:3237:9: USING RELEVANCE MODEL IDENT prop_list[KeyType.STRING_LITERAL_AND_IDENT] (model= relevance_model )?
            {
            root_0 = (Object)adaptor.nil();


            USING459=(Token)match(input,USING,FOLLOW_USING_in_relevance_model_clause11052); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            USING459_tree = 
            (Object)adaptor.create(USING459)
            ;
            adaptor.addChild(root_0, USING459_tree);
            }

            RELEVANCE460=(Token)match(input,RELEVANCE,FOLLOW_RELEVANCE_in_relevance_model_clause11054); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RELEVANCE460_tree = 
            (Object)adaptor.create(RELEVANCE460)
            ;
            adaptor.addChild(root_0, RELEVANCE460_tree);
            }

            MODEL461=(Token)match(input,MODEL,FOLLOW_MODEL_in_relevance_model_clause11056); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            MODEL461_tree = 
            (Object)adaptor.create(MODEL461)
            ;
            adaptor.addChild(root_0, MODEL461_tree);
            }

            IDENT462=(Token)match(input,IDENT,FOLLOW_IDENT_in_relevance_model_clause11058); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            IDENT462_tree = 
            (Object)adaptor.create(IDENT462)
            ;
            adaptor.addChild(root_0, IDENT462_tree);
            }

            pushFollow(FOLLOW_prop_list_in_relevance_model_clause11060);
            prop_list463=prop_list(KeyType.STRING_LITERAL_AND_IDENT);

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, prop_list463.getTree());

            // com/senseidb/bql/parsers/BQL.g:3237:86: (model= relevance_model )?
            int alt143=2;
            switch ( input.LA(1) ) {
                case DEFINED:
                    {
                    alt143=1;
                    }
                    break;
            }

            switch (alt143) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3237:86: model= relevance_model
                    {
                    pushFollow(FOLLOW_relevance_model_in_relevance_model_clause11065);
                    model=relevance_model();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, model.getTree());

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        try {
                            if (model == null) {
                                retval.json.put("predefined_model", (IDENT462!=null?IDENT462.getText():null));
                                retval.json.put("values", (prop_list463!=null?prop_list463.json:null));
                            }
                            else {
                                JSONObject modelInfo = (model!=null?model.json:null);
                                JSONObject modelJson = new FastJSONObject();
                                modelJson.put("function", (model!=null?model.functionBody:null));

                                JSONArray funcParams = modelInfo.optJSONArray("function_params");
                                if (funcParams != null) {
                                    modelJson.put("function_params", funcParams);
                                }

                                JSONObject facets = modelInfo.optJSONObject("facets");
                                if (facets != null) {
                                    modelJson.put("facets", facets);
                                }

                                JSONObject variables = modelInfo.optJSONObject("variables");
                                if (variables != null) {
                                    modelJson.put("variables", variables);
                                }

                                retval.json.put("model", modelJson);
                                retval.json.put("values", (prop_list463!=null?prop_list463.json:null));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "relevance_model_clause",
                                                               "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 124, relevance_model_clause_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "relevance_model_clause"


    public static class facet_param_list_return extends ParserRuleReturnScope {
        public JSONObject json;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "facet_param_list"
    // com/senseidb/bql/parsers/BQL.g:3275:1: facet_param_list returns [JSONObject json] : p= facet_param ( COMMA p= facet_param )* ;
    public final BQLParser.facet_param_list_return facet_param_list() throws RecognitionException {
        BQLParser.facet_param_list_return retval = new BQLParser.facet_param_list_return();
        retval.start = input.LT(1);

        int facet_param_list_StartIndex = input.index();

        Object root_0 = null;

        Token COMMA464=null;
        BQLParser.facet_param_return p =null;


        Object COMMA464_tree=null;


            retval.json = new FastJSONObject();

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 125) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3279:5: (p= facet_param ( COMMA p= facet_param )* )
            // com/senseidb/bql/parsers/BQL.g:3279:9: p= facet_param ( COMMA p= facet_param )*
            {
            root_0 = (Object)adaptor.nil();


            pushFollow(FOLLOW_facet_param_in_facet_param_list11106);
            p=facet_param();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

            if ( state.backtracking==0 ) {
                        try {
                            if (!retval.json.has((p!=null?p.facet:null))) {
                                retval.json.put((p!=null?p.facet:null), (p!=null?p.param:null));
                            }
                            else {
                                JSONObject currentParam = (JSONObject) retval.json.get((p!=null?p.facet:null));
                                String paramName = (String) (p!=null?p.param:null).keys().next();
                                currentParam.put(paramName, (p!=null?p.param:null).get(paramName));
                            }
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "facet_param_list", "JSONException: " + err.getMessage());
                        }
                    }

            // com/senseidb/bql/parsers/BQL.g:3295:9: ( COMMA p= facet_param )*
            loop144:
            do {
                int alt144=2;
                switch ( input.LA(1) ) {
                case COMMA:
                    {
                    alt144=1;
                    }
                    break;

                }

                switch (alt144) {
            	case 1 :
            	    // com/senseidb/bql/parsers/BQL.g:3295:10: COMMA p= facet_param
            	    {
            	    COMMA464=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_param_list11127); if (state.failed) return retval;
            	    if ( state.backtracking==0 ) {
            	    COMMA464_tree = 
            	    (Object)adaptor.create(COMMA464)
            	    ;
            	    adaptor.addChild(root_0, COMMA464_tree);
            	    }

            	    pushFollow(FOLLOW_facet_param_in_facet_param_list11131);
            	    p=facet_param();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, p.getTree());

            	    if ( state.backtracking==0 ) {
            	                    try {
            	                        if (!retval.json.has((p!=null?p.facet:null))) {
            	                            retval.json.put((p!=null?p.facet:null), (p!=null?p.param:null));
            	                        }
            	                        else {
            	                            JSONObject currentParam = (JSONObject) retval.json.get((p!=null?p.facet:null));
            	                            String paramName = (String) (p!=null?p.param:null).keys().next();
            	                            currentParam.put(paramName, (p!=null?p.param:null).get(paramName));
            	                        }
            	                    }
            	                    catch (JSONException err) {
            	                        throw new FailedPredicateException(input, "facet_param_list", "JSONException: " + err.getMessage());
            	                    }
            	                }

            	    }
            	    break;

            	default :
            	    break loop144;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 125, facet_param_list_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "facet_param_list"


    public static class facet_param_return extends ParserRuleReturnScope {
        public String facet;
        public JSONObject param;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "facet_param"
    // com/senseidb/bql/parsers/BQL.g:3314:1: facet_param returns [String facet, JSONObject param] : LPAR column_name COMMA STRING_LITERAL COMMA facet_param_type COMMA (val= value |valList= non_variable_value_list ) RPAR ;
    public final BQLParser.facet_param_return facet_param() throws RecognitionException {
        BQLParser.facet_param_return retval = new BQLParser.facet_param_return();
        retval.start = input.LT(1);

        int facet_param_StartIndex = input.index();

        Object root_0 = null;

        Token LPAR465=null;
        Token COMMA467=null;
        Token STRING_LITERAL468=null;
        Token COMMA469=null;
        Token COMMA471=null;
        Token RPAR472=null;
        BQLParser.value_return val =null;

        BQLParser.non_variable_value_list_return valList =null;

        BQLParser.column_name_return column_name466 =null;

        BQLParser.facet_param_type_return facet_param_type470 =null;


        Object LPAR465_tree=null;
        Object COMMA467_tree=null;
        Object STRING_LITERAL468_tree=null;
        Object COMMA469_tree=null;
        Object COMMA471_tree=null;
        Object RPAR472_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 126) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3315:5: ( LPAR column_name COMMA STRING_LITERAL COMMA facet_param_type COMMA (val= value |valList= non_variable_value_list ) RPAR )
            // com/senseidb/bql/parsers/BQL.g:3315:9: LPAR column_name COMMA STRING_LITERAL COMMA facet_param_type COMMA (val= value |valList= non_variable_value_list ) RPAR
            {
            root_0 = (Object)adaptor.nil();


            LPAR465=(Token)match(input,LPAR,FOLLOW_LPAR_in_facet_param11179); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            LPAR465_tree = 
            (Object)adaptor.create(LPAR465)
            ;
            adaptor.addChild(root_0, LPAR465_tree);
            }

            pushFollow(FOLLOW_column_name_in_facet_param11181);
            column_name466=column_name();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, column_name466.getTree());

            COMMA467=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_param11183); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            COMMA467_tree = 
            (Object)adaptor.create(COMMA467)
            ;
            adaptor.addChild(root_0, COMMA467_tree);
            }

            STRING_LITERAL468=(Token)match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_facet_param11185); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            STRING_LITERAL468_tree = 
            (Object)adaptor.create(STRING_LITERAL468)
            ;
            adaptor.addChild(root_0, STRING_LITERAL468_tree);
            }

            COMMA469=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_param11187); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            COMMA469_tree = 
            (Object)adaptor.create(COMMA469)
            ;
            adaptor.addChild(root_0, COMMA469_tree);
            }

            pushFollow(FOLLOW_facet_param_type_in_facet_param11189);
            facet_param_type470=facet_param_type();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, facet_param_type470.getTree());

            COMMA471=(Token)match(input,COMMA,FOLLOW_COMMA_in_facet_param11191); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            COMMA471_tree = 
            (Object)adaptor.create(COMMA471)
            ;
            adaptor.addChild(root_0, COMMA471_tree);
            }

            // com/senseidb/bql/parsers/BQL.g:3315:76: (val= value |valList= non_variable_value_list )
            int alt145=2;
            switch ( input.LA(1) ) {
            case EOF:
            case AGO:
            case AND:
            case BROWSE:
            case DATE:
            case DISTINCT:
            case EXECUTE:
            case FALSE:
            case FETCHING:
            case GIVEN:
            case GROUP:
            case INTEGER:
            case LIMIT:
            case NOW:
            case OR:
            case ORDER:
            case REAL:
            case ROUTE:
            case RPAR:
            case SEMI:
            case STRING_LITERAL:
            case TRUE:
            case USING:
            case VARIABLE:
                {
                alt145=1;
                }
                break;
            case LPAR:
                {
                alt145=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 145, 0, input);

                throw nvae;

            }

            switch (alt145) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3315:77: val= value
                    {
                    pushFollow(FOLLOW_value_in_facet_param11196);
                    val=value();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, val.getTree());

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3315:89: valList= non_variable_value_list
                    {
                    pushFollow(FOLLOW_non_variable_value_list_in_facet_param11202);
                    valList=non_variable_value_list();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, valList.getTree());

                    }
                    break;

            }


            RPAR472=(Token)match(input,RPAR,FOLLOW_RPAR_in_facet_param11205); if (state.failed) return retval;
            if ( state.backtracking==0 ) {
            RPAR472_tree = 
            (Object)adaptor.create(RPAR472)
            ;
            adaptor.addChild(root_0, RPAR472_tree);
            }

            if ( state.backtracking==0 ) {
                        retval.facet = (column_name466!=null?column_name466.text:null); // XXX Check error here?
                        try {
                            Object valArray = null;
                            if (val != null) {
                                String varName = (val!=null?input.toString(val.start,val.stop):null);
                                if (varName.matches("\\$[^$].*")) {
                                    // Here "value" is a variable.  In this case, it
                                    // is REQUIRED that the variable should be
                                    // replaced by a list, NOT a scalar value.
                                    valArray = varName;
                                }
                                else {
                                    valArray = new FastJSONArray().put(val.val);
                                }
                            }
                            else {
                                valArray = (valList!=null?valList.json:null);
                            }

                            String orig = (STRING_LITERAL468!=null?STRING_LITERAL468.getText():null);
                            orig = orig.substring(1, orig.length() - 1);
                            retval.param = new FastJSONObject().put(orig,
                                                              new FastJSONObject().put("type", (facet_param_type470!=null?facet_param_type470.paramType:null))
                                                                                  .put("values", valArray));
                        }
                        catch (JSONException err) {
                            throw new FailedPredicateException(input, "facet_param", "JSONException: " + err.getMessage());
                        }
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 126, facet_param_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "facet_param"


    public static class facet_param_type_return extends ParserRuleReturnScope {
        public String paramType;
        Object tree;
        public Object getTree() { return tree; }
    };


    // $ANTLR start "facet_param_type"
    // com/senseidb/bql/parsers/BQL.g:3348:1: facet_param_type returns [String paramType] : (t= BOOLEAN |t= INT |t= LONG |t= STRING |t= BYTEARRAY |t= DOUBLE ) ;
    public final BQLParser.facet_param_type_return facet_param_type() throws RecognitionException {
        BQLParser.facet_param_type_return retval = new BQLParser.facet_param_type_return();
        retval.start = input.LT(1);

        int facet_param_type_StartIndex = input.index();

        Object root_0 = null;

        Token t=null;

        Object t_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 127) ) { return retval; }

            // com/senseidb/bql/parsers/BQL.g:3349:5: ( (t= BOOLEAN |t= INT |t= LONG |t= STRING |t= BYTEARRAY |t= DOUBLE ) )
            // com/senseidb/bql/parsers/BQL.g:3349:9: (t= BOOLEAN |t= INT |t= LONG |t= STRING |t= BYTEARRAY |t= DOUBLE )
            {
            root_0 = (Object)adaptor.nil();


            // com/senseidb/bql/parsers/BQL.g:3349:9: (t= BOOLEAN |t= INT |t= LONG |t= STRING |t= BYTEARRAY |t= DOUBLE )
            int alt146=6;
            switch ( input.LA(1) ) {
            case BOOLEAN:
                {
                alt146=1;
                }
                break;
            case INT:
                {
                alt146=2;
                }
                break;
            case LONG:
                {
                alt146=3;
                }
                break;
            case STRING:
                {
                alt146=4;
                }
                break;
            case BYTEARRAY:
                {
                alt146=5;
                }
                break;
            case DOUBLE:
                {
                alt146=6;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 146, 0, input);

                throw nvae;

            }

            switch (alt146) {
                case 1 :
                    // com/senseidb/bql/parsers/BQL.g:3349:10: t= BOOLEAN
                    {
                    t=(Token)match(input,BOOLEAN,FOLLOW_BOOLEAN_in_facet_param_type11274); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t_tree = 
                    (Object)adaptor.create(t)
                    ;
                    adaptor.addChild(root_0, t_tree);
                    }

                    }
                    break;
                case 2 :
                    // com/senseidb/bql/parsers/BQL.g:3349:22: t= INT
                    {
                    t=(Token)match(input,INT,FOLLOW_INT_in_facet_param_type11280); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t_tree = 
                    (Object)adaptor.create(t)
                    ;
                    adaptor.addChild(root_0, t_tree);
                    }

                    }
                    break;
                case 3 :
                    // com/senseidb/bql/parsers/BQL.g:3349:30: t= LONG
                    {
                    t=(Token)match(input,LONG,FOLLOW_LONG_in_facet_param_type11286); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t_tree = 
                    (Object)adaptor.create(t)
                    ;
                    adaptor.addChild(root_0, t_tree);
                    }

                    }
                    break;
                case 4 :
                    // com/senseidb/bql/parsers/BQL.g:3349:39: t= STRING
                    {
                    t=(Token)match(input,STRING,FOLLOW_STRING_in_facet_param_type11292); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t_tree = 
                    (Object)adaptor.create(t)
                    ;
                    adaptor.addChild(root_0, t_tree);
                    }

                    }
                    break;
                case 5 :
                    // com/senseidb/bql/parsers/BQL.g:3349:50: t= BYTEARRAY
                    {
                    t=(Token)match(input,BYTEARRAY,FOLLOW_BYTEARRAY_in_facet_param_type11298); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t_tree = 
                    (Object)adaptor.create(t)
                    ;
                    adaptor.addChild(root_0, t_tree);
                    }

                    }
                    break;
                case 6 :
                    // com/senseidb/bql/parsers/BQL.g:3349:64: t= DOUBLE
                    {
                    t=(Token)match(input,DOUBLE,FOLLOW_DOUBLE_in_facet_param_type11304); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    t_tree = 
                    (Object)adaptor.create(t)
                    ;
                    adaptor.addChild(root_0, t_tree);
                    }

                    }
                    break;

            }


            if ( state.backtracking==0 ) {
                        retval.paramType = (t!=null?t.getText():null);
                    }

            }

            retval.stop = input.LT(-1);


            if ( state.backtracking==0 ) {

            retval.tree = (Object)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }

            catch (RecognitionException err) {
                throw err;
            }

        finally {
        	// do for sure before leaving
            if ( state.backtracking>0 ) { memoize(input, 127, facet_param_type_StartIndex); }

        }
        return retval;
    }
    // $ANTLR end "facet_param_type"

    // $ANTLR start synpred105_BQL
    public final void synpred105_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2632:11: ({...}? => STRING_LITERAL )
        // com/senseidb/bql/parsers/BQL.g:2632:11: {...}? => STRING_LITERAL
        {
        if ( !(( ((key_value_pair_scope)key_value_pair_stack.peek()).type == KeyType.STRING_LITERAL ||
                    ((key_value_pair_scope)key_value_pair_stack.peek()).type == KeyType.STRING_LITERAL_AND_IDENT)) ) {
            if (state.backtracking>0) {state.failed=true; return ;}
            throw new FailedPredicateException(input, "synpred105_BQL", " $key_value_pair::type == KeyType.STRING_LITERAL ||\n            $key_value_pair::type == KeyType.STRING_LITERAL_AND_IDENT");
        }

        match(input,STRING_LITERAL,FOLLOW_STRING_LITERAL_in_synpred105_BQL7715); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred105_BQL

    // $ANTLR start synpred118_BQL
    public final void synpred118_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2711:9: ( primitive_type ( '[' ']' )* )
        // com/senseidb/bql/parsers/BQL.g:2711:9: primitive_type ( '[' ']' )*
        {
        pushFollow(FOLLOW_primitive_type_in_synpred118_BQL8099);
        primitive_type();

        state._fsp--;
        if (state.failed) return ;

        // com/senseidb/bql/parsers/BQL.g:2711:24: ( '[' ']' )*
        loop154:
        do {
            int alt154=2;
            switch ( input.LA(1) ) {
            case 141:
                {
                alt154=1;
                }
                break;

            }

            switch (alt154) {
        	case 1 :
        	    // com/senseidb/bql/parsers/BQL.g:2711:25: '[' ']'
        	    {
        	    match(input,141,FOLLOW_141_in_synpred118_BQL8102); if (state.failed) return ;

        	    match(input,142,FOLLOW_142_in_synpred118_BQL8104); if (state.failed) return ;

        	    }
        	    break;

        	default :
        	    break loop154;
            }
        } while (true);


        }

    }
    // $ANTLR end synpred118_BQL

    // $ANTLR start synpred120_BQL
    public final void synpred120_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2713:9: ( boxed_type ( '[' ']' )* )
        // com/senseidb/bql/parsers/BQL.g:2713:9: boxed_type ( '[' ']' )*
        {
        pushFollow(FOLLOW_boxed_type_in_synpred120_BQL8126);
        boxed_type();

        state._fsp--;
        if (state.failed) return ;

        // com/senseidb/bql/parsers/BQL.g:2713:20: ( '[' ']' )*
        loop155:
        do {
            int alt155=2;
            switch ( input.LA(1) ) {
            case 141:
                {
                alt155=1;
                }
                break;

            }

            switch (alt155) {
        	case 1 :
        	    // com/senseidb/bql/parsers/BQL.g:2713:21: '[' ']'
        	    {
        	    match(input,141,FOLLOW_141_in_synpred120_BQL8129); if (state.failed) return ;

        	    match(input,142,FOLLOW_142_in_synpred120_BQL8131); if (state.failed) return ;

        	    }
        	    break;

        	default :
        	    break loop155;
            }
        } while (true);


        }

    }
    // $ANTLR end synpred120_BQL

    // $ANTLR start synpred148_BQL
    public final void synpred148_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2956:45: ( else_statement )
        // com/senseidb/bql/parsers/BQL.g:2956:45: else_statement
        {
        pushFollow(FOLLOW_else_statement_in_synpred148_BQL9053);
        else_statement();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred148_BQL

    // $ANTLR start synpred161_BQL
    public final void synpred161_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2986:9: ( switch_label )
        // com/senseidb/bql/parsers/BQL.g:2986:9: switch_label
        {
        pushFollow(FOLLOW_switch_label_in_synpred161_BQL9281);
        switch_label();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred161_BQL

    // $ANTLR start synpred163_BQL
    public final void synpred163_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2990:9: ( 'case' constant_expression COLON )
        // com/senseidb/bql/parsers/BQL.g:2990:9: 'case' constant_expression COLON
        {
        match(input,146,FOLLOW_146_in_synpred163_BQL9304); if (state.failed) return ;

        pushFollow(FOLLOW_constant_expression_in_synpred163_BQL9306);
        constant_expression();

        state._fsp--;
        if (state.failed) return ;

        match(input,COLON,FOLLOW_COLON_in_synpred163_BQL9308); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred163_BQL

    // $ANTLR start synpred164_BQL
    public final void synpred164_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2991:9: ( 'case' enum_constant_name COLON )
        // com/senseidb/bql/parsers/BQL.g:2991:9: 'case' enum_constant_name COLON
        {
        match(input,146,FOLLOW_146_in_synpred164_BQL9318); if (state.failed) return ;

        pushFollow(FOLLOW_enum_constant_name_in_synpred164_BQL9320);
        enum_constant_name();

        state._fsp--;
        if (state.failed) return ;

        match(input,COLON,FOLLOW_COLON_in_synpred164_BQL9322); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred164_BQL

    // $ANTLR start synpred165_BQL
    public final void synpred165_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:2997:9: ( enhanced_for_control )
        // com/senseidb/bql/parsers/BQL.g:2997:9: enhanced_for_control
        {
        pushFollow(FOLLOW_enhanced_for_control_in_synpred165_BQL9360);
        enhanced_for_control();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred165_BQL

    // $ANTLR start synpred171_BQL
    public final void synpred171_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3035:33: ( assignment_operator expression )
        // com/senseidb/bql/parsers/BQL.g:3035:33: assignment_operator expression
        {
        pushFollow(FOLLOW_assignment_operator_in_synpred171_BQL9584);
        assignment_operator();

        state._fsp--;
        if (state.failed) return ;

        pushFollow(FOLLOW_expression_in_synpred171_BQL9586);
        expression();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred171_BQL

    // $ANTLR start synpred181_BQL
    public final void synpred181_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3048:9: ( '<' '<' '=' )
        // com/senseidb/bql/parsers/BQL.g:3048:10: '<' '<' '='
        {
        match(input,LT,FOLLOW_LT_in_synpred181_BQL9698); if (state.failed) return ;

        match(input,LT,FOLLOW_LT_in_synpred181_BQL9700); if (state.failed) return ;

        match(input,EQUAL,FOLLOW_EQUAL_in_synpred181_BQL9702); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred181_BQL

    // $ANTLR start synpred182_BQL
    public final void synpred182_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3053:9: ( '>' '>' '>' '=' )
        // com/senseidb/bql/parsers/BQL.g:3053:10: '>' '>' '>' '='
        {
        match(input,GT,FOLLOW_GT_in_synpred182_BQL9738); if (state.failed) return ;

        match(input,GT,FOLLOW_GT_in_synpred182_BQL9740); if (state.failed) return ;

        match(input,GT,FOLLOW_GT_in_synpred182_BQL9742); if (state.failed) return ;

        match(input,EQUAL,FOLLOW_EQUAL_in_synpred182_BQL9744); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred182_BQL

    // $ANTLR start synpred183_BQL
    public final void synpred183_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3060:9: ( '>' '>' '=' )
        // com/senseidb/bql/parsers/BQL.g:3060:10: '>' '>' '='
        {
        match(input,GT,FOLLOW_GT_in_synpred183_BQL9783); if (state.failed) return ;

        match(input,GT,FOLLOW_GT_in_synpred183_BQL9785); if (state.failed) return ;

        match(input,EQUAL,FOLLOW_EQUAL_in_synpred183_BQL9787); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred183_BQL

    // $ANTLR start synpred194_BQL
    public final void synpred194_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3104:9: ( '<' '=' )
        // com/senseidb/bql/parsers/BQL.g:3104:10: '<' '='
        {
        match(input,LT,FOLLOW_LT_in_synpred194_BQL10088); if (state.failed) return ;

        match(input,EQUAL,FOLLOW_EQUAL_in_synpred194_BQL10090); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred194_BQL

    // $ANTLR start synpred195_BQL
    public final void synpred195_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3107:9: ( '>' '=' )
        // com/senseidb/bql/parsers/BQL.g:3107:10: '>' '='
        {
        match(input,GT,FOLLOW_GT_in_synpred195_BQL10122); if (state.failed) return ;

        match(input,EQUAL,FOLLOW_EQUAL_in_synpred195_BQL10124); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred195_BQL

    // $ANTLR start synpred198_BQL
    public final void synpred198_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3119:9: ( '<' '<' )
        // com/senseidb/bql/parsers/BQL.g:3119:10: '<' '<'
        {
        match(input,LT,FOLLOW_LT_in_synpred198_BQL10213); if (state.failed) return ;

        match(input,LT,FOLLOW_LT_in_synpred198_BQL10215); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred198_BQL

    // $ANTLR start synpred199_BQL
    public final void synpred199_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3122:9: ( '>' '>' '>' )
        // com/senseidb/bql/parsers/BQL.g:3122:10: '>' '>' '>'
        {
        match(input,GT,FOLLOW_GT_in_synpred199_BQL10247); if (state.failed) return ;

        match(input,GT,FOLLOW_GT_in_synpred199_BQL10249); if (state.failed) return ;

        match(input,GT,FOLLOW_GT_in_synpred199_BQL10251); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred199_BQL

    // $ANTLR start synpred200_BQL
    public final void synpred200_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3127:9: ( '>' '>' )
        // com/senseidb/bql/parsers/BQL.g:3127:10: '>' '>'
        {
        match(input,GT,FOLLOW_GT_in_synpred200_BQL10287); if (state.failed) return ;

        match(input,GT,FOLLOW_GT_in_synpred200_BQL10289); if (state.failed) return ;

        }

    }
    // $ANTLR end synpred200_BQL

    // $ANTLR start synpred212_BQL
    public final void synpred212_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3151:9: ( cast_expression )
        // com/senseidb/bql/parsers/BQL.g:3151:9: cast_expression
        {
        pushFollow(FOLLOW_cast_expression_in_synpred212_BQL10497);
        cast_expression();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred212_BQL

    // $ANTLR start synpred216_BQL
    public final void synpred216_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3156:8: ( '(' primitive_type ')' unary_expression )
        // com/senseidb/bql/parsers/BQL.g:3156:8: '(' primitive_type ')' unary_expression
        {
        match(input,LPAR,FOLLOW_LPAR_in_synpred216_BQL10535); if (state.failed) return ;

        pushFollow(FOLLOW_primitive_type_in_synpred216_BQL10537);
        primitive_type();

        state._fsp--;
        if (state.failed) return ;

        match(input,RPAR,FOLLOW_RPAR_in_synpred216_BQL10539); if (state.failed) return ;

        pushFollow(FOLLOW_unary_expression_in_synpred216_BQL10541);
        unary_expression();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred216_BQL

    // $ANTLR start synpred217_BQL
    public final void synpred217_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3157:13: ( type )
        // com/senseidb/bql/parsers/BQL.g:3157:13: type
        {
        pushFollow(FOLLOW_type_in_synpred217_BQL10553);
        type();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred217_BQL

    // $ANTLR start synpred220_BQL
    public final void synpred220_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3163:9: ( java_method identifier_suffix )
        // com/senseidb/bql/parsers/BQL.g:3163:9: java_method identifier_suffix
        {
        pushFollow(FOLLOW_java_method_in_synpred220_BQL10604);
        java_method();

        state._fsp--;
        if (state.failed) return ;

        pushFollow(FOLLOW_identifier_suffix_in_synpred220_BQL10606);
        identifier_suffix();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred220_BQL

    // $ANTLR start synpred221_BQL
    public final void synpred221_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3164:21: ( '.' java_method )
        // com/senseidb/bql/parsers/BQL.g:3164:21: '.' java_method
        {
        match(input,129,FOLLOW_129_in_synpred221_BQL10619); if (state.failed) return ;

        pushFollow(FOLLOW_java_method_in_synpred221_BQL10621);
        java_method();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred221_BQL

    // $ANTLR start synpred222_BQL
    public final void synpred222_BQL_fragment() throws RecognitionException {
        // com/senseidb/bql/parsers/BQL.g:3164:39: ( identifier_suffix )
        // com/senseidb/bql/parsers/BQL.g:3164:39: identifier_suffix
        {
        pushFollow(FOLLOW_identifier_suffix_in_synpred222_BQL10625);
        identifier_suffix();

        state._fsp--;
        if (state.failed) return ;

        }

    }
    // $ANTLR end synpred222_BQL

    // Delegated rules

    public final boolean synpred216_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred216_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred171_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred171_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred194_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred194_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred212_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred212_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred181_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred181_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred220_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred220_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred198_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred198_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred165_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred165_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred120_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred120_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred118_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred118_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred222_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred222_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred105_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred105_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred164_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred164_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred183_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred183_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred199_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred199_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred217_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred217_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred148_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred148_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred200_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred200_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred163_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred163_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred182_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred182_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred221_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred221_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred161_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred161_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred195_BQL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred195_BQL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }


    protected DFA8 dfa8 = new DFA8(this);
    protected DFA9 dfa9 = new DFA9(this);
    protected DFA34 dfa34 = new DFA34(this);
    protected DFA48 dfa48 = new DFA48(this);
    protected DFA93 dfa93 = new DFA93(this);
    protected DFA106 dfa106 = new DFA106(this);
    protected DFA107 dfa107 = new DFA107(this);
    static final String DFA8_eotS =
        "\10\uffff";
    static final String DFA8_eofS =
        "\1\uffff\2\5\3\uffff\2\5";
    static final String DFA8_minS =
        "\1\73\2\21\1\uffff\1\73\1\uffff\2\21";
    static final String DFA8_maxS =
        "\1\u009e\2\u0081\1\uffff\1\146\1\uffff\2\u0081";
    static final String DFA8_acceptS =
        "\3\uffff\1\2\1\uffff\1\1\2\uffff";
    static final String DFA8_specialS =
        "\10\uffff}>";
    static final String[] DFA8_transitionS = {
            "\1\1\52\uffff\1\2\67\uffff\1\3",
            "\1\5\6\uffff\1\5\10\uffff\1\5\10\uffff\1\5\4\uffff\1\5\2\uffff"+
            "\3\5\16\uffff\1\5\2\uffff\1\3\17\uffff\1\5\6\uffff\2\5\3\uffff"+
            "\1\5\10\uffff\1\5\3\uffff\1\5\21\uffff\1\4",
            "\1\5\6\uffff\1\5\10\uffff\1\5\10\uffff\1\5\4\uffff\1\5\2\uffff"+
            "\3\5\16\uffff\1\5\2\uffff\1\3\17\uffff\1\5\6\uffff\2\5\3\uffff"+
            "\1\5\10\uffff\1\5\3\uffff\1\5\21\uffff\1\4",
            "",
            "\1\6\52\uffff\1\7",
            "",
            "\1\5\6\uffff\1\5\10\uffff\1\5\10\uffff\1\5\4\uffff\1\5\2\uffff"+
            "\3\5\16\uffff\1\5\2\uffff\1\3\17\uffff\1\5\6\uffff\2\5\3\uffff"+
            "\1\5\10\uffff\1\5\3\uffff\1\5\21\uffff\1\4",
            "\1\5\6\uffff\1\5\10\uffff\1\5\10\uffff\1\5\4\uffff\1\5\2\uffff"+
            "\3\5\16\uffff\1\5\2\uffff\1\3\17\uffff\1\5\6\uffff\2\5\3\uffff"+
            "\1\5\10\uffff\1\5\3\uffff\1\5\21\uffff\1\4"
    };

    static final short[] DFA8_eot = DFA.unpackEncodedString(DFA8_eotS);
    static final short[] DFA8_eof = DFA.unpackEncodedString(DFA8_eofS);
    static final char[] DFA8_min = DFA.unpackEncodedStringToUnsignedChars(DFA8_minS);
    static final char[] DFA8_max = DFA.unpackEncodedStringToUnsignedChars(DFA8_maxS);
    static final short[] DFA8_accept = DFA.unpackEncodedString(DFA8_acceptS);
    static final short[] DFA8_special = DFA.unpackEncodedString(DFA8_specialS);
    static final short[][] DFA8_transition;

    static {
        int numStates = DFA8_transitionS.length;
        DFA8_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA8_transition[i] = DFA.unpackEncodedString(DFA8_transitionS[i]);
        }
    }

    class DFA8 extends DFA {

        public DFA8(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 8;
            this.eot = DFA8_eot;
            this.eof = DFA8_eof;
            this.min = DFA8_min;
            this.max = DFA8_max;
            this.accept = DFA8_accept;
            this.special = DFA8_special;
            this.transition = DFA8_transition;
        }
        public String getDescription() {
            return "1451:10: (col= column_name |agrFunction= aggregation_function )";
        }
    }
    static final String DFA9_eotS =
        "\10\uffff";
    static final String DFA9_eofS =
        "\1\uffff\2\5\3\uffff\2\5";
    static final String DFA9_minS =
        "\1\73\2\21\1\uffff\1\73\1\uffff\2\21";
    static final String DFA9_maxS =
        "\1\u009e\2\u0081\1\uffff\1\146\1\uffff\2\u0081";
    static final String DFA9_acceptS =
        "\3\uffff\1\2\1\uffff\1\1\2\uffff";
    static final String DFA9_specialS =
        "\10\uffff}>";
    static final String[] DFA9_transitionS = {
            "\1\1\52\uffff\1\2\67\uffff\1\3",
            "\1\5\6\uffff\1\5\10\uffff\1\5\10\uffff\1\5\4\uffff\1\5\2\uffff"+
            "\3\5\16\uffff\1\5\2\uffff\1\3\17\uffff\1\5\6\uffff\2\5\3\uffff"+
            "\1\5\10\uffff\1\5\3\uffff\1\5\21\uffff\1\4",
            "\1\5\6\uffff\1\5\10\uffff\1\5\10\uffff\1\5\4\uffff\1\5\2\uffff"+
            "\3\5\16\uffff\1\5\2\uffff\1\3\17\uffff\1\5\6\uffff\2\5\3\uffff"+
            "\1\5\10\uffff\1\5\3\uffff\1\5\21\uffff\1\4",
            "",
            "\1\6\52\uffff\1\7",
            "",
            "\1\5\6\uffff\1\5\10\uffff\1\5\10\uffff\1\5\4\uffff\1\5\2\uffff"+
            "\3\5\16\uffff\1\5\2\uffff\1\3\17\uffff\1\5\6\uffff\2\5\3\uffff"+
            "\1\5\10\uffff\1\5\3\uffff\1\5\21\uffff\1\4",
            "\1\5\6\uffff\1\5\10\uffff\1\5\10\uffff\1\5\4\uffff\1\5\2\uffff"+
            "\3\5\16\uffff\1\5\2\uffff\1\3\17\uffff\1\5\6\uffff\2\5\3\uffff"+
            "\1\5\10\uffff\1\5\3\uffff\1\5\21\uffff\1\4"
    };

    static final short[] DFA9_eot = DFA.unpackEncodedString(DFA9_eotS);
    static final short[] DFA9_eof = DFA.unpackEncodedString(DFA9_eofS);
    static final char[] DFA9_min = DFA.unpackEncodedStringToUnsignedChars(DFA9_minS);
    static final char[] DFA9_max = DFA.unpackEncodedStringToUnsignedChars(DFA9_maxS);
    static final short[] DFA9_accept = DFA.unpackEncodedString(DFA9_acceptS);
    static final short[] DFA9_special = DFA.unpackEncodedString(DFA9_specialS);
    static final short[][] DFA9_transition;

    static {
        int numStates = DFA9_transitionS.length;
        DFA9_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA9_transition[i] = DFA.unpackEncodedString(DFA9_transitionS[i]);
        }
    }

    class DFA9 extends DFA {

        public DFA9(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 9;
            this.eot = DFA9_eot;
            this.eof = DFA9_eof;
            this.min = DFA9_min;
            this.max = DFA9_max;
            this.accept = DFA9_accept;
            this.special = DFA9_special;
            this.transition = DFA9_transition;
        }
        public String getDescription() {
            return "1464:18: (col= column_name |agrFunction= aggregation_function )";
        }
    }
    static final String DFA34_eotS =
        "\24\uffff";
    static final String DFA34_eofS =
        "\24\uffff";
    static final String DFA34_minS =
        "\1\73\2\4\3\uffff\1\73\1\4\1\101\10\uffff\2\4\1\uffff";
    static final String DFA34_maxS =
        "\1\155\2\u0081\3\uffff\1\146\1\143\1\155\10\uffff\2\u0081\1\uffff";
    static final String DFA34_acceptS =
        "\3\uffff\1\5\1\11\1\14\3\uffff\1\2\1\3\1\4\1\6\1\7\1\10\1\12\1\13"+
        "\2\uffff\1\1";
    static final String DFA34_specialS =
        "\24\uffff}>";
    static final String[] DFA34_transitionS = {
            "\1\1\12\uffff\1\5\2\uffff\1\4\5\uffff\1\4\11\uffff\1\3\14\uffff"+
            "\1\2\6\uffff\1\5",
            "\1\16\10\uffff\1\16\1\uffff\1\14\12\uffff\1\11\13\uffff\1\12"+
            "\16\uffff\2\15\5\uffff\1\10\3\uffff\1\20\1\uffff\1\17\4\uffff"+
            "\2\15\6\uffff\1\7\1\13\22\uffff\1\16\35\uffff\1\6",
            "\1\16\10\uffff\1\16\1\uffff\1\14\12\uffff\1\11\13\uffff\1\12"+
            "\16\uffff\2\15\5\uffff\1\10\3\uffff\1\20\1\uffff\1\17\4\uffff"+
            "\2\15\6\uffff\1\7\1\13\22\uffff\1\16\35\uffff\1\6",
            "",
            "",
            "",
            "\1\21\52\uffff\1\22",
            "\1\16\10\uffff\1\16\1\uffff\1\14\54\uffff\1\10\5\uffff\1\17"+
            "\40\uffff\1\16",
            "\1\16\4\uffff\1\23\46\uffff\1\23",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\16\10\uffff\1\16\1\uffff\1\14\12\uffff\1\11\13\uffff\1\12"+
            "\16\uffff\2\15\5\uffff\1\10\3\uffff\1\20\1\uffff\1\17\4\uffff"+
            "\2\15\6\uffff\1\7\1\13\22\uffff\1\16\35\uffff\1\6",
            "\1\16\10\uffff\1\16\1\uffff\1\14\12\uffff\1\11\13\uffff\1\12"+
            "\16\uffff\2\15\5\uffff\1\10\3\uffff\1\20\1\uffff\1\17\4\uffff"+
            "\2\15\6\uffff\1\7\1\13\22\uffff\1\16\35\uffff\1\6",
            ""
    };

    static final short[] DFA34_eot = DFA.unpackEncodedString(DFA34_eotS);
    static final short[] DFA34_eof = DFA.unpackEncodedString(DFA34_eofS);
    static final char[] DFA34_min = DFA.unpackEncodedStringToUnsignedChars(DFA34_minS);
    static final char[] DFA34_max = DFA.unpackEncodedStringToUnsignedChars(DFA34_maxS);
    static final short[] DFA34_accept = DFA.unpackEncodedString(DFA34_acceptS);
    static final short[] DFA34_special = DFA.unpackEncodedString(DFA34_specialS);
    static final short[][] DFA34_transition;

    static {
        int numStates = DFA34_transitionS.length;
        DFA34_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA34_transition[i] = DFA.unpackEncodedString(DFA34_transitionS[i]);
        }
    }

    class DFA34 extends DFA {

        public DFA34(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 34;
            this.eot = DFA34_eot;
            this.eof = DFA34_eof;
            this.min = DFA34_min;
            this.max = DFA34_max;
            this.accept = DFA34_accept;
            this.special = DFA34_special;
            this.transition = DFA34_transition;
        }
        public String getDescription() {
            return "1870:1: predicate returns [JSONObject json] : ( in_predicate | contains_all_predicate | equal_predicate | not_equal_predicate | query_predicate | between_predicate | range_predicate | time_predicate | match_predicate | like_predicate | null_predicate | empty_predicate );";
        }
    }
    static final String DFA48_eotS =
        "\11\uffff";
    static final String DFA48_eofS =
        "\11\uffff";
    static final String DFA48_minS =
        "\1\73\2\4\1\73\1\4\2\uffff\2\4";
    static final String DFA48_maxS =
        "\1\146\2\u0081\1\146\1\143\2\uffff\2\u0081";
    static final String DFA48_acceptS =
        "\5\uffff\1\1\1\2\2\uffff";
    static final String DFA48_specialS =
        "\11\uffff}>";
    static final String[] DFA48_transitionS = {
            "\1\1\52\uffff\1\2",
            "\1\6\10\uffff\1\6\56\uffff\1\5\22\uffff\1\4\23\uffff\1\6\35"+
            "\uffff\1\3",
            "\1\6\10\uffff\1\6\56\uffff\1\5\22\uffff\1\4\23\uffff\1\6\35"+
            "\uffff\1\3",
            "\1\7\52\uffff\1\10",
            "\1\6\10\uffff\1\6\56\uffff\1\5\46\uffff\1\6",
            "",
            "",
            "\1\6\10\uffff\1\6\56\uffff\1\5\22\uffff\1\4\23\uffff\1\6\35"+
            "\uffff\1\3",
            "\1\6\10\uffff\1\6\56\uffff\1\5\22\uffff\1\4\23\uffff\1\6\35"+
            "\uffff\1\3"
    };

    static final short[] DFA48_eot = DFA.unpackEncodedString(DFA48_eotS);
    static final short[] DFA48_eof = DFA.unpackEncodedString(DFA48_eofS);
    static final char[] DFA48_min = DFA.unpackEncodedStringToUnsignedChars(DFA48_minS);
    static final char[] DFA48_max = DFA.unpackEncodedStringToUnsignedChars(DFA48_maxS);
    static final short[] DFA48_accept = DFA.unpackEncodedString(DFA48_acceptS);
    static final short[] DFA48_special = DFA.unpackEncodedString(DFA48_specialS);
    static final short[][] DFA48_transition;

    static {
        int numStates = DFA48_transitionS.length;
        DFA48_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA48_transition[i] = DFA.unpackEncodedString(DFA48_transitionS[i]);
        }
    }

    class DFA48 extends DFA {

        public DFA48(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 48;
            this.eot = DFA48_eot;
            this.eof = DFA48_eof;
            this.min = DFA48_min;
            this.max = DFA48_max;
            this.accept = DFA48_accept;
            this.special = DFA48_special;
            this.transition = DFA48_transition;
        }
        public String getDescription() {
            return "2223:1: time_predicate returns [JSONObject json] : ( column_name ( NOT )? IN LAST time_span | column_name ( NOT )? (since= SINCE |since= AFTER |before= BEFORE ) time_expr );";
        }
    }
    static final String DFA93_eotS =
        "\36\uffff";
    static final String DFA93_eofS =
        "\36\uffff";
    static final String DFA93_minS =
        "\1\20\1\uffff\11\46\1\uffff\6\20\6\73\6\u008e";
    static final String DFA93_maxS =
        "\1\u00aa\1\uffff\11\u00a8\1\uffff\6\u00aa\6\u008d\6\u008e";
    static final String DFA93_acceptS =
        "\1\uffff\1\1\11\uffff\1\2\22\uffff";
    static final String DFA93_specialS =
        "\36\uffff}>";
    static final String[] DFA93_transitionS = {
            "\1\2\2\uffff\1\3\1\uffff\1\13\4\uffff\1\13\7\uffff\1\5\12\uffff"+
            "\1\13\1\1\1\uffff\1\13\7\uffff\1\13\2\uffff\1\13\1\uffff\1\1"+
            "\1\13\6\uffff\1\4\1\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1"+
            "\13\6\uffff\1\13\3\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10"+
            "\uffff\2\13\1\uffff\2\13\6\uffff\1\6\1\11\1\10\1\12\1\7\2\12"+
            "\4\uffff\1\13\1\uffff\1\1\1\uffff\1\13\1\uffff\1\13\1\uffff"+
            "\2\1\2\13\2\uffff\1\13\1\1\1\uffff\1\13\1\uffff\2\13\4\uffff"+
            "\1\13",
            "",
            "\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\14\1\uffff\2\13\14\uffff\1\13\10\uffff"+
            "\3\13",
            "\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\15\1\uffff\2\13\14\uffff\1\13\10\uffff"+
            "\3\13",
            "\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\16\1\uffff\2\13\14\uffff\1\13\10\uffff"+
            "\3\13",
            "\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\17\1\uffff\2\13\14\uffff\1\13\10\uffff"+
            "\3\13",
            "\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff\1\13\10\uffff"+
            "\3\13",
            "\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff\1\13\10\uffff"+
            "\3\13",
            "\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff\1\13\10\uffff"+
            "\3\13",
            "\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff\1\13\10\uffff"+
            "\3\13",
            "\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13\32\uffff\1\13"+
            "\20\uffff\23\13\7\uffff\1\21\1\uffff\2\13\14\uffff\1\13\10\uffff"+
            "\3\13",
            "",
            "\1\13\2\uffff\1\13\1\uffff\1\13\4\uffff\1\13\7\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1"+
            "\13\6\uffff\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff"+
            "\2\13\6\uffff\7\13\1\uffff\1\22\33\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\4\uffff\1\13\7\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1"+
            "\13\6\uffff\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff"+
            "\2\13\6\uffff\7\13\1\uffff\1\23\33\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\4\uffff\1\13\7\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1"+
            "\13\6\uffff\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff"+
            "\2\13\6\uffff\7\13\1\uffff\1\24\33\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\4\uffff\1\13\7\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1"+
            "\13\6\uffff\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff"+
            "\2\13\6\uffff\7\13\1\uffff\1\25\33\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\4\uffff\1\13\7\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1"+
            "\13\6\uffff\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff"+
            "\2\13\6\uffff\7\13\1\uffff\1\26\33\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\4\uffff\1\13\7\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1"+
            "\13\6\uffff\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff"+
            "\2\13\6\uffff\7\13\1\uffff\1\27\33\uffff\1\13",
            "\1\1\105\uffff\1\13\13\uffff\1\30",
            "\1\1\105\uffff\1\13\13\uffff\1\31",
            "\1\1\105\uffff\1\13\13\uffff\1\32",
            "\1\1\105\uffff\1\13\13\uffff\1\33",
            "\1\1\105\uffff\1\13\13\uffff\1\34",
            "\1\1\105\uffff\1\13\13\uffff\1\35",
            "\1\22",
            "\1\23",
            "\1\24",
            "\1\25",
            "\1\26",
            "\1\27"
    };

    static final short[] DFA93_eot = DFA.unpackEncodedString(DFA93_eotS);
    static final short[] DFA93_eof = DFA.unpackEncodedString(DFA93_eofS);
    static final char[] DFA93_min = DFA.unpackEncodedStringToUnsignedChars(DFA93_minS);
    static final char[] DFA93_max = DFA.unpackEncodedStringToUnsignedChars(DFA93_maxS);
    static final short[] DFA93_accept = DFA.unpackEncodedString(DFA93_acceptS);
    static final short[] DFA93_special = DFA.unpackEncodedString(DFA93_specialS);
    static final short[][] DFA93_transition;

    static {
        int numStates = DFA93_transitionS.length;
        DFA93_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA93_transition[i] = DFA.unpackEncodedString(DFA93_transitionS[i]);
        }
    }

    class DFA93 extends DFA {

        public DFA93(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 93;
            this.eot = DFA93_eot;
            this.eof = DFA93_eof;
            this.min = DFA93_min;
            this.max = DFA93_max;
            this.accept = DFA93_accept;
            this.special = DFA93_special;
            this.transition = DFA93_transition;
        }
        public String getDescription() {
            return "2908:1: block_statement : ( local_variable_declaration_stmt | java_statement );";
        }
    }
    static final String DFA106_eotS =
        "\u02a0\uffff";
    static final String DFA106_eofS =
        "\u02a0\uffff";
    static final String DFA106_minS =
        "\2\20\1\73\1\30\1\73\1\30\2\73\1\30\1\73\6\30\22\uffff\16\73\1\20"+
        "\1\u008e\1\26\1\20\1\26\32\uffff\1\u008e\1\26\1\20\1\26\32\uffff"+
        "\1\u008e\1\26\1\u008e\1\26\1\20\1\26\32\uffff\1\u008e\1\26\1\20"+
        "\1\26\32\uffff\1\20\1\26\32\uffff\1\20\1\26\32\uffff\1\20\1\26\32"+
        "\uffff\1\20\1\26\32\uffff\1\20\1\26\32\uffff\54\0\5\uffff\1\0\37"+
        "\uffff\1\0\5\uffff\1\0\37\uffff\1\0\5\uffff\1\0\5\uffff\1\0\37\uffff"+
        "\1\0\5\uffff\1\0\37\uffff\1\0\37\uffff\1\0\37\uffff\1\0\37\uffff"+
        "\1\0\37\uffff\1\0\37\uffff";
    static final String DFA106_maxS =
        "\1\u00aa\1\u00a0\1\u008d\1\u00a8\1\u008d\1\u00a8\2\u008d\1\u00a8"+
        "\1\u008d\6\u00a8\22\uffff\16\u008d\1\u00a0\1\u008e\1\u008d\1\u00aa"+
        "\1\u008d\32\uffff\1\u008e\1\u008d\1\u00aa\1\u008d\32\uffff\1\u008e"+
        "\1\u008d\1\u008e\1\u008d\1\u00aa\1\u008d\32\uffff\1\u008e\1\u008d"+
        "\1\u00aa\1\u008d\32\uffff\1\u00aa\1\u008d\32\uffff\1\u00aa\1\u008d"+
        "\32\uffff\1\u00aa\1\u008d\32\uffff\1\u00aa\1\u008d\32\uffff\1\u00aa"+
        "\1\u008d\32\uffff\54\0\5\uffff\1\0\37\uffff\1\0\5\uffff\1\0\37\uffff"+
        "\1\0\5\uffff\1\0\5\uffff\1\0\37\uffff\1\0\5\uffff\1\0\37\uffff\1"+
        "\0\37\uffff\1\0\37\uffff\1\0\37\uffff\1\0\37\uffff\1\0\37\uffff";
    static final String DFA106_acceptS =
        "\20\uffff\1\2\u0152\uffff\1\1\u013c\uffff";
    static final String DFA106_specialS =
        "\u0137\uffff\1\0\1\1\1\2\1\3\1\4\1\5\1\6\1\7\1\10\1\11\1\12\1\13"+
        "\1\14\1\15\1\16\1\17\1\20\1\21\1\22\1\23\1\24\1\25\1\26\1\27\1\30"+
        "\1\31\1\32\1\33\1\34\1\35\1\36\1\37\1\40\1\41\1\42\1\43\1\44\1\45"+
        "\1\46\1\47\1\50\1\51\1\52\1\53\5\uffff\1\54\37\uffff\1\55\5\uffff"+
        "\1\56\37\uffff\1\57\5\uffff\1\60\5\uffff\1\61\37\uffff\1\62\5\uffff"+
        "\1\63\37\uffff\1\64\37\uffff\1\65\37\uffff\1\66\37\uffff\1\67\37"+
        "\uffff\1\70\37\uffff}>";
    static final String[] DFA106_transitionS = {
            "\1\3\2\uffff\1\5\1\uffff\1\20\4\uffff\1\20\7\uffff\1\12\12\uffff"+
            "\1\20\1\2\1\uffff\1\20\7\uffff\1\20\2\uffff\1\20\1\uffff\1\7"+
            "\1\20\6\uffff\1\10\1\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1"+
            "\20\6\uffff\1\20\3\uffff\1\20\2\uffff\1\20\10\uffff\1\20\10"+
            "\uffff\2\20\1\uffff\2\20\6\uffff\1\13\1\16\1\15\1\17\1\14\2"+
            "\17\6\uffff\1\4\5\uffff\1\1\1\11\5\uffff\1\6\11\uffff\1\20",
            "\1\43\2\uffff\1\45\16\uffff\1\52\13\uffff\1\42\16\uffff\1\47"+
            "\7\uffff\1\50\100\uffff\1\53\1\56\1\55\1\57\1\54\2\57\6\uffff"+
            "\1\44\5\uffff\1\60\1\51\5\uffff\1\46",
            "\1\62\121\uffff\1\61",
            "\1\20\15\uffff\1\20\16\uffff\1\20\5\uffff\1\64\12\uffff\2\20"+
            "\32\uffff\1\20\20\uffff\23\20\7\uffff\1\63\1\uffff\2\20\14\uffff"+
            "\1\20\10\uffff\3\20",
            "\1\120\121\uffff\1\117",
            "\1\20\15\uffff\1\20\16\uffff\1\20\5\uffff\1\122\12\uffff\2"+
            "\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\121\1\uffff\2\20"+
            "\14\uffff\1\20\10\uffff\3\20",
            "\1\156\121\uffff\1\155",
            "\1\160\121\uffff\1\157",
            "\1\20\15\uffff\1\20\16\uffff\1\20\5\uffff\1\162\12\uffff\2"+
            "\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\161\1\uffff\2\20"+
            "\14\uffff\1\20\10\uffff\3\20",
            "\1\u008e\121\uffff\1\u008d",
            "\1\20\15\uffff\1\20\16\uffff\1\20\5\uffff\1\u0090\12\uffff"+
            "\2\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\u008f\1\uffff\2"+
            "\20\14\uffff\1\20\10\uffff\3\20",
            "\1\20\15\uffff\1\20\16\uffff\1\20\5\uffff\1\u00ac\12\uffff"+
            "\2\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\u00ab\1\uffff\2"+
            "\20\14\uffff\1\20\10\uffff\3\20",
            "\1\20\15\uffff\1\20\16\uffff\1\20\5\uffff\1\u00c8\12\uffff"+
            "\2\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\u00c7\1\uffff\2"+
            "\20\14\uffff\1\20\10\uffff\3\20",
            "\1\20\15\uffff\1\20\16\uffff\1\20\5\uffff\1\u00e4\12\uffff"+
            "\2\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\u00e3\1\uffff\2"+
            "\20\14\uffff\1\20\10\uffff\3\20",
            "\1\20\15\uffff\1\20\16\uffff\1\20\5\uffff\1\u0100\12\uffff"+
            "\2\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\u00ff\1\uffff\2"+
            "\20\14\uffff\1\20\10\uffff\3\20",
            "\1\20\15\uffff\1\20\16\uffff\1\20\5\uffff\1\u011c\12\uffff"+
            "\2\20\32\uffff\1\20\20\uffff\23\20\7\uffff\1\u011b\1\uffff\2"+
            "\20\14\uffff\1\20\10\uffff\3\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\u0138\121\uffff\1\u0137",
            "\1\u013a\121\uffff\1\u0139",
            "\1\u013c\121\uffff\1\u013b",
            "\1\u013e\121\uffff\1\u013d",
            "\1\u0140\121\uffff\1\u013f",
            "\1\u0142\121\uffff\1\u0141",
            "\1\u0144\121\uffff\1\u0143",
            "\1\u0146\121\uffff\1\u0145",
            "\1\u0148\121\uffff\1\u0147",
            "\1\u014a\121\uffff\1\u0149",
            "\1\u014c\121\uffff\1\u014b",
            "\1\u014e\121\uffff\1\u014d",
            "\1\u0150\121\uffff\1\u014f",
            "\1\u0152\121\uffff\1\u0151",
            "\1\u0154\2\uffff\1\u0156\16\uffff\1\u015b\13\uffff\1\u0153"+
            "\16\uffff\1\u0158\7\uffff\1\u0159\100\uffff\1\u015c\1\u015f"+
            "\1\u015e\1\u0160\1\u015d\2\u0160\6\uffff\1\u0155\5\uffff\1\u0161"+
            "\1\u015a\5\uffff\1\u0157",
            "\1\u0162",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "\1\20\2\uffff\1\20\1\uffff\1\20\4\uffff\1\20\7\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1"+
            "\20\6\uffff\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff"+
            "\2\20\6\uffff\7\20\1\uffff\1\u0168\33\uffff\1\20",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\u0188",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "\1\20\2\uffff\1\20\1\uffff\1\20\4\uffff\1\20\7\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1"+
            "\20\6\uffff\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff"+
            "\2\20\6\uffff\7\20\1\uffff\1\u018e\33\uffff\1\20",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\u01ae",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "\1\u01b4",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "\1\20\2\uffff\1\20\1\uffff\1\20\4\uffff\1\20\7\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1"+
            "\20\6\uffff\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff"+
            "\2\20\6\uffff\7\20\1\uffff\1\u01ba\33\uffff\1\20",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\u01da",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "\1\20\2\uffff\1\20\1\uffff\1\20\4\uffff\1\20\7\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1"+
            "\20\6\uffff\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff"+
            "\2\20\6\uffff\7\20\1\uffff\1\u01e0\33\uffff\1\20",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\20\2\uffff\1\20\1\uffff\1\20\4\uffff\1\20\7\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1"+
            "\20\6\uffff\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff"+
            "\2\20\6\uffff\7\20\1\uffff\1\u0200\33\uffff\1\20",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\20\2\uffff\1\20\1\uffff\1\20\4\uffff\1\20\7\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1"+
            "\20\6\uffff\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff"+
            "\2\20\6\uffff\7\20\1\uffff\1\u0220\33\uffff\1\20",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\20\2\uffff\1\20\1\uffff\1\20\4\uffff\1\20\7\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1"+
            "\20\6\uffff\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff"+
            "\2\20\6\uffff\7\20\1\uffff\1\u0240\33\uffff\1\20",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\20\2\uffff\1\20\1\uffff\1\20\4\uffff\1\20\7\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1"+
            "\20\6\uffff\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff"+
            "\2\20\6\uffff\7\20\1\uffff\1\u0260\33\uffff\1\20",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\20\2\uffff\1\20\1\uffff\1\20\4\uffff\1\20\7\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\7\uffff\1\20\2\uffff\1\20\2\uffff\1"+
            "\20\6\uffff\2\20\13\uffff\1\20\1\uffff\1\20\6\uffff\1\20\12"+
            "\uffff\1\20\2\uffff\1\20\10\uffff\1\20\10\uffff\2\20\1\uffff"+
            "\2\20\6\uffff\7\20\1\uffff\1\u0280\33\uffff\1\20",
            "\1\u0163\1\uffff\1\20\15\uffff\1\20\73\uffff\1\20\52\uffff"+
            "\1\20",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            ""
    };

    static final short[] DFA106_eot = DFA.unpackEncodedString(DFA106_eotS);
    static final short[] DFA106_eof = DFA.unpackEncodedString(DFA106_eofS);
    static final char[] DFA106_min = DFA.unpackEncodedStringToUnsignedChars(DFA106_minS);
    static final char[] DFA106_max = DFA.unpackEncodedStringToUnsignedChars(DFA106_maxS);
    static final short[] DFA106_accept = DFA.unpackEncodedString(DFA106_acceptS);
    static final short[] DFA106_special = DFA.unpackEncodedString(DFA106_specialS);
    static final short[][] DFA106_transition;

    static {
        int numStates = DFA106_transitionS.length;
        DFA106_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA106_transition[i] = DFA.unpackEncodedString(DFA106_transitionS[i]);
        }
    }

    class DFA106 extends DFA {

        public DFA106(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 106;
            this.eot = DFA106_eot;
            this.eof = DFA106_eof;
            this.min = DFA106_min;
            this.max = DFA106_max;
            this.accept = DFA106_accept;
            this.special = DFA106_special;
            this.transition = DFA106_transition;
        }
        public String getDescription() {
            return "2995:1: for_control options {k=3; } : ( enhanced_for_control | ( for_init )? SEMI ( expression )? SEMI ( for_update )? );";
        }
        public int specialStateTransition(int s, IntStream _input) throws NoViableAltException {
            TokenStream input = (TokenStream)_input;
        	int _s = s;
            switch ( s ) {
                    case 0 : 
                        int LA106_311 = input.LA(1);

                         
                        int index106_311 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_311);

                        if ( s>=0 ) return s;
                        break;
                    case 1 : 
                        int LA106_312 = input.LA(1);

                         
                        int index106_312 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_312);

                        if ( s>=0 ) return s;
                        break;
                    case 2 : 
                        int LA106_313 = input.LA(1);

                         
                        int index106_313 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_313);

                        if ( s>=0 ) return s;
                        break;
                    case 3 : 
                        int LA106_314 = input.LA(1);

                         
                        int index106_314 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_314);

                        if ( s>=0 ) return s;
                        break;
                    case 4 : 
                        int LA106_315 = input.LA(1);

                         
                        int index106_315 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_315);

                        if ( s>=0 ) return s;
                        break;
                    case 5 : 
                        int LA106_316 = input.LA(1);

                         
                        int index106_316 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_316);

                        if ( s>=0 ) return s;
                        break;
                    case 6 : 
                        int LA106_317 = input.LA(1);

                         
                        int index106_317 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_317);

                        if ( s>=0 ) return s;
                        break;
                    case 7 : 
                        int LA106_318 = input.LA(1);

                         
                        int index106_318 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_318);

                        if ( s>=0 ) return s;
                        break;
                    case 8 : 
                        int LA106_319 = input.LA(1);

                         
                        int index106_319 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_319);

                        if ( s>=0 ) return s;
                        break;
                    case 9 : 
                        int LA106_320 = input.LA(1);

                         
                        int index106_320 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_320);

                        if ( s>=0 ) return s;
                        break;
                    case 10 : 
                        int LA106_321 = input.LA(1);

                         
                        int index106_321 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_321);

                        if ( s>=0 ) return s;
                        break;
                    case 11 : 
                        int LA106_322 = input.LA(1);

                         
                        int index106_322 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_322);

                        if ( s>=0 ) return s;
                        break;
                    case 12 : 
                        int LA106_323 = input.LA(1);

                         
                        int index106_323 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_323);

                        if ( s>=0 ) return s;
                        break;
                    case 13 : 
                        int LA106_324 = input.LA(1);

                         
                        int index106_324 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_324);

                        if ( s>=0 ) return s;
                        break;
                    case 14 : 
                        int LA106_325 = input.LA(1);

                         
                        int index106_325 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_325);

                        if ( s>=0 ) return s;
                        break;
                    case 15 : 
                        int LA106_326 = input.LA(1);

                         
                        int index106_326 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_326);

                        if ( s>=0 ) return s;
                        break;
                    case 16 : 
                        int LA106_327 = input.LA(1);

                         
                        int index106_327 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_327);

                        if ( s>=0 ) return s;
                        break;
                    case 17 : 
                        int LA106_328 = input.LA(1);

                         
                        int index106_328 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_328);

                        if ( s>=0 ) return s;
                        break;
                    case 18 : 
                        int LA106_329 = input.LA(1);

                         
                        int index106_329 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_329);

                        if ( s>=0 ) return s;
                        break;
                    case 19 : 
                        int LA106_330 = input.LA(1);

                         
                        int index106_330 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_330);

                        if ( s>=0 ) return s;
                        break;
                    case 20 : 
                        int LA106_331 = input.LA(1);

                         
                        int index106_331 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_331);

                        if ( s>=0 ) return s;
                        break;
                    case 21 : 
                        int LA106_332 = input.LA(1);

                         
                        int index106_332 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_332);

                        if ( s>=0 ) return s;
                        break;
                    case 22 : 
                        int LA106_333 = input.LA(1);

                         
                        int index106_333 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_333);

                        if ( s>=0 ) return s;
                        break;
                    case 23 : 
                        int LA106_334 = input.LA(1);

                         
                        int index106_334 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_334);

                        if ( s>=0 ) return s;
                        break;
                    case 24 : 
                        int LA106_335 = input.LA(1);

                         
                        int index106_335 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_335);

                        if ( s>=0 ) return s;
                        break;
                    case 25 : 
                        int LA106_336 = input.LA(1);

                         
                        int index106_336 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_336);

                        if ( s>=0 ) return s;
                        break;
                    case 26 : 
                        int LA106_337 = input.LA(1);

                         
                        int index106_337 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_337);

                        if ( s>=0 ) return s;
                        break;
                    case 27 : 
                        int LA106_338 = input.LA(1);

                         
                        int index106_338 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_338);

                        if ( s>=0 ) return s;
                        break;
                    case 28 : 
                        int LA106_339 = input.LA(1);

                         
                        int index106_339 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_339);

                        if ( s>=0 ) return s;
                        break;
                    case 29 : 
                        int LA106_340 = input.LA(1);

                         
                        int index106_340 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_340);

                        if ( s>=0 ) return s;
                        break;
                    case 30 : 
                        int LA106_341 = input.LA(1);

                         
                        int index106_341 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_341);

                        if ( s>=0 ) return s;
                        break;
                    case 31 : 
                        int LA106_342 = input.LA(1);

                         
                        int index106_342 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_342);

                        if ( s>=0 ) return s;
                        break;
                    case 32 : 
                        int LA106_343 = input.LA(1);

                         
                        int index106_343 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_343);

                        if ( s>=0 ) return s;
                        break;
                    case 33 : 
                        int LA106_344 = input.LA(1);

                         
                        int index106_344 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_344);

                        if ( s>=0 ) return s;
                        break;
                    case 34 : 
                        int LA106_345 = input.LA(1);

                         
                        int index106_345 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_345);

                        if ( s>=0 ) return s;
                        break;
                    case 35 : 
                        int LA106_346 = input.LA(1);

                         
                        int index106_346 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_346);

                        if ( s>=0 ) return s;
                        break;
                    case 36 : 
                        int LA106_347 = input.LA(1);

                         
                        int index106_347 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_347);

                        if ( s>=0 ) return s;
                        break;
                    case 37 : 
                        int LA106_348 = input.LA(1);

                         
                        int index106_348 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_348);

                        if ( s>=0 ) return s;
                        break;
                    case 38 : 
                        int LA106_349 = input.LA(1);

                         
                        int index106_349 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_349);

                        if ( s>=0 ) return s;
                        break;
                    case 39 : 
                        int LA106_350 = input.LA(1);

                         
                        int index106_350 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_350);

                        if ( s>=0 ) return s;
                        break;
                    case 40 : 
                        int LA106_351 = input.LA(1);

                         
                        int index106_351 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_351);

                        if ( s>=0 ) return s;
                        break;
                    case 41 : 
                        int LA106_352 = input.LA(1);

                         
                        int index106_352 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_352);

                        if ( s>=0 ) return s;
                        break;
                    case 42 : 
                        int LA106_353 = input.LA(1);

                         
                        int index106_353 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_353);

                        if ( s>=0 ) return s;
                        break;
                    case 43 : 
                        int LA106_354 = input.LA(1);

                         
                        int index106_354 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_354);

                        if ( s>=0 ) return s;
                        break;
                    case 44 : 
                        int LA106_360 = input.LA(1);

                         
                        int index106_360 = input.index();
                        input.rewind();

                        s = -1;
                        if ( ((((( "Boolean".equals(input.LT(1).getText()) )||( "boolean".equals(input.LT(1).getText()) ))&&(( "Boolean".equals(input.LT(1).getText()) )||( "boolean".equals(input.LT(1).getText()) )))&&synpred165_BQL())) ) {s = 355;}

                        else if ( ((( "Boolean".equals(input.LT(1).getText()) )||( "boolean".equals(input.LT(1).getText()) ))) ) {s = 16;}

                         
                        input.seek(index106_360);

                        if ( s>=0 ) return s;
                        break;
                    case 45 : 
                        int LA106_392 = input.LA(1);

                         
                        int index106_392 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_392);

                        if ( s>=0 ) return s;
                        break;
                    case 46 : 
                        int LA106_398 = input.LA(1);

                         
                        int index106_398 = input.index();
                        input.rewind();

                        s = -1;
                        if ( ((((( "Byte".equals(input.LT(1).getText()) )||( "byte".equals(input.LT(1).getText()) ))&&(( "Byte".equals(input.LT(1).getText()) )||( "byte".equals(input.LT(1).getText()) )))&&synpred165_BQL())) ) {s = 355;}

                        else if ( ((( "Byte".equals(input.LT(1).getText()) )||( "byte".equals(input.LT(1).getText()) ))) ) {s = 16;}

                         
                        input.seek(index106_398);

                        if ( s>=0 ) return s;
                        break;
                    case 47 : 
                        int LA106_430 = input.LA(1);

                         
                        int index106_430 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_430);

                        if ( s>=0 ) return s;
                        break;
                    case 48 : 
                        int LA106_436 = input.LA(1);

                         
                        int index106_436 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (((( "int".equals(input.LT(1).getText()) )&&( "int".equals(input.LT(1).getText()) ))&&synpred165_BQL())) ) {s = 355;}

                        else if ( (( "int".equals(input.LT(1).getText()) )) ) {s = 16;}

                         
                        input.seek(index106_436);

                        if ( s>=0 ) return s;
                        break;
                    case 49 : 
                        int LA106_442 = input.LA(1);

                         
                        int index106_442 = input.index();
                        input.rewind();

                        s = -1;
                        if ( ((((( "long".equals(input.LT(1).getText()) )||( "Long".equals(input.LT(1).getText()) ))&&(( "long".equals(input.LT(1).getText()) )||( "Long".equals(input.LT(1).getText()) )))&&synpred165_BQL())) ) {s = 355;}

                        else if ( ((( "long".equals(input.LT(1).getText()) )||( "Long".equals(input.LT(1).getText()) ))) ) {s = 16;}

                         
                        input.seek(index106_442);

                        if ( s>=0 ) return s;
                        break;
                    case 50 : 
                        int LA106_474 = input.LA(1);

                         
                        int index106_474 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_474);

                        if ( s>=0 ) return s;
                        break;
                    case 51 : 
                        int LA106_480 = input.LA(1);

                         
                        int index106_480 = input.index();
                        input.rewind();

                        s = -1;
                        if ( ((((( "double".equals(input.LT(1).getText()) )||( "Double".equals(input.LT(1).getText()) ))&&(( "double".equals(input.LT(1).getText()) )||( "Double".equals(input.LT(1).getText()) )))&&synpred165_BQL())) ) {s = 355;}

                        else if ( ((( "double".equals(input.LT(1).getText()) )||( "Double".equals(input.LT(1).getText()) ))) ) {s = 16;}

                         
                        input.seek(index106_480);

                        if ( s>=0 ) return s;
                        break;
                    case 52 : 
                        int LA106_512 = input.LA(1);

                         
                        int index106_512 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_512);

                        if ( s>=0 ) return s;
                        break;
                    case 53 : 
                        int LA106_544 = input.LA(1);

                         
                        int index106_544 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_544);

                        if ( s>=0 ) return s;
                        break;
                    case 54 : 
                        int LA106_576 = input.LA(1);

                         
                        int index106_576 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_576);

                        if ( s>=0 ) return s;
                        break;
                    case 55 : 
                        int LA106_608 = input.LA(1);

                         
                        int index106_608 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_608);

                        if ( s>=0 ) return s;
                        break;
                    case 56 : 
                        int LA106_640 = input.LA(1);

                         
                        int index106_640 = input.index();
                        input.rewind();

                        s = -1;
                        if ( (synpred165_BQL()) ) {s = 355;}

                        else if ( (true) ) {s = 16;}

                         
                        input.seek(index106_640);

                        if ( s>=0 ) return s;
                        break;
            }
            if (state.backtracking>0) {state.failed=true; return -1;}

            NoViableAltException nvae =
                new NoViableAltException(getDescription(), 106, _s, input);
            error(nvae);
            throw nvae;
        }

    }
    static final String DFA107_eotS =
        "\36\uffff";
    static final String DFA107_eofS =
        "\2\uffff\11\13\23\uffff";
    static final String DFA107_minS =
        "\1\20\1\uffff\11\30\1\uffff\6\20\6\73\6\u008e";
    static final String DFA107_maxS =
        "\1\u00aa\1\uffff\11\u00a8\1\uffff\6\u00aa\6\u008d\6\u008e";
    static final String DFA107_acceptS =
        "\1\uffff\1\1\11\uffff\1\2\22\uffff";
    static final String DFA107_specialS =
        "\36\uffff}>";
    static final String[] DFA107_transitionS = {
            "\1\2\2\uffff\1\3\1\uffff\1\13\4\uffff\1\13\7\uffff\1\5\12\uffff"+
            "\1\13\1\1\1\uffff\1\13\7\uffff\1\13\2\uffff\1\13\1\uffff\1\1"+
            "\1\13\6\uffff\1\4\1\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1"+
            "\13\12\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1"+
            "\uffff\2\13\6\uffff\1\6\1\11\1\10\1\12\1\7\2\12\6\uffff\1\1"+
            "\5\uffff\2\1\5\uffff\1\1\11\uffff\1\13",
            "",
            "\1\13\15\uffff\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\14\1\uffff\2\13\14\uffff"+
            "\1\13\10\uffff\3\13",
            "\1\13\15\uffff\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\15\1\uffff\2\13\14\uffff"+
            "\1\13\10\uffff\3\13",
            "\1\13\15\uffff\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\16\1\uffff\2\13\14\uffff"+
            "\1\13\10\uffff\3\13",
            "\1\13\15\uffff\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\17\1\uffff\2\13\14\uffff"+
            "\1\13\10\uffff\3\13",
            "\1\13\15\uffff\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff"+
            "\1\13\10\uffff\3\13",
            "\1\13\15\uffff\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff"+
            "\1\13\10\uffff\3\13",
            "\1\13\15\uffff\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff"+
            "\1\13\10\uffff\3\13",
            "\1\13\15\uffff\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\20\1\uffff\2\13\14\uffff"+
            "\1\13\10\uffff\3\13",
            "\1\13\15\uffff\1\13\16\uffff\1\13\5\uffff\1\1\12\uffff\2\13"+
            "\32\uffff\1\13\20\uffff\23\13\7\uffff\1\21\1\uffff\2\13\14\uffff"+
            "\1\13\10\uffff\3\13",
            "",
            "\1\13\2\uffff\1\13\1\uffff\1\13\4\uffff\1\13\7\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1"+
            "\13\6\uffff\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff"+
            "\2\13\6\uffff\7\13\1\uffff\1\22\33\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\4\uffff\1\13\7\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1"+
            "\13\6\uffff\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff"+
            "\2\13\6\uffff\7\13\1\uffff\1\23\33\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\4\uffff\1\13\7\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1"+
            "\13\6\uffff\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff"+
            "\2\13\6\uffff\7\13\1\uffff\1\24\33\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\4\uffff\1\13\7\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1"+
            "\13\6\uffff\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff"+
            "\2\13\6\uffff\7\13\1\uffff\1\25\33\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\4\uffff\1\13\7\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1"+
            "\13\6\uffff\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff"+
            "\2\13\6\uffff\7\13\1\uffff\1\26\33\uffff\1\13",
            "\1\13\2\uffff\1\13\1\uffff\1\13\4\uffff\1\13\7\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\7\uffff\1\13\2\uffff\1\13\2\uffff\1"+
            "\13\6\uffff\2\13\13\uffff\1\13\1\uffff\1\13\6\uffff\1\13\12"+
            "\uffff\1\13\2\uffff\1\13\10\uffff\1\13\10\uffff\2\13\1\uffff"+
            "\2\13\6\uffff\7\13\1\uffff\1\27\33\uffff\1\13",
            "\1\1\105\uffff\1\13\13\uffff\1\30",
            "\1\1\105\uffff\1\13\13\uffff\1\31",
            "\1\1\105\uffff\1\13\13\uffff\1\32",
            "\1\1\105\uffff\1\13\13\uffff\1\33",
            "\1\1\105\uffff\1\13\13\uffff\1\34",
            "\1\1\105\uffff\1\13\13\uffff\1\35",
            "\1\22",
            "\1\23",
            "\1\24",
            "\1\25",
            "\1\26",
            "\1\27"
    };

    static final short[] DFA107_eot = DFA.unpackEncodedString(DFA107_eotS);
    static final short[] DFA107_eof = DFA.unpackEncodedString(DFA107_eofS);
    static final char[] DFA107_min = DFA.unpackEncodedStringToUnsignedChars(DFA107_minS);
    static final char[] DFA107_max = DFA.unpackEncodedStringToUnsignedChars(DFA107_maxS);
    static final short[] DFA107_accept = DFA.unpackEncodedString(DFA107_acceptS);
    static final short[] DFA107_special = DFA.unpackEncodedString(DFA107_specialS);
    static final short[][] DFA107_transition;

    static {
        int numStates = DFA107_transitionS.length;
        DFA107_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA107_transition[i] = DFA.unpackEncodedString(DFA107_transitionS[i]);
        }
    }

    class DFA107 extends DFA {

        public DFA107(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 107;
            this.eot = DFA107_eot;
            this.eof = DFA107_eof;
            this.min = DFA107_min;
            this.max = DFA107_max;
            this.accept = DFA107_accept;
            this.special = DFA107_special;
            this.transition = DFA107_transition;
        }
        public String getDescription() {
            return "3001:1: for_init : ( local_variable_declaration | expression_list );";
        }
    }
 

    public static final BitSet FOLLOW_select_stmt_in_statement3650 = new BitSet(new long[]{0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_describe_stmt_in_statement3666 = new BitSet(new long[]{0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_SEMI_in_statement3680 = new BitSet(new long[]{0x0000000000000000L});
    public static final BitSet FOLLOW_EOF_in_statement3683 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_SELECT_in_select_stmt3711 = new BitSet(new long[]{0x0800000000000000L,0x0200004000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_121_in_select_stmt3714 = new BitSet(new long[]{0x001C840200020002L,0x0000880020400008L});
    public static final BitSet FOLLOW_selection_list_in_select_stmt3720 = new BitSet(new long[]{0x001C840200020002L,0x0000880020400008L});
    public static final BitSet FOLLOW_FROM_in_select_stmt3732 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_set_in_select_stmt3734 = new BitSet(new long[]{0x0018840200020002L,0x0000880020400008L});
    public static final BitSet FOLLOW_where_in_select_stmt3754 = new BitSet(new long[]{0x0018840200020002L,0x0000080020400008L});
    public static final BitSet FOLLOW_given_clause_in_select_stmt3767 = new BitSet(new long[]{0x0010840200020002L,0x0000080020400008L});
    public static final BitSet FOLLOW_order_by_clause_in_select_stmt3786 = new BitSet(new long[]{0x0010840200020002L,0x0000080020400008L});
    public static final BitSet FOLLOW_limit_clause_in_select_stmt3819 = new BitSet(new long[]{0x0010840200020002L,0x0000080020400008L});
    public static final BitSet FOLLOW_group_by_clause_in_select_stmt3851 = new BitSet(new long[]{0x0010840200020002L,0x0000080020400008L});
    public static final BitSet FOLLOW_distinct_clause_in_select_stmt3883 = new BitSet(new long[]{0x0010840200020002L,0x0000080020400008L});
    public static final BitSet FOLLOW_execute_clause_in_select_stmt3915 = new BitSet(new long[]{0x0010840200020002L,0x0000080020400008L});
    public static final BitSet FOLLOW_browse_by_clause_in_select_stmt3947 = new BitSet(new long[]{0x0010840200020002L,0x0000080020400008L});
    public static final BitSet FOLLOW_fetching_stored_clause_in_select_stmt3979 = new BitSet(new long[]{0x0010840200020002L,0x0000080020400008L});
    public static final BitSet FOLLOW_route_by_clause_in_select_stmt4011 = new BitSet(new long[]{0x0010840200020002L,0x0000080020400008L});
    public static final BitSet FOLLOW_relevance_model_clause_in_select_stmt4043 = new BitSet(new long[]{0x0010840200020002L,0x0000080020400008L});
    public static final BitSet FOLLOW_DESCRIBE_in_describe_stmt4107 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_set_in_describe_stmt4109 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_selection_list4147 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_aggregation_function_in_selection_list4163 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_selection_list4190 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_column_name_in_selection_list4197 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_aggregation_function_in_selection_list4213 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_function_name_in_aggregation_function4249 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_LPAR_in_aggregation_function4251 = new BitSet(new long[]{0x0800000000000000L,0x0200004000000000L});
    public static final BitSet FOLLOW_column_name_in_aggregation_function4256 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_121_in_aggregation_function4260 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_aggregation_function4263 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_IDENT_in_column_name4292 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000002L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_column_name4298 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000002L});
    public static final BitSet FOLLOW_129_in_column_name4320 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_IDENT_in_column_name4325 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000002L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_column_name4331 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000002L});
    public static final BitSet FOLLOW_158_in_function_name4391 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_function_name4395 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_WHERE_in_where4428 = new BitSet(new long[]{0x0800000000000000L,0x0000204002008240L});
    public static final BitSet FOLLOW_search_expr_in_where4431 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_ORDER_in_order_by_clause4464 = new BitSet(new long[]{0x0000000000040000L});
    public static final BitSet FOLLOW_BY_in_order_by_clause4466 = new BitSet(new long[]{0x0800000000000000L,0x0000004010000000L});
    public static final BitSet FOLLOW_RELEVANCE_in_order_by_clause4469 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_sort_specs_in_order_by_clause4473 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_sort_spec_in_sort_specs4514 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_sort_specs4535 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_sort_spec_in_sort_specs4539 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_column_name_in_sort_spec4600 = new BitSet(new long[]{0x0000000040001002L});
    public static final BitSet FOLLOW_ASC_in_sort_spec4605 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DESC_in_sort_spec4611 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LIMIT_in_limit_clause4646 = new BitSet(new long[]{0x4000000000000000L});
    public static final BitSet FOLLOW_INTEGER_in_limit_clause4651 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_limit_clause4653 = new BitSet(new long[]{0x4000000000000000L});
    public static final BitSet FOLLOW_INTEGER_in_limit_clause4659 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_comma_column_name_list4698 = new BitSet(new long[]{0x0000000001000002L,0x0000000000200000L});
    public static final BitSet FOLLOW_set_in_comma_column_name_list4719 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_column_name_in_comma_column_name_list4729 = new BitSet(new long[]{0x0000000001000002L,0x0000000000200000L});
    public static final BitSet FOLLOW_column_name_in_or_column_name_list4783 = new BitSet(new long[]{0x0000000000000002L,0x0000000000200000L});
    public static final BitSet FOLLOW_OR_in_or_column_name_list4804 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_column_name_in_or_column_name_list4808 = new BitSet(new long[]{0x0000000000000002L,0x0000000000200000L});
    public static final BitSet FOLLOW_GROUP_in_group_by_clause4856 = new BitSet(new long[]{0x0000000000040000L});
    public static final BitSet FOLLOW_BY_in_group_by_clause4858 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_comma_column_name_list_in_group_by_clause4860 = new BitSet(new long[]{0x0000000000000002L,0x0000010000000000L});
    public static final BitSet FOLLOW_TOP_in_group_by_clause4863 = new BitSet(new long[]{0x4000000000000000L});
    public static final BitSet FOLLOW_INTEGER_in_group_by_clause4867 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DISTINCT_in_distinct_clause4902 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_or_column_name_list_in_distinct_clause4904 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BROWSE_in_browse_by_clause4942 = new BitSet(new long[]{0x0000000000040000L});
    public static final BitSet FOLLOW_BY_in_browse_by_clause4944 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_facet_spec_in_browse_by_clause4948 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_browse_by_clause4969 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_facet_spec_in_browse_by_clause4973 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_EXECUTE_in_execute_clause5025 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_LPAR_in_execute_clause5027 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_function_name_in_execute_clause5031 = new BitSet(new long[]{0x0000000001000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_COMMA_in_execute_clause5053 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000002000000000L});
    public static final BitSet FOLLOW_python_style_dict_in_execute_clause5057 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_COMMA_in_execute_clause5096 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_key_value_pair_in_execute_clause5100 = new BitSet(new long[]{0x0000000001000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_execute_clause5147 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_facet_spec5180 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000040L});
    public static final BitSet FOLLOW_LPAR_in_facet_spec5204 = new BitSet(new long[]{0x0000200000000000L,0x0000020000000000L});
    public static final BitSet FOLLOW_TRUE_in_facet_spec5220 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_FALSE_in_facet_spec5226 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_facet_spec5229 = new BitSet(new long[]{0x4000000000000000L});
    public static final BitSet FOLLOW_INTEGER_in_facet_spec5245 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_facet_spec5247 = new BitSet(new long[]{0x4000000000000000L});
    public static final BitSet FOLLOW_INTEGER_in_facet_spec5263 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_facet_spec5265 = new BitSet(new long[]{0x0200000000000000L,0x0000100000000000L});
    public static final BitSet FOLLOW_HITS_in_facet_spec5280 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_VALUE_in_facet_spec5284 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_facet_spec5301 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000040L});
    public static final BitSet FOLLOW_FETCHING_in_fetching_stored_clause5350 = new BitSet(new long[]{0x0000000000000000L,0x0000001000000000L});
    public static final BitSet FOLLOW_STORED_in_fetching_stored_clause5352 = new BitSet(new long[]{0x0000200000000002L,0x0000020000000000L});
    public static final BitSet FOLLOW_TRUE_in_fetching_stored_clause5364 = new BitSet(new long[]{0x0000200000000002L,0x0000020000000000L});
    public static final BitSet FOLLOW_FALSE_in_fetching_stored_clause5368 = new BitSet(new long[]{0x0000200000000002L,0x0000020000000000L});
    public static final BitSet FOLLOW_ROUTE_in_route_by_clause5405 = new BitSet(new long[]{0x0000000000040000L});
    public static final BitSet FOLLOW_BY_in_route_by_clause5407 = new BitSet(new long[]{0x0000000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_route_by_clause5409 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_term_expr_in_search_expr5450 = new BitSet(new long[]{0x0000000000000002L,0x0000000000200000L});
    public static final BitSet FOLLOW_OR_in_search_expr5463 = new BitSet(new long[]{0x0800000000000000L,0x0000204002008240L});
    public static final BitSet FOLLOW_term_expr_in_search_expr5467 = new BitSet(new long[]{0x0000000000000002L,0x0000000000200000L});
    public static final BitSet FOLLOW_factor_expr_in_term_expr5543 = new BitSet(new long[]{0x0000000000000202L});
    public static final BitSet FOLLOW_AND_in_term_expr5556 = new BitSet(new long[]{0x0800000000000000L,0x0000204002008240L});
    public static final BitSet FOLLOW_factor_expr_in_term_expr5560 = new BitSet(new long[]{0x0000000000000202L});
    public static final BitSet FOLLOW_predicate_in_factor_expr5629 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_factor_expr5641 = new BitSet(new long[]{0x0800000000000000L,0x0000204002008240L});
    public static final BitSet FOLLOW_search_expr_in_factor_expr5643 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_factor_expr5645 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_in_predicate_in_predicate5690 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_contains_all_predicate_in_predicate5702 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_equal_predicate_in_predicate5714 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_not_equal_predicate_in_predicate5726 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_query_predicate_in_predicate5738 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_between_predicate_in_predicate5750 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_range_predicate_in_predicate5762 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_time_predicate_in_predicate5774 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_match_predicate_in_predicate5786 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_like_predicate_in_predicate5798 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_null_predicate_in_predicate5810 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_empty_predicate_in_predicate5822 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_in_predicate5847 = new BitSet(new long[]{0x1000000000000000L,0x0000000000008000L});
    public static final BitSet FOLLOW_NOT_in_in_predicate5851 = new BitSet(new long[]{0x1000000000000000L});
    public static final BitSet FOLLOW_IN_in_in_predicate5854 = new BitSet(new long[]{0x0000000000000000L,0x0000200000000040L});
    public static final BitSet FOLLOW_value_list_in_in_predicate5856 = new BitSet(new long[]{0x0000020000000002L,0x0001000000000000L});
    public static final BitSet FOLLOW_except_clause_in_in_predicate5860 = new BitSet(new long[]{0x0000000000000002L,0x0001000000000000L});
    public static final BitSet FOLLOW_predicate_props_in_in_predicate5863 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_value_list_in_empty_predicate5926 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000001L});
    public static final BitSet FOLLOW_IS_in_empty_predicate5928 = new BitSet(new long[]{0x0000001000000000L,0x0000000000008000L});
    public static final BitSet FOLLOW_NOT_in_empty_predicate5931 = new BitSet(new long[]{0x0000001000000000L});
    public static final BitSet FOLLOW_EMPTY_in_empty_predicate5935 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_contains_all_predicate5972 = new BitSet(new long[]{0x0000000004000000L});
    public static final BitSet FOLLOW_CONTAINS_in_contains_all_predicate5974 = new BitSet(new long[]{0x0000000000000080L});
    public static final BitSet FOLLOW_ALL_in_contains_all_predicate5976 = new BitSet(new long[]{0x0000000000000000L,0x0000200000000040L});
    public static final BitSet FOLLOW_value_list_in_contains_all_predicate5978 = new BitSet(new long[]{0x0000020000000002L,0x0001000000000000L});
    public static final BitSet FOLLOW_except_clause_in_contains_all_predicate5982 = new BitSet(new long[]{0x0000000000000002L,0x0001000000000000L});
    public static final BitSet FOLLOW_predicate_props_in_contains_all_predicate5985 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_equal_predicate6046 = new BitSet(new long[]{0x0000004000000000L});
    public static final BitSet FOLLOW_EQUAL_in_equal_predicate6048 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020000L});
    public static final BitSet FOLLOW_value_in_equal_predicate6050 = new BitSet(new long[]{0x0000000000000002L,0x0001000000000000L});
    public static final BitSet FOLLOW_predicate_props_in_equal_predicate6054 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_not_equal_predicate6109 = new BitSet(new long[]{0x0000000000000000L,0x0000000000010000L});
    public static final BitSet FOLLOW_NOT_EQUAL_in_not_equal_predicate6111 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020000L});
    public static final BitSet FOLLOW_value_in_not_equal_predicate6113 = new BitSet(new long[]{0x0000000000000002L,0x0001000000000000L});
    public static final BitSet FOLLOW_predicate_props_in_not_equal_predicate6115 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_QUERY_in_query_predicate6170 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000001L});
    public static final BitSet FOLLOW_IS_in_query_predicate6172 = new BitSet(new long[]{0x0000000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_query_predicate6174 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_between_predicate6223 = new BitSet(new long[]{0x0000000000008000L,0x0000000000008000L});
    public static final BitSet FOLLOW_NOT_in_between_predicate6227 = new BitSet(new long[]{0x0000000000008000L});
    public static final BitSet FOLLOW_BETWEEN_in_between_predicate6230 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020000L});
    public static final BitSet FOLLOW_value_in_between_predicate6234 = new BitSet(new long[]{0x0000000000000200L});
    public static final BitSet FOLLOW_AND_in_between_predicate6236 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020000L});
    public static final BitSet FOLLOW_value_in_between_predicate6240 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_range_predicate6296 = new BitSet(new long[]{0x0060000000000000L,0x0000000000000180L});
    public static final BitSet FOLLOW_GT_in_range_predicate6301 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020000L});
    public static final BitSet FOLLOW_GTE_in_range_predicate6307 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020000L});
    public static final BitSet FOLLOW_LT_in_range_predicate6313 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020000L});
    public static final BitSet FOLLOW_LTE_in_range_predicate6319 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020000L});
    public static final BitSet FOLLOW_value_in_range_predicate6324 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_time_predicate6376 = new BitSet(new long[]{0x1000000000000000L,0x0000000000008000L});
    public static final BitSet FOLLOW_NOT_in_time_predicate6379 = new BitSet(new long[]{0x1000000000000000L});
    public static final BitSet FOLLOW_IN_in_time_predicate6383 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000002L});
    public static final BitSet FOLLOW_LAST_in_time_predicate6385 = new BitSet(new long[]{0x4000000000000000L});
    public static final BitSet FOLLOW_time_span_in_time_predicate6387 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_time_predicate6407 = new BitSet(new long[]{0x0000000000002010L,0x0000000800008000L});
    public static final BitSet FOLLOW_NOT_in_time_predicate6410 = new BitSet(new long[]{0x0000000000002010L,0x0000000800000000L});
    public static final BitSet FOLLOW_SINCE_in_time_predicate6417 = new BitSet(new long[]{0x4000000008000040L,0x0000000000020000L});
    public static final BitSet FOLLOW_AFTER_in_time_predicate6423 = new BitSet(new long[]{0x4000000008000040L,0x0000000000020000L});
    public static final BitSet FOLLOW_BEFORE_in_time_predicate6429 = new BitSet(new long[]{0x4000000008000040L,0x0000000000020000L});
    public static final BitSet FOLLOW_time_expr_in_time_predicate6432 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_time_week_part_in_time_span6467 = new BitSet(new long[]{0x4000000000000002L});
    public static final BitSet FOLLOW_time_day_part_in_time_span6472 = new BitSet(new long[]{0x4000000000000002L});
    public static final BitSet FOLLOW_time_hour_part_in_time_span6477 = new BitSet(new long[]{0x4000000000000002L});
    public static final BitSet FOLLOW_time_minute_part_in_time_span6491 = new BitSet(new long[]{0x4000000000000002L});
    public static final BitSet FOLLOW_time_second_part_in_time_span6496 = new BitSet(new long[]{0x4000000000000002L});
    public static final BitSet FOLLOW_time_millisecond_part_in_time_span6501 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_time_week_part6535 = new BitSet(new long[]{0x0000000000000000L,0x0000400000000000L});
    public static final BitSet FOLLOW_WEEKS_in_time_week_part6537 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_time_day_part6570 = new BitSet(new long[]{0x0000000010000000L});
    public static final BitSet FOLLOW_DAYS_in_time_day_part6572 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_time_hour_part6605 = new BitSet(new long[]{0x0400000000000000L});
    public static final BitSet FOLLOW_HOURS_in_time_hour_part6607 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_time_minute_part6640 = new BitSet(new long[]{0x0000000000000000L,0x0000000000001800L});
    public static final BitSet FOLLOW_set_in_time_minute_part6642 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_time_second_part6681 = new BitSet(new long[]{0x0000000000000000L,0x0000000180000000L});
    public static final BitSet FOLLOW_set_in_time_second_part6683 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_time_millisecond_part6723 = new BitSet(new long[]{0x0000000000000000L,0x0000000000004400L});
    public static final BitSet FOLLOW_set_in_time_millisecond_part6725 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_time_span_in_time_expr6764 = new BitSet(new long[]{0x0000000000000040L});
    public static final BitSet FOLLOW_AGO_in_time_expr6766 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_date_time_string_in_time_expr6786 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_NOW_in_time_expr6806 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DATE_in_date_time_string6839 = new BitSet(new long[]{0x0000000000000002L,0x0000008000000000L});
    public static final BitSet FOLLOW_TIME_in_date_time_string6841 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_NOT_in_match_predicate6876 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000200L});
    public static final BitSet FOLLOW_MATCH_in_match_predicate6880 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_LPAR_in_match_predicate6882 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_selection_list_in_match_predicate6884 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_match_predicate6886 = new BitSet(new long[]{0x0000000000000020L});
    public static final BitSet FOLLOW_AGAINST_in_match_predicate6888 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_LPAR_in_match_predicate6890 = new BitSet(new long[]{0x0000000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_match_predicate6892 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_match_predicate6894 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_like_predicate6927 = new BitSet(new long[]{0x0000000000000000L,0x0000000000008004L});
    public static final BitSet FOLLOW_NOT_in_like_predicate6930 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000004L});
    public static final BitSet FOLLOW_LIKE_in_like_predicate6934 = new BitSet(new long[]{0x0000000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_like_predicate6936 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_column_name_in_null_predicate6969 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000001L});
    public static final BitSet FOLLOW_IS_in_null_predicate6971 = new BitSet(new long[]{0x0000000000000000L,0x0000000000048000L});
    public static final BitSet FOLLOW_NOT_in_null_predicate6974 = new BitSet(new long[]{0x0000000000000000L,0x0000000000040000L});
    public static final BitSet FOLLOW_NULL_in_null_predicate6978 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_non_variable_value_list7016 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020000L});
    public static final BitSet FOLLOW_value_in_non_variable_value_list7020 = new BitSet(new long[]{0x0000000001000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_COMMA_in_non_variable_value_list7041 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020000L});
    public static final BitSet FOLLOW_value_in_non_variable_value_list7045 = new BitSet(new long[]{0x0000000001000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_non_variable_value_list7072 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_non_variable_value_list7082 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_non_variable_value_list7084 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_141_in_python_style_list7112 = new BitSet(new long[]{0x4000200009000040L,0x0000224008020000L,0x0000002000006000L});
    public static final BitSet FOLLOW_python_style_value_in_python_style_list7116 = new BitSet(new long[]{0x0000000001000000L,0x0000000000000000L,0x0000000000004000L});
    public static final BitSet FOLLOW_COMMA_in_python_style_list7138 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020000L,0x0000002000002000L});
    public static final BitSet FOLLOW_python_style_value_in_python_style_list7142 = new BitSet(new long[]{0x0000000001000000L,0x0000000000000000L,0x0000000000004000L});
    public static final BitSet FOLLOW_142_in_python_style_list7169 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_165_in_python_style_dict7197 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000020000000000L});
    public static final BitSet FOLLOW_169_in_python_style_dict7198 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_165_in_python_style_dict7217 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_key_value_pair_in_python_style_dict7221 = new BitSet(new long[]{0x0000000001000000L,0x0000000000000000L,0x0000020000000000L});
    public static final BitSet FOLLOW_COMMA_in_python_style_dict7243 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_key_value_pair_in_python_style_dict7247 = new BitSet(new long[]{0x0000000001000000L,0x0000000000000000L,0x0000020000000000L});
    public static final BitSet FOLLOW_169_in_python_style_dict7275 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_value_in_python_style_value7298 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_python_style_list_in_python_style_value7310 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_python_style_dict_in_python_style_value7322 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_non_variable_value_list_in_value_list7347 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_VARIABLE_in_value_list7367 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_numeric_in_value7400 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_value7412 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_TRUE_in_value7432 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_FALSE_in_value7444 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_VARIABLE_in_value7456 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_time_expr_in_numeric7489 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INTEGER_in_numeric7501 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_REAL_in_numeric7513 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_EXCEPT_in_except_clause7538 = new BitSet(new long[]{0x0000000000000000L,0x0000200000000040L});
    public static final BitSet FOLLOW_value_list_in_except_clause7541 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_WITH_in_predicate_props7576 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_prop_list_in_predicate_props7579 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_prop_list7619 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_key_value_pair_in_prop_list7623 = new BitSet(new long[]{0x0000000001000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_COMMA_in_prop_list7645 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_key_value_pair_in_prop_list7649 = new BitSet(new long[]{0x0000000001000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_prop_list7677 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_key_value_pair7715 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_IDENT_in_key_value_pair7730 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_key_value_pair7750 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020000L,0x0000002000002000L});
    public static final BitSet FOLLOW_value_in_key_value_pair7755 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_python_style_list_in_key_value_pair7761 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_python_style_dict_in_key_value_pair7767 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GIVEN_in_given_clause7801 = new BitSet(new long[]{0x0000100000000000L});
    public static final BitSet FOLLOW_FACET_in_given_clause7803 = new BitSet(new long[]{0x0000000000000000L,0x0000000001000000L});
    public static final BitSet FOLLOW_PARAM_in_given_clause7805 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_facet_param_list_in_given_clause7807 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_variable_declarator_in_variable_declarators7852 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_variable_declarators7873 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_variable_declarator_in_variable_declarators7877 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_variable_declarator_id_in_variable_declarator7925 = new BitSet(new long[]{0x0000004000000002L});
    public static final BitSet FOLLOW_EQUAL_in_variable_declarator7928 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000042000001FC0L});
    public static final BitSet FOLLOW_variable_initializer_in_variable_declarator7930 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_IDENT_in_variable_declarator_id7965 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_141_in_variable_declarator_id7968 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000004000L});
    public static final BitSet FOLLOW_142_in_variable_declarator_id7970 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_array_initializer_in_variable_initializer8001 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_expression_in_variable_initializer8011 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_165_in_array_initializer8030 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000062000001FC0L});
    public static final BitSet FOLLOW_variable_initializer_in_array_initializer8033 = new BitSet(new long[]{0x0000000001000000L,0x0000000000000000L,0x0000020000000000L});
    public static final BitSet FOLLOW_COMMA_in_array_initializer8036 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000042000001FC0L});
    public static final BitSet FOLLOW_variable_initializer_in_array_initializer8038 = new BitSet(new long[]{0x0000000001000000L,0x0000000000000000L,0x0000020000000000L});
    public static final BitSet FOLLOW_COMMA_in_array_initializer8043 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000020000000000L});
    public static final BitSet FOLLOW_169_in_array_initializer8049 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_class_or_interface_type_in_type8072 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_141_in_type8075 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000004000L});
    public static final BitSet FOLLOW_142_in_type8077 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_primitive_type_in_type8099 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_141_in_type8102 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000004000L});
    public static final BitSet FOLLOW_142_in_type8104 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_boxed_type_in_type8126 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_141_in_type8129 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000004000L});
    public static final BitSet FOLLOW_142_in_type8131 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_limited_type_in_type8153 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_141_in_type8156 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000004000L});
    public static final BitSet FOLLOW_142_in_type8158 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_FAST_UTIL_DATA_TYPE_in_class_or_interface_type8193 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LT_in_type_arguments8231 = new BitSet(new long[]{0x2000400400090000L,0x0000000000000020L,0x0000000104081FE0L});
    public static final BitSet FOLLOW_type_argument_in_type_arguments8243 = new BitSet(new long[]{0x0020000001000000L});
    public static final BitSet FOLLOW_COMMA_in_type_arguments8264 = new BitSet(new long[]{0x2000400400090000L,0x0000000000000020L,0x0000000104081FE0L});
    public static final BitSet FOLLOW_type_argument_in_type_arguments8268 = new BitSet(new long[]{0x0020000001000000L});
    public static final BitSet FOLLOW_GT_in_type_arguments8295 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_type_in_type_argument8324 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_133_in_type_argument8334 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000201000000L});
    public static final BitSet FOLLOW_set_in_type_argument8337 = new BitSet(new long[]{0x2000400400090000L,0x0000000000000020L,0x0000000104081FC0L});
    public static final BitSet FOLLOW_type_in_type_argument8345 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_formal_parameters8370 = new BitSet(new long[]{0x2000400400090000L,0x0000000000000020L,0x0000000106081FC0L});
    public static final BitSet FOLLOW_formal_parameter_decls_in_formal_parameters8372 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_formal_parameters8374 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_formal_parameter_decl_in_formal_parameter_decls8414 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_formal_parameter_decls8435 = new BitSet(new long[]{0x2000400400090000L,0x0000000000000020L,0x0000000106081FC0L});
    public static final BitSet FOLLOW_formal_parameter_decl_in_formal_parameter_decls8439 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_variable_modifiers_in_formal_parameter_decl8491 = new BitSet(new long[]{0x2000400400090000L,0x0000000000000020L,0x0000000104081FC0L});
    public static final BitSet FOLLOW_type_in_formal_parameter_decl8493 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_variable_declarator_id_in_formal_parameter_decl8495 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BOOLEAN_in_primitive_type8526 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_147_in_primitive_type8536 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BYTE_in_primitive_type8548 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_160_in_primitive_type8558 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INT_in_primitive_type8570 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LONG_in_primitive_type8582 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_154_in_primitive_type8592 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DOUBLE_in_primitive_type8604 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BOOLEAN_in_boxed_type8625 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_134_in_boxed_type8635 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BYTE_in_boxed_type8647 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_138_in_boxed_type8657 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_136_in_boxed_type8667 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LONG_in_boxed_type8679 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_135_in_boxed_type8689 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DOUBLE_in_boxed_type8701 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_153_in_variable_modifier8764 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DEFINED_in_relevance_model8792 = new BitSet(new long[]{0x0000000000000800L});
    public static final BitSet FOLLOW_AS_in_relevance_model8794 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_formal_parameters_in_relevance_model8798 = new BitSet(new long[]{0x0000000000004000L});
    public static final BitSet FOLLOW_BEGIN_in_relevance_model8818 = new BitSet(new long[]{0x6901600404290000L,0xD804024408140060L,0x000004359EAA1FC0L});
    public static final BitSet FOLLOW_model_block_in_relevance_model8820 = new BitSet(new long[]{0x0000002000000000L});
    public static final BitSet FOLLOW_END_in_relevance_model8822 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_block_statement_in_model_block8851 = new BitSet(new long[]{0x6901600404290002L,0xD804024408140060L,0x000004359EAA1FC0L});
    public static final BitSet FOLLOW_165_in_block8871 = new BitSet(new long[]{0x6901600404290000L,0xD804024408140060L,0x000006359EAA1FC0L});
    public static final BitSet FOLLOW_block_statement_in_block8892 = new BitSet(new long[]{0x6901600404290000L,0xD804024408140060L,0x000006359EAA1FC0L});
    public static final BitSet FOLLOW_169_in_block8914 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_local_variable_declaration_stmt_in_block_statement8933 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_java_statement_in_block_statement8943 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_local_variable_declaration_in_local_variable_declaration_stmt8962 = new BitSet(new long[]{0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_SEMI_in_local_variable_declaration_stmt8964 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_variable_modifiers_in_local_variable_declaration8983 = new BitSet(new long[]{0x2000400400090000L,0x0000000000000020L,0x0000000104081FC0L});
    public static final BitSet FOLLOW_type_in_local_variable_declaration8985 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_variable_declarators_in_local_variable_declaration8987 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_variable_modifier_in_variable_modifiers9016 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000002000000L});
    public static final BitSet FOLLOW_block_in_java_statement9036 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_156_in_java_statement9046 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_par_expression_in_java_statement9048 = new BitSet(new long[]{0x4901200404290000L,0xD804024408140060L,0x0000043498A21FC0L});
    public static final BitSet FOLLOW_java_statement_in_java_statement9050 = new BitSet(new long[]{0x0000000800000002L});
    public static final BitSet FOLLOW_else_statement_in_java_statement9053 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_155_in_java_statement9065 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_LPAR_in_java_statement9067 = new BitSet(new long[]{0x6901600404290000L,0xD804024408140060L,0x0000040106081FC0L});
    public static final BitSet FOLLOW_for_control_in_java_statement9087 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_java_statement9089 = new BitSet(new long[]{0x4901200404290000L,0xD804024408140060L,0x0000043498A21FC0L});
    public static final BitSet FOLLOW_java_statement_in_java_statement9091 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_164_in_java_statement9111 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_par_expression_in_java_statement9113 = new BitSet(new long[]{0x4901200404290000L,0xD804024408140060L,0x0000043498A21FC0L});
    public static final BitSet FOLLOW_java_statement_in_java_statement9115 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_151_in_java_statement9125 = new BitSet(new long[]{0x4901200404290000L,0xD804024408140060L,0x0000043498A21FC0L});
    public static final BitSet FOLLOW_java_statement_in_java_statement9127 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000001000000000L});
    public static final BitSet FOLLOW_164_in_java_statement9129 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_par_expression_in_java_statement9131 = new BitSet(new long[]{0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_SEMI_in_java_statement9133 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_162_in_java_statement9143 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_par_expression_in_java_statement9145 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000002000000000L});
    public static final BitSet FOLLOW_165_in_java_statement9147 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000020000440000L});
    public static final BitSet FOLLOW_switch_block_statement_groups_in_java_statement9149 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000020000000000L});
    public static final BitSet FOLLOW_169_in_java_statement9151 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_159_in_java_statement9161 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_expression_in_java_statement9163 = new BitSet(new long[]{0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_SEMI_in_java_statement9165 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_145_in_java_statement9175 = new BitSet(new long[]{0x0800000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_IDENT_in_java_statement9177 = new BitSet(new long[]{0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_SEMI_in_java_statement9180 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_149_in_java_statement9190 = new BitSet(new long[]{0x0800000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_IDENT_in_java_statement9192 = new BitSet(new long[]{0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_SEMI_in_java_statement9195 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_SEMI_in_java_statement9205 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_statement_expression_in_java_statement9215 = new BitSet(new long[]{0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_SEMI_in_java_statement9217 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_ELSE_in_else_statement9238 = new BitSet(new long[]{0x4901200404290000L,0xD804024408140060L,0x0000043498A21FC0L});
    public static final BitSet FOLLOW_java_statement_in_else_statement9240 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_switch_block_statement_group_in_switch_block_statement_groups9260 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000440000L});
    public static final BitSet FOLLOW_switch_label_in_switch_block_statement_group9281 = new BitSet(new long[]{0x6901600404290002L,0xD804024408140060L,0x000004359EEE1FC0L});
    public static final BitSet FOLLOW_block_statement_in_switch_block_statement_group9284 = new BitSet(new long[]{0x6901600404290002L,0xD804024408140060L,0x000004359EAA1FC0L});
    public static final BitSet FOLLOW_146_in_switch_label9304 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_constant_expression_in_switch_label9306 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_switch_label9308 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_146_in_switch_label9318 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_enum_constant_name_in_switch_label9320 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_switch_label9322 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_150_in_switch_label9332 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_switch_label9334 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_enhanced_for_control_in_for_control9360 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_for_init_in_for_control9370 = new BitSet(new long[]{0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_SEMI_in_for_control9373 = new BitSet(new long[]{0x4901200404290000L,0xD804024408140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_expression_in_for_control9375 = new BitSet(new long[]{0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_SEMI_in_for_control9378 = new BitSet(new long[]{0x4901200404290002L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_for_update_in_for_control9380 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_local_variable_declaration_in_for_init9400 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_expression_list_in_for_init9410 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_variable_modifiers_in_enhanced_for_control9429 = new BitSet(new long[]{0x2000400400090000L,0x0000000000000020L,0x0000000104081FC0L});
    public static final BitSet FOLLOW_type_in_enhanced_for_control9431 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_IDENT_in_enhanced_for_control9433 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_enhanced_for_control9435 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_expression_in_enhanced_for_control9437 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_expression_list_in_for_update9456 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_par_expression9475 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_expression_in_par_expression9477 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_par_expression9479 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_expression_in_expression_list9498 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_expression_list9501 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_expression_in_expression_list9503 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_expression_in_statement_expression9524 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_expression_in_constant_expression9543 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_IDENT_in_enum_constant_name9562 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_conditional_expression_in_expression9581 = new BitSet(new long[]{0x0020004000000002L,0x2520000000000080L,0x0000008000010009L});
    public static final BitSet FOLLOW_assignment_operator_in_expression9584 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_expression_in_expression9586 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_EQUAL_in_assignment_operator9607 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_125_in_assignment_operator9617 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_128_in_assignment_operator9627 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_122_in_assignment_operator9637 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_131_in_assignment_operator9647 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_120_in_assignment_operator9657 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_167_in_assignment_operator9667 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_144_in_assignment_operator9677 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_117_in_assignment_operator9687 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LT_in_assignment_operator9708 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000080L});
    public static final BitSet FOLLOW_LT_in_assignment_operator9712 = new BitSet(new long[]{0x0000004000000000L});
    public static final BitSet FOLLOW_EQUAL_in_assignment_operator9716 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_assignment_operator9750 = new BitSet(new long[]{0x0020000000000000L});
    public static final BitSet FOLLOW_GT_in_assignment_operator9754 = new BitSet(new long[]{0x0020000000000000L});
    public static final BitSet FOLLOW_GT_in_assignment_operator9758 = new BitSet(new long[]{0x0000004000000000L});
    public static final BitSet FOLLOW_EQUAL_in_assignment_operator9762 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_assignment_operator9793 = new BitSet(new long[]{0x0020000000000000L});
    public static final BitSet FOLLOW_GT_in_assignment_operator9797 = new BitSet(new long[]{0x0000004000000000L});
    public static final BitSet FOLLOW_EQUAL_in_assignment_operator9801 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_conditional_or_expression_in_conditional_expression9830 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000020L});
    public static final BitSet FOLLOW_133_in_conditional_expression9834 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_expression_in_conditional_expression9836 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_conditional_expression9838 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_expression_in_conditional_expression9840 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_conditional_and_expression_in_conditional_or_expression9862 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000010000000000L});
    public static final BitSet FOLLOW_168_in_conditional_or_expression9866 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_conditional_and_expression_in_conditional_or_expression9868 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000010000000000L});
    public static final BitSet FOLLOW_inclusive_or_expression_in_conditional_and_expression9890 = new BitSet(new long[]{0x0000000000000002L,0x0040000000000000L});
    public static final BitSet FOLLOW_118_in_conditional_and_expression9893 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_inclusive_or_expression_in_conditional_and_expression9895 = new BitSet(new long[]{0x0000000000000002L,0x0040000000000000L});
    public static final BitSet FOLLOW_exclusive_or_expression_in_inclusive_or_expression9917 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_166_in_inclusive_or_expression9920 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_exclusive_or_expression_in_inclusive_or_expression9922 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_and_expression_in_exclusive_or_expression9944 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000008000L});
    public static final BitSet FOLLOW_143_in_exclusive_or_expression9947 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_and_expression_in_exclusive_or_expression9949 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000008000L});
    public static final BitSet FOLLOW_equality_expression_in_and_expression9971 = new BitSet(new long[]{0x0000000000000002L,0x0080000000000000L});
    public static final BitSet FOLLOW_119_in_and_expression9975 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_equality_expression_in_and_expression9977 = new BitSet(new long[]{0x0000000000000002L,0x0080000000000000L});
    public static final BitSet FOLLOW_instanceof_expression_in_equality_expression9999 = new BitSet(new long[]{0x0000000000000002L,0x0008000000000000L,0x0000000000000010L});
    public static final BitSet FOLLOW_set_in_equality_expression10003 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_instanceof_expression_in_equality_expression10011 = new BitSet(new long[]{0x0000000000000002L,0x0008000000000000L,0x0000000000000010L});
    public static final BitSet FOLLOW_relational_expression_in_instanceof_expression10033 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000020000000L});
    public static final BitSet FOLLOW_157_in_instanceof_expression10036 = new BitSet(new long[]{0x2000400400090000L,0x0000000000000020L,0x0000000104081FC0L});
    public static final BitSet FOLLOW_type_in_instanceof_expression10038 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_shift_expression_in_relational_expression10059 = new BitSet(new long[]{0x0020000000000002L,0x0000000000000080L});
    public static final BitSet FOLLOW_relational_op_in_relational_expression10063 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_shift_expression_in_relational_expression10065 = new BitSet(new long[]{0x0020000000000002L,0x0000000000000080L});
    public static final BitSet FOLLOW_LT_in_relational_op10096 = new BitSet(new long[]{0x0000004000000000L});
    public static final BitSet FOLLOW_EQUAL_in_relational_op10100 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_relational_op10130 = new BitSet(new long[]{0x0000004000000000L});
    public static final BitSet FOLLOW_EQUAL_in_relational_op10134 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LT_in_relational_op10155 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_relational_op10165 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_additive_expression_in_shift_expression10184 = new BitSet(new long[]{0x0020000000000002L,0x0000000000000080L});
    public static final BitSet FOLLOW_shift_op_in_shift_expression10188 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_additive_expression_in_shift_expression10190 = new BitSet(new long[]{0x0020000000000002L,0x0000000000000080L});
    public static final BitSet FOLLOW_LT_in_shift_op10221 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000080L});
    public static final BitSet FOLLOW_LT_in_shift_op10225 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_shift_op10257 = new BitSet(new long[]{0x0020000000000000L});
    public static final BitSet FOLLOW_GT_in_shift_op10261 = new BitSet(new long[]{0x0020000000000000L});
    public static final BitSet FOLLOW_GT_in_shift_op10265 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_shift_op10295 = new BitSet(new long[]{0x0020000000000000L});
    public static final BitSet FOLLOW_GT_in_shift_op10299 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_multiplicative_expression_in_additive_expression10328 = new BitSet(new long[]{0x0000000000000002L,0x4800000000000000L});
    public static final BitSet FOLLOW_set_in_additive_expression10332 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_multiplicative_expression_in_additive_expression10340 = new BitSet(new long[]{0x0000000000000002L,0x4800000000000000L});
    public static final BitSet FOLLOW_unary_expression_in_multiplicative_expression10362 = new BitSet(new long[]{0x0000000000000002L,0x0210000000000000L,0x0000000000000004L});
    public static final BitSet FOLLOW_set_in_multiplicative_expression10366 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_unary_expression_in_multiplicative_expression10380 = new BitSet(new long[]{0x0000000000000002L,0x0210000000000000L,0x0000000000000004L});
    public static final BitSet FOLLOW_123_in_unary_expression10406 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_unary_expression_in_unary_expression10408 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_126_in_unary_expression10418 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_unary_expression_in_unary_expression10420 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_124_in_unary_expression10430 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_unary_expression_in_unary_expression10432 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_127_in_unary_expression10442 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_unary_expression_in_unary_expression10444 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_unary_expression_not_plus_minus_in_unary_expression10454 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_170_in_unary_expression_not_plus_minus10473 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_unary_expression_in_unary_expression_not_plus_minus10475 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_114_in_unary_expression_not_plus_minus10485 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_unary_expression_in_unary_expression_not_plus_minus10487 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_cast_expression_in_unary_expression_not_plus_minus10497 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_primary_in_unary_expression_not_plus_minus10507 = new BitSet(new long[]{0x0000000000000002L,0x9000000000000000L,0x0000000000002002L});
    public static final BitSet FOLLOW_selector_in_unary_expression_not_plus_minus10509 = new BitSet(new long[]{0x0000000000000002L,0x9000000000000000L,0x0000000000002002L});
    public static final BitSet FOLLOW_LPAR_in_cast_expression10535 = new BitSet(new long[]{0x2000000400090000L,0x0000000000000020L,0x0000000104080000L});
    public static final BitSet FOLLOW_primitive_type_in_cast_expression10537 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_cast_expression10539 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_unary_expression_in_cast_expression10541 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_cast_expression10550 = new BitSet(new long[]{0x6901600404290000L,0xD804024008140060L,0x0000040104081FC0L});
    public static final BitSet FOLLOW_type_in_cast_expression10553 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_expression_in_cast_expression10557 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_cast_expression10560 = new BitSet(new long[]{0x4901200404290000L,0x0004024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_unary_expression_not_plus_minus_in_cast_expression10562 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_par_expression_in_primary10581 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_literal_in_primary10591 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_java_method_in_primary10604 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L,0x0000000000002002L});
    public static final BitSet FOLLOW_identifier_suffix_in_primary10606 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_java_ident_in_primary10616 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000040L,0x0000000000002002L});
    public static final BitSet FOLLOW_129_in_primary10619 = new BitSet(new long[]{0x0800000004000000L});
    public static final BitSet FOLLOW_java_method_in_primary10621 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000040L,0x0000000000002002L});
    public static final BitSet FOLLOW_identifier_suffix_in_primary10625 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_boxed_type_in_java_ident10663 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_limited_type_in_java_ident10673 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_IDENT_in_java_ident10683 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_CONTAINS_in_java_method10706 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_IDENT_in_java_method10716 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_141_in_identifier_suffix10736 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000004000L});
    public static final BitSet FOLLOW_142_in_identifier_suffix10738 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000002002L});
    public static final BitSet FOLLOW_129_in_identifier_suffix10742 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000100000L});
    public static final BitSet FOLLOW_148_in_identifier_suffix10744 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_arguments_in_identifier_suffix10754 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_129_in_identifier_suffix10764 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000100000L});
    public static final BitSet FOLLOW_148_in_identifier_suffix10766 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_129_in_identifier_suffix10776 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_163_in_identifier_suffix10778 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_129_in_identifier_suffix10788 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000200000000L});
    public static final BitSet FOLLOW_161_in_identifier_suffix10790 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_arguments_in_identifier_suffix10792 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_integer_literal_in_literal10812 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_REAL_in_literal10822 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_FLOATING_POINT_LITERAL_in_literal10832 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_CHARACTER_LITERAL_in_literal10842 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_literal10852 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_boolean_literal_in_literal10862 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_NULL_in_literal10874 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_TRUE_in_boolean_literal10934 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_FALSE_in_boolean_literal10946 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_129_in_selector10965 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_IDENT_in_selector10967 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000040L});
    public static final BitSet FOLLOW_arguments_in_selector10969 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_129_in_selector10980 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_163_in_selector10982 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_141_in_selector10992 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_expression_in_selector10994 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000004000L});
    public static final BitSet FOLLOW_142_in_selector10996 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_arguments11015 = new BitSet(new long[]{0x4901200404290000L,0xD804024048140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_expression_list_in_arguments11017 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_arguments11020 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_USING_in_relevance_model_clause11052 = new BitSet(new long[]{0x0000000000000000L,0x0000000010000000L});
    public static final BitSet FOLLOW_RELEVANCE_in_relevance_model_clause11054 = new BitSet(new long[]{0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_MODEL_in_relevance_model_clause11056 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_IDENT_in_relevance_model_clause11058 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_prop_list_in_relevance_model_clause11060 = new BitSet(new long[]{0x0000000020000002L});
    public static final BitSet FOLLOW_relevance_model_in_relevance_model_clause11065 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_facet_param_in_facet_param_list11106 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_COMMA_in_facet_param_list11127 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_facet_param_in_facet_param_list11131 = new BitSet(new long[]{0x0000000001000002L});
    public static final BitSet FOLLOW_LPAR_in_facet_param11179 = new BitSet(new long[]{0x0800000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_column_name_in_facet_param11181 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_facet_param11183 = new BitSet(new long[]{0x0000000000000000L,0x0000004000000000L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_facet_param11185 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_facet_param11187 = new BitSet(new long[]{0x2000000400110000L,0x0000002000000020L});
    public static final BitSet FOLLOW_facet_param_type_in_facet_param11189 = new BitSet(new long[]{0x0000000001000000L});
    public static final BitSet FOLLOW_COMMA_in_facet_param11191 = new BitSet(new long[]{0x4000200008000040L,0x0000224008020040L});
    public static final BitSet FOLLOW_value_in_facet_param11196 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_non_variable_value_list_in_facet_param11202 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_facet_param11205 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BOOLEAN_in_facet_param_type11274 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_INT_in_facet_param_type11280 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LONG_in_facet_param_type11286 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_STRING_in_facet_param_type11292 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_BYTEARRAY_in_facet_param_type11298 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_DOUBLE_in_facet_param_type11304 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_STRING_LITERAL_in_synpred105_BQL7715 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_primitive_type_in_synpred118_BQL8099 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_141_in_synpred118_BQL8102 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000004000L});
    public static final BitSet FOLLOW_142_in_synpred118_BQL8104 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_boxed_type_in_synpred120_BQL8126 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_141_in_synpred120_BQL8129 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000004000L});
    public static final BitSet FOLLOW_142_in_synpred120_BQL8131 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_else_statement_in_synpred148_BQL9053 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_switch_label_in_synpred161_BQL9281 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_146_in_synpred163_BQL9304 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_constant_expression_in_synpred163_BQL9306 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_synpred163_BQL9308 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_146_in_synpred164_BQL9318 = new BitSet(new long[]{0x0800000000000000L});
    public static final BitSet FOLLOW_enum_constant_name_in_synpred164_BQL9320 = new BitSet(new long[]{0x0000000000400000L});
    public static final BitSet FOLLOW_COLON_in_synpred164_BQL9322 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_enhanced_for_control_in_synpred165_BQL9360 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_assignment_operator_in_synpred171_BQL9584 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_expression_in_synpred171_BQL9586 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LT_in_synpred181_BQL9698 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000080L});
    public static final BitSet FOLLOW_LT_in_synpred181_BQL9700 = new BitSet(new long[]{0x0000004000000000L});
    public static final BitSet FOLLOW_EQUAL_in_synpred181_BQL9702 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_synpred182_BQL9738 = new BitSet(new long[]{0x0020000000000000L});
    public static final BitSet FOLLOW_GT_in_synpred182_BQL9740 = new BitSet(new long[]{0x0020000000000000L});
    public static final BitSet FOLLOW_GT_in_synpred182_BQL9742 = new BitSet(new long[]{0x0000004000000000L});
    public static final BitSet FOLLOW_EQUAL_in_synpred182_BQL9744 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_synpred183_BQL9783 = new BitSet(new long[]{0x0020000000000000L});
    public static final BitSet FOLLOW_GT_in_synpred183_BQL9785 = new BitSet(new long[]{0x0000004000000000L});
    public static final BitSet FOLLOW_EQUAL_in_synpred183_BQL9787 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LT_in_synpred194_BQL10088 = new BitSet(new long[]{0x0000004000000000L});
    public static final BitSet FOLLOW_EQUAL_in_synpred194_BQL10090 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_synpred195_BQL10122 = new BitSet(new long[]{0x0000004000000000L});
    public static final BitSet FOLLOW_EQUAL_in_synpred195_BQL10124 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LT_in_synpred198_BQL10213 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000080L});
    public static final BitSet FOLLOW_LT_in_synpred198_BQL10215 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_synpred199_BQL10247 = new BitSet(new long[]{0x0020000000000000L});
    public static final BitSet FOLLOW_GT_in_synpred199_BQL10249 = new BitSet(new long[]{0x0020000000000000L});
    public static final BitSet FOLLOW_GT_in_synpred199_BQL10251 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_GT_in_synpred200_BQL10287 = new BitSet(new long[]{0x0020000000000000L});
    public static final BitSet FOLLOW_GT_in_synpred200_BQL10289 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_cast_expression_in_synpred212_BQL10497 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_LPAR_in_synpred216_BQL10535 = new BitSet(new long[]{0x2000000400090000L,0x0000000000000020L,0x0000000104080000L});
    public static final BitSet FOLLOW_primitive_type_in_synpred216_BQL10537 = new BitSet(new long[]{0x0000000000000000L,0x0000000040000000L});
    public static final BitSet FOLLOW_RPAR_in_synpred216_BQL10539 = new BitSet(new long[]{0x4901200404290000L,0xD804024008140060L,0x0000040000001FC0L});
    public static final BitSet FOLLOW_unary_expression_in_synpred216_BQL10541 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_type_in_synpred217_BQL10553 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_java_method_in_synpred220_BQL10604 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000040L,0x0000000000002002L});
    public static final BitSet FOLLOW_identifier_suffix_in_synpred220_BQL10606 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_129_in_synpred221_BQL10619 = new BitSet(new long[]{0x0800000004000000L});
    public static final BitSet FOLLOW_java_method_in_synpred221_BQL10621 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_identifier_suffix_in_synpred222_BQL10625 = new BitSet(new long[]{0x0000000000000002L});

}